@@ -46,6 +46,9 @@
 #include "ZycoreFormat.h"
 #include "ZydisShortString.h"
 #include "ZydisStatus.h"
+#include <wtf/Compiler.h>
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
 
 #ifdef __cplusplus
 extern "C" {
@@ -461,4 +464,6 @@ ZYAN_INLINE ZyanStatus ZydisStringAppendHexS(ZyanString* string, ZyanI64 value,
 }
 #endif
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
+
 #endif // ZYDIS_INTERNAL_STRING_H
