@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2013-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2022 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,6 +28,7 @@
 
 #if ENABLE(FTL_JIT)
 
+#include "AssemblyHelpersSpoolers.h"
 #include "BytecodeStructs.h"
 #include "CheckpointOSRExitSideState.h"
 #include "DFGOSRExitCompilerCommon.h"
@@ -45,6 +46,8 @@
 
 #include <wtf/Scope.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC { namespace FTL {
 
 using namespace DFG;
@@ -88,7 +91,7 @@ static void reboxAccordingToFormat(
     case DataFormatDouble: {
         jit.moveDoubleTo64(FPRInfo::fpRegT0, scratch1);
         jit.move64ToDouble(value, FPRInfo::fpRegT0);
-        jit.purifyNaN(FPRInfo::fpRegT0);
+        jit.purifyNaN(FPRInfo::fpRegT0, FPRInfo::fpRegT0);
         jit.boxDouble(FPRInfo::fpRegT0, value);
         jit.move64ToDouble(scratch1, FPRInfo::fpRegT0);
         break;
@@ -104,7 +107,7 @@ static void compileRecovery(
     CCallHelpers& jit, const ExitValue& value,
     const FixedVector<B3::ValueRep>& valueReps,
     char* registerScratch,
-    const HashMap<ExitTimeObjectMaterialization*, EncodedJSValue*>& materializationToPointer)
+    const UncheckedKeyHashMap<ExitTimeObjectMaterialization*, EncodedJSValue*>& materializationToPointer)
 {
     switch (value.kind()) {
     case ExitValueDead:
@@ -147,6 +150,15 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
 
     CCallHelpers jit(codeBlock);
 
+    if (UNLIKELY(Options::printEachOSRExit())) {
+        SpeculationFailureDebugInfo* debugInfo = new SpeculationFailureDebugInfo;
+        debugInfo->codeBlock = jit.codeBlock();
+        debugInfo->kind = exit.m_kind;
+        debugInfo->exitIndex = exitID;
+        debugInfo->bytecodeIndex = exit.m_codeOrigin.bytecodeIndex();
+        jit.probe(tagCFunction<JITProbePtrTag>(operationDebugPrintSpeculationFailure), debugInfo, SavedFPWidth::DontSaveVectors);
+    }
+
     // The first thing we need to do is restablish our frame in the case of an exception.
     if (exit.isGenericUnwindHandler()) {
         RELEASE_ASSERT(vm.callFrameForCatch); // The first time we hit this exit, like at all other times, this field should be non-null.
@@ -181,14 +193,14 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         sizeof(EncodedJSValue) * (
             exit.m_descriptor->m_values.size() + numMaterializations + maxMaterializationNumArguments) +
         requiredScratchMemorySizeInBytes() +
-        codeBlock->jitCode()->calleeSaveRegisters()->size() * sizeof(uint64_t));
+        codeBlock->jitCode()->calleeSaveRegisters()->sizeOfAreaInBytes());
     EncodedJSValue* scratch = scratchBuffer ? static_cast<EncodedJSValue*>(scratchBuffer->dataBuffer()) : nullptr;
     EncodedJSValue* materializationPointers = scratch + exit.m_descriptor->m_values.size();
     EncodedJSValue* materializationArguments = materializationPointers + numMaterializations;
-    char* registerScratch = bitwise_cast<char*>(materializationArguments + maxMaterializationNumArguments);
-    uint64_t* unwindScratch = bitwise_cast<uint64_t*>(registerScratch + requiredScratchMemorySizeInBytes());
+    char* registerScratch = std::bit_cast<char*>(materializationArguments + maxMaterializationNumArguments);
+    uint64_t* unwindScratch = std::bit_cast<uint64_t*>(registerScratch + requiredScratchMemorySizeInBytes());
     
-    HashMap<ExitTimeObjectMaterialization*, EncodedJSValue*> materializationToPointer;
+    UncheckedKeyHashMap<ExitTimeObjectMaterialization*, EncodedJSValue*> materializationToPointer;
     unsigned materializationCount = 0;
     for (ExitTimeObjectMaterialization* materialization : exit.m_descriptor->m_materializations) {
         materializationToPointer.add(
@@ -241,28 +253,20 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     
     // Get the call frame and tag thingies.
     // Restore the exiting function's callFrame value into a regT4
-    jit.move(MacroAssembler::TrustedImm64(JSValue::NumberTag), GPRInfo::numberTagRegister);
-    jit.move(MacroAssembler::TrustedImm64(JSValue::NotCellMask), GPRInfo::notCellMaskRegister);
+    jit.emitMaterializeTagCheckRegisters();
     
     // Do some value profiling.
     if (exit.m_descriptor->m_profileDataFormat != DataFormatNone) {
         Location::forValueRep(exit.m_valueReps[0]).restoreInto(jit, registerScratch, GPRInfo::regT0);
-        reboxAccordingToFormat(
-            exit.m_descriptor->m_profileDataFormat, jit, GPRInfo::regT0, GPRInfo::regT1, GPRInfo::regT2);
+        reboxAccordingToFormat(exit.m_descriptor->m_profileDataFormat, jit, GPRInfo::regT0, GPRInfo::regT1, GPRInfo::regT2);
         
         if (exit.m_kind == BadCache || exit.m_kind == BadIndexingType) {
             CodeOrigin codeOrigin = exit.m_codeOriginForExitProfile;
             CodeBlock* codeBlock = jit.baselineCodeBlockFor(codeOrigin);
-            if (ArrayProfile* arrayProfile = codeBlock->getArrayProfile(codeOrigin.bytecodeIndex())) {
-                const Instruction* instruction = codeBlock->instructions().at(codeOrigin.bytecodeIndex()).ptr();
-                CCallHelpers::Jump skipProfile;
-                if (instruction->is<OpGetById>()) {
-                    auto& metadata = instruction->as<OpGetById>().metadata(codeBlock);
-                    skipProfile = jit.branch8(CCallHelpers::NotEqual, CCallHelpers::AbsoluteAddress(&metadata.m_modeMetadata.mode), CCallHelpers::TrustedImm32(static_cast<uint8_t>(GetByIdMode::ArrayLength)));
-                }
-
+            if (ArrayProfile* arrayProfile = codeBlock->getArrayProfile(ConcurrentJSLocker(codeBlock->m_lock), codeOrigin.bytecodeIndex())) {
+                jit.move(CCallHelpers::TrustedImmPtr(arrayProfile), GPRInfo::regT3);
                 jit.load32(MacroAssembler::Address(GPRInfo::regT0, JSCell::structureIDOffset()), GPRInfo::regT1);
-                jit.store32(GPRInfo::regT1, arrayProfile->addressOfLastSeenStructureID());
+                jit.store32(GPRInfo::regT1, CCallHelpers::Address(GPRInfo::regT3, ArrayProfile::offsetOfSpeculationFailureStructureID()));
 
                 jit.load8(MacroAssembler::Address(GPRInfo::regT0, JSCell::typeInfoTypeOffset()), GPRInfo::regT2);
                 jit.sub32(MacroAssembler::TrustedImm32(FirstTypedArrayType), GPRInfo::regT2);
@@ -277,15 +281,12 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
                 jit.move(MacroAssembler::TrustedImm32(1), GPRInfo::regT2);
                 jit.lshift32(GPRInfo::regT1, GPRInfo::regT2);
                 storeArrayModes.link(&jit);
-                jit.or32(GPRInfo::regT2, MacroAssembler::AbsoluteAddress(arrayProfile->addressOfArrayModes()));
-
-                if (skipProfile.isSet())
-                    skipProfile.link(&jit);
+                jit.or32(GPRInfo::regT2, CCallHelpers::Address(GPRInfo::regT3, ArrayProfile::offsetOfArrayModes()));
             }
         }
 
         if (exit.m_descriptor->m_valueProfile)
-            exit.m_descriptor->m_valueProfile.emitReportValue(jit, JSValueRegs(GPRInfo::regT0), GPRInfo::regT1);
+            exit.m_descriptor->m_valueProfile.emitReportValue(jit, jit.codeBlock(), JSValueRegs(GPRInfo::regT0), GPRInfo::regT1);
     }
 
     // Materialize all objects. Don't materialize an object until all
@@ -294,7 +295,7 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     // needing another object if the later is needed for the
     // allocation of the former.
 
-    HashSet<ExitTimeObjectMaterialization*> toMaterialize;
+    UncheckedKeyHashSet<ExitTimeObjectMaterialization*> toMaterialize;
     for (ExitTimeObjectMaterialization* materialization : exit.m_descriptor->m_materializations)
         toMaterialize.add(materialization);
 
@@ -377,9 +378,65 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     // Save all state from wherever the exit data tells us it was, into the appropriate place in
     // the scratch buffer. This also does the reboxing.
     
-    for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
-        recoverValue(exit.m_descriptor->m_values[index]);
-        jit.store64(GPRInfo::regT0, scratch + index);
+    {
+        std::optional<GPRReg> undefinedGPR;
+        jit.move(CCallHelpers::TrustedImmPtr(scratch), GPRInfo::regT3);
+        CCallHelpers::CopySpooler spooler(jit, CCallHelpers::framePointerRegister, GPRInfo::regT3, GPRInfo::regT0, GPRInfo::regT1);
+        for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
+            auto& value = exit.m_descriptor->m_values[index];
+            if (value.dataFormat() == DataFormatJS) {
+                switch (value.kind()) {
+                case ExitValueDead:
+                    if (UNLIKELY(!undefinedGPR)) {
+                        jit.move(CCallHelpers::TrustedImm64(JSValue::encode(jsUndefined())), GPRInfo::regT4);
+                        undefinedGPR = GPRInfo::regT4;
+                    }
+                    spooler.copyGPR(undefinedGPR.value());
+                    spooler.storeGPR(index * sizeof(EncodedJSValue));
+                    break;
+
+                case ExitValueConstant: {
+                    EncodedJSValue currentConstant = JSValue::encode(value.constant());
+                    if (currentConstant == encodedJSUndefined()) {
+                        if (UNLIKELY(!undefinedGPR)) {
+                            jit.move(CCallHelpers::TrustedImm64(JSValue::encode(jsUndefined())), GPRInfo::regT4);
+                            undefinedGPR = GPRInfo::regT4;
+                        }
+                        spooler.copyGPR(undefinedGPR.value());
+                    } else
+                        spooler.moveConstant(currentConstant);
+                    spooler.storeGPR(index * sizeof(EncodedJSValue));
+                    break;
+                }
+
+                case ExitValueArgument:
+                    Location::forValueRep(exit.m_valueReps[value.exitArgument().argument()]).restoreInto(jit, registerScratch, GPRInfo::regT0);
+                    jit.store64(GPRInfo::regT0, CCallHelpers::Address(GPRInfo::regT3, index * sizeof(EncodedJSValue)));
+                    break;
+
+                case ExitValueInJSStack:
+                case ExitValueInJSStackAsInt32:
+                case ExitValueInJSStackAsInt52:
+                case ExitValueInJSStackAsDouble:
+                    spooler.loadGPR(value.virtualRegister().offset() * sizeof(EncodedJSValue));
+                    spooler.storeGPR(index * sizeof(EncodedJSValue));
+                    break;
+
+                case ExitValueMaterializeNewObject:
+                    jit.loadPtr(materializationToPointer.get(value.objectMaterialization()), GPRInfo::regT0);
+                    jit.store64(GPRInfo::regT0, CCallHelpers::Address(GPRInfo::regT3, index * sizeof(EncodedJSValue)));
+                    break;
+
+                default:
+                    RELEASE_ASSERT_NOT_REACHED();
+                    break;
+                }
+            } else {
+                recoverValue(value);
+                jit.store64(GPRInfo::regT0, CCallHelpers::Address(GPRInfo::regT3, index * sizeof(EncodedJSValue)));
+            }
+        }
+        spooler.finalizeGPR();
     }
     
     // Henceforth we make it look like the exiting function was called through a register
@@ -390,12 +447,18 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     
     // Before we start messing with the frame, we need to set aside any registers that the
     // FTL code was preserving.
-    for (unsigned i = codeBlock->jitCode()->calleeSaveRegisters()->size(); i--;) {
-        RegisterAtOffset entry = codeBlock->jitCode()->calleeSaveRegisters()->at(i);
-        jit.load64(
-            MacroAssembler::Address(MacroAssembler::framePointerRegister, entry.offset()),
-            GPRInfo::regT0);
-        jit.store64(GPRInfo::regT0, unwindScratch + i);
+    {
+        constexpr GPRReg srcBufferGPR = GPRInfo::regT2;
+        constexpr GPRReg destBufferGPR = GPRInfo::regT3;
+        jit.move(CCallHelpers::framePointerRegister, srcBufferGPR);
+        jit.move(CCallHelpers::TrustedImmPtr(unwindScratch), destBufferGPR);
+        CCallHelpers::CopySpooler spooler(CCallHelpers::CopySpooler::BufferRegs::AllowModification, jit, srcBufferGPR, destBufferGPR, GPRInfo::regT0, GPRInfo::regT1);
+        for (unsigned i = codeBlock->jitCode()->calleeSaveRegisters()->registerCount(); i--;) {
+            RegisterAtOffset entry = codeBlock->jitCode()->calleeSaveRegisters()->at(i);
+            spooler.loadGPR(entry.offset());
+            spooler.storeGPR(i * sizeof(uint64_t));
+        }
+        spooler.finalizeGPR();
     }
     
     CodeBlock* baselineCodeBlock = jit.baselineCodeBlockFor(exit.m_codeOrigin);
@@ -411,45 +474,112 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
         MacroAssembler::framePointerRegister, MacroAssembler::stackPointerRegister);
     jit.checkStackPointerAlignment();
 
-    RegisterSet allFTLCalleeSaves = RegisterSet::ftlCalleeSaveRegisters();
-    const RegisterAtOffsetList* baselineCalleeSaves = baselineCodeBlock->jitCode()->calleeSaveRegisters();
-
-    for (Reg reg = Reg::first(); reg <= Reg::last(); reg = reg.next()) {
-        if (!allFTLCalleeSaves.get(reg)) {
-            continue;
-        }
-        unsigned unwindIndex = codeBlock->jitCode()->calleeSaveRegisters()->indexOf(reg);
-        const RegisterAtOffset* baselineRegisterOffset = baselineCalleeSaves->find(reg);
-
-        if (reg.isGPR()) {
-            GPRReg regToLoad = baselineRegisterOffset ? GPRInfo::regT0 : reg.gpr();
-
-            if (unwindIndex == UINT_MAX) {
-                // The FTL compilation didn't preserve this register. This means that it also
-                // didn't use the register. So its value at the beginning of OSR exit should be
-                // preserved by the thunk. Luckily, we saved all registers into the register
-                // scratch buffer, so we can restore them from there.
-                jit.load64(registerScratch + offsetOfReg(reg), regToLoad);
-            } else {
-                // The FTL compilation preserved the register. Its new value is therefore
-                // irrelevant, but we can get the value that was preserved by using the unwind
-                // data. We've already copied all unwind-able preserved registers into the unwind
-                // scratch buffer, so we can get it from there.
-                jit.load64(unwindScratch + unwindIndex, regToLoad);
+    {
+        auto allFTLCalleeSaves = RegisterSetBuilder::ftlCalleeSaveRegisters();
+        const RegisterAtOffsetList* baselineCalleeSaves = baselineCodeBlock->jitCode()->calleeSaveRegisters();
+        auto iterateCalleeSavesImpl = [&](auto check, auto func) {
+            for (Reg reg = Reg::first(); reg <= Reg::last(); reg = reg.next()) {
+                if (!allFTLCalleeSaves.contains(reg, IgnoreVectors))
+                    continue;
+                if (!check(reg))
+                    continue;
+                unsigned unwindIndex = codeBlock->jitCode()->calleeSaveRegisters()->indexOf(reg);
+                const RegisterAtOffset* baselineRegisterOffset = baselineCalleeSaves->find(reg);
+                func(reg, unwindIndex, baselineRegisterOffset);
             }
+        };
 
-            if (baselineRegisterOffset)
-                jit.store64(regToLoad, MacroAssembler::Address(MacroAssembler::framePointerRegister, baselineRegisterOffset->offset()));
-        } else {
-            FPRReg fpRegToLoad = baselineRegisterOffset ? FPRInfo::fpRegT0 : reg.fpr();
+        auto iterateGPRCalleeSaves = [&](auto func) {
+            iterateCalleeSavesImpl([](Reg reg) { return reg.isGPR(); }, func);
+        };
 
-            if (unwindIndex == UINT_MAX)
-                jit.loadDouble(MacroAssembler::TrustedImmPtr(registerScratch + offsetOfReg(reg)), fpRegToLoad);
-            else
-                jit.loadDouble(MacroAssembler::TrustedImmPtr(unwindScratch + unwindIndex), fpRegToLoad);
+        auto iterateFPRCalleeSaves = [&](auto func) {
+            iterateCalleeSavesImpl([](Reg reg) { return reg.isFPR(); }, func);
+        };
 
-            if (baselineRegisterOffset)
-                jit.storeDouble(fpRegToLoad, MacroAssembler::Address(MacroAssembler::framePointerRegister, baselineRegisterOffset->offset()));
+        {
+            // unwindIndex == UINT_MAX indicates that the FTL compilation didn't preserve these registers.
+            // This means that it also didn't use them. Their values at the beginning of OSR exit should
+            // be the ones to retain. We saved all registers into the register scratch buffer at the beginning
+            // of the thunk. So we can restore them from there.
+            ASSERT(!allFTLCalleeSaves.contains(GPRInfo::regT3, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(GPRInfo::regT0, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(GPRInfo::regT1, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(FPRInfo::fpRegT0, IgnoreVectors));
+            ASSERT(!allFTLCalleeSaves.contains(FPRInfo::fpRegT1, IgnoreVectors));
+            jit.move(CCallHelpers::TrustedImmPtr(registerScratch), GPRInfo::regT3);
+            {
+                // Load from registerScratch buffer to callee-save registers.
+                CCallHelpers::LoadRegSpooler spooler(jit, GPRInfo::regT3);
+                iterateGPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadGPR({ reg, static_cast<ptrdiff_t>(offsetOfReg(reg)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadFPR({ reg, static_cast<ptrdiff_t>(offsetOfReg(reg)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeFPR();
+            }
+            {
+                // Copy from registerScratch buffer to call frame.
+                CCallHelpers::CopySpooler spooler(jit, GPRInfo::regT3, CCallHelpers::framePointerRegister, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0, FPRInfo::fpRegT1);
+                iterateGPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadGPR(offsetOfReg(reg));
+                        spooler.storeGPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex == UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadFPR(offsetOfReg(reg));
+                        spooler.storeFPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeFPR();
+            }
+        }
+        {
+            // The FTL compilation preserved these registers. Their new values are therefore irrelevant,
+            // but we can get their values that were preserved by using the unwind data. We've already
+            // copied all unwind-able preserved registers into the unwind scratch buffer, so we can get
+            // the values to restore from there.
+            ASSERT((std::bit_cast<uintptr_t>(unwindScratch) - std::bit_cast<uintptr_t>(registerScratch)) == requiredScratchMemorySizeInBytes());
+            jit.addPtr(CCallHelpers::TrustedImm32(requiredScratchMemorySizeInBytes()), GPRInfo::regT3); // Change registerScratch to unwindScratch.
+            {
+                // Load from unwindScratch buffer to callee-save registers.
+                CCallHelpers::LoadRegSpooler spooler(jit, GPRInfo::regT3);
+                iterateGPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadGPR({ reg, static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && !baselineRegisterOffset)
+                        spooler.loadFPR({ reg, static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)), conservativeWidthWithoutVectors(reg) });
+                });
+                spooler.finalizeFPR();
+            }
+            {
+                // Copy from unwindScratch buffer to call frame.
+                CCallHelpers::CopySpooler spooler(jit, GPRInfo::regT3, CCallHelpers::framePointerRegister, GPRInfo::regT0, GPRInfo::regT1, FPRInfo::fpRegT0, FPRInfo::fpRegT1);
+                iterateGPRCalleeSaves([&](Reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadGPR(static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)));
+                        spooler.storeGPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeGPR();
+                iterateFPRCalleeSaves([&](Reg, unsigned unwindIndex, const RegisterAtOffset* baselineRegisterOffset) {
+                    if (unwindIndex != UINT_MAX && baselineRegisterOffset) {
+                        spooler.loadFPR(static_cast<ptrdiff_t>(unwindIndex * sizeof(uint64_t)));
+                        spooler.storeFPR(baselineRegisterOffset->offset());
+                    }
+                });
+                spooler.finalizeFPR();
+            }
         }
     }
 
@@ -457,7 +587,7 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
 
     if (exit.m_codeOrigin.inlineStackContainsActiveCheckpoint()) {
         EncodedJSValue* tmpScratch = scratch + exit.m_descriptor->m_values.tmpIndex(0);
-        jit.setupArguments<decltype(operationMaterializeOSRExitSideState)>(&vm, &exit, tmpScratch);
+        jit.setupArguments<decltype(operationMaterializeOSRExitSideState)>(CCallHelpers::TrustedImmPtr(&vm), CCallHelpers::TrustedImmPtr(&exit), CCallHelpers::TrustedImmPtr(tmpScratch));
         jit.prepareCallOperation(vm);
         jit.move(AssemblyHelpers::TrustedImmPtr(tagCFunction<OperationPtrTag>(operationMaterializeOSRExitSideState)), GPRInfo::nonArgGPR0);
         jit.call(GPRInfo::nonArgGPR0, OperationPtrTag);
@@ -465,17 +595,25 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
 
     // Now get state out of the scratch buffer and place it back into the stack. The values are
     // already reboxed so we just move them.
-    for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
-        Operand operand = exit.m_descriptor->m_values.operandForIndex(index);
+    {
+        constexpr GPRReg srcBufferGPR = GPRInfo::regT2;
+        constexpr GPRReg destBufferGPR = GPRInfo::regT3;
+        jit.move(CCallHelpers::TrustedImmPtr(scratch), srcBufferGPR);
+        jit.move(GPRInfo::callFrameRegister, destBufferGPR);
+        CCallHelpers::CopySpooler spooler(CCallHelpers::CopySpooler::BufferRegs::AllowModification, jit, srcBufferGPR, destBufferGPR, GPRInfo::regT0, GPRInfo::regT1);
+        for (unsigned index = exit.m_descriptor->m_values.size(); index--;) {
+            Operand operand = exit.m_descriptor->m_values.operandForIndex(index);
 
-        if (operand.isTmp())
-            continue;
+            if (operand.isTmp())
+                continue;
 
-        if (operand.isLocal() && operand.toLocal() < static_cast<int>(baselineVirtualRegistersForCalleeSaves))
-            continue;
+            if (operand.isLocal() && operand.toLocal() < static_cast<int>(baselineVirtualRegistersForCalleeSaves))
+                continue;
 
-        jit.load64(scratch + index, GPRInfo::regT0);
-        jit.store64(GPRInfo::regT0, AssemblyHelpers::addressFor(operand.virtualRegister()));
+            spooler.loadGPR(index * sizeof(EncodedJSValue));
+            spooler.storeGPR(operand.virtualRegister().offset() * sizeof(EncodedJSValue));
+        }
+        spooler.finalizeGPR();
     }
     
     handleExitCounts(vm, jit, exit);
@@ -485,18 +623,17 @@ static void compileStub(VM& vm, unsigned exitID, JITCode* jitCode, OSRExit& exit
     LinkBuffer patchBuffer(jit, codeBlock, LinkBuffer::Profile::FTLOSRExit);
     exit.m_code = FINALIZE_CODE_IF(
         shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit(),
-        patchBuffer, OSRExitPtrTag,
+        patchBuffer, OSRExitPtrTag, nullptr,
         "FTL OSR exit #%u (D@%u, %s, %s) from %s, with operands = %s",
             exitID, exit.m_dfgNodeIndex, toCString(exit.m_codeOrigin).data(),
-            exitKindToString(exit.m_kind), toCString(*codeBlock).data(),
+            toCString(exit.m_kind).data(), toCString(*codeBlock).data(),
             toCString(ignoringContext<DumpContext>(exit.m_descriptor->m_values)).data()
         );
 }
 
-JSC_DEFINE_JIT_OPERATION(operationCompileFTLOSRExit, void*, (CallFrame* callFrame, unsigned exitID))
+JSC_DEFINE_NOEXCEPT_JIT_OPERATION(operationCompileFTLOSRExit, void*, (CallFrame* callFrame, unsigned exitID))
 {
-    if (shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit())
-        dataLog("Compiling OSR exit with exitID = ", exitID, "\n");
+    dataLogLnIf(shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit(), "Compiling OSR exit with exitID = ", exitID);
 
     VM& vm = callFrame->deprecatedVM();
     // Don't need an ActiveScratchBufferScope here because we DeferGCForAWhile below.
@@ -523,19 +660,21 @@ JSC_DEFINE_JIT_OPERATION(operationCompileFTLOSRExit, void*, (CallFrame* callFram
     OSRExit& exit = jitCode->m_osrExit[exitID];
     
     if (shouldDumpDisassembly() || Options::verboseOSR() || Options::verboseFTLOSRExit()) {
-        dataLog("    Owning block: ", pointerDump(codeBlock), "\n");
-        dataLog("    Origin: ", exit.m_codeOrigin, "\n");
+        dataLogLn(
+            "    Owning block: ", pointerDump(codeBlock), "\n",
+            "    Origin: ", exit.m_codeOrigin);
         if (exit.m_codeOriginForExitProfile != exit.m_codeOrigin)
-            dataLog("    Origin for exit profile: ", exit.m_codeOriginForExitProfile, "\n");
-        dataLog("    Current call site index: ", callFrame->callSiteIndex().bits(), "\n");
-        dataLog("    Exit is exception handler: ", exit.isExceptionHandler(), "\n");
-        dataLog("    Is unwind handler: ", exit.isGenericUnwindHandler(), "\n");
-        dataLog("    Exit values: ", exit.m_descriptor->m_values, "\n");
-        dataLog("    Value reps: ", listDump(exit.m_valueReps), "\n");
+            dataLogLn("    Origin for exit profile: ", exit.m_codeOriginForExitProfile);
+        dataLogLn(
+            "    Current call site index: ", callFrame->callSiteIndex().bits(), "\n",
+            "    Exit is exception handler: ", exit.isExceptionHandler(), "\n",
+            "    Is unwind handler: ", exit.isGenericUnwindHandler(), "\n",
+            "    Exit values: ", exit.m_descriptor->m_values, "\n",
+            "    Value reps: ", listDump(exit.m_valueReps));
         if (!exit.m_descriptor->m_materializations.isEmpty()) {
-            dataLog("    Materializations:\n");
+            dataLogLn("    Materializations:");
             for (ExitTimeObjectMaterialization* materialization : exit.m_descriptor->m_materializations)
-                dataLog("        ", pointerDump(materialization), "\n");
+                dataLogLn("        ", pointerDump(materialization));
         }
     }
 
@@ -544,10 +683,11 @@ JSC_DEFINE_JIT_OPERATION(operationCompileFTLOSRExit, void*, (CallFrame* callFram
     MacroAssembler::repatchJump(
         exit.codeLocationForRepatch(codeBlock), CodeLocationLabel<OSRExitPtrTag>(exit.m_code.code()));
     
-    return exit.m_code.code().executableAddress();
+    return exit.m_code.code().taggedPtr();
 }
 
 } } // namespace JSC::FTL
 
-#endif // ENABLE(FTL_JIT)
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
 
+#endif // ENABLE(FTL_JIT)
