@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -28,8 +28,8 @@
 
 #include "DeferGCInlines.h"
 #include "HeapInlines.h"
-#include "MarkedBlock.h"
-#include "VM.h"
+#include "MarkedBlockInlines.h"
+#include <wtf/SystemTracing.h>
 
 namespace JSC {
 
@@ -42,36 +42,54 @@ void IncrementalSweeper::scheduleTimer()
     setTimeUntilFire(sweepTimeSlice * sweepTimeMultiplier);
 }
 
-IncrementalSweeper::IncrementalSweeper(Heap* heap)
+IncrementalSweeper::IncrementalSweeper(JSC::Heap* heap)
     : Base(heap->vm())
     , m_currentDirectory(nullptr)
 {
 }
 
-void IncrementalSweeper::doWork(VM& vm)
+void IncrementalSweeper::doWorkUntil(VM& vm, MonotonicTime deadline)
 {
-    doSweep(vm, MonotonicTime::now());
+    if (!m_currentDirectory)
+        m_currentDirectory = vm.heap.objectSpace().firstDirectory();
+
+    if (m_currentDirectory)
+        doSweep(vm, deadline, SweepTrigger::OpportunisticTask);
 }
 
-void IncrementalSweeper::doSweep(VM& vm, MonotonicTime sweepBeginTime)
+void IncrementalSweeper::doWork(VM& vm)
 {
-    while (sweepNextBlock(vm)) {
-        Seconds elapsedTime = MonotonicTime::now() - sweepBeginTime;
-        if (elapsedTime < sweepTimeSlice)
-            continue;
-
+    if (m_lastOpportunisticTaskDidFinishSweeping) {
+        m_lastOpportunisticTaskDidFinishSweeping = false;
         scheduleTimer();
         return;
     }
+    doSweep(vm, MonotonicTime::now() + sweepTimeSlice, SweepTrigger::Timer);
+}
 
-    if (m_shouldFreeFastMallocMemoryAfterSweeping) {
-        WTF::releaseFastMallocFreeMemory();
-        m_shouldFreeFastMallocMemoryAfterSweeping = false;
+void IncrementalSweeper::doSweep(VM& vm, MonotonicTime deadline, SweepTrigger trigger)
+{
+    std::optional<TraceScope> traceScope;
+    if (UNLIKELY(Options::useTracePoints()))
+        traceScope.emplace(IncrementalSweepStart, IncrementalSweepEnd, vm.heap.size(), vm.heap.capacity());
+
+    while (sweepNextBlock(vm, trigger)) {
+        if (MonotonicTime::now() < deadline)
+            continue;
+
+        if (trigger == SweepTrigger::Timer)
+            scheduleTimer();
+        else
+            m_lastOpportunisticTaskDidFinishSweeping = false;
+        return;
     }
+    if (trigger == SweepTrigger::OpportunisticTask)
+        m_lastOpportunisticTaskDidFinishSweeping = true;
+
     cancelTimer();
 }
 
-bool IncrementalSweeper::sweepNextBlock(VM& vm)
+bool IncrementalSweeper::sweepNextBlock(VM& vm, SweepTrigger trigger)
 {
     vm.heap.stopIfNecessary();
 
@@ -86,14 +104,26 @@ bool IncrementalSweeper::sweepNextBlock(VM& vm)
     if (block) {
         DeferGCForAWhile deferGC(vm);
         block->sweep(nullptr);
-        vm.heap.objectSpace().freeOrShrinkBlock(block);
+
+        bool blockIsFreed = false;
+        if (trigger == SweepTrigger::Timer) {
+            if (!block->isEmpty())
+                block->shrink();
+            else {
+                vm.heap.objectSpace().freeBlock(block);
+                blockIsFreed = true;
+            }
+        }
+
+        if (!blockIsFreed)
+            m_currentDirectory->didFinishUsingBlock(block);
         return true;
     }
 
     return vm.heap.sweepNextLogicallyEmptyWeakBlock();
 }
 
-void IncrementalSweeper::startSweeping(Heap& heap)
+void IncrementalSweeper::startSweeping(JSC::Heap& heap)
 {
     scheduleTimer();
     m_currentDirectory = heap.objectSpace().firstDirectory();
