@@ -30,8 +30,11 @@
 #include "InspectorFrontendRouter.h"
 #include <wtf/JSONValues.h>
 #include <wtf/SetForScope.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/WTFString.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace Inspector {
 
 SupplementalBackendDispatcher::SupplementalBackendDispatcher(BackendDispatcher& backendDispatcher)
@@ -39,9 +42,7 @@ SupplementalBackendDispatcher::SupplementalBackendDispatcher(BackendDispatcher&
 {
 }
 
-SupplementalBackendDispatcher::~SupplementalBackendDispatcher()
-{
-}
+SupplementalBackendDispatcher::~SupplementalBackendDispatcher() = default;
 
 BackendDispatcher::CallbackBase::CallbackBase(Ref<BackendDispatcher>&& backendDispatcher, long requestId)
     : m_backendDispatcher(WTFMove(backendDispatcher))
@@ -68,7 +69,7 @@ void BackendDispatcher::CallbackBase::sendFailure(const ErrorString& error)
     m_backendDispatcher->sendPendingErrors();
 }
 
-void BackendDispatcher::CallbackBase::sendSuccess(RefPtr<JSON::Object>&& partialMessage)
+void BackendDispatcher::CallbackBase::sendSuccess(Ref<JSON::Object>&& partialMessage)
 {
     if (m_alreadySent)
         return;
@@ -107,54 +108,58 @@ void BackendDispatcher::dispatch(const String& message)
 
     ASSERT(!m_protocolErrors.size());
 
-    long requestId = 0;
+    int requestId = 0;
     RefPtr<JSON::Object> messageObject;
 
     {
         // In case this is a re-entrant call from a nested run loop, we don't want to lose
         // the outer request's id just because the inner request is bogus.
-        SetForScope<Optional<long>> scopedRequestId(m_currentRequestId, WTF::nullopt);
+        SetForScope scopedRequestId(m_currentRequestId, std::nullopt);
 
-        RefPtr<JSON::Value> parsedMessage;
-        if (!JSON::Value::parseJSON(message, parsedMessage)) {
+        auto parsedMessage = JSON::Value::parseJSON(message);
+        if (!parsedMessage) {
             reportProtocolError(ParseError, "Message must be in JSON format"_s);
             sendPendingErrors();
             return;
         }
 
-        if (!parsedMessage->asObject(messageObject)) {
+        messageObject = parsedMessage->asObject();
+        if (!messageObject) {
             reportProtocolError(InvalidRequest, "Message must be a JSONified object"_s);
             sendPendingErrors();
             return;
         }
 
-        RefPtr<JSON::Value> requestIdValue;
-        if (!messageObject->getValue("id"_s, requestIdValue)) {
+        auto requestIdValue = messageObject->getValue("id"_s);
+        if (!requestIdValue) {
             reportProtocolError(InvalidRequest, "'id' property was not found"_s);
             sendPendingErrors();
             return;
         }
 
-        if (!requestIdValue->asInteger(requestId)) {
+        auto requestIdInt = requestIdValue->asInteger();
+        if (!requestIdInt) {
             reportProtocolError(InvalidRequest, "The type of 'id' property must be integer"_s);
             sendPendingErrors();
             return;
         }
+
+        requestId = *requestIdInt;
     }
 
     {
         // We could be called re-entrantly from a nested run loop, so restore the previous id.
-        SetForScope<Optional<long>> scopedRequestId(m_currentRequestId, requestId);
+        SetForScope scopedRequestId(m_currentRequestId, requestId);
 
-        RefPtr<JSON::Value> methodValue;
-        if (!messageObject->getValue("method"_s, methodValue)) {
+        auto methodValue = messageObject->getValue("method"_s);
+        if (!methodValue) {
             reportProtocolError(InvalidRequest, "'method' property wasn't found"_s);
             sendPendingErrors();
             return;
         }
 
-        String methodString;
-        if (!methodValue->asString(methodString)) {
+        auto methodString = methodValue->asString();
+        if (!methodString) {
             reportProtocolError(InvalidRequest, "The type of 'method' property must be string"_s);
             sendPendingErrors();
             return;
@@ -170,7 +175,7 @@ void BackendDispatcher::dispatch(const String& message)
         String domain = domainAndMethod[0];
         SupplementalBackendDispatcher* domainDispatcher = m_dispatchers.get(domain);
         if (!domainDispatcher) {
-            reportProtocolError(MethodNotFound, "'" + domain + "' domain was not found");
+            reportProtocolError(MethodNotFound, makeString('\'', domain, "' domain was not found"_s));
             sendPendingErrors();
             return;
         }
@@ -183,13 +188,27 @@ void BackendDispatcher::dispatch(const String& message)
     }
 }
 
-// FIXME: remove this function when legacy InspectorObject symbols are no longer needed <http://webkit.org/b/179847>.
+// FIXME: <http://webkit.org/b/179847> remove this function when legacy InspectorObject symbols are no longer needed.
 void BackendDispatcher::sendResponse(long requestId, RefPtr<JSON::Object>&& result)
+{
+    ASSERT(result);
+    sendResponse(requestId, result.releaseNonNull(), false);
+}
+
+// FIXME: <http://webkit.org/b/179847> remove this function when legacy InspectorObject symbols are no longer needed.
+void BackendDispatcher::sendResponse(long requestId, RefPtr<JSON::Object>&& result, bool)
+{
+    ASSERT(result);
+    sendResponse(requestId, result.releaseNonNull(), false);
+}
+
+// FIXME: <http://webkit.org/b/179847> remove this function when legacy InspectorObject symbols are no longer needed.
+void BackendDispatcher::sendResponse(long requestId, Ref<JSON::Object>&& result)
 {
     sendResponse(requestId, WTFMove(result), false);
 }
 
-void BackendDispatcher::sendResponse(long requestId, RefPtr<JSON::Object>&& result, bool)
+void BackendDispatcher::sendResponse(long requestId, Ref<JSON::Object>&& result, bool)
 {
     ASSERT(!m_protocolErrors.size());
 
@@ -224,7 +243,7 @@ void BackendDispatcher::sendPendingErrors()
         errorCode = std::get<0>(data);
         errorMessage = std::get<1>(data);
 
-        ASSERT_ARG(errorCode, (unsigned)errorCode < WTF_ARRAY_LENGTH(errorCodes));
+        ASSERT_ARG(errorCode, (unsigned)errorCode < std::size(errorCodes));
         ASSERT_ARG(errorCode, errorCodes[errorCode]);
 
         Ref<JSON::Object> error = JSON::Object::create();
@@ -250,7 +269,7 @@ void BackendDispatcher::sendPendingErrors()
     m_frontendRouter->sendResponse(message->toJSONString());
 
     m_protocolErrors.clear();
-    m_currentRequestId = WTF::nullopt;
+    m_currentRequestId = std::nullopt;
 }
     
 void BackendDispatcher::reportProtocolError(CommonErrorCode errorCode, const String& errorMessage)
@@ -258,7 +277,7 @@ void BackendDispatcher::reportProtocolError(CommonErrorCode errorCode, const Str
     reportProtocolError(m_currentRequestId, errorCode, errorMessage);
 }
 
-void BackendDispatcher::reportProtocolError(Optional<long> relatedRequestId, CommonErrorCode errorCode, const String& errorMessage)
+void BackendDispatcher::reportProtocolError(std::optional<long> relatedRequestId, CommonErrorCode errorCode, const String& errorMessage)
 {
     ASSERT_ARG(errorCode, errorCode >= 0);
 
@@ -270,74 +289,74 @@ void BackendDispatcher::reportProtocolError(Optional<long> relatedRequestId, Com
 }
 
 template<typename T>
-T BackendDispatcher::getPropertyValue(JSON::Object* object, const String& name, bool* out_optionalValueFound, T defaultValue, std::function<bool(JSON::Value&, T&)> asMethod, const char* typeName)
+T BackendDispatcher::getPropertyValue(JSON::Object* params, const String& name, bool required, std::function<T(JSON::Value&)> converter, ASCIILiteral typeName)
 {
-    T result(defaultValue);
-    // out_optionalValueFound signals to the caller whether an optional property was found.
-    // if out_optionalValueFound == nullptr, then this is a required property.
-    if (out_optionalValueFound)
-        *out_optionalValueFound = false;
+    T result;
 
-    if (!object) {
-        if (!out_optionalValueFound)
-            reportProtocolError(BackendDispatcher::InvalidParams, makeString("'params' object must contain required parameter '", name, "' with type '", typeName, "'."));
+    if (!params) {
+        if (required)
+            reportProtocolError(BackendDispatcher::InvalidParams, makeString("'params' object must contain required parameter '"_s, name, "' with type '"_s, typeName, "'."_s));
         return result;
     }
 
-    auto findResult = object->find(name);
-    if (findResult == object->end()) {
-        if (!out_optionalValueFound)
-            reportProtocolError(BackendDispatcher::InvalidParams, makeString("Parameter '", name, "' with type '", typeName, "' was not found."));
+    auto findResult = params->find(name);
+    if (findResult == params->end()) {
+        if (required)
+            reportProtocolError(BackendDispatcher::InvalidParams, makeString("Parameter '"_s, name, "' with type '"_s, typeName, "' was not found."_s));
         return result;
     }
 
-    if (!asMethod(*findResult->value, result)) {
-        reportProtocolError(BackendDispatcher::InvalidParams, makeString("Parameter '", name, "' has wrong type. It must be '", typeName, "'."));
-        return result;
-    }
+    result = converter(findResult->value);
 
-    if (out_optionalValueFound)
-        *out_optionalValueFound = true;
+    if (!result)
+        reportProtocolError(BackendDispatcher::InvalidParams, makeString("Parameter '"_s, name, "' has wrong type. It must be '"_s, typeName, "'."_s));
 
     return result;
 }
 
-static bool castToInteger(JSON::Value& value, int& result) { return value.asInteger(result); }
-static bool castToNumber(JSON::Value& value, double& result) { return value.asDouble(result); }
-
-int BackendDispatcher::getInteger(JSON::Object* object, const String& name, bool* valueFound)
+std::optional<bool> BackendDispatcher::getBoolean(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<int>(object, name, valueFound, 0, &castToInteger, "Integer");
+    return getPropertyValue<std::optional<bool>>(params, name, required, &JSON::Value::asBoolean, "Boolean"_s);
 }
 
-double BackendDispatcher::getDouble(JSON::Object* object, const String& name, bool* valueFound)
+std::optional<int> BackendDispatcher::getInteger(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<double>(object, name, valueFound, 0, &castToNumber, "Number");
+    // FIXME: <http://webkit.org/b/179847> simplify this when legacy InspectorObject symbols are no longer needed.
+    std::optional<int> (JSON::Value::*asInteger)() const = &JSON::Value::asInteger;
+    return getPropertyValue<std::optional<int>>(params, name, required, asInteger, "Integer"_s);
 }
 
-String BackendDispatcher::getString(JSON::Object* object, const String& name, bool* valueFound)
+std::optional<double> BackendDispatcher::getDouble(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<String>(object, name, valueFound, "", &JSON::Value::asString, "String");
+    // FIXME: <http://webkit.org/b/179847> simplify this when legacy InspectorObject symbols are no longer needed.
+    std::optional<double> (JSON::Value::*asDouble)() const = &JSON::Value::asDouble;
+    return getPropertyValue<std::optional<double>>(params, name, required, asDouble, "Number"_s);
 }
 
-bool BackendDispatcher::getBoolean(JSON::Object* object, const String& name, bool* valueFound)
+String BackendDispatcher::getString(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<bool>(object, name, valueFound, false, &JSON::Value::asBoolean, "Boolean");
+    // FIXME: <http://webkit.org/b/179847> simplify this when legacy InspectorObject symbols are no longer needed.
+    String (JSON::Value::*asString)() const = &JSON::Value::asString;
+    return getPropertyValue<String>(params, name, required, asString, "String"_s);
 }
 
-RefPtr<JSON::Object> BackendDispatcher::getObject(JSON::Object* object, const String& name, bool* valueFound)
+RefPtr<JSON::Value> BackendDispatcher::getValue(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<RefPtr<JSON::Object>>(object, name, valueFound, nullptr, &JSON::Value::asObject, "Object");
+    return getPropertyValue<RefPtr<JSON::Value>>(params, name, required, &JSON::Value::asValue, "Value"_s);
 }
 
-RefPtr<JSON::Array> BackendDispatcher::getArray(JSON::Object* object, const String& name, bool* valueFound)
+RefPtr<JSON::Object> BackendDispatcher::getObject(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<RefPtr<JSON::Array>>(object, name, valueFound, nullptr, &JSON::Value::asArray, "Array");
+    return getPropertyValue<RefPtr<JSON::Object>>(params, name, required, [](JSON::Value& value) {
+        return value.asObject();
+    }, "Object"_s);
 }
 
-RefPtr<JSON::Value> BackendDispatcher::getValue(JSON::Object* object, const String& name, bool* valueFound)
+RefPtr<JSON::Array> BackendDispatcher::getArray(JSON::Object* params, const String& name, bool required)
 {
-    return getPropertyValue<RefPtr<JSON::Value>>(object, name, valueFound, nullptr, &JSON::Value::asValue, "Value");
+    return getPropertyValue<RefPtr<JSON::Array>>(params, name, required, &JSON::Value::asArray, "Array"_s);
 }
 
 } // namespace Inspector
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
