@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2008-2018 Apple Inc. All rights reserved.
+ * Copyright (C) 2008-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -31,19 +31,20 @@
 
 #include <wtf/PrintStream.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace JSC {
 
 DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(DirectJITCode);
 
-JITCode::JITCode(JITType jitType, ShareAttribute shareAttribute)
+JITCode::JITCode(JITType jitType, CodePtr<JSEntryPtrTag> code, ShareAttribute shareAttribute)
     : m_jitType(jitType)
     , m_shareAttribute(shareAttribute)
+    , m_addressForCall(code)
 {
 }
 
-JITCode::~JITCode()
-{
-}
+JITCode::~JITCode() = default;
 
 ASCIILiteral JITCode::typeName(JITType jitType)
 {
@@ -100,6 +101,12 @@ DFG::CommonData* JITCode::dfgCommon()
     return nullptr;
 }
 
+const DFG::CommonData* JITCode::dfgCommon() const
+{
+    RELEASE_ASSERT_NOT_REACHED();
+    return nullptr;
+}
+
 DFG::JITCode* JITCode::dfg()
 {
     RELEASE_ASSERT_NOT_REACHED();
@@ -118,7 +125,7 @@ FTL::ForOSREntryJITCode* JITCode::ftlForOSREntry()
     return nullptr;
 }
 
-void JITCode::shrinkToFit(const ConcurrentJSLocker&)
+void JITCode::shrinkToFit()
 {
 }
 
@@ -139,60 +146,78 @@ const RegisterAtOffsetList* JITCode::calleeSaveRegisters() const
 #endif
 }
 
+JITCode::CodeRef<JSEntryPtrTag> JITCode::swapCodeRefForDebugger(JITCode::CodeRef<JSEntryPtrTag>)
+{
+    return CodeRef<JSEntryPtrTag>();
+}
+
 JITCodeWithCodeRef::JITCodeWithCodeRef(JITType jitType)
     : JITCode(jitType)
 {
 }
 
 JITCodeWithCodeRef::JITCodeWithCodeRef(CodeRef<JSEntryPtrTag> ref, JITType jitType, JITCode::ShareAttribute shareAttribute)
-    : JITCode(jitType, shareAttribute)
-    , m_ref(ref)
+    : JITCode(jitType, ref.code(), shareAttribute)
+    , m_executableMemory(ref.executableMemory())
 {
 }
 
 JITCodeWithCodeRef::~JITCodeWithCodeRef()
 {
-    if ((Options::dumpDisassembly() || (isOptimizingJIT(jitType()) && Options::dumpDFGDisassembly()))
-        && m_ref.executableMemory())
-        dataLog("Destroying JIT code at ", pointerDump(m_ref.executableMemory()), "\n");
+    if ((Options::dumpDisassembly() || ((jitType() == JITType::BaselineJIT) && Options::dumpBaselineDisassembly()) || (isOptimizingJIT(jitType()) && Options::dumpDFGDisassembly()))
+        && m_executableMemory)
+        dataLog("Destroying JIT code at ", pointerDump(m_executableMemory.get()), "\n");
 }
 
 void* JITCodeWithCodeRef::executableAddressAtOffset(size_t offset)
 {
-    RELEASE_ASSERT(m_ref);
-    assertIsTaggedWith<JSEntryPtrTag>(m_ref.code().executableAddress());
+    RELEASE_ASSERT(m_addressForCall);
+    assertIsTaggedWith<JSEntryPtrTag>(m_addressForCall.taggedPtr());
     if (!offset)
-        return m_ref.code().executableAddress();
+        return m_addressForCall.taggedPtr();
 
-    char* executableAddress = untagCodePtr<char*, JSEntryPtrTag>(m_ref.code().executableAddress());
+    char* executableAddress = untagCodePtr<char*, JSEntryPtrTag>(m_addressForCall.taggedPtr());
     return tagCodePtr<JSEntryPtrTag>(executableAddress + offset);
 }
 
 void* JITCodeWithCodeRef::dataAddressAtOffset(size_t offset)
 {
-    RELEASE_ASSERT(m_ref);
+    RELEASE_ASSERT(m_addressForCall);
     ASSERT(offset <= size()); // use <= instead of < because it is valid to ask for an address at the exclusive end of the code.
-    return m_ref.code().dataLocation<char*>() + offset;
+    return m_addressForCall.dataLocation<char*>() + offset;
 }
 
 unsigned JITCodeWithCodeRef::offsetOf(void* pointerIntoCode)
 {
-    RELEASE_ASSERT(m_ref);
-    intptr_t result = reinterpret_cast<intptr_t>(pointerIntoCode) - m_ref.code().executableAddress<intptr_t>();
+    RELEASE_ASSERT(m_addressForCall);
+    intptr_t result = reinterpret_cast<intptr_t>(pointerIntoCode) - m_addressForCall.taggedPtr<intptr_t>();
     ASSERT(static_cast<intptr_t>(static_cast<unsigned>(result)) == result);
     return static_cast<unsigned>(result);
 }
 
 size_t JITCodeWithCodeRef::size()
 {
-    RELEASE_ASSERT(m_ref);
-    return m_ref.size();
+    if (RefPtr memory = m_executableMemory)
+        return memory->sizeInBytes();
+    return 0;
 }
 
 bool JITCodeWithCodeRef::contains(void* address)
 {
-    RELEASE_ASSERT(m_ref);
-    return m_ref.executableMemory()->contains(address);
+    RELEASE_ASSERT(m_executableMemory);
+    return m_executableMemory->contains(address);
+}
+
+JITCode::CodeRef<JSEntryPtrTag> JITCodeWithCodeRef::swapCodeRefForDebugger(JITCode::CodeRef<JSEntryPtrTag> ref)
+{
+    ASSERT(canSwapCodeRefForDebugger());
+    RELEASE_ASSERT(m_addressForCall);
+    RELEASE_ASSERT(ref);
+    auto old = CodeRef<JSEntryPtrTag>::createSelfManagedCodeRef(m_addressForCall);
+    ASSERT(!m_executableMemory);
+    m_addressForCall = ref.code();
+    m_executableMemory = ref.executableMemory();
+    return old;
 }
 
 DirectJITCode::DirectJITCode(JITType jitType)
@@ -200,42 +225,41 @@ DirectJITCode::DirectJITCode(JITType jitType)
 {
 }
 
-DirectJITCode::DirectJITCode(JITCode::CodeRef<JSEntryPtrTag> ref, JITCode::CodePtr<JSEntryPtrTag> withArityCheck, JITType jitType, JITCode::ShareAttribute shareAttribute)
+DirectJITCode::DirectJITCode(JITCode::CodeRef<JSEntryPtrTag> ref, CodePtr<JSEntryPtrTag> withArityCheck, JITType jitType, JITCode::ShareAttribute shareAttribute)
     : JITCodeWithCodeRef(ref, jitType, shareAttribute)
     , m_withArityCheck(withArityCheck)
 {
-    ASSERT(m_ref);
+    ASSERT(m_addressForCall);
     ASSERT(m_withArityCheck);
 }
 
-DirectJITCode::DirectJITCode(JITCode::CodeRef<JSEntryPtrTag> ref, JITCode::CodePtr<JSEntryPtrTag> withArityCheck, JITType jitType, Intrinsic intrinsic, JITCode::ShareAttribute shareAttribute)
+DirectJITCode::DirectJITCode(JITCode::CodeRef<JSEntryPtrTag> ref, CodePtr<JSEntryPtrTag> withArityCheck, JITType jitType, Intrinsic intrinsic, JITCode::ShareAttribute shareAttribute)
     : JITCodeWithCodeRef(ref, jitType, shareAttribute)
     , m_withArityCheck(withArityCheck)
 {
     m_intrinsic = intrinsic;
-    ASSERT(m_ref);
+    ASSERT(m_addressForCall);
     ASSERT(m_withArityCheck);
 }
 
-DirectJITCode::~DirectJITCode()
-{
-}
+DirectJITCode::~DirectJITCode() = default;
 
-void DirectJITCode::initializeCodeRefForDFG(JITCode::CodeRef<JSEntryPtrTag> ref, JITCode::CodePtr<JSEntryPtrTag> withArityCheck)
+void DirectJITCode::initializeCodeRefForDFG(JITCode::CodeRef<JSEntryPtrTag> ref, CodePtr<JSEntryPtrTag> withArityCheck)
 {
-    RELEASE_ASSERT(!m_ref);
-    m_ref = ref;
+    RELEASE_ASSERT(!m_addressForCall);
+    m_addressForCall = ref.code();
+    m_executableMemory = ref.executableMemory();
     m_withArityCheck = withArityCheck;
-    ASSERT(m_ref);
+    ASSERT(m_addressForCall);
     ASSERT(m_withArityCheck);
 }
 
-JITCode::CodePtr<JSEntryPtrTag> DirectJITCode::addressForCall(ArityCheckMode arity)
+CodePtr<JSEntryPtrTag> DirectJITCode::addressForCall(ArityCheckMode arity)
 {
     switch (arity) {
     case ArityCheckNotRequired:
-        RELEASE_ASSERT(m_ref);
-        return m_ref.code();
+        RELEASE_ASSERT(m_addressForCall);
+        return m_addressForCall;
     case MustCheckArity:
         RELEASE_ASSERT(m_withArityCheck);
         return m_withArityCheck;
@@ -255,21 +279,13 @@ NativeJITCode::NativeJITCode(CodeRef<JSEntryPtrTag> ref, JITType jitType, Intrin
     m_intrinsic = intrinsic;
 }
 
-NativeJITCode::~NativeJITCode()
-{
-}
+NativeJITCode::~NativeJITCode() = default;
 
-JITCode::CodePtr<JSEntryPtrTag> NativeJITCode::addressForCall(ArityCheckMode arity)
+CodePtr<JSEntryPtrTag> NativeJITCode::addressForCall(ArityCheckMode arity)
 {
-    RELEASE_ASSERT(m_ref);
-    switch (arity) {
-    case ArityCheckNotRequired:
-        return m_ref.code();
-    case MustCheckArity:
-        return m_ref.code();
-    }
-    RELEASE_ASSERT_NOT_REACHED();
-    return CodePtr<JSEntryPtrTag>();
+    RELEASE_ASSERT(m_addressForCall);
+    ASSERT_UNUSED(arity, arity == ArityCheckNotRequired || arity == MustCheckArity);
+    return m_addressForCall;
 }
 
 NativeDOMJITCode::NativeDOMJITCode(CodeRef<JSEntryPtrTag> ref, JITType type, Intrinsic intrinsic, const DOMJIT::Signature* signature)
@@ -279,7 +295,7 @@ NativeDOMJITCode::NativeDOMJITCode(CodeRef<JSEntryPtrTag> ref, JITType type, Int
 }
 
 #if ENABLE(JIT)
-RegisterSet JITCode::liveRegistersToPreserveAtExceptionHandlingCallSite(CodeBlock*, CallSiteIndex)
+RegisterSetBuilder JITCode::liveRegistersToPreserveAtExceptionHandlingCallSite(CodeBlock*, CallSiteIndex)
 {
     return { };
 }
@@ -296,3 +312,4 @@ void printInternal(PrintStream& out, JSC::JITType type)
 
 } // namespace WTF
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
