@@ -212,6 +212,10 @@ class Immediate < NoChildren
     def dump
         "#{value}"
     end
+
+    def name
+        "0x%x" % value
+    end
     
     def ==(other)
         other.is_a? Immediate and other.value == @value
@@ -698,6 +702,48 @@ class FPRegisterID < NoChildren
     end
 end
 
+class VecRegisterID < NoChildren
+    attr_reader :name
+    
+    def initialize(codeOrigin, name)
+        super(codeOrigin)
+        @name = name
+    end
+    
+    @@mapping = {}
+    
+    def self.forName(codeOrigin, name)
+        unless @@mapping[name]
+            @@mapping[name] = VecRegisterID.new(codeOrigin, name)
+        end
+        @@mapping[name]
+    end
+    
+    def dump
+        name
+    end
+    
+    def address?
+        false
+    end
+    
+    def label?
+        false
+    end
+    
+    def immediate?
+        false
+    end
+    
+    def immediateOperand?
+        false
+    end
+    
+    def register?
+        true
+    end
+end
+
 class SpecialRegister < NoChildren
     attr_reader :name
 
@@ -1046,6 +1092,9 @@ class Label < NoChildren
         @definedInFile = definedInFile
         @extern = true
         @global = false
+        @aligned = true
+        @alignTo = false
+        @export = false
     end
     
     def self.forName(codeOrigin, name, definedInFile = false)
@@ -1072,6 +1121,54 @@ class Label < NoChildren
         end
     end
 
+    def self.setAsGlobalExport(codeOrigin, name)
+        if $labelMapping[name]
+            label = $labelMapping[name]
+            raise "Label: #{name} declared global multiple times" unless not label.global?
+            label.setGlobalExport()
+        else
+            newLabel = Label.new(codeOrigin, name)
+            newLabel.setGlobalExport()
+            $labelMapping[name] = newLabel
+        end
+    end
+
+    def self.setAsUnalignedGlobal(codeOrigin, name)
+        if $labelMapping[name]
+            label = $labelMapping[name]
+            raise "Label: #{name} declared global multiple times" unless not label.global?
+            label.setUnalignedGlobal()
+        else
+            newLabel = Label.new(codeOrigin, name)
+            newLabel.setUnalignedGlobal()
+            $labelMapping[name] = newLabel
+        end
+    end
+
+    def self.setAsAligned(codeOrigin, name, alignTo)
+        if $labelMapping[name]
+            label = $labelMapping[name]
+            raise "Label: #{name} declared aligned multiple times" unless not label.aligned?
+            label.setAligned(alignTo)
+        else
+            newLabel = Label.new(codeOrigin, name)
+            newLabel.setAligned(alignTo)
+            $labelMapping[name] = newLabel
+        end
+    end
+
+    def self.setAsUnalignedGlobalExport(codeOrigin, name)
+        if $labelMapping[name]
+            label = $labelMapping[name]
+            raise "Label: #{name} declared global multiple times" unless not label.global?
+            label.setUnalignedGlobalExport()
+        else
+            newLabel = Label.new(codeOrigin, name)
+            newLabel.setUnalignedGlobalExport()
+            $labelMapping[name] = newLabel
+        end
+    end
+
     def self.resetReferenced
         $referencedExternLabels = Array.new
     end
@@ -1095,10 +1192,41 @@ class Label < NoChildren
         @global = true
     end
 
+    def setGlobalExport
+        @global = true
+        @export = true
+    end
+
+    def setUnalignedGlobal
+        @global = true
+        @aligned = false
+    end
+
+    def setAligned(alignTo)
+        @aligned = true
+        @alignTo = alignTo
+        # You must use this from cpp for the alignment to work on all linkers
+        @global = true
+    end
+
+    def setUnalignedGlobalExport
+        @global = true
+        @aligned = false
+        @export = true
+    end
+
     def global?
         @global
     end
 
+    def export?
+        @export
+    end
+
+    def aligned?
+        @aligned
+    end
+
     def name
         @name
     end
@@ -1122,22 +1250,20 @@ class LocalLabel < NoChildren
         if $labelMapping[name]
             raise "Label name collision: #{name}" unless $labelMapping[name].is_a? LocalLabel
         else
+            raise "nil codeOrigin" if codeOrigin.nil?
             $labelMapping[name] = LocalLabel.new(codeOrigin, name)
         end
         $labelMapping[name]
     end
     
-    def self.unique(comment)
+    def self.unique(codeOrigin, comment)
         newName = "_#{comment}"
-        if $emitWinAsm and newName.length > 90
-            newName = newName[0...45] + "___" + newName[-45..-1]
-        end
         if $labelMapping[newName]
             while $labelMapping[newName = "_#{@@uniqueNameCounter}_#{comment}"]
                 @@uniqueNameCounter += 1
             end
         end
-        forName(nil, newName)
+        forName(codeOrigin, newName)
     end
     
     def cleanName
@@ -1223,6 +1349,9 @@ class LocalLabelReference < NoChildren
     
     def initialize(codeOrigin, label)
         super(codeOrigin)
+        if label.is_a? LocalLabelReference
+            label = label.label
+        end
         @label = label
     end
     
