@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2011-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2011-2021 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -25,6 +25,10 @@
 
 #pragma once
 
+#include <wtf/Compiler.h>
+
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 #include "CatchScope.h"
 #include "Error.h"
 #include "ExceptionHelpers.h"
@@ -34,29 +38,34 @@
 #include "JSCJSValue.h"
 #include "JSCellInlines.h"
 #include "JSFunction.h"
+#include "JSGlobalProxy.h"
 #include "JSObject.h"
-#include "JSProxy.h"
 #include "JSStringInlines.h"
 #include "MathCommon.h"
-#include <wtf/Variant.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/StringImpl.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
+
 namespace JSC {
 
 ALWAYS_INLINE int32_t JSValue::toInt32(JSGlobalObject* globalObject) const
 {
     if (isInt32())
         return asInt32();
-    return JSC::toInt32(toNumber(globalObject));
+
+    double d = toNumber(globalObject);
+    return JSC::toInt32(d);
 }
 
 inline uint32_t JSValue::toUInt32(JSGlobalObject* globalObject) const
 {
-    // See comment on JSC::toUInt32, in JSCJSValue.h.
+    // The only difference between toInt32 and toUint32 is that toUint32 reinterprets resulted int32_t value as uint32_t.
+    // https://tc39.es/ecma262/#sec-touint32
     return toInt32(globalObject);
 }
 
-inline uint32_t JSValue::toIndex(JSGlobalObject* globalObject, const char* errorName) const
+inline uint32_t JSValue::toIndex(JSGlobalObject* globalObject, ASCIILiteral errorName) const
 {
     VM& vm = getVM(globalObject);
     auto scope = DECLARE_THROW_SCOPE(vm);
@@ -64,19 +73,69 @@ inline uint32_t JSValue::toIndex(JSGlobalObject* globalObject, const char* error
     double d = toNumber(globalObject);
     RETURN_IF_EXCEPTION(scope, 0);
     if (d <= -1) {
-        throwException(globalObject, scope, createRangeError(globalObject, makeString(errorName, " cannot be negative")));
-        return 0;
-    }
-    if (d > std::numeric_limits<unsigned>::max()) {
-        throwException(globalObject, scope, createRangeError(globalObject, makeString(errorName, " too large")));
+        throwException(globalObject, scope, createRangeError(globalObject, makeString(errorName, " cannot be negative"_s)));
         return 0;
     }
 
     if (isInt32())
         return asInt32();
+
+    if (d > static_cast<double>(std::numeric_limits<unsigned>::max())) {
+        throwException(globalObject, scope, createRangeError(globalObject, makeString(errorName, " too large"_s)));
+        return 0;
+    }
+
     RELEASE_AND_RETURN(scope, JSC::toInt32(d));
 }
 
+inline size_t JSValue::toTypedArrayIndex(JSGlobalObject* globalObject, ASCIILiteral errorName) const
+{
+    VM& vm = getVM(globalObject);
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    double d = toNumber(globalObject);
+    RETURN_IF_EXCEPTION(scope, 0);
+    if (d <= -1) {
+        throwException(globalObject, scope, createRangeError(globalObject, makeString(errorName, " cannot be negative"_s)));
+        return 0;
+    }
+
+    if (isInt32())
+        return asInt32();
+
+    if (d > static_cast<double>(MAX_ARRAY_BUFFER_SIZE)) {
+        throwException(globalObject, scope, createRangeError(globalObject, makeString(errorName, " too large"_s)));
+        return 0;
+    }
+
+    // All of this monstrosity is just to give the correct result on 1<<32.
+    size_t outputOffset = 0;
+    double inputOffset = 0;
+    size_t int32Max = std::numeric_limits<int32_t>::max();
+    if (d > static_cast<double>(int32Max)) {
+        outputOffset = int32Max;
+        inputOffset = int32Max;
+    }
+    RELEASE_AND_RETURN(scope, outputOffset + static_cast<size_t>(static_cast<uint32_t>(JSC::toInt32(d - inputOffset))));
+}
+
+// https://tc39.es/proposal-temporal/#sec-tointegerwithtruncation
+inline double JSValue::toIntegerWithTruncation(JSGlobalObject* globalObject) const
+{
+    if (isInt32())
+        return asInt32();
+    return trunc(toNumber(globalObject) + 0.0);
+}
+
+// https://tc39.es/ecma262/#sec-tointegerorinfinity
+inline double JSValue::toIntegerOrInfinity(JSGlobalObject* globalObject) const
+{
+    if (isInt32())
+        return asInt32();
+    double d = toNumber(globalObject);
+    return trunc(std::isnan(d) ? 0.0 : d + 0.0);
+}
+
 inline bool JSValue::isUInt32() const
 {
     return isInt32() && asInt32() >= 0;
@@ -94,6 +153,34 @@ inline double JSValue::asNumber() const
     return isInt32() ? asInt32() : asDouble();
 }
 
+inline std::optional<uint32_t> JSValue::tryGetAsUint32Index()
+{
+    if (isUInt32()) {
+        ASSERT(isIndex(asUInt32()));
+        return asUInt32();
+    }
+    if (isNumber()) {
+        double number = asNumber();
+        uint32_t asUint = static_cast<uint32_t>(number);
+        if (static_cast<double>(asUint) == number && isIndex(asUint))
+            return asUint;
+    }
+    return std::nullopt;
+}
+
+inline std::optional<int32_t> JSValue::tryGetAsInt32()
+{
+    if (isInt32())
+        return asInt32();
+    if (isNumber()) {
+        double number = asNumber();
+        int32_t asInt = static_cast<int32_t>(number);
+        if (static_cast<double>(asInt) == number)
+            return asInt;
+    }
+    return std::nullopt;
+}
+
 inline JSValue jsNaN()
 {
     return JSValue(JSValue::EncodeAsDouble, PNaN);
@@ -251,11 +338,6 @@ inline bool JSValue::operator==(const JSValue& other) const
     return u.asInt64 == other.u.asInt64;
 }
 
-inline bool JSValue::operator!=(const JSValue& other) const
-{
-    return u.asInt64 != other.u.asInt64;
-}
-
 inline bool JSValue::isEmpty() const
 {
     return tag() == EmptyValueTag;
@@ -329,6 +411,12 @@ ALWAYS_INLINE JSCell* JSValue::asCell() const
     return reinterpret_cast<JSCell*>(u.asBits.payload);
 }
 
+ALWAYS_INLINE JSBigInt* JSValue::asHeapBigInt() const
+{
+    ASSERT(isHeapBigInt());
+    return reinterpret_cast<JSBigInt*>(u.asBits.payload);
+}
+
 ALWAYS_INLINE JSValue::JSValue(EncodeAsDoubleTag, double d)
 {
     ASSERT(!isImpureNaN(d));
@@ -397,11 +485,6 @@ inline bool JSValue::operator==(const JSValue& other) const
     return u.asInt64 == other.u.asInt64;
 }
 
-inline bool JSValue::operator!=(const JSValue& other) const
-{
-    return u.asInt64 != other.u.asInt64;
-}
-
 inline bool JSValue::isEmpty() const
 {
     return u.asInt64 == ValueEmpty;
@@ -487,11 +570,11 @@ inline bool JSValue::isInt32() const
 
 inline int64_t reinterpretDoubleToInt64(double value)
 {
-    return bitwise_cast<int64_t>(value);
+    return std::bit_cast<int64_t>(value);
 }
 inline double reinterpretInt64ToDouble(int64_t value)
 {
-    return bitwise_cast<double>(value);
+    return std::bit_cast<double>(value);
 }
 
 ALWAYS_INLINE JSValue::JSValue(EncodeAsDoubleTag, double d)
@@ -522,8 +605,30 @@ ALWAYS_INLINE JSCell* JSValue::asCell() const
     return u.ptr;
 }
 
+ALWAYS_INLINE JSBigInt* JSValue::asHeapBigInt() const
+{
+    ASSERT(isHeapBigInt());
+    return static_cast<JSBigInt*>(u.ptr);
+}
+
 #endif // USE(JSVALUE64)
 
+#if USE(BIGINT32)
+inline JSValue::JSValue(EncodeAsBigInt32Tag, int32_t value)
+{
+    uint64_t shiftedValue = static_cast<uint64_t>(static_cast<uint32_t>(value)) << 16;
+    ASSERT(!(shiftedValue & NumberTag));
+    u.asInt64 = shiftedValue | BigInt32Tag;
+}
+#endif // USE(BIGINT32)
+
+#if ENABLE(WEBASSEMBLY) && USE(JSVALUE32_64)
+inline JSValue::JSValue(EncodeAsUnboxedFloatTag, float value)
+{
+    u.asBits.payload = std::bit_cast<int32_t>(value);
+}
+#endif
+
 inline int64_t tryConvertToInt52(double number)
 {
     if (number != number)
@@ -610,7 +715,51 @@ inline bool JSValue::isString() const
 
 inline bool JSValue::isBigInt() const
 {
-    return isCell() && asCell()->isBigInt();
+    return isBigInt32() || isHeapBigInt();
+}
+
+inline bool JSValue::isHeapBigInt() const
+{
+    return isCell() && asCell()->isHeapBigInt();
+}
+
+inline bool JSValue::isBigInt32() const
+{
+#if USE(BIGINT32)
+    return (u.asInt64 & BigInt32Mask) == BigInt32Tag;
+#else
+    return false;
+#endif
+}
+
+#if USE(BIGINT32)
+inline int32_t JSValue::bigInt32AsInt32() const
+{
+    ASSERT(isBigInt32());
+    return static_cast<int32_t>(u.asInt64 >> 16);
+}
+#endif // USE(BIGINT32)
+
+inline bool JSValue::isZeroBigInt() const
+{
+    ASSERT(isBigInt());
+#if USE(BIGINT32)
+    if (isBigInt32())
+        return !bigInt32AsInt32();
+#endif
+    ASSERT(isHeapBigInt());
+    return asHeapBigInt()->isZero();
+}
+
+inline bool JSValue::isNegativeBigInt() const
+{
+    ASSERT(isBigInt());
+#if USE(BIGINT32)
+    if (isBigInt32())
+        return bigInt32AsInt32() < 0;
+#endif
+    ASSERT(isHeapBigInt());
+    return asHeapBigInt()->sign();
 }
 
 inline bool JSValue::isSymbol() const
@@ -620,7 +769,7 @@ inline bool JSValue::isSymbol() const
 
 inline bool JSValue::isPrimitive() const
 {
-    return !isCell() || asCell()->isString() || asCell()->isSymbol() || asCell()->isBigInt();
+    return !isCell() || asCell()->isString() || asCell()->isSymbol() || asCell()->isHeapBigInt();
 }
 
 inline bool JSValue::isGetterSetter() const
@@ -655,7 +804,7 @@ template <typename Base> String HandleConverter<Base, Unknown>::getString(JSGlob
 
 inline JSObject* JSValue::getObject() const
 {
-    return isCell() ? asCell()->getObject() : 0;
+    return isCell() ? asCell()->getObject() : nullptr;
 }
 
 ALWAYS_INLINE bool JSValue::getUInt32(uint32_t& v) const
@@ -722,50 +871,20 @@ inline PreferredPrimitiveType toPreferredPrimitiveType(JSGlobalObject* globalObj
         return NoPreference;
     }
 
-    StringImpl* hintString = asString(value)->value(globalObject).impl();
+    auto hintString = asString(value)->view(globalObject);
     RETURN_IF_EXCEPTION(scope, NoPreference);
 
-    if (WTF::equal(hintString, "default"))
+    if (WTF::equal(hintString, "default"_s))
         return NoPreference;
-    if (WTF::equal(hintString, "number"))
+    if (WTF::equal(hintString, "number"_s))
         return PreferNumber;
-    if (WTF::equal(hintString, "string"))
+    if (WTF::equal(hintString, "string"_s))
         return PreferString;
 
     throwTypeError(globalObject, scope, "Expected primitive hint to match one of 'default', 'number', 'string'."_s);
     return NoPreference;
 }
 
-inline bool JSValue::getPrimitiveNumber(JSGlobalObject* globalObject, double& number, JSValue& value)
-{
-    if (isInt32()) {
-        number = asInt32();
-        value = *this;
-        return true;
-    }
-    if (isDouble()) {
-        number = asDouble();
-        value = *this;
-        return true;
-    }
-    if (isCell())
-        return asCell()->getPrimitiveNumber(globalObject, number, value);
-    if (isTrue()) {
-        number = 1.0;
-        value = *this;
-        return true;
-    }
-    if (isFalse() || isNull()) {
-        number = 0.0;
-        value = *this;
-        return true;
-    }
-    ASSERT(isUndefined());
-    number = PNaN;
-    value = *this;
-    return true;
-}
-
 ALWAYS_INLINE double JSValue::toNumber(JSGlobalObject* globalObject) const
 {
     if (isInt32())
@@ -775,44 +894,97 @@ ALWAYS_INLINE double JSValue::toNumber(JSGlobalObject* globalObject) const
     return toNumberSlowCase(globalObject);
 }
 
-ALWAYS_INLINE Variant<JSBigInt*, double> JSValue::toNumeric(JSGlobalObject* globalObject) const
+ALWAYS_INLINE JSValue JSValue::toNumeric(JSGlobalObject* globalObject) const
+{
+    VM& vm = globalObject->vm();
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    if (isInt32() || isDouble() || isBigInt())
+        return *this;
+
+    if (isString())
+        RELEASE_AND_RETURN(scope, jsNumber(asString(*this)->toNumber(globalObject)));
+
+    JSValue primValue = this->toPrimitive(globalObject, PreferNumber);
+    RETURN_IF_EXCEPTION(scope, { });
+
+    if (primValue.isDouble() || primValue.isBigInt())
+        return primValue;
+
+    double value = primValue.toNumber(globalObject);
+    RETURN_IF_EXCEPTION(scope, { });
+
+    return jsNumber(value);
+}
+
+ALWAYS_INLINE std::optional<uint32_t> JSValue::toUInt32AfterToNumeric(JSGlobalObject* globalObject) const
+{
+    VM& vm = getVM(globalObject);
+    auto scope = DECLARE_THROW_SCOPE(vm);
+    JSValue result = toBigIntOrInt32(globalObject);
+    RETURN_IF_EXCEPTION(scope, { });
+    if (LIKELY(result.isInt32()))
+        return static_cast<uint32_t>(result.asInt32());
+    return std::nullopt;
+}
+
+ALWAYS_INLINE JSValue JSValue::toBigIntOrInt32(JSGlobalObject* globalObject) const
+{
+    VM& vm = getVM(globalObject);
+    auto scope = DECLARE_THROW_SCOPE(vm);
+
+    if (isInt32() || isBigInt())
+        return *this;
+    if (isDouble() && canBeInt32(asDouble()))
+        return jsNumber(static_cast<int32_t>(asDouble()));
+
+    JSValue primValue = this->toPrimitive(globalObject, PreferNumber);
+    RETURN_IF_EXCEPTION(scope, { });
+
+    if (primValue.isInt32() || primValue.isBigInt())
+        return primValue;
+
+    int32_t value = primValue.toInt32(globalObject);
+    RETURN_IF_EXCEPTION(scope, { });
+    return jsNumber(value);
+}
+
+inline bool JSValue::toBoolean(JSGlobalObject* globalObject) const
 {
     if (isInt32())
         return asInt32();
     if (isDouble())
-        return asDouble();
-    if (isBigInt())
-        return asBigInt(*this);
-
-    VM& vm = getVM(globalObject);
-    auto scope = DECLARE_THROW_SCOPE(vm);
-    JSValue primValue = this->toPrimitive(globalObject, PreferNumber);
-    RETURN_IF_EXCEPTION(scope, 0);
-    if (primValue.isBigInt())
-        return asBigInt(primValue);
-    double value = primValue.toNumber(globalObject);
-    RETURN_IF_EXCEPTION(scope, 0);
-    return value;
+        return asDouble() > 0.0 || asDouble() < 0.0; // false for NaN
+    if (isCell())
+        return asCell()->toBoolean(globalObject);
+#if USE(BIGINT32)
+    if (isBigInt32())
+        return !!bigInt32AsInt32();
+#endif
+    return isTrue(); // false, null, and undefined all convert to false.
 }
 
-ALWAYS_INLINE Variant<JSBigInt*, int32_t> JSValue::toBigIntOrInt32(JSGlobalObject* globalObject) const
+inline JSString* JSValue::toString(JSGlobalObject* globalObject) const
 {
-    if (isInt32())
-        return asInt32();
-    if (isDouble() && canBeInt32(asDouble()))
-        return static_cast<int32_t>(asDouble());
-    if (isBigInt())
-        return asBigInt(*this);
+    if (isString())
+        return asString(asCell());
+    bool returnEmptyStringOnError = true;
+    return toStringSlowCase(globalObject, returnEmptyStringOnError);
+}
 
-    VM& vm = getVM(globalObject);
-    auto scope = DECLARE_THROW_SCOPE(vm);
-    JSValue primValue = this->toPrimitive(globalObject, PreferNumber);
-    RETURN_IF_EXCEPTION(scope, 0);
-    if (primValue.isBigInt())
-        return asBigInt(primValue);
-    int32_t value = primValue.toInt32(globalObject);
-    RETURN_IF_EXCEPTION(scope, 0);
-    return value;
+inline JSString* JSValue::toStringOrNull(JSGlobalObject* globalObject) const
+{
+    if (isString())
+        return asString(asCell());
+    bool returnEmptyStringOnError = false;
+    return toStringSlowCase(globalObject, returnEmptyStringOnError);
+}
+
+inline String JSValue::toWTFString(JSGlobalObject* globalObject) const
+{
+    if (isString())
+        return static_cast<JSString*>(asCell())->value(globalObject);
+    return toWTFStringSlowCase(globalObject);
 }
 
 inline JSObject* JSValue::toObject(JSGlobalObject* globalObject) const
@@ -820,54 +992,65 @@ inline JSObject* JSValue::toObject(JSGlobalObject* globalObject) const
     return isCell() ? asCell()->toObject(globalObject) : toObjectSlowCase(globalObject);
 }
 
-inline bool JSValue::isFunction(VM& vm) const
+inline bool JSValue::isCallable() const
 {
-    if (!isCell())
-        return false;
-    return asCell()->isFunction(vm);
+    return isCell() && asCell()->isCallable();
 }
 
-inline bool JSValue::isCallable(VM& vm, CallType& callType, CallData& callData) const
+template<Concurrency concurrency>
+inline TriState JSValue::isCallableWithConcurrency() const
 {
     if (!isCell())
-        return false;
-    return asCell()->isCallable(vm, callType, callData);
+        return TriState::False;
+    return asCell()->isCallableWithConcurrency<concurrency>();
 }
 
-inline bool JSValue::isConstructor(VM& vm) const
+inline bool JSValue::isConstructor() const
 {
-    if (!isCell())
-        return false;
-    return asCell()->isConstructor(vm);
+    return isCell() && asCell()->isConstructor();
 }
 
-inline bool JSValue::isConstructor(VM& vm, ConstructType& constructType, ConstructData& constructData) const
+template<Concurrency concurrency>
+inline TriState JSValue::isConstructorWithConcurrency() const
 {
     if (!isCell())
-        return false;
-    return asCell()->isConstructor(vm, constructType, constructData);
+        return TriState::False;
+    return asCell()->isConstructorWithConcurrency<concurrency>();
 }
 
 // this method is here to be after the inline declaration of JSCell::inherits
-inline bool JSValue::inherits(VM& vm, const ClassInfo* classInfo) const
+inline bool JSValue::inherits(const ClassInfo* classInfo) const
 {
-    return isCell() && asCell()->inherits(vm, classInfo);
+    return isCell() && asCell()->inherits(classInfo);
 }
 
 template<typename Target>
-inline bool JSValue::inherits(VM& vm) const
+inline bool JSValue::inherits() const
 {
-    return isCell() && asCell()->inherits<Target>(vm);
+    return isCell() && asCell()->inherits<Target>();
 }
 
-inline const ClassInfo* JSValue::classInfoOrNull(VM& vm) const
+inline const ClassInfo* JSValue::classInfoOrNull() const
 {
-    return isCell() ? asCell()->classInfo(vm) : nullptr;
+    return isCell() ? asCell()->classInfo() : nullptr;
 }
 
 inline JSValue JSValue::toThis(JSGlobalObject* globalObject, ECMAMode ecmaMode) const
 {
-    return isCell() ? asCell()->methodTable(getVM(globalObject))->toThis(asCell(), globalObject, ecmaMode) : toThisSlowCase(globalObject, ecmaMode);
+    if (isObject()) {
+        if (asObject(*this)->inherits<JSScope>())
+            return ecmaMode.isStrict() ? jsUndefined() : globalObject->globalThis();
+        return *this;
+    }
+
+    if (ecmaMode.isStrict())
+        return *this;
+
+    ASSERT(!ecmaMode.isStrict());
+    if (isUndefinedOrNull())
+        return globalObject->globalThis();
+
+    return toThisSloppySlowCase(globalObject);
 }
 
 ALWAYS_INLINE JSValue JSValue::get(JSGlobalObject* globalObject, PropertyName propertyName) const
@@ -887,14 +1070,14 @@ ALWAYS_INLINE JSValue JSValue::get(JSGlobalObject* globalObject, PropertyName pr
 }
 
 template<typename CallbackWhenNoException>
-ALWAYS_INLINE typename std::result_of<CallbackWhenNoException(bool, PropertySlot&)>::type JSValue::getPropertySlot(JSGlobalObject* globalObject, PropertyName propertyName, CallbackWhenNoException callback) const
+ALWAYS_INLINE typename std::invoke_result<CallbackWhenNoException, bool, PropertySlot&>::type JSValue::getPropertySlot(JSGlobalObject* globalObject, PropertyName propertyName, CallbackWhenNoException callback) const
 {
     PropertySlot slot(asValue(), PropertySlot::InternalMethodType::Get);
     return getPropertySlot(globalObject, propertyName, slot, callback);
 }
 
 template<typename CallbackWhenNoException>
-ALWAYS_INLINE typename std::result_of<CallbackWhenNoException(bool, PropertySlot&)>::type JSValue::getPropertySlot(JSGlobalObject* globalObject, PropertyName propertyName, PropertySlot& slot, CallbackWhenNoException callback) const
+ALWAYS_INLINE typename std::invoke_result<CallbackWhenNoException, bool, PropertySlot&>::type JSValue::getPropertySlot(JSGlobalObject* globalObject, PropertyName propertyName, PropertySlot& slot, CallbackWhenNoException callback) const
 {
     auto scope = DECLARE_THROW_SCOPE(getVM(globalObject));
     bool found = getPropertySlot(globalObject, propertyName, slot);
@@ -981,12 +1164,24 @@ ALWAYS_INLINE JSValue JSValue::get(JSGlobalObject* globalObject, uint64_t proper
     return get(globalObject, Identifier::from(getVM(globalObject), static_cast<double>(propertyName)));
 }
 
+template<typename T, typename PropertyNameType>
+ALWAYS_INLINE T JSValue::getAs(JSGlobalObject* globalObject, PropertyNameType propertyName) const
+{
+    JSValue value = get(globalObject, propertyName);
+#if ASSERT_ENABLED || ENABLE(SECURITY_ASSERTIONS)
+    VM& vm = getVM(globalObject);
+    if (vm.exceptionForInspection())
+        return nullptr;
+#endif
+    return jsCast<T>(value);
+}
+
 inline bool JSValue::put(JSGlobalObject* globalObject, PropertyName propertyName, JSValue value, PutPropertySlot& slot)
 {
     if (UNLIKELY(!isCell()))
         return putToPrimitive(globalObject, propertyName, value, slot);
 
-    return asCell()->methodTable(getVM(globalObject))->put(asCell(), globalObject, propertyName, value, slot);
+    return asCell()->methodTable()->put(asCell(), globalObject, propertyName, value, slot);
 }
 
 ALWAYS_INLINE bool JSValue::putInline(JSGlobalObject* globalObject, PropertyName propertyName, JSValue value, PutPropertySlot& slot)
@@ -1001,7 +1196,15 @@ inline bool JSValue::putByIndex(JSGlobalObject* globalObject, unsigned propertyN
     if (UNLIKELY(!isCell()))
         return putToPrimitiveByIndex(globalObject, propertyName, value, shouldThrow);
 
-    return asCell()->methodTable(getVM(globalObject))->putByIndex(asCell(), globalObject, propertyName, value, shouldThrow);
+    return asCell()->methodTable()->putByIndex(asCell(), globalObject, propertyName, value, shouldThrow);
+}
+
+ALWAYS_INLINE JSValue JSValue::getPrototype(JSGlobalObject* globalObject) const
+{
+    VM& vm = getVM(globalObject);
+    if (isObject())
+        return asObject(asCell())->getPrototype(vm, globalObject);
+    return synthesizePrototype(globalObject);
 }
 
 inline Structure* JSValue::structureOrNull() const
@@ -1011,13 +1214,6 @@ inline Structure* JSValue::structureOrNull() const
     return nullptr;
 }
 
-inline JSValue JSValue::structureOrUndefined() const
-{
-    if (isCell())
-        return JSValue(asCell()->structure());
-    return jsUndefined();
-}
-
 // ECMA 11.9.3
 inline bool JSValue::equal(JSGlobalObject* globalObject, JSValue v1, JSValue v2)
 {
@@ -1032,55 +1228,35 @@ ALWAYS_INLINE bool JSValue::equalSlowCaseInline(JSGlobalObject* globalObject, JS
     VM& vm = getVM(globalObject);
     auto scope = DECLARE_THROW_SCOPE(vm);
     do {
-        if (v1.isNumber() && v2.isNumber())
-            return v1.asNumber() == v2.asNumber();
-
-        bool s1 = v1.isString();
-        bool s2 = v2.isString();
-        if (s1 && s2)
-            RELEASE_AND_RETURN(scope, asString(v1)->equal(globalObject, asString(v2)));
-
-        if (v1.isBigInt() && s2) {
-            String v2String = asString(v2)->value(globalObject);
-            RETURN_IF_EXCEPTION(scope, false);
-            JSBigInt* n = JSBigInt::stringToBigInt(globalObject, v2String);
-            RETURN_IF_EXCEPTION(scope, false);
-            if (!n)
-                return false;
-            
-            v2 = JSValue(n);
-            continue;
+        if (v1.isNumber()) {
+            if (v2.isNumber())
+                return v1.asNumber() == v2.asNumber();
+            // Guaranteeing that if we have a number it is v2 makes some of the cases below simpler.
+            std::swap(v1, v2);
         }
 
-        if (s1 && v2.isBigInt()) {
-            String v1String = asString(v1)->value(globalObject);
-            RETURN_IF_EXCEPTION(scope, false);
-            JSBigInt* n = JSBigInt::stringToBigInt(globalObject, v1String);
-            RETURN_IF_EXCEPTION(scope, false);
-            if (!n)
-                return false;
-            
-            v1 = JSValue(n);
-            continue;
-        }
+        // This deals with Booleans, BigInt32, Objects, and is a shortcut for a few more types.
+        // It has to come here and not before, because it is NOT true that NaN == NaN
+        if (v1 == v2)
+            return true;
 
         if (v1.isUndefinedOrNull()) {
             if (v2.isUndefinedOrNull())
                 return true;
             if (!v2.isCell())
                 return false;
-            return v2.asCell()->structure(vm)->masqueradesAsUndefined(globalObject);
+            return v2.asCell()->structure()->masqueradesAsUndefined(globalObject);
         }
 
         if (v2.isUndefinedOrNull()) {
             if (!v1.isCell())
                 return false;
-            return v1.asCell()->structure(vm)->masqueradesAsUndefined(globalObject);
+            return v1.asCell()->structure()->masqueradesAsUndefined(globalObject);
         }
 
         if (v1.isObject()) {
             if (v2.isObject())
-                return v1 == v2;
+                return false; // v1 == v2 is already dealt with previously
             JSValue p1 = v1.toPrimitive(globalObject);
             RETURN_IF_EXCEPTION(scope, false);
             v1 = p1;
@@ -1102,56 +1278,86 @@ ALWAYS_INLINE bool JSValue::equalSlowCaseInline(JSGlobalObject* globalObject, JS
         bool sym2 = v2.isSymbol();
         if (sym1 || sym2) {
             if (sym1 && sym2)
-                return asSymbol(v1) == asSymbol(v2);
+                return false; // v1 == v2 is already dealt with previously
             return false;
         }
 
+        bool s1 = v1.isString();
+        bool s2 = v2.isString();
+        if (s1) {
+            if (s2)
+                RELEASE_AND_RETURN(scope, asString(v1)->equal(globalObject, asString(v2)));
+            std::swap(v1, v2);
+            // We are guaranteed to enter the next case, so losing the invariant of only v2 being a number is fine
+        }
         if (s1 || s2) {
-            double d1 = v1.toNumber(globalObject);
-            RETURN_IF_EXCEPTION(scope, false);
-            double d2 = v2.toNumber(globalObject);
-            RETURN_IF_EXCEPTION(scope, false);
-            return d1 == d2;
+            // We are guaranteed that the string is v2 (thanks to the swap above)
+            if (v1.isBigInt()) {
+                auto v2String = asString(v2)->value(globalObject);
+                RETURN_IF_EXCEPTION(scope, false);
+                v2 = JSBigInt::stringToBigInt(globalObject, v2String);
+                RETURN_IF_EXCEPTION(scope, false);
+                if (!v2)
+                    return false;
+                if (v1 == v2)
+                    return true; // For BigInt32
+                // We fallthrough to the generic code for comparing BigInts (which is only missing the BigInt32/BigInt32 case, hence the check above)
+            } else {
+                ASSERT(v1.isNumber() || v1.isBoolean());
+                double d1 = v1.toNumber(globalObject);
+                RETURN_IF_EXCEPTION(scope, false);
+                double d2 = v2.toNumber(globalObject);
+                RETURN_IF_EXCEPTION(scope, false);
+                return d1 == d2;
+            }
         }
 
         if (v1.isBoolean()) {
             if (v2.isNumber())
                 return static_cast<double>(v1.asBoolean()) == v2.asNumber();
-            else if (v2.isBigInt()) {
-                v1 = JSValue(v1.toNumber(globalObject));
-                continue;
-            }
+            v1 = JSValue(v1.toNumber(globalObject));
+            RETURN_IF_EXCEPTION(scope, false);
+            // We fallthrough to the BigInt/Number comparison below
+            // We just need one more swap to repair the rule that only v2 is allowed to be a number in these comparisons
+            std::swap(v1, v2);
         } else if (v2.isBoolean()) {
-            if (v1.isNumber())
-                return v1.asNumber() == static_cast<double>(v2.asBoolean());
-            else if (v1.isBigInt()) {
-                v2 = JSValue(v2.toNumber(globalObject));
-                continue;
-            }
+            v2 = JSValue(v2.toNumber(globalObject));
+            RETURN_IF_EXCEPTION(scope, false);
+            // We fallthrough to the BigInt/Number comparison below
         }
-        
-        if (v1.isBigInt() && v2.isBigInt())
-            return JSBigInt::equals(asBigInt(v1), asBigInt(v2));
-        
-        if (v1.isBigInt() && v2.isNumber())
-            return asBigInt(v1)->equalsToNumber(v2);
 
-        if (v2.isBigInt() && v1.isNumber())
-            return asBigInt(v2)->equalsToNumber(v1);
+#if USE(BIGINT32)
+        if (v1.isBigInt32()) {
+            if (v2.isInt32())
+                return v1.bigInt32AsInt32() == v2.asInt32();
+            if (v2.isDouble())
+                return static_cast<double>(v1.bigInt32AsInt32()) == v2.asDouble();
+            std::swap(v1, v2);
+        }
+#endif // USE(BIGINT32)
 
-        return v1 == v2;
+        if (v1.isHeapBigInt()) {
+            if (v2.isHeapBigInt())
+                return JSBigInt::equals(v1.asHeapBigInt(), v2.asHeapBigInt());
+#if USE(BIGINT32)
+            if (v2.isBigInt32())
+                return v1.asHeapBigInt()->equalsToInt32(v2.bigInt32AsInt32());
+#endif // USE(BIGINT32)
+            if (v2.isNumber())
+                return v1.asHeapBigInt()->equalsToNumber(v2);
+        }
+
+        return false;
     } while (true);
 }
 
 // ECMA 11.9.3
-ALWAYS_INLINE bool JSValue::strictEqualSlowCaseInline(JSGlobalObject* globalObject, JSValue v1, JSValue v2)
+ALWAYS_INLINE bool JSValue::strictEqualForCells(JSGlobalObject* globalObject, JSCell* v1, JSCell* v2)
 {
-    ASSERT(v1.isCell() && v2.isCell());
-
-    if (v1.asCell()->isString() && v2.asCell()->isString())
+    if (v1->isString() && v2->isString())
         return asString(v1)->equal(globalObject, asString(v2));
-    if (v1.isBigInt() && v2.isBigInt())
-        return JSBigInt::equals(asBigInt(v1), asBigInt(v2));
+    if (v1->isHeapBigInt() && v2->isHeapBigInt())
+        return JSBigInt::equals(static_cast<JSBigInt*>(v1), static_cast<JSBigInt*>(v2));
     return v1 == v2;
 }
 
@@ -1163,10 +1369,17 @@ inline bool JSValue::strictEqual(JSGlobalObject* globalObject, JSValue v1, JSVal
     if (v1.isNumber() && v2.isNumber())
         return v1.asNumber() == v2.asNumber();
 
-    if (!v1.isCell() || !v2.isCell())
-        return v1 == v2;
+#if USE(BIGINT32)
+    if (v1.isHeapBigInt() && v2.isBigInt32())
+        return v1.asHeapBigInt()->equalsToInt32(v2.bigInt32AsInt32());
+    if (v1.isBigInt32() && v2.isHeapBigInt())
+        return v2.asHeapBigInt()->equalsToInt32(v1.bigInt32AsInt32());
+#endif
 
-    return strictEqualSlowCaseInline(globalObject, v1, v2);
+    if (v1.isCell() && v2.isCell())
+        return strictEqualForCells(globalObject, v1.asCell(), v2.asCell());
+
+    return v1 == v2;
 }
 
 inline int32_t JSValue::asInt32ForArithmetic() const
@@ -1184,15 +1397,23 @@ inline TriState JSValue::pureStrictEqual(JSValue v1, JSValue v2)
     if (v1.isNumber() && v2.isNumber())
         return triState(v1.asNumber() == v2.asNumber());
 
-    if (!v1.isCell() || !v2.isCell())
-        return triState(v1 == v2);
-    
-    if (v1.asCell()->isString() && v2.asCell()->isString()) {
-        const StringImpl* v1String = asString(v1)->tryGetValueImpl();
-        const StringImpl* v2String = asString(v2)->tryGetValueImpl();
-        if (!v1String || !v2String)
-            return MixedTriState;
-        return triState(WTF::equal(*v1String, *v2String));
+#if USE(BIGINT32)
+    if (v1.isHeapBigInt() && v2.isBigInt32())
+        return triState(v1.asHeapBigInt()->equalsToInt32(v2.bigInt32AsInt32()));
+    if (v1.isBigInt32() && v2.isHeapBigInt())
+        return triState(v2.asHeapBigInt()->equalsToInt32(v1.bigInt32AsInt32()));
+#endif
+
+    if (v1.isCell() && v2.isCell()) {
+        if (v1.asCell()->isString() && v2.asCell()->isString()) {
+            const StringImpl* v1String = asString(v1)->tryGetValueImpl();
+            const StringImpl* v2String = asString(v2)->tryGetValueImpl();
+            if (!v1String || !v2String)
+                return TriState::Indeterminate;
+            return triState(WTF::equal(*v1String, *v2String));
+        }
+        if (v1.asCell()->isHeapBigInt() && v2.asCell()->isHeapBigInt())
+            return triState(JSBigInt::equals(v1.asHeapBigInt(), v2.asHeapBigInt()));
     }
     
     return triState(v1 == v2);
@@ -1201,12 +1422,16 @@ inline TriState JSValue::pureStrictEqual(JSValue v1, JSValue v2)
 inline TriState JSValue::pureToBoolean() const
 {
     if (isInt32())
-        return asInt32() ? TrueTriState : FalseTriState;
+        return asInt32() ? TriState::True : TriState::False;
     if (isDouble())
-        return isNotZeroAndOrdered(asDouble()) ? TrueTriState : FalseTriState; // false for NaN
+        return isNotZeroAndOrdered(asDouble()) ? TriState::True : TriState::False; // false for NaN
     if (isCell())
         return asCell()->pureToBoolean();
-    return isTrue() ? TrueTriState : FalseTriState;
+#if USE(BIGINT32)
+    if (isBigInt32())
+        return bigInt32AsInt32() ? TriState::True : TriState::False;
+#endif
+    return isTrue() ? TriState::True : TriState::False;
 }
 
 ALWAYS_INLINE bool JSValue::requireObjectCoercible(JSGlobalObject* globalObject) const
@@ -1229,8 +1454,8 @@ ALWAYS_INLINE bool isThisValueAltered(const PutPropertySlot& slot, JSObject* bas
     if (!thisValue.isObject())
         return true;
     JSObject* thisObject = asObject(thisValue);
-    // Only PureForwardingProxyType can be seen as the same to the original target object.
-    if (thisObject->type() == PureForwardingProxyType && jsCast<JSProxy*>(thisObject)->target() == baseObject)
+    // Only GlobalProxyType can be seen as the same to the original target object.
+    if (thisObject->type() == GlobalProxyType && jsCast<JSGlobalProxy*>(thisObject)->target() == baseObject)
         return false;
     return true;
 }
@@ -1238,6 +1463,9 @@ ALWAYS_INLINE bool isThisValueAltered(const PutPropertySlot& slot, JSObject* bas
 // See section 7.2.9: https://tc39.github.io/ecma262/#sec-samevalue
 ALWAYS_INLINE bool sameValue(JSGlobalObject* globalObject, JSValue a, JSValue b)
 {
+    if (a == b)
+        return true;
+
     if (!a.isNumber())
         return JSValue::strictEqual(globalObject, a, b);
     if (!b.isNumber())
@@ -1248,7 +1476,29 @@ ALWAYS_INLINE bool sameValue(JSGlobalObject* globalObject, JSValue a, JSValue b)
     bool yIsNaN = std::isnan(y);
     if (xIsNaN || yIsNaN)
         return xIsNaN && yIsNaN;
-    return bitwise_cast<uint64_t>(x) == bitwise_cast<uint64_t>(y);
+    return std::bit_cast<uint64_t>(x) == std::bit_cast<uint64_t>(y);
+}
+
+ALWAYS_INLINE bool sameValueZero(JSGlobalObject* globalObject, JSValue a, JSValue b)
+{
+    if (a == b)
+        return true;
+
+    if (!a.isNumber())
+        return JSValue::strictEqual(globalObject, a, b);
+    if (!b.isNumber())
+        return false;
+    double x = a.asNumber();
+    double y = b.asNumber();
+    if (std::isnan(x))
+        return std::isnan(y);
+    if (std::isnan(y))
+        return std::isnan(x);
+    if (!x && y == -0)
+        return true;
+    if (x == -0 && !y)
+        return true;
+    return std::bit_cast<uint64_t>(x) == std::bit_cast<uint64_t>(y);
 }
 
 } // namespace JSC
