@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2013-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2013-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2020 Sony Interactive Entertainment Inc.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -29,48 +30,45 @@
 #include "Error.h"
 #include "IteratorOperations.h"
 #include "JSArrayBuffer.h"
+#include "JSArrayBufferPrototypeInlines.h"
 #include "JSCJSValueInlines.h"
 #include "JSDataView.h"
 #include "JSGenericTypedArrayViewConstructor.h"
 #include "JSGlobalObject.h"
+#include "JSTypedArrays.h"
 #include "StructureInlines.h"
 
 namespace JSC {
 
-template<typename ViewClass>
-static EncodedJSValue JSC_HOST_CALL callGenericTypedArrayView(JSGlobalObject*, CallFrame*);
-
-template<typename ViewClass>
-EncodedJSValue JSC_HOST_CALL constructGenericTypedArrayView(JSGlobalObject*, CallFrame*);
-
 template<typename ViewClass>
 JSGenericTypedArrayViewConstructor<ViewClass>::JSGenericTypedArrayViewConstructor(VM& vm, Structure* structure)
-    : Base(vm, structure, callGenericTypedArrayView<ViewClass>, constructGenericTypedArrayView<ViewClass>)
+    : Base(vm, structure, callConstructor(), constructConstructor())
 {
 }
 
 template<typename ViewClass>
-void JSGenericTypedArrayViewConstructor<ViewClass>::finishCreation(VM& vm, JSGlobalObject* globalObject, JSObject* prototype, const String& name, FunctionExecutable* privateAllocator)
+void JSGenericTypedArrayViewConstructor<ViewClass>::finishCreation(VM& vm, JSGlobalObject* globalObject, JSObject* prototype, const String& name)
 {
-    Base::finishCreation(vm, name, NameAdditionMode::WithoutStructureTransition);
+    Base::finishCreation(vm, ViewClass::TypedArrayStorageType == TypeDataView ? 1 : 3, name, PropertyAdditionMode::WithoutStructureTransition);
     putDirectWithoutTransition(vm, vm.propertyNames->prototype, prototype, PropertyAttribute::DontEnum | PropertyAttribute::DontDelete | PropertyAttribute::ReadOnly);
-    putDirectWithoutTransition(vm, vm.propertyNames->length, jsNumber(3), PropertyAttribute::DontEnum | PropertyAttribute::ReadOnly);
     putDirectWithoutTransition(vm, vm.propertyNames->BYTES_PER_ELEMENT, jsNumber(ViewClass::elementSize), PropertyAttribute::DontEnum | PropertyAttribute::ReadOnly | PropertyAttribute::DontDelete);
 
-    if (privateAllocator)
-        putDirectBuiltinFunction(vm, globalObject, vm.propertyNames->builtinNames().allocateTypedArrayPrivateName(), privateAllocator, PropertyAttribute::DontEnum | PropertyAttribute::DontDelete | PropertyAttribute::ReadOnly);
+    if constexpr (std::is_same_v<ViewClass, JSUint8Array>) {
+        JSC_NATIVE_FUNCTION_WITHOUT_TRANSITION("fromBase64"_s, uint8ArrayConstructorFromBase64, static_cast<unsigned>(PropertyAttribute::DontEnum), 1, ImplementationVisibility::Public);
+        JSC_NATIVE_FUNCTION_WITHOUT_TRANSITION("fromHex"_s, uint8ArrayConstructorFromHex, static_cast<unsigned>(PropertyAttribute::DontEnum), 1, ImplementationVisibility::Public);
+    }
 }
 
 template<typename ViewClass>
 JSGenericTypedArrayViewConstructor<ViewClass>*
 JSGenericTypedArrayViewConstructor<ViewClass>::create(
     VM& vm, JSGlobalObject* globalObject, Structure* structure, JSObject* prototype,
-    const String& name, FunctionExecutable* privateAllocator)
+    const String& name)
 {
     JSGenericTypedArrayViewConstructor* result =
-        new (NotNull, allocateCell<JSGenericTypedArrayViewConstructor>(vm.heap))
+        new (NotNull, allocateCell<JSGenericTypedArrayViewConstructor>(vm))
         JSGenericTypedArrayViewConstructor(vm, structure);
-    result->finishCreation(vm, globalObject, prototype, name, privateAllocator);
+    result->finishCreation(vm, globalObject, prototype, name);
     return result;
 }
 
@@ -114,32 +112,43 @@ inline JSObject* constructGenericTypedArrayViewFromIterator(JSGlobalObject* glob
 }
 
 template<typename ViewClass>
-inline JSObject* constructGenericTypedArrayViewWithArguments(JSGlobalObject* globalObject, Structure* structure, EncodedJSValue firstArgument, unsigned offset, Optional<unsigned> lengthOpt)
+inline JSObject* constructGenericTypedArrayViewWithArguments(JSGlobalObject* globalObject, Structure* structure, JSValue firstValue, size_t offset, std::optional<size_t> lengthOpt)
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
-    JSValue firstValue = JSValue::decode(firstArgument);
-
-    if (JSArrayBuffer* jsBuffer = jsDynamicCast<JSArrayBuffer*>(vm, firstValue)) {
+    // https://tc39.es/proposal-resizablearraybuffer/#sec-initializetypedarrayfromarraybuffer
+    if (JSArrayBuffer* jsBuffer = jsDynamicCast<JSArrayBuffer*>(firstValue)) {
         RefPtr<ArrayBuffer> buffer = jsBuffer->impl();
-        unsigned length = 0;
+        if (buffer->isDetached()) {
+            throwTypeError(globalObject, scope, "Buffer is already detached"_s);
+            return nullptr;
+        }
 
+        std::optional<size_t> length;
         if (lengthOpt)
-            length = lengthOpt.value();
+            length = lengthOpt;
         else {
-            if (UNLIKELY((buffer->byteLength() - offset) % ViewClass::elementSize)) {
-                throwRangeError(globalObject, scope, "ArrayBuffer length minus the byteOffset is not a multiple of the element size"_s);
-                return nullptr;
+            size_t byteLength = buffer->byteLength();
+            if (buffer->isResizableOrGrowableShared()) {
+                if (UNLIKELY(offset > byteLength)) {
+                    throwRangeError(globalObject, scope, "byteOffset exceeds source ArrayBuffer byteLength"_s);
+                    return nullptr;
+                }
+            } else {
+                if (UNLIKELY((byteLength - offset) % ViewClass::elementSize)) {
+                    throwRangeError(globalObject, scope, "ArrayBuffer length minus the byteOffset is not a multiple of the element size"_s);
+                    return nullptr;
+                }
+                length = (byteLength - offset) / ViewClass::elementSize;
             }
-            length = (buffer->byteLength() - offset) / ViewClass::elementSize;
         }
 
         RELEASE_AND_RETURN(scope, ViewClass::create(globalObject, structure, WTFMove(buffer), offset, length));
     }
     ASSERT(!offset && !lengthOpt);
-    
-    if (UNLIKELY(ViewClass::TypedArrayStorageType == TypeDataView)) {
+
+    if constexpr (ViewClass::TypedArrayStorageType == TypeDataView) {
         throwTypeError(globalObject, scope, "Expected ArrayBuffer for the first argument."_s);
         return nullptr;
     }
@@ -148,14 +157,42 @@ inline JSObject* constructGenericTypedArrayViewWithArguments(JSGlobalObject* glo
     // - Another array. This creates a copy of the of that array.
     // - A primitive. This creates a new typed array of that length and zero-initializes it.
 
-    if (JSObject* object = jsDynamicCast<JSObject*>(vm, firstValue)) {
-        unsigned length;
+    if (JSObject* object = jsDynamicCast<JSObject*>(firstValue)) {
+        size_t length;
 
-        if (isTypedView(object->classInfo(vm)->typedArrayStorageType))
-            length = jsCast<JSArrayBufferView*>(object)->length();
+        // https://tc39.es/proposal-resizablearraybuffer/#sec-initializetypedarrayfromtypedarray
+        if (isTypedView(object->type())) {
+            auto* view = jsCast<JSArrayBufferView*>(object);
+
+            length = view->length();
+
+            ViewClass* result = ViewClass::createUninitialized(globalObject, structure, length);
+            EXCEPTION_ASSERT(!!scope.exception() == !result);
+            if (UNLIKELY(!result))
+                return nullptr;
+
+            IdempotentArrayBufferByteLengthGetter<std::memory_order_seq_cst> getter;
+            if (isIntegerIndexedObjectOutOfBounds(view, getter)) {
+                throwTypeError(globalObject, scope, typedArrayBufferHasBeenDetachedErrorMessage);
+                return nullptr;
+            }
+
+            if (contentType(object->type()) != ViewClass::contentType) {
+                throwTypeError(globalObject, scope, "Content types of source and new typed array are different"_s);
+                return nullptr;
+            }
+
+            scope.release();
+            if (!result->setFromTypedArray(globalObject, 0, view, 0, length, CopyType::Unobservable))
+                return nullptr;
+            return result;
+        }
+
+        // This getPropertySlot operation should not be observed by the Proxy.
+        // So we use VMInquiry. And purge the opaque object cases (proxy and namespace object) by isTaintedByOpaqueObject() guard.
+        if (JSArray* array = jsDynamicCast<JSArray*>(object); LIKELY(array && isJSArray(array) && array->isIteratorProtocolFastAndNonObservable()))
+            length = array->length();
         else {
-            // This getPropertySlot operation should not be observed by the Proxy.
-            // So we use VMInquiry. And purge the opaque object cases (proxy and namespace object) by isTaintedByOpaqueObject() guard.
             PropertySlot lengthSlot(object, PropertySlot::InternalMethodType::VMInquiry, &vm);
             object->getPropertySlot(globalObject, vm.propertyNames->length, lengthSlot);
             RETURN_IF_EXCEPTION(scope, nullptr);
@@ -168,13 +205,14 @@ inline JSObject* constructGenericTypedArrayViewWithArguments(JSGlobalObject* glo
             // 1) The iterator is not a known iterator.
             // 2) The base object does not have a length getter.
             // 3) The base object might have indexed getters.
+            // 4) The iterator protocol is still intact.
             // it should not be observable that we do not use the iterator.
 
             if (!iteratorFunc.isUndefinedOrNull()
-                && (iteratorFunc != object->globalObject(vm)->arrayProtoValuesFunction()
+                && (iteratorFunc != object->globalObject()->arrayProtoValuesFunction()
                     || lengthSlot.isAccessor() || lengthSlot.isCustom() || lengthSlot.isTaintedByOpaqueObject()
-                    || hasAnyArrayStorage(object->indexingType()))) {
-
+                    || hasAnyArrayStorage(object->indexingType())
+                    || !object->globalObject()->arrayIteratorProtocolWatchpointSet().isStillValid())) {
                     RELEASE_AND_RETURN(scope, constructGenericTypedArrayViewFromIterator<ViewClass>(globalObject, structure, object, iteratorFunc));
             }
 
@@ -183,72 +221,92 @@ inline JSObject* constructGenericTypedArrayViewWithArguments(JSGlobalObject* glo
             else {
                 JSValue value = lengthSlot.getValue(globalObject, vm.propertyNames->length);
                 RETURN_IF_EXCEPTION(scope, nullptr);
-                length = value.toUInt32(globalObject);
+                length = value.toLength(globalObject);
                 RETURN_IF_EXCEPTION(scope, nullptr);
             }
         }
 
-        
         ViewClass* result = ViewClass::createUninitialized(globalObject, structure, length);
         EXCEPTION_ASSERT(!!scope.exception() == !result);
         if (UNLIKELY(!result))
             return nullptr;
-        
+
         scope.release();
-        if (!result->set(globalObject, 0, object, 0, length))
+        if (!result->setFromArrayLike(globalObject, 0, object, 0, length))
             return nullptr;
-        
         return result;
     }
 
-    unsigned length = firstValue.toIndex(globalObject, "length");
+    size_t length = firstValue.toTypedArrayIndex(globalObject, "length"_s);
     RETURN_IF_EXCEPTION(scope, nullptr);
+
     RELEASE_AND_RETURN(scope, ViewClass::create(globalObject, structure, length));
 }
 
 template<typename ViewClass>
-EncodedJSValue JSC_HOST_CALL constructGenericTypedArrayView(JSGlobalObject* globalObject, CallFrame* callFrame)
+ALWAYS_INLINE EncodedJSValue constructGenericTypedArrayViewImpl(JSGlobalObject* globalObject, CallFrame* callFrame)
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
 
     JSObject* newTarget = asObject(callFrame->newTarget());
-    Structure* structure = newTarget == callFrame->jsCallee()
-        ? globalObject->typedArrayStructure(ViewClass::TypedArrayStorageType)
-        : InternalFunction::createSubclassStructure(globalObject, newTarget, getFunctionRealm(vm, newTarget)->typedArrayStructure(ViewClass::TypedArrayStorageType));
-    RETURN_IF_EXCEPTION(scope, { });
 
     size_t argCount = callFrame->argumentCount();
 
     if (!argCount) {
-        if (ViewClass::TypedArrayStorageType == TypeDataView)
+        Structure* structure = JSC_GET_DERIVED_STRUCTURE(vm, typedArrayStructureWithTypedArrayType<ViewClass::TypedArrayStorageType>, newTarget, callFrame->jsCallee());
+        RETURN_IF_EXCEPTION(scope, { });
+
+        if constexpr (ViewClass::TypedArrayStorageType == TypeDataView)
             return throwVMTypeError(globalObject, scope, "DataView constructor requires at least one argument."_s);
 
         RELEASE_AND_RETURN(scope, JSValue::encode(ViewClass::create(globalObject, structure, 0)));
     }
 
+    Structure* structure = nullptr;
     JSValue firstValue = callFrame->uncheckedArgument(0);
-    unsigned offset = 0;
-    Optional<unsigned> length = WTF::nullopt;
-    if (jsDynamicCast<JSArrayBuffer*>(vm, firstValue) && argCount > 1) {
-        offset = callFrame->uncheckedArgument(1).toIndex(globalObject, "byteOffset");
-        RETURN_IF_EXCEPTION(scope, encodedJSValue());
+    size_t offset = 0;
+    std::optional<size_t> length;
+    if (auto* arrayBuffer = jsDynamicCast<JSArrayBuffer*>(firstValue)) {
+        if (argCount > 1) {
+            offset = callFrame->uncheckedArgument(1).toTypedArrayIndex(globalObject, "byteOffset"_s);
+            RETURN_IF_EXCEPTION(scope, { });
+
+            if constexpr (ViewClass::TypedArrayStorageType == TypeDataView) {
+                RefPtr<ArrayBuffer> buffer = arrayBuffer->impl();
+                if (UNLIKELY(offset > buffer->byteLength())) {
+                    throwRangeError(globalObject, scope, "byteOffset exceeds source ArrayBuffer byteLength"_s);
+                    RETURN_IF_EXCEPTION(scope, { });
+                }
+            }
+        }
+
+        if (arrayBuffer->isResizableOrGrowableShared()) {
+            structure = JSC_GET_DERIVED_STRUCTURE(vm, resizableOrGrowableSharedTypedArrayStructureWithTypedArrayType<ViewClass::TypedArrayStorageType>, newTarget, callFrame->jsCallee());
+            RETURN_IF_EXCEPTION(scope, { });
+        } else {
+            structure = JSC_GET_DERIVED_STRUCTURE(vm, typedArrayStructureWithTypedArrayType<ViewClass::TypedArrayStorageType>, newTarget, callFrame->jsCallee());
+            RETURN_IF_EXCEPTION(scope, { });
+        }
 
         if (argCount > 2) {
             // If the length value is present but undefined, treat it as missing.
             JSValue lengthValue = callFrame->uncheckedArgument(2);
             if (!lengthValue.isUndefined()) {
-                length = lengthValue.toIndex(globalObject, ViewClass::TypedArrayStorageType == TypeDataView ? "byteLength" : "length");
+                length = lengthValue.toTypedArrayIndex(globalObject, ViewClass::TypedArrayStorageType == TypeDataView ? "byteLength"_s : "length"_s);
                 RETURN_IF_EXCEPTION(scope, encodedJSValue());
             }
         }
+    } else {
+        structure = JSC_GET_DERIVED_STRUCTURE(vm, typedArrayStructureWithTypedArrayType<ViewClass::TypedArrayStorageType>, newTarget, callFrame->jsCallee());
+        RETURN_IF_EXCEPTION(scope, { });
     }
 
-    RELEASE_AND_RETURN(scope, JSValue::encode(constructGenericTypedArrayViewWithArguments<ViewClass>(globalObject, structure, JSValue::encode(firstValue), offset, length)));
+    RELEASE_AND_RETURN(scope, JSValue::encode(constructGenericTypedArrayViewWithArguments<ViewClass>(globalObject, structure, firstValue, offset, length)));
 }
 
 template<typename ViewClass>
-static EncodedJSValue JSC_HOST_CALL callGenericTypedArrayView(JSGlobalObject* globalObject, CallFrame*)
+ALWAYS_INLINE EncodedJSValue callGenericTypedArrayViewImpl(JSGlobalObject* globalObject, CallFrame*)
 {
     VM& vm = globalObject->vm();
     auto scope = DECLARE_THROW_SCOPE(vm);
