@@ -17,16 +17,22 @@ import xml.etree.ElementTree as etree
 
 from enum import Enum
 
-xml_inputs = [
-    'cl.xml',
-    'gl.xml',
+khronos_xml_inputs = [
+    '../third_party/EGL-Registry/src/api/egl.xml',
+    '../third_party/OpenCL-Docs/src/xml/cl.xml',
+    '../third_party/OpenGL-Registry/src/xml/gl.xml',
+    '../third_party/OpenGL-Registry/src/xml/glx.xml',
+    '../third_party/OpenGL-Registry/src/xml/wgl.xml',
+]
+
+angle_xml_inputs = [
     'gl_angle_ext.xml',
-    'egl.xml',
     'egl_angle_ext.xml',
-    'wgl.xml',
     'registry_xml.py',
 ]
 
+xml_inputs = sorted(khronos_xml_inputs + angle_xml_inputs)
+
 # Notes on categories of extensions:
 # 'Requestable' extensions are extensions that can be enabled with ANGLE_request_extension
 # 'ES-Only' extensions are always implicitly enabled.
@@ -41,32 +47,41 @@ angle_toggleable_extensions = [
 angle_requestable_extensions = [
     "GL_ANGLE_base_vertex_base_instance",
     "GL_ANGLE_base_vertex_base_instance_shader_builtin",
+    "GL_ANGLE_blob_cache",
+    "GL_ANGLE_clip_cull_distance",
     "GL_ANGLE_compressed_texture_etc",
     "GL_ANGLE_copy_texture_3d",
     "GL_ANGLE_framebuffer_multisample",
     "GL_ANGLE_get_image",
     "GL_ANGLE_get_tex_level_parameter",
+    "GL_ANGLE_logic_op",
     "GL_ANGLE_lossy_etc_decode",
     "GL_ANGLE_memory_object_flags",
     "GL_ANGLE_memory_object_fuchsia",
     "GL_ANGLE_memory_size",
     "GL_ANGLE_multi_draw",
     "GL_ANGLE_multiview_multisample",
+    "GL_ANGLE_polygon_mode",
     "GL_ANGLE_provoking_vertex",
     "GL_ANGLE_read_only_depth_stencil_feedback_loops",
+    "GL_ANGLE_renderability_validation",
     "GL_ANGLE_robust_fragment_shader_output",
     "GL_ANGLE_semaphore_fuchsia",
+    "GL_ANGLE_shader_pixel_local_storage",
+    "GL_ANGLE_shader_pixel_local_storage_coherent",
+    "GL_ANGLE_stencil_texturing",
     "GL_ANGLE_texture_compression_dxt3",
     "GL_ANGLE_texture_compression_dxt5",
     "GL_ANGLE_texture_external_update",
     "GL_ANGLE_texture_multisample",
+    "GL_ANGLE_variable_rasterization_rate_metal",
     "GL_ANGLE_vulkan_image",
     "GL_ANGLE_yuv_internal_format",
     "GL_CHROMIUM_color_buffer_float_rgb",
     "GL_CHROMIUM_color_buffer_float_rgba",
     "GL_CHROMIUM_lose_context",
     "GL_CHROMIUM_sync_query",
-    "GL_CHROMIUM_texture_filtering_hint",
+    "GL_WEBKIT_explicit_resolve_target",
 ]
 
 gles_requestable_extensions = [
@@ -76,16 +91,22 @@ gles_requestable_extensions = [
     "GL_ANGLE_texture_usage",
     "GL_APPLE_clip_distance",
     "GL_ARB_sync",
+    "GL_ARM_rgba8",
+    "GL_ARM_shader_framebuffer_fetch",
+    "GL_ARM_shader_framebuffer_fetch_depth_stencil",
     "GL_EXT_base_instance",
     "GL_EXT_blend_func_extended",
     "GL_EXT_blend_minmax",
     "GL_EXT_buffer_storage",
+    "GL_EXT_clear_texture",
     "GL_EXT_clip_control",
     "GL_EXT_clip_cull_distance",
     "GL_EXT_color_buffer_float",
     "GL_EXT_color_buffer_half_float",
     "GL_EXT_compressed_ETC1_RGB8_sub_texture",
+    "GL_EXT_conservative_depth",
     "GL_EXT_copy_image",
+    "GL_EXT_depth_clamp",
     "GL_EXT_disjoint_timer_query",
     "GL_EXT_draw_buffers",
     "GL_EXT_draw_buffers_indexed",
@@ -93,6 +114,7 @@ gles_requestable_extensions = [
     "GL_EXT_EGL_image_array",
     "GL_EXT_EGL_image_external_wrap_modes",
     "GL_EXT_EGL_image_storage",
+    "GL_EXT_EGL_image_storage_compression",
     "GL_EXT_external_buffer",
     "GL_EXT_float_blend",
     "GL_EXT_frag_depth",
@@ -106,11 +128,14 @@ gles_requestable_extensions = [
     "GL_EXT_multisampled_render_to_texture",
     "GL_EXT_multisampled_render_to_texture2",
     "GL_EXT_occlusion_query_boolean",
+    "GL_EXT_polygon_offset_clamp",
     "GL_EXT_protected_textures",
     "GL_EXT_pvrtc_sRGB",
     "GL_EXT_read_format_bgra",
+    "GL_EXT_render_snorm",
     "GL_EXT_semaphore",
     "GL_EXT_semaphore_fd",
+    "GL_EXT_separate_depth_stencil",
     "GL_EXT_separate_shader_objects",
     "GL_EXT_shader_framebuffer_fetch",
     "GL_EXT_shader_framebuffer_fetch_non_coherent",
@@ -122,6 +147,8 @@ gles_requestable_extensions = [
     "GL_EXT_tessellation_shader",
     "GL_EXT_texture_border_clamp",
     "GL_EXT_texture_buffer",
+    "GL_EXT_texture_compression_astc_decode_mode",
+    "GL_EXT_texture_compression_astc_decode_mode_rgb9e5",
     "GL_EXT_texture_compression_bptc",
     "GL_EXT_texture_compression_dxt1",
     "GL_EXT_texture_compression_rgtc",
@@ -129,12 +156,17 @@ gles_requestable_extensions = [
     "GL_EXT_texture_compression_s3tc_srgb",
     "GL_EXT_texture_cube_map_array",
     "GL_EXT_texture_filter_anisotropic",
+    "GL_EXT_texture_filter_minmax",
     "GL_EXT_texture_format_BGRA8888",
+    "GL_EXT_texture_mirror_clamp_to_edge",
     "GL_EXT_texture_norm16",
+    "GL_EXT_texture_query_lod",
     "GL_EXT_texture_rg",
+    "GL_EXT_texture_shadow_lod",
     "GL_EXT_texture_sRGB_R8",
     "GL_EXT_texture_sRGB_RG8",
     "GL_EXT_texture_storage",
+    "GL_EXT_texture_storage_compression",
     "GL_EXT_texture_type_2_10_10_10_REV",
     "GL_EXT_unpack_subimage",
     "GL_EXT_YUV_target",
@@ -150,6 +182,7 @@ gles_requestable_extensions = [
     "GL_NV_framebuffer_blit",
     "GL_NV_pack_subimage",
     "GL_NV_pixel_buffer_object",
+    "GL_NV_polygon_mode",
     "GL_NV_read_depth",
     "GL_NV_read_depth_stencil",
     "GL_NV_read_stencil",
@@ -165,6 +198,7 @@ gles_requestable_extensions = [
     "GL_OES_compressed_ETC2_RGBA8_texture",
     "GL_OES_compressed_ETC2_sRGB8_alpha8_texture",
     "GL_OES_compressed_ETC2_sRGB8_texture",
+    "GL_OES_compressed_paletted_texture",
     "GL_OES_copy_image",
     "GL_OES_depth_texture_cube_map",
     "GL_OES_draw_buffers_indexed",
@@ -176,7 +210,9 @@ gles_requestable_extensions = [
     "GL_OES_fbo_render_mipmap",
     "GL_OES_geometry_shader",
     "GL_OES_get_program_binary",
+    "GL_OES_gpu_shader5",
     "GL_OES_mapbuffer",
+    "GL_OES_required_internalformat",
     "GL_OES_rgb8_rgba8",
     "GL_OES_sample_shading",
     "GL_OES_sample_variables",
@@ -184,6 +220,7 @@ gles_requestable_extensions = [
     "GL_OES_shader_io_blocks",
     "GL_OES_shader_multisample_interpolation",
     "GL_OES_standard_derivatives",
+    "GL_OES_tessellation_shader",
     "GL_OES_texture_3D",
     "GL_OES_texture_border_clamp",
     "GL_OES_texture_buffer",
@@ -201,7 +238,11 @@ gles_requestable_extensions = [
     "GL_OES_vertex_type_10_10_10_2",
     "GL_OVR_multiview",
     "GL_OVR_multiview2",
+    "GL_QCOM_framebuffer_foveated",
+    "GL_QCOM_render_shared_exponent",
     "GL_QCOM_shading_rate",
+    "GL_QCOM_texture_foveated",
+    "GL_QCOM_tiled_rendering",
     "GL_WEBGL_video_texture",
 ]
 
@@ -209,12 +250,14 @@ angle_es_only_extensions = [
     "GL_ANGLE_client_arrays",
     "GL_ANGLE_get_serialized_context_string",
     "GL_ANGLE_program_binary",
+    "GL_ANGLE_program_binary_readiness_query",
     "GL_ANGLE_program_cache_control",
     "GL_ANGLE_relaxed_vertex_attribute_type",
     "GL_ANGLE_request_extension",
     "GL_ANGLE_rgbx_internal_format",
     "GL_ANGLE_robust_client_memory",
     "GL_ANGLE_robust_resource_initialization",
+    "GL_ANGLE_shader_binary",
     "GL_ANGLE_webgl_compatibility",
     "GL_CHROMIUM_bind_generates_resource",
     "GL_CHROMIUM_bind_uniform_location",
@@ -238,9 +281,11 @@ gles_es_only_extensions = [
     "GL_EXT_texture_format_sRGB_override",
     "GL_EXT_texture_sRGB_decode",
     "GL_KHR_blend_equation_advanced",
+    "GL_KHR_blend_equation_advanced_coherent",
     "GL_KHR_debug",
     "GL_KHR_no_error",
     "GL_KHR_robust_buffer_access_behavior",
+    "GL_KHR_robustness",
     "GL_NV_fence",
     "GL_NV_robustness_video_memory_purge",
     "GL_OES_depth24",
@@ -254,6 +299,7 @@ gles_es_only_extensions = [
 
 # ES1 (Possibly the min set of extensions needed by Android)
 gles1_extensions = [
+    "GL_OES_blend_subtract",
     "GL_OES_draw_texture",
     "GL_OES_framebuffer_object",
     "GL_OES_matrix_palette",
@@ -261,6 +307,7 @@ gles1_extensions = [
     "GL_OES_point_sprite",
     "GL_OES_query_matrix",
     "GL_OES_texture_cube_map",
+    "GL_OES_texture_mirrored_repeat",
 ]
 
 
@@ -294,11 +341,18 @@ supported_egl_extensions = [
     "EGL_ANGLE_d3d_share_handle_client_buffer",
     "EGL_ANGLE_device_creation",
     "EGL_ANGLE_device_d3d",
+    "EGL_ANGLE_device_d3d11",
+    "EGL_ANGLE_device_d3d9",
+    "EGL_ANGLE_device_vulkan",
     "EGL_ANGLE_display_semaphore_share_group",
     "EGL_ANGLE_display_texture_share_group",
+    "EGL_ANGLE_external_context_and_surface",
     "EGL_ANGLE_feature_control",
     "EGL_ANGLE_ggp_stream_descriptor",
+    "EGL_ANGLE_memory_usage_report",
     "EGL_ANGLE_metal_create_context_ownership_identity",
+    "EGL_ANGLE_metal_shared_event_sync",
+    "EGL_ANGLE_no_error",
     "EGL_ANGLE_power_preference",
     "EGL_ANGLE_prepare_swap_buffers",
     "EGL_ANGLE_program_cache_control",
@@ -308,10 +362,14 @@ supported_egl_extensions = [
     "EGL_ANGLE_swap_with_frame_token",
     "EGL_ANGLE_sync_control_rate",
     "EGL_ANGLE_vulkan_image",
+    "EGL_ANGLE_wait_until_work_scheduled",
     "EGL_ANGLE_window_fixed_size",
     "EGL_CHROMIUM_sync_control",
     "EGL_EXT_create_context_robustness",
     "EGL_EXT_device_query",
+    "EGL_EXT_gl_colorspace_bt2020_hlg",
+    "EGL_EXT_gl_colorspace_bt2020_linear",
+    "EGL_EXT_gl_colorspace_bt2020_pq",
     "EGL_EXT_gl_colorspace_display_p3",
     "EGL_EXT_gl_colorspace_display_p3_linear",
     "EGL_EXT_gl_colorspace_display_p3_passthrough",
@@ -324,6 +382,7 @@ supported_egl_extensions = [
     "EGL_EXT_platform_base",
     "EGL_EXT_platform_device",
     "EGL_EXT_protected_content",
+    "EGL_EXT_surface_compression",
     "EGL_IMG_context_priority",
     "EGL_KHR_debug",
     "EGL_KHR_fence_sync",
@@ -367,16 +426,13 @@ supported_cl_extensions = [
 ]
 
 # Strip these suffixes from Context entry point names. NV is excluded (for now).
-strip_suffixes = ["AMD", "ANDROID", "ANGLE", "CHROMIUM", "EXT", "KHR", "OES", "OVR"]
+strip_suffixes = ["AMD", "ANDROID", "ANGLE", "CHROMIUM", "EXT", "KHR", "OES", "OVR", "QCOM"]
 check_sorted('strip_suffixes', strip_suffixes)
 
 # The EGL_ANGLE_explicit_context extension is generated differently from other extensions.
 # Toggle generation here.
 support_EGL_ANGLE_explicit_context = True
 
-# For ungrouped GLenum types
-default_enum_group_name = "DefaultGroup"
-
 # Group names that appear in command/param, but not present in groups/group
 unsupported_enum_group_names = {
     'GetMultisamplePNameNV',
@@ -394,12 +450,8 @@ unsupported_enum_group_names = {
 }
 
 # Versions (major, minor). Note that GLES intentionally places 1.0 last.
-DESKTOP_GL_VERSIONS = [(1, 0), (1, 1), (1, 2), (1, 3), (1, 4), (1, 5), (2, 0), (2, 1), (3, 0),
-                       (3, 1), (3, 2), (3, 3), (4, 0), (4, 1), (4, 2), (4, 3), (4, 4), (4, 5),
-                       (4, 6)]
 GLES_VERSIONS = [(2, 0), (3, 0), (3, 1), (3, 2), (1, 0)]
 EGL_VERSIONS = [(1, 0), (1, 1), (1, 2), (1, 3), (1, 4), (1, 5)]
-WGL_VERSIONS = [(1, 0)]
 CL_VERSIONS = [(1, 0), (1, 1), (1, 2), (2, 0), (2, 1), (2, 2), (3, 0)]
 
 
@@ -408,9 +460,14 @@ class apis:
     GL = 'GL'
     GLES = 'GLES'
     WGL = 'WGL'
+    GLX = 'GLX'
     EGL = 'EGL'
     CL = 'CL'
 
+# For GLenum types
+api_enums = {apis.GL: 'BigGLEnum', apis.GLES: 'GLESEnum'}
+default_enum_group_name = 'AllEnums'
+
 
 def script_relative(path):
     return os.path.join(os.path.dirname(sys.argv[0]), path)
@@ -424,6 +481,13 @@ def strip_api_prefix(cmd_name):
     return cmd_name.lstrip("cwegl")
 
 
+def find_xml_input(xml_file):
+    for found_xml in xml_inputs:
+        if found_xml == xml_file or found_xml.endswith('/' + xml_file):
+            return found_xml
+    raise Exception('Could not find XML input: ' + xml_file)
+
+
 def get_cmd_name(command_node):
     proto = command_node.find('proto')
     cmd_name = proto.find('name').text
@@ -457,10 +521,10 @@ class CommandNames:
 class RegistryXML:
 
     def __init__(self, xml_file, ext_file=None):
-        tree = etree.parse(script_relative(xml_file))
+        tree = etree.parse(script_relative(find_xml_input(xml_file)))
         self.root = tree.getroot()
         if (ext_file):
-            self._AppendANGLEExts(ext_file)
+            self._AppendANGLEExts(find_xml_input(ext_file))
         self.all_commands = self.root.findall('commands/command')
         self.all_cmd_names = CommandNames()
         self.commands = {}
@@ -469,6 +533,10 @@ class RegistryXML:
         angle_ext_tree = etree.parse(script_relative(ext_file))
         angle_ext_root = angle_ext_tree.getroot()
 
+        insertion_point = self.root.findall("./types")[0]
+        for t in angle_ext_root.iter('types'):
+            insertion_point.extend(t)
+
         insertion_point = self.root.findall("./commands")[0]
         for command in angle_ext_root.iter('commands'):
             insertion_point.extend(command)
@@ -494,6 +562,9 @@ class RegistryXML:
 
     def _ClassifySupport(self, extension):
         supported = extension.attrib['supported']
+        # Desktop GL extensions exposed in ANGLE GLES for Chrome.
+        if extension.attrib['name'] in ['GL_ARB_sync', 'GL_NV_robustness_video_memory_purge']:
+            supported += "|gles2"
         if 'gles2' in supported:
             return 'gl2ext'
         elif 'gles1' in supported:
@@ -502,6 +573,8 @@ class RegistryXML:
             return 'eglext'
         elif 'wgl' in supported:
             return 'wglext'
+        elif 'glx' in supported:
+            return 'glxext'
         elif 'cl' in supported:
             return 'clext'
         else:
@@ -558,6 +631,15 @@ class RegistryXML:
             self.ext_dupes[extension_name] = dupes
             self.all_cmd_names.add_commands(ext_annotations[extension_name], ext_cmd_names)
 
+    def GetEnums(self, override_prefix=None):
+        cmd_names = []
+        for cmd in self.all_cmd_names.get_all_commands():
+            stripped = strip_api_prefix(cmd)
+            prefix = override_prefix or cmd[:(len(cmd) - len(stripped))]
+            cmd_names.append(
+                ('%s%s' % (prefix.upper(), stripped), '%s%s' % (prefix.lower(), stripped)))
+        return cmd_names
+
 
 class EntryPoints:
 
@@ -569,7 +651,7 @@ class EntryPoints:
             cmd_name = get_cmd_name(command_node)
 
             if api == apis.WGL:
-                cmd_name = cmd_name if cmd_name[:3] == 'wgl' else 'wgl' + cmd_name
+                cmd_name = cmd_name if cmd_name.startswith('wgl') else 'wgl' + cmd_name
 
             if cmd_name not in commands:
                 continue
@@ -587,3 +669,27 @@ class EntryPoints:
 
     def get_infos(self):
         return self._cmd_info
+
+
+def GetEGL():
+    egl = RegistryXML('egl.xml', 'egl_angle_ext.xml')
+    for major_version, minor_version in EGL_VERSIONS:
+        version = "%d_%d" % (major_version, minor_version)
+        name_prefix = "EGL_VERSION_"
+        feature_name = "%s%s" % (name_prefix, version)
+        egl.AddCommands(feature_name, version)
+    egl.AddExtensionCommands(supported_egl_extensions, ['egl'])
+    return egl
+
+
+def GetGLES():
+    gles = RegistryXML('gl.xml', 'gl_angle_ext.xml')
+    for major_version, minor_version in GLES_VERSIONS:
+        version = "{}_{}".format(major_version, minor_version)
+        name_prefix = "GL_ES_VERSION_"
+        if major_version == 1:
+            name_prefix = "GL_VERSION_ES_CM_"
+        feature_name = "{}{}".format(name_prefix, version)
+        gles.AddCommands(feature_name, version)
+    gles.AddExtensionCommands(supported_extensions, ['gles2', 'gles1'])
+    return gles
