@@ -54,7 +54,6 @@ enum TBasicType
 {
     EbtVoid,
     EbtFloat,
-    EbtDouble,
     EbtInt,
     EbtUInt,
     EbtBool,
@@ -87,21 +86,13 @@ enum TBasicType
     EbtSampler2DShadow,
     EbtSamplerCubeShadow,
     EbtSampler2DArrayShadow,
-    EbtSampler1D,  // Desktop GLSL sampler types
-    EbtSampler1DArray,
-    EbtSampler1DArrayShadow,
     EbtSamplerBuffer,
     EbtSamplerCubeArray,
     EbtSamplerCubeArrayShadow,
-    EbtSampler1DShadow,
     EbtSampler2DRectShadow,
-    EbtISampler1D,
-    EbtISampler1DArray,
     EbtISampler2DRect,
     EbtISamplerBuffer,
     EbtISamplerCubeArray,
-    EbtUSampler1D,
-    EbtUSampler1DArray,
     EbtUSampler2DRect,
     EbtUSamplerBuffer,
     EbtUSamplerCubeArray,
@@ -114,8 +105,6 @@ enum TBasicType
     EbtImage3D,
     EbtImage2DArray,
     EbtImageCube,
-    EbtImage1D,
-    EbtImage1DArray,
     EbtImage2DMS,
     EbtImage2DMSArray,
     EbtImageCubeArray,
@@ -125,8 +114,6 @@ enum TBasicType
     EbtIImage3D,
     EbtIImage2DArray,
     EbtIImageCube,
-    EbtIImage1D,
-    EbtIImage1DArray,
     EbtIImage2DMS,
     EbtIImage2DMSArray,
     EbtIImageCubeArray,
@@ -137,8 +124,6 @@ enum TBasicType
     EbtUImage3D,
     EbtUImage2DArray,
     EbtUImageCube,
-    EbtUImage1D,
-    EbtUImage1DArray,
     EbtUImage2DMS,
     EbtUImage2DMSArray,
     EbtUImageCubeArray,
@@ -147,15 +132,19 @@ enum TBasicType
     EbtGuardUIntImageEnd = EbtUImageBuffer,
     EbtGuardImageEnd     = EbtGuardUIntImageEnd,
 
+    // ANGLE_shader_pixel_local_storage
+    EbtGuardPixelLocalBegin,
+    EbtPixelLocalANGLE = EbtGuardPixelLocalBegin,
+    EbtIPixelLocalANGLE,
+    EbtUPixelLocalANGLE,
+    EbtGuardPixelLocalEnd = EbtUPixelLocalANGLE,
+
     // Subpass Input
     EbtGuardSubpassInputBegin,
     EbtSubpassInput = EbtGuardSubpassInputBegin,
     EbtISubpassInput,
     EbtUSubpassInput,
-    EbtSubpassInputMS,
-    EbtISubpassInputMS,
-    EbtUSubpassInputMS,
-    EbtGuardSubpassInputEnd = EbtUSubpassInputMS,
+    EbtGuardSubpassInputEnd = EbtUSubpassInput,
 
     EbtLastSimpleType = EbtGuardSubpassInputEnd,
 
@@ -228,6 +217,11 @@ inline bool IsAtomicCounter(TBasicType type)
     return type == EbtAtomicCounter;
 }
 
+inline bool IsPixelLocal(TBasicType type)
+{
+    return type >= EbtGuardPixelLocalBegin && type <= EbtGuardPixelLocalEnd;
+}
+
 inline bool IsSubpassInputType(TBasicType type)
 {
     return type >= EbtGuardSubpassInputBegin && type <= EbtGuardSubpassInputEnd;
@@ -235,7 +229,8 @@ inline bool IsSubpassInputType(TBasicType type)
 
 inline bool IsOpaqueType(TBasicType type)
 {
-    return IsSampler(type) || IsImage(type) || IsAtomicCounter(type) || IsSubpassInputType(type);
+    return IsSampler(type) || IsImage(type) || IsAtomicCounter(type) || IsPixelLocal(type) ||
+           IsSubpassInputType(type);
 }
 
 inline bool IsIntegerSampler(TBasicType type)
@@ -254,13 +249,9 @@ inline bool IsIntegerSampler(TBasicType type)
         case EbtUSampler2DArray:
         case EbtUSampler2DMS:
         case EbtUSampler2DMSArray:
-        case EbtISampler1D:
-        case EbtISampler1DArray:
         case EbtISampler2DRect:
         case EbtISamplerBuffer:
         case EbtISamplerCubeArray:
-        case EbtUSampler1D:
-        case EbtUSampler1DArray:
         case EbtUSampler2DRect:
         case EbtUSamplerBuffer:
         case EbtUSamplerCubeArray:
@@ -277,13 +268,9 @@ inline bool IsIntegerSampler(TBasicType type)
         case EbtSampler2DArrayShadow:
         case EbtSampler2DMS:
         case EbtSampler2DMSArray:
-        case EbtSampler1D:
-        case EbtSampler1DArray:
-        case EbtSampler1DArrayShadow:
         case EbtSamplerBuffer:
         case EbtSamplerCubeArray:
         case EbtSamplerCubeArrayShadow:
-        case EbtSampler1DShadow:
         case EbtSampler2DRectShadow:
         case EbtSamplerVideoWEBGL:
             return false;
@@ -304,8 +291,6 @@ inline bool IsIntegerSamplerUnsigned(TBasicType type)
         case EbtISampler2DArray:
         case EbtISampler2DMS:
         case EbtISampler2DMSArray:
-        case EbtISampler1D:
-        case EbtISampler1DArray:
         case EbtISampler2DRect:
         case EbtISamplerBuffer:
         case EbtISamplerCubeArray:
@@ -316,8 +301,6 @@ inline bool IsIntegerSamplerUnsigned(TBasicType type)
         case EbtUSampler2DArray:
         case EbtUSampler2DMS:
         case EbtUSampler2DMSArray:
-        case EbtUSampler1D:
-        case EbtUSampler1DArray:
         case EbtUSampler2DRect:
         case EbtUSamplerBuffer:
         case EbtUSamplerCubeArray:
@@ -380,10 +363,8 @@ inline bool IsFloatImage(TBasicType type)
 {
     switch (type)
     {
-        case EbtImage1D:
         case EbtImage2D:
         case EbtImage3D:
-        case EbtImage1DArray:
         case EbtImage2DArray:
         case EbtImageCube:
         case EbtImage2DMS:
@@ -404,10 +385,8 @@ inline bool IsIntegerImage(TBasicType type)
 
     switch (type)
     {
-        case EbtIImage1D:
         case EbtIImage2D:
         case EbtIImage3D:
-        case EbtIImage1DArray:
         case EbtIImage2DArray:
         case EbtIImageCube:
         case EbtIImage2DMS:
@@ -428,10 +407,8 @@ inline bool IsUnsignedImage(TBasicType type)
 
     switch (type)
     {
-        case EbtUImage1D:
         case EbtUImage2D:
         case EbtUImage3D:
-        case EbtUImage1DArray:
         case EbtUImage2DArray:
         case EbtUImageCube:
         case EbtUImage2DMS:
@@ -482,19 +459,11 @@ inline bool IsSampler2D(TBasicType type)
         case EbtUSamplerCube:
         case EbtSamplerCube:
         case EbtSamplerCubeShadow:
-        case EbtSampler1D:
-        case EbtSampler1DArray:
-        case EbtSampler1DArrayShadow:
         case EbtSamplerBuffer:
         case EbtSamplerCubeArray:
         case EbtSamplerCubeArrayShadow:
-        case EbtSampler1DShadow:
-        case EbtISampler1D:
-        case EbtISampler1DArray:
         case EbtISamplerBuffer:
         case EbtISamplerCubeArray:
-        case EbtUSampler1D:
-        case EbtUSampler1DArray:
         case EbtUSamplerBuffer:
         case EbtUSamplerCubeArray:
             return false;
@@ -534,21 +503,13 @@ inline bool IsSamplerCube(TBasicType type)
         case EbtUSampler2DMSArray:
         case EbtSampler2DShadow:
         case EbtSampler2DArrayShadow:
-        case EbtSampler1D:
-        case EbtSampler1DArray:
-        case EbtSampler1DArrayShadow:
         case EbtSamplerBuffer:
         case EbtSamplerCubeArray:
         case EbtSamplerCubeArrayShadow:
-        case EbtSampler1DShadow:
         case EbtSampler2DRectShadow:
-        case EbtISampler1D:
-        case EbtISampler1DArray:
         case EbtISampler2DRect:
         case EbtISamplerBuffer:
         case EbtISamplerCubeArray:
-        case EbtUSampler1D:
-        case EbtUSampler1DArray:
         case EbtUSampler2DRect:
         case EbtUSamplerBuffer:
         case EbtUSamplerCubeArray:
@@ -590,21 +551,13 @@ inline bool IsSampler3D(TBasicType type)
         case EbtSampler2DShadow:
         case EbtSamplerCubeShadow:
         case EbtSampler2DArrayShadow:
-        case EbtSampler1D:
-        case EbtSampler1DArray:
-        case EbtSampler1DArrayShadow:
         case EbtSamplerBuffer:
         case EbtSamplerCubeArray:
         case EbtSamplerCubeArrayShadow:
-        case EbtSampler1DShadow:
         case EbtSampler2DRectShadow:
-        case EbtISampler1D:
-        case EbtISampler1DArray:
         case EbtISampler2DRect:
         case EbtISamplerBuffer:
         case EbtISamplerCubeArray:
-        case EbtUSampler1D:
-        case EbtUSampler1DArray:
         case EbtUSampler2DRect:
         case EbtUSamplerBuffer:
         case EbtUSamplerCubeArray:
@@ -621,10 +574,6 @@ inline bool IsSamplerArray(TBasicType type)
 {
     switch (type)
     {
-        case EbtSampler1DArray:
-        case EbtISampler1DArray:
-        case EbtUSampler1DArray:
-        case EbtSampler1DArrayShadow:
         case EbtSampler2DArray:
         case EbtISampler2DArray:
         case EbtUSampler2DArray:
@@ -654,14 +603,10 @@ inline bool IsSamplerArray(TBasicType type)
         case EbtSampler2DMS:
         case EbtISampler2DMS:
         case EbtUSampler2DMS:
-        case EbtSampler1D:
         case EbtSamplerBuffer:
-        case EbtSampler1DShadow:
         case EbtSampler2DRectShadow:
-        case EbtISampler1D:
         case EbtISampler2DRect:
         case EbtISamplerBuffer:
-        case EbtUSampler1D:
         case EbtUSampler2DRect:
         case EbtUSamplerBuffer:
         case EbtSamplerVideoWEBGL:
@@ -673,62 +618,6 @@ inline bool IsSamplerArray(TBasicType type)
     return false;
 }
 
-inline bool IsSampler1D(TBasicType type)
-{
-    switch (type)
-    {
-        case EbtSampler1D:
-        case EbtISampler1D:
-        case EbtUSampler1D:
-        case EbtSampler1DShadow:
-            return true;
-        case EbtSampler2D:
-        case EbtSamplerCube:
-        case EbtSampler3D:
-        case EbtISampler3D:
-        case EbtUSampler3D:
-        case EbtSamplerExternalOES:
-        case EbtSamplerExternal2DY2YEXT:
-        case EbtSampler2DRect:
-        case EbtSampler2DArray:
-        case EbtSampler2DMS:
-        case EbtSampler2DMSArray:
-        case EbtISampler2D:
-        case EbtISamplerCube:
-        case EbtISampler2DArray:
-        case EbtISampler2DMS:
-        case EbtISampler2DMSArray:
-        case EbtUSampler2D:
-        case EbtUSamplerCube:
-        case EbtUSampler2DArray:
-        case EbtUSampler2DMS:
-        case EbtUSampler2DMSArray:
-        case EbtSampler2DShadow:
-        case EbtSamplerCubeShadow:
-        case EbtSampler2DArrayShadow:
-        case EbtSampler1DArray:
-        case EbtSampler1DArrayShadow:
-        case EbtSamplerBuffer:
-        case EbtSamplerCubeArray:
-        case EbtSamplerCubeArrayShadow:
-        case EbtSampler2DRectShadow:
-        case EbtISampler1DArray:
-        case EbtISampler2DRect:
-        case EbtISamplerBuffer:
-        case EbtISamplerCubeArray:
-        case EbtUSampler1DArray:
-        case EbtUSampler2DRect:
-        case EbtUSamplerBuffer:
-        case EbtUSamplerCubeArray:
-        case EbtSamplerVideoWEBGL:
-            return false;
-        default:
-            ASSERT(!IsSampler(type));
-    }
-
-    return false;
-}
-
 inline bool IsSamplerBuffer(TBasicType type)
 {
     switch (type)
@@ -746,10 +635,8 @@ inline bool IsShadowSampler(TBasicType type)
 {
     switch (type)
     {
-        case EbtSampler1DShadow:
         case EbtSampler2DShadow:
         case EbtSamplerCubeShadow:
-        case EbtSampler1DArrayShadow:
         case EbtSampler2DArrayShadow:
         case EbtSamplerCubeArrayShadow:
         case EbtSampler2DRectShadow:
@@ -775,17 +662,11 @@ inline bool IsShadowSampler(TBasicType type)
         case EbtSampler2DArray:
         case EbtSampler2DMS:
         case EbtSampler2DMSArray:
-        case EbtSampler1D:
-        case EbtSampler1DArray:
         case EbtSamplerBuffer:
         case EbtSamplerCubeArray:
-        case EbtISampler1D:
-        case EbtISampler1DArray:
         case EbtISampler2DRect:
         case EbtISamplerBuffer:
         case EbtISamplerCubeArray:
-        case EbtUSampler1D:
-        case EbtUSampler1DArray:
         case EbtUSampler2DRect:
         case EbtUSamplerBuffer:
         case EbtUSamplerCubeArray:
@@ -818,12 +699,6 @@ inline bool IsImage2D(TBasicType type)
         case EbtImageCube:
         case EbtIImageCube:
         case EbtUImageCube:
-        case EbtImage1D:
-        case EbtIImage1D:
-        case EbtUImage1D:
-        case EbtImage1DArray:
-        case EbtIImage1DArray:
-        case EbtUImage1DArray:
         case EbtImage2DMSArray:
         case EbtIImage2DMSArray:
         case EbtUImage2DMSArray:
@@ -861,12 +736,6 @@ inline bool IsImage3D(TBasicType type)
         case EbtImageCube:
         case EbtIImageCube:
         case EbtUImageCube:
-        case EbtImage1D:
-        case EbtIImage1D:
-        case EbtUImage1D:
-        case EbtImage1DArray:
-        case EbtIImage1DArray:
-        case EbtUImage1DArray:
         case EbtImage2DMS:
         case EbtIImage2DMS:
         case EbtUImage2DMS:
@@ -910,12 +779,6 @@ inline bool IsImage2DArray(TBasicType type)
         case EbtImageCube:
         case EbtIImageCube:
         case EbtUImageCube:
-        case EbtImage1D:
-        case EbtIImage1D:
-        case EbtUImage1D:
-        case EbtImage1DArray:
-        case EbtIImage1DArray:
-        case EbtUImage1DArray:
         case EbtImage2DMS:
         case EbtIImage2DMS:
         case EbtUImage2DMS:
@@ -953,12 +816,6 @@ inline bool IsImageCube(TBasicType type)
         case EbtImage2DArray:
         case EbtIImage2DArray:
         case EbtUImage2DArray:
-        case EbtImage1D:
-        case EbtIImage1D:
-        case EbtUImage1D:
-        case EbtImage1DArray:
-        case EbtIImage1DArray:
-        case EbtUImage1DArray:
         case EbtImage2DMS:
         case EbtIImage2DMS:
         case EbtUImage2DMS:
@@ -1060,32 +917,66 @@ enum TQualifier
     EvqSecondaryFragColorEXT,  // EXT_blend_func_extended
     EvqSecondaryFragDataEXT,   // EXT_blend_func_extended
 
-    EvqViewIDOVR,      // OVR_multiview
-    EvqViewportIndex,  // gl_ViewportIndex
+    EvqViewIDOVR,  // OVR_multiview
 
-    EvqClipDistance,  // APPLE_clip_distance/EXT_clip_cull_distance
-    EvqCullDistance,  // EXT_clip_cull_distance
+    EvqClipDistance,  // APPLE_clip_distance / EXT_clip_cull_distance / ANGLE_clip_cull_distance
+    EvqCullDistance,  // EXT_clip_cull_distance / ANGLE_clip_cull_distance
 
     // built-ins written by the shader_framebuffer_fetch extension(s)
     EvqLastFragColor,
     EvqLastFragData,
+    // built-ins written by the shader_framebuffer_fetch_depth_stencil extension
+    EvqLastFragDepth,
+    EvqLastFragStencil,
 
     // GLSL ES 3.0 vertex output and fragment input
-    EvqSmooth,         // Incomplete qualifier, smooth is the default
-    EvqFlat,           // Incomplete qualifier
-    EvqNoPerspective,  // Incomplete qualifier
-    EvqCentroid,       // Incomplete qualifier
-    EvqSample,
+
+    // This section combines
+    // * storage (in/out),
+    // * auxiliary storage (<none>/centroid/sample), and
+    // * interpolation (<none>/smooth/flat/noperspective)
+    // qualifiers into a flat list.
+
+    // Qualifiers not ending with 'In' or 'Out' are considered incomplete
+    // and are used only internally. Some combinations are redundant
+    // because they represent the same effective qualifiers. Specifically:
+    // * 'smooth' is implied when an interpolation qualifier is omitted
+    // * 'flat' makes 'centroid' and 'sample' irrelevant
+
+    // <none>        <none>   -> original storage qualifier, e.g., EvqFragmentIn, implies smooth
+    // <none>        centroid -> EvqCentroid
+    // <none>        sample   -> EvqSample
+    // smooth        <none>   -> EvqSmooth
+    // smooth        centroid -> EvqCentroid
+    // smooth        sample   -> EvqSample
+    // flat          <none>   -> EvqFlat
+    // flat          centroid -> EvqFlat
+    // flat          sample   -> EvqFlat
+    // noperspective <none>   -> EvqNoPerspective
+    // noperspective centroid -> EvqNoPerspectiveCentroid
+    // noperspective sample   -> EvqNoPerspectiveSample
+
+    EvqSmooth,                 // Incomplete
+    EvqFlat,                   // Incomplete
+    EvqNoPerspective,          // Incomplete
+    EvqCentroid,               // Incomplete
+    EvqSample,                 // Incomplete
+    EvqNoPerspectiveCentroid,  // Incomplete
+    EvqNoPerspectiveSample,    // Incomplete
     EvqSmoothOut,
     EvqFlatOut,
     EvqNoPerspectiveOut,
     EvqCentroidOut,  // Implies smooth
-    EvqSampleOut,
+    EvqSampleOut,    // Implies smooth
+    EvqNoPerspectiveCentroidOut,
+    EvqNoPerspectiveSampleOut,
     EvqSmoothIn,
     EvqFlatIn,
     EvqNoPerspectiveIn,
     EvqCentroidIn,  // Implies smooth
-    EvqSampleIn,
+    EvqSampleIn,    // Implies smooth
+    EvqNoPerspectiveCentroidIn,
+    EvqNoPerspectiveSampleIn,
 
     // GLSL ES 3.0 extension OES_sample_variables
     EvqSampleID,
@@ -1147,6 +1038,9 @@ enum TQualifier
     // constant_id.
     EvqSpecConst,
 
+    // __pixel_localEXT from EXT_shader_pixel_local_storage.
+    EvqPixelLocalEXT,
+
     // end of list
     EvqLast
 };
@@ -1178,6 +1072,8 @@ inline bool IsShaderIn(TQualifier qualifier)
         case EvqNoPerspectiveIn:
         case EvqCentroidIn:
         case EvqSampleIn:
+        case EvqNoPerspectiveCentroidIn:
+        case EvqNoPerspectiveSampleIn:
         case EvqPatchIn:
             return true;
         default:
@@ -1201,6 +1097,8 @@ inline bool IsShaderOut(TQualifier qualifier)
         case EvqNoPerspectiveOut:
         case EvqCentroidOut:
         case EvqSampleOut:
+        case EvqNoPerspectiveCentroidOut:
+        case EvqNoPerspectiveSampleOut:
         case EvqPatchOut:
         case EvqFragmentInOut:
             return true;
@@ -1271,6 +1169,15 @@ enum TLayoutBlockStorage
     EbsLast = EbsStd430,
 };
 
+enum TLayoutDepth
+{
+    EdUnspecified,
+    EdAny,
+    EdGreater,
+    EdLess,
+    EdUnchanged,
+};
+
 enum TYuvCscStandardEXT
 {
     EycsUndefined,
@@ -1315,6 +1222,7 @@ class AdvancedBlendEquations
     {}
 
     bool any() const;
+    bool all() const;
     bool anyHsl() const;
 
     void setAll();
@@ -1331,6 +1239,9 @@ class AdvancedBlendEquations
         return *this;
     }
 
+    static const char *GetLayoutString(uint32_t blendEquation);
+    static const char *GetAllEquationsLayoutString();
+
   private:
     uint32_t mEnabledBlendEquations;
 };
@@ -1348,10 +1259,11 @@ struct TLayoutQualifier
                earlyFragmentTests == false && matrixPacking == EmpUnspecified &&
                blockStorage == EbsUnspecified && !localSize.isAnyValueSet() &&
                imageInternalFormat == EiifUnspecified && primitiveType == EptUndefined &&
-               invocations == 0 && maxVertices == -1 && vertices == 0 &&
+               invocations == 0 && maxVertices == -1 && vertices == 0 && depth == EdUnspecified &&
                tesPrimitiveType == EtetUndefined && tesVertexSpacingType == EtetUndefined &&
                tesOrderingType == EtetUndefined && tesPointType == EtetUndefined && index == -1 &&
-               inputAttachmentIndex == -1 && noncoherent == false && !advancedBlendEquations.any();
+               inputAttachmentIndex == -1 && noncoherent == false &&
+               !advancedBlendEquations.any() && !pushConstant;
     }
 
     bool isCombinationValid() const
@@ -1392,6 +1304,11 @@ struct TLayoutQualifier
     int binding;
     int offset;
 
+    bool pushConstant;
+
+    // Depth layout qualifier
+    TLayoutDepth depth;
+
     // Image format layout qualifier
     TLayoutImageInternalFormat imageInternalFormat;
 
@@ -1426,6 +1343,10 @@ struct TLayoutQualifier
     // KHR_blend_equation_advanced layout qualifiers.
     AdvancedBlendEquations advancedBlendEquations;
 
+    // D3D 11.3 Rasterizer Order Views (ROVs).
+    // This qualifier is only used internally by ANGLE; it is not visible to the application.
+    bool rasterOrdered;
+
   private:
     explicit constexpr TLayoutQualifier(int /*placeholder*/)
         : location(-1),
@@ -1435,6 +1356,8 @@ struct TLayoutQualifier
           localSize(-1),
           binding(-1),
           offset(-1),
+          pushConstant(false),
+          depth(EdUnspecified),
           imageInternalFormat(EiifUnspecified),
           numViews(-1),
           yuv(false),
@@ -1450,7 +1373,8 @@ struct TLayoutQualifier
           index(-1),
           inputAttachmentIndex(-1),
           noncoherent(false),
-          advancedBlendEquations(0)
+          advancedBlendEquations(0),
+          rasterOrdered(false)
     {}
 };
 
@@ -1466,6 +1390,33 @@ struct TMemoryQualifier
 
     constexpr static TMemoryQualifier Create() { return TMemoryQualifier(0); }
 
+    // Used for GLSL generation, debugging and error messages.
+    inline const char *getAnyQualifierString() const
+    {
+        if (readonly)
+        {
+            return "readonly";
+        }
+        if (writeonly)
+        {
+            return "writeonly";
+        }
+        if (coherent)
+        {
+            return "coherent";
+        }
+        if (restrictQualifier)
+        {
+            return "restrict";
+        }
+        if (volatileQualifier)
+        {
+            return "volatile";
+        }
+        ASSERT(isEmpty());
+        return "";
+    }
+
     // GLSL ES 3.10 Revision 4, 4.9 Memory Access Qualifiers
     // An image can be qualified as both readonly and writeonly. It still can be can be used with
     // imageSize().
@@ -1509,97 +1460,108 @@ inline const char *getQualifierString(TQualifier q)
     // clang-format off
     switch(q)
     {
-    case EvqTemporary:              return "Temporary";
-    case EvqGlobal:                 return "Global";
-    case EvqConst:                  return "const";
-    case EvqAttribute:              return "attribute";
-    case EvqVaryingIn:              return "varying";
-    case EvqVaryingOut:             return "varying";
-    case EvqUniform:                return "uniform";
-    case EvqBuffer:                 return "buffer";
-    case EvqPatch:                  return "patch";
-    case EvqVertexIn:               return "in";
-    case EvqFragmentOut:            return "out";
-    case EvqVertexOut:              return "out";
-    case EvqFragmentIn:             return "in";
-    case EvqParamIn:                return "in";
-    case EvqParamOut:               return "out";
-    case EvqParamInOut:             return "inout";
-    case EvqParamConst:             return "const";
-    case EvqInstanceID:             return "InstanceID";
-    case EvqVertexID:               return "VertexID";
-    case EvqPosition:               return "Position";
-    case EvqPointSize:              return "PointSize";
-    case EvqDrawID:                 return "DrawID";
-    case EvqFragCoord:              return "FragCoord";
-    case EvqFrontFacing:            return "FrontFacing";
-    case EvqHelperInvocation:       return "HelperInvocation";
-    case EvqPointCoord:             return "PointCoord";
-    case EvqFragColor:              return "FragColor";
-    case EvqFragData:               return "FragData";
-    case EvqFragDepth:              return "FragDepth";
-    case EvqSecondaryFragColorEXT:  return "SecondaryFragColorEXT";
-    case EvqSecondaryFragDataEXT:   return "SecondaryFragDataEXT";
-    case EvqViewIDOVR:              return "ViewIDOVR";
-    case EvqViewportIndex:          return "ViewportIndex";
-    case EvqLayerOut:               return "LayerOut";
-    case EvqLayerIn:                return "LayerIn";
-    case EvqLastFragColor:          return "LastFragColor";
-    case EvqLastFragData:           return "LastFragData";
-    case EvqFragmentInOut:          return "inout";
-    case EvqSmoothOut:              return "smooth out";
-    case EvqCentroidOut:            return "smooth centroid out";
-    case EvqFlatOut:                return "flat out";
-    case EvqNoPerspectiveOut:       return "noperspective out";
-    case EvqSmoothIn:               return "smooth in";
-    case EvqFlatIn:                 return "flat in";
-    case EvqNoPerspectiveIn:        return "noperspective in";
-    case EvqCentroidIn:             return "smooth centroid in";
-    case EvqCentroid:               return "centroid";
-    case EvqFlat:                   return "flat";
-    case EvqNoPerspective:          return "noperspective";
-    case EvqSmooth:                 return "smooth";
-    case EvqShared:                 return "shared";
-    case EvqComputeIn:              return "in";
-    case EvqNumWorkGroups:          return "NumWorkGroups";
-    case EvqWorkGroupSize:          return "WorkGroupSize";
-    case EvqWorkGroupID:            return "WorkGroupID";
-    case EvqLocalInvocationID:      return "LocalInvocationID";
-    case EvqGlobalInvocationID:     return "GlobalInvocationID";
-    case EvqLocalInvocationIndex:   return "LocalInvocationIndex";
-    case EvqReadOnly:               return "readonly";
-    case EvqWriteOnly:              return "writeonly";
-    case EvqGeometryIn:             return "in";
-    case EvqGeometryOut:            return "out";
-    case EvqPerVertexIn:            return "gl_in";
-    case EvqPrimitiveIDIn:          return "gl_PrimitiveIDIn";
-    case EvqInvocationID:           return "gl_InvocationID";
-    case EvqPrimitiveID:            return "gl_PrimitiveID";
-    case EvqPrecise:                return "precise";
-    case EvqClipDistance:           return "ClipDistance";
-    case EvqCullDistance:           return "CullDistance";
-    case EvqSample:                 return "sample";
-    case EvqSampleIn:               return "sample in";
-    case EvqSampleOut:              return "sample out";
-    case EvqSampleID:               return "SampleID";
-    case EvqSamplePosition:         return "SamplePosition";
-    case EvqSampleMaskIn:           return "SampleMaskIn";
-    case EvqSampleMask:             return "SampleMask";
-    case EvqNumSamples:             return "NumSamples";
-    case EvqPatchIn:                return "patch in";
-    case EvqPatchOut:               return "patch out";
-    case EvqTessControlIn:          return "in";
-    case EvqTessControlOut:         return "out";
-    case EvqPerVertexOut:           return "gl_out";
-    case EvqPatchVerticesIn:        return "PatchVerticesIn";
-    case EvqTessLevelOuter:         return "TessLevelOuter";
-    case EvqTessLevelInner:         return "TessLevelInner";
-    case EvqBoundingBox:            return "BoundingBox";
-    case EvqTessEvaluationIn:       return "in";
-    case EvqTessEvaluationOut:      return "out";
-    case EvqTessCoord:              return "TessCoord";
-    case EvqSpecConst:              return "const";
-    default: UNREACHABLE();         return "unknown qualifier";
+    case EvqTemporary:                 return "Temporary";
+    case EvqGlobal:                    return "Global";
+    case EvqConst:                     return "const";
+    case EvqAttribute:                 return "attribute";
+    case EvqVaryingIn:                 return "varying";
+    case EvqVaryingOut:                return "varying";
+    case EvqUniform:                   return "uniform";
+    case EvqBuffer:                    return "buffer";
+    case EvqPatch:                     return "patch";
+    case EvqVertexIn:                  return "in";
+    case EvqFragmentOut:               return "out";
+    case EvqVertexOut:                 return "out";
+    case EvqFragmentIn:                return "in";
+    case EvqParamIn:                   return "in";
+    case EvqParamOut:                  return "out";
+    case EvqParamInOut:                return "inout";
+    case EvqParamConst:                return "const";
+    case EvqInstanceID:                return "InstanceID";
+    case EvqVertexID:                  return "VertexID";
+    case EvqPosition:                  return "Position";
+    case EvqPointSize:                 return "PointSize";
+    case EvqDrawID:                    return "DrawID";
+    case EvqFragCoord:                 return "FragCoord";
+    case EvqFrontFacing:               return "FrontFacing";
+    case EvqHelperInvocation:          return "HelperInvocation";
+    case EvqPointCoord:                return "PointCoord";
+    case EvqFragColor:                 return "FragColor";
+    case EvqFragData:                  return "FragData";
+    case EvqFragDepth:                 return "FragDepth";
+    case EvqSecondaryFragColorEXT:     return "SecondaryFragColorEXT";
+    case EvqSecondaryFragDataEXT:      return "SecondaryFragDataEXT";
+    case EvqViewIDOVR:                 return "ViewIDOVR";
+    case EvqLayerOut:                  return "LayerOut";
+    case EvqLayerIn:                   return "LayerIn";
+    case EvqLastFragColor:             return "LastFragColor";
+    case EvqLastFragData:              return "LastFragData";
+    case EvqLastFragDepth:             return "LastFragDepthARM";
+    case EvqLastFragStencil:           return "LastFragStencilARM";
+    case EvqFragmentInOut:             return "inout";
+    case EvqSmoothOut:                 return "smooth out";
+    case EvqCentroidOut:               return "smooth centroid out";
+    case EvqFlatOut:                   return "flat out";
+    case EvqNoPerspectiveOut:          return "noperspective out";
+    case EvqNoPerspectiveCentroidOut:  return "noperspective centroid out";
+    case EvqNoPerspectiveSampleOut:    return "noperspective sample out";
+    case EvqSmoothIn:                  return "smooth in";
+    case EvqFlatIn:                    return "flat in";
+    case EvqNoPerspectiveIn:           return "noperspective in";
+    case EvqNoPerspectiveCentroidIn:   return "noperspective centroid in";
+    case EvqNoPerspectiveSampleIn:     return "noperspective sample in";
+    case EvqCentroidIn:                return "smooth centroid in";
+    case EvqCentroid:                  return "centroid";
+    case EvqFlat:                      return "flat";
+    case EvqNoPerspective:             return "noperspective";
+    case EvqNoPerspectiveCentroid:     return "noperspective centroid";
+    case EvqNoPerspectiveSample:       return "noperspective sample";
+    case EvqSmooth:                    return "smooth";
+    case EvqShared:                    return "shared";
+    case EvqComputeIn:                 return "in";
+    case EvqNumWorkGroups:             return "NumWorkGroups";
+    case EvqWorkGroupSize:             return "WorkGroupSize";
+    case EvqWorkGroupID:               return "WorkGroupID";
+    case EvqLocalInvocationID:         return "LocalInvocationID";
+    case EvqGlobalInvocationID:        return "GlobalInvocationID";
+    case EvqLocalInvocationIndex:      return "LocalInvocationIndex";
+    case EvqReadOnly:                  return "readonly";
+    case EvqWriteOnly:                 return "writeonly";
+    case EvqCoherent:                  return "coherent";
+    case EvqRestrict:                  return "restrict";
+    case EvqVolatile:                  return "volatile";
+    case EvqGeometryIn:                return "in";
+    case EvqGeometryOut:               return "out";
+    case EvqPerVertexIn:               return "gl_in";
+    case EvqPrimitiveIDIn:             return "gl_PrimitiveIDIn";
+    case EvqInvocationID:              return "gl_InvocationID";
+    case EvqPrimitiveID:               return "gl_PrimitiveID";
+    case EvqPrecise:                   return "precise";
+    case EvqClipDistance:              return "ClipDistance";
+    case EvqCullDistance:              return "CullDistance";
+    case EvqSample:                    return "sample";
+    case EvqSampleIn:                  return "sample in";
+    case EvqSampleOut:                 return "sample out";
+    case EvqSampleID:                  return "SampleID";
+    case EvqSamplePosition:            return "SamplePosition";
+    case EvqSampleMaskIn:              return "SampleMaskIn";
+    case EvqSampleMask:                return "SampleMask";
+    case EvqNumSamples:                return "NumSamples";
+    case EvqPatchIn:                   return "patch in";
+    case EvqPatchOut:                  return "patch out";
+    case EvqTessControlIn:             return "in";
+    case EvqTessControlOut:            return "out";
+    case EvqPerVertexOut:              return "out";
+    case EvqPatchVerticesIn:           return "PatchVerticesIn";
+    case EvqTessLevelOuter:            return "TessLevelOuter";
+    case EvqTessLevelInner:            return "TessLevelInner";
+    case EvqBoundingBox:               return "BoundingBox";
+    case EvqTessEvaluationIn:          return "in";
+    case EvqTessEvaluationOut:         return "out";
+    case EvqTessCoord:                 return "TessCoord";
+    case EvqSpecConst:                 return "const";
+    case EvqPixelLocalEXT:             return "__pixel_localEXT";
+    default: UNREACHABLE();            return "unknown qualifier";
     }
     // clang-format on
 }
@@ -1676,6 +1638,26 @@ inline const char *getImageInternalFormatString(TLayoutImageInternalFormat iifq)
     }
 }
 
+inline const char *getDepthString(TLayoutDepth depth)
+{
+    switch (depth)
+    {
+        case EdUnspecified:
+            return "depth_unspecified";
+        case EdAny:
+            return "depth_any";
+        case EdGreater:
+            return "depth_greater";
+        case EdLess:
+            return "depth_less";
+        case EdUnchanged:
+            return "depth_unchanged";
+        default:
+            UNREACHABLE();
+            return "unknown depth";
+    }
+}
+
 inline TYuvCscStandardEXT getYuvCscStandardEXT(const ImmutableString &str)
 {
     if (str == "itu_601")
