@@ -82,6 +82,63 @@ struct ETC2Block
         }
     }
 
+    // Transcodes  block to BC4
+    // For simplicity, R11 alpha use the same formula as Alpha8 to decode,
+    // the result R8 may have some precision issue like multiplier == 0 case.
+    void transcodeAsBC4(uint8_t *dest, size_t x, size_t y, size_t w, size_t h, bool isSigned) const
+    {
+        static constexpr int kIndexMap[] = {1, 7, 6, 5, 4, 3, 2, 0};
+        int alpha[16];
+        size_t k     = 0;
+        int minAlpha = std::numeric_limits<int>::max();
+        int maxAlpha = std::numeric_limits<int>::min();
+        for (size_t j = 0; j < 4; j++)
+        {
+            for (size_t i = 0; i < 4; i++)
+            {
+                if (isSigned)
+                {
+                    alpha[k] = clampSByte(getSingleETC2Channel(i, j, isSigned));
+                }
+                else
+                {
+                    alpha[k] = clampByte(getSingleETC2Channel(i, j, isSigned));
+                }
+                minAlpha = std::min(minAlpha, alpha[k]);
+                maxAlpha = std::max(maxAlpha, alpha[k]);
+                k++;
+            }
+        }
+        uint64_t *result = (uint64_t *)dest;
+        *result          = (maxAlpha & 0xff) | ((minAlpha & 0xff) << 8);
+        if (minAlpha != maxAlpha)
+        {
+            float dist = static_cast<float>(maxAlpha - minAlpha);
+            for (size_t i = 0; i < 16; i++)
+            {
+                int ind = int(roundf((alpha[i] - minAlpha) / dist * 7.0f));
+                // 0 : maxAlpha
+                // 1 : minAlpha
+                // 2 : 6/7*maxAlpha + 1/7*minAlpha;
+                // 3 : 5/7*maxAlpha + 2/7*minAlpha;
+                // 4 : 4/7*maxAlpha + 3/7*minAlpha;
+                // 5 : 3/7*maxAlpha + 4/7*minAlpha;
+                // 6 : 2/7*maxAlpha + 5/7*minAlpha;
+                // 7 : 1/7*maxAlpha + 6/7*minAlpha;
+                // so the mapping is
+                // 0 -> 1
+                // 1 -> 7
+                // 2 -> 6
+                // 3 -> 5
+                // 4 -> 4
+                // 5 -> 3
+                // 6 -> 2
+                // 7 -> 0
+                *result |= ((uint64_t)kIndexMap[ind]) << ((3 * i) + 16);
+            }
+        }
+    }
+
     // Decodes unsigned single or dual channel EAC block to 16-bit color
     void decodeAsSingleEACChannel(uint16_t *dest,
                                   size_t x,
@@ -249,9 +306,9 @@ struct ETC2Block
                 {
                     // Byte 1
                     unsigned char TR1b : 2;
-                    unsigned char TdummyB : 1;
+                    unsigned char TunusedB : 1;
                     unsigned char TR1a : 2;
-                    unsigned char TdummyA : 3;
+                    unsigned char TunusedA : 3;
                     // Byte 2
                     unsigned char TB1 : 4;
                     unsigned char TG1 : 4;
@@ -270,13 +327,13 @@ struct ETC2Block
                     // Byte 1
                     unsigned char HG1a : 3;
                     unsigned char HR1 : 4;
-                    unsigned char HdummyA : 1;
+                    unsigned char HunusedA : 1;
                     // Byte 2
                     unsigned char HB1b : 2;
-                    unsigned char HdummyC : 1;
+                    unsigned char HunusedC : 1;
                     unsigned char HB1a : 1;
                     unsigned char HG1b : 1;
-                    unsigned char HdummyB : 3;
+                    unsigned char HunusedB : 3;
                     // Byte 3
                     unsigned char HG2a : 3;
                     unsigned char HR2 : 4;
@@ -298,16 +355,16 @@ struct ETC2Block
             // Byte 1
             unsigned char GO1 : 1;
             unsigned char RO : 6;
-            unsigned char PdummyA : 1;
+            unsigned char PunusedA : 1;
             // Byte 2
             unsigned char BO1 : 1;
             unsigned char GO2 : 6;
-            unsigned char PdummyB : 1;
+            unsigned char PunusedB : 1;
             // Byte 3
             unsigned char BO3a : 2;
-            unsigned char PdummyD : 1;
+            unsigned char PunusedD : 1;
             unsigned char BO2 : 2;
-            unsigned char PdummyC : 3;
+            unsigned char PunusedC : 3;
             // Byte 4
             unsigned char RH2 : 1;
             bool Pflipbit : 1;
@@ -1084,7 +1141,7 @@ struct ETC2Block
             // From the eigenvalue and eigenvector, choose the axis to project
             // colors on. When projecting colors we want to do integer computations
             // for speed, so we normalize the eigenvector to the [0, 512] range.
-            float magn = std::max(std::max(std::abs(vfr), std::abs(vfg)), std::abs(vfb));
+            float magn = std::max({std::abs(vfr), std::abs(vfg), std::abs(vfb)});
             magn       = kQuantizeRange / magn;
             vr         = static_cast<int>(vfr * magn);
             vg         = static_cast<int>(vfg * magn);
@@ -1413,7 +1470,8 @@ static const uint8_t DefaultETCAlphaValues[4][4] =
 };
 
 // clang-format on
-void LoadR11EACToR8(size_t width,
+void LoadR11EACToR8(const ImageLoadContext &context,
+                    size_t width,
                     size_t height,
                     size_t depth,
                     const uint8_t *input,
@@ -1445,7 +1503,8 @@ void LoadR11EACToR8(size_t width,
     }
 }
 
-void LoadRG11EACToRG8(size_t width,
+void LoadRG11EACToRG8(const ImageLoadContext &context,
+                      size_t width,
                       size_t height,
                       size_t depth,
                       const uint8_t *input,
@@ -1481,7 +1540,8 @@ void LoadRG11EACToRG8(size_t width,
     }
 }
 
-void LoadR11EACToR16(size_t width,
+void LoadR11EACToR16(const ImageLoadContext &context,
+                     size_t width,
                      size_t height,
                      size_t depth,
                      const uint8_t *input,
@@ -1514,7 +1574,8 @@ void LoadR11EACToR16(size_t width,
     }
 }
 
-void LoadRG11EACToRG16(size_t width,
+void LoadRG11EACToRG16(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1551,7 +1612,8 @@ void LoadRG11EACToRG16(size_t width,
     }
 }
 
-void LoadETC2RGB8ToRGBA8(size_t width,
+void LoadETC2RGB8ToRGBA8(const ImageLoadContext &context,
+                         size_t width,
                          size_t height,
                          size_t depth,
                          const uint8_t *input,
@@ -1583,7 +1645,8 @@ void LoadETC2RGB8ToRGBA8(size_t width,
     }
 }
 
-void LoadETC2RGB8ToBC1(size_t width,
+void LoadETC2RGB8ToBC1(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1615,7 +1678,47 @@ void LoadETC2RGB8ToBC1(size_t width,
     }
 }
 
-void LoadETC2RGBA8ToRGBA8(size_t width,
+void LoadETC2RGBA8ToBC3(const ImageLoadContext &context,
+                        size_t width,
+                        size_t height,
+                        size_t depth,
+                        const uint8_t *input,
+                        size_t inputRowPitch,
+                        size_t inputDepthPitch,
+                        uint8_t *output,
+                        size_t outputRowPitch,
+                        size_t outputDepthPitch,
+                        bool punchthroughAlpha,
+                        bool isSigned)
+{
+    for (size_t z = 0; z < depth; z++)
+    {
+        for (size_t y = 0; y < height; y += 4)
+        {
+            const ETC2Block *sourceRow =
+                priv::OffsetDataPointer<ETC2Block>(input, y / 4, z, inputRowPitch, inputDepthPitch);
+            uint8_t *destRow = priv::OffsetDataPointer<uint8_t>(output, y / 4, z, outputRowPitch,
+                                                                outputDepthPitch);
+
+            for (size_t x = 0; x < width; x += 4)
+            {
+                const ETC2Block *sourceAlphaBlock = sourceRow + (x / 4) * 2;
+                uint8_t *destAlphaPixels          = destRow + (x * 4);
+
+                const ETC2Block *sourceRgbBlock = sourceAlphaBlock + 1;
+                uint8_t *destRgbPixels          = destAlphaPixels + 8;
+
+                sourceRgbBlock->transcodeAsBC1(destRgbPixels, x, y, width, height,
+                                               DefaultETCAlphaValues, punchthroughAlpha);
+
+                sourceAlphaBlock->transcodeAsBC4(destAlphaPixels, x, y, width, height, isSigned);
+            }
+        }
+    }
+}
+
+void LoadETC2RGBA8ToRGBA8(const ImageLoadContext &context,
+                          size_t width,
                           size_t height,
                           size_t depth,
                           const uint8_t *input,
@@ -1655,7 +1758,8 @@ void LoadETC2RGBA8ToRGBA8(size_t width,
 
 }  // anonymous namespace
 
-void LoadETC1RGB8ToRGBA8(size_t width,
+void LoadETC1RGB8ToRGBA8(const ImageLoadContext &context,
+                         size_t width,
                          size_t height,
                          size_t depth,
                          const uint8_t *input,
@@ -1665,11 +1769,12 @@ void LoadETC1RGB8ToRGBA8(size_t width,
                          size_t outputRowPitch,
                          size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                        outputRowPitch, outputDepthPitch, false);
+    LoadETC2RGB8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                        output, outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadETC1RGB8ToBC1(size_t width,
+void LoadETC1RGB8ToBC1(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1679,11 +1784,12 @@ void LoadETC1RGB8ToBC1(size_t width,
                        size_t outputRowPitch,
                        size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToBC1(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadETC2RGB8ToBC1(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadEACR11ToR8(size_t width,
+void LoadEACR11ToR8(const ImageLoadContext &context,
+                    size_t width,
                     size_t height,
                     size_t depth,
                     const uint8_t *input,
@@ -1693,11 +1799,12 @@ void LoadEACR11ToR8(size_t width,
                     size_t outputRowPitch,
                     size_t outputDepthPitch)
 {
-    LoadR11EACToR8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadR11EACToR8(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                    outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadEACR11SToR8(size_t width,
+void LoadEACR11SToR8(const ImageLoadContext &context,
+                     size_t width,
                      size_t height,
                      size_t depth,
                      const uint8_t *input,
@@ -1707,11 +1814,12 @@ void LoadEACR11SToR8(size_t width,
                      size_t outputRowPitch,
                      size_t outputDepthPitch)
 {
-    LoadR11EACToR8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadR11EACToR8(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                    outputRowPitch, outputDepthPitch, true);
 }
 
-void LoadEACRG11ToRG8(size_t width,
+void LoadEACRG11ToRG8(const ImageLoadContext &context,
+                      size_t width,
                       size_t height,
                       size_t depth,
                       const uint8_t *input,
@@ -1721,11 +1829,12 @@ void LoadEACRG11ToRG8(size_t width,
                       size_t outputRowPitch,
                       size_t outputDepthPitch)
 {
-    LoadRG11EACToRG8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadRG11EACToRG8(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                      outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadEACRG11SToRG8(size_t width,
+void LoadEACRG11SToRG8(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1735,11 +1844,12 @@ void LoadEACRG11SToRG8(size_t width,
                        size_t outputRowPitch,
                        size_t outputDepthPitch)
 {
-    LoadRG11EACToRG8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadRG11EACToRG8(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                      outputRowPitch, outputDepthPitch, true);
 }
 
-void LoadEACR11ToR16(size_t width,
+void LoadEACR11ToR16(const ImageLoadContext &context,
+                     size_t width,
                      size_t height,
                      size_t depth,
                      const uint8_t *input,
@@ -1749,11 +1859,12 @@ void LoadEACR11ToR16(size_t width,
                      size_t outputRowPitch,
                      size_t outputDepthPitch)
 {
-    LoadR11EACToR16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadR11EACToR16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                     outputRowPitch, outputDepthPitch, false, false);
 }
 
-void LoadEACR11SToR16(size_t width,
+void LoadEACR11SToR16(const ImageLoadContext &context,
+                      size_t width,
                       size_t height,
                       size_t depth,
                       const uint8_t *input,
@@ -1763,11 +1874,12 @@ void LoadEACR11SToR16(size_t width,
                       size_t outputRowPitch,
                       size_t outputDepthPitch)
 {
-    LoadR11EACToR16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadR11EACToR16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                     outputRowPitch, outputDepthPitch, true, false);
 }
 
-void LoadEACRG11ToRG16(size_t width,
+void LoadEACRG11ToRG16(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1777,11 +1889,12 @@ void LoadEACRG11ToRG16(size_t width,
                        size_t outputRowPitch,
                        size_t outputDepthPitch)
 {
-    LoadRG11EACToRG16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadRG11EACToRG16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, false, false);
 }
 
-void LoadEACRG11SToRG16(size_t width,
+void LoadEACRG11SToRG16(const ImageLoadContext &context,
+                        size_t width,
                         size_t height,
                         size_t depth,
                         const uint8_t *input,
@@ -1791,11 +1904,12 @@ void LoadEACRG11SToRG16(size_t width,
                         size_t outputRowPitch,
                         size_t outputDepthPitch)
 {
-    LoadRG11EACToRG16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadRG11EACToRG16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, true, false);
 }
 
-void LoadEACR11ToR16F(size_t width,
+void LoadEACR11ToR16F(const ImageLoadContext &context,
+                      size_t width,
                       size_t height,
                       size_t depth,
                       const uint8_t *input,
@@ -1805,11 +1919,12 @@ void LoadEACR11ToR16F(size_t width,
                       size_t outputRowPitch,
                       size_t outputDepthPitch)
 {
-    LoadR11EACToR16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadR11EACToR16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                     outputRowPitch, outputDepthPitch, false, true);
 }
 
-void LoadEACR11SToR16F(size_t width,
+void LoadEACR11SToR16F(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1819,11 +1934,12 @@ void LoadEACR11SToR16F(size_t width,
                        size_t outputRowPitch,
                        size_t outputDepthPitch)
 {
-    LoadR11EACToR16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadR11EACToR16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                     outputRowPitch, outputDepthPitch, true, true);
 }
 
-void LoadEACRG11ToRG16F(size_t width,
+void LoadEACRG11ToRG16F(const ImageLoadContext &context,
+                        size_t width,
                         size_t height,
                         size_t depth,
                         const uint8_t *input,
@@ -1833,11 +1949,12 @@ void LoadEACRG11ToRG16F(size_t width,
                         size_t outputRowPitch,
                         size_t outputDepthPitch)
 {
-    LoadRG11EACToRG16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadRG11EACToRG16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, false, true);
 }
 
-void LoadEACRG11SToRG16F(size_t width,
+void LoadEACRG11SToRG16F(const ImageLoadContext &context,
+                         size_t width,
                          size_t height,
                          size_t depth,
                          const uint8_t *input,
@@ -1847,11 +1964,12 @@ void LoadEACRG11SToRG16F(size_t width,
                          size_t outputRowPitch,
                          size_t outputDepthPitch)
 {
-    LoadRG11EACToRG16(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadRG11EACToRG16(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, true, true);
 }
 
-void LoadETC2RGB8ToRGBA8(size_t width,
+void LoadETC2RGB8ToRGBA8(const ImageLoadContext &context,
+                         size_t width,
                          size_t height,
                          size_t depth,
                          const uint8_t *input,
@@ -1861,11 +1979,12 @@ void LoadETC2RGB8ToRGBA8(size_t width,
                          size_t outputRowPitch,
                          size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                        outputRowPitch, outputDepthPitch, false);
+    LoadETC2RGB8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                        output, outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadETC2RGB8ToBC1(size_t width,
+void LoadETC2RGB8ToBC1(const ImageLoadContext &context,
+                       size_t width,
                        size_t height,
                        size_t depth,
                        const uint8_t *input,
@@ -1875,11 +1994,12 @@ void LoadETC2RGB8ToBC1(size_t width,
                        size_t outputRowPitch,
                        size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToBC1(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadETC2RGB8ToBC1(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadETC2SRGB8ToRGBA8(size_t width,
+void LoadETC2SRGB8ToRGBA8(const ImageLoadContext &context,
+                          size_t width,
                           size_t height,
                           size_t depth,
                           const uint8_t *input,
@@ -1889,11 +2009,12 @@ void LoadETC2SRGB8ToRGBA8(size_t width,
                           size_t outputRowPitch,
                           size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                        outputRowPitch, outputDepthPitch, false);
+    LoadETC2RGB8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                        output, outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadETC2SRGB8ToBC1(size_t width,
+void LoadETC2SRGB8ToBC1(const ImageLoadContext &context,
+                        size_t width,
                         size_t height,
                         size_t depth,
                         const uint8_t *input,
@@ -1903,11 +2024,12 @@ void LoadETC2SRGB8ToBC1(size_t width,
                         size_t outputRowPitch,
                         size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToBC1(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadETC2RGB8ToBC1(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadETC2RGB8A1ToRGBA8(size_t width,
+void LoadETC2RGB8A1ToRGBA8(const ImageLoadContext &context,
+                           size_t width,
                            size_t height,
                            size_t depth,
                            const uint8_t *input,
@@ -1917,11 +2039,12 @@ void LoadETC2RGB8A1ToRGBA8(size_t width,
                            size_t outputRowPitch,
                            size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                        outputRowPitch, outputDepthPitch, true);
+    LoadETC2RGB8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                        output, outputRowPitch, outputDepthPitch, true);
 }
 
-void LoadETC2RGB8A1ToBC1(size_t width,
+void LoadETC2RGB8A1ToBC1(const ImageLoadContext &context,
+                         size_t width,
                          size_t height,
                          size_t depth,
                          const uint8_t *input,
@@ -1931,11 +2054,12 @@ void LoadETC2RGB8A1ToBC1(size_t width,
                          size_t outputRowPitch,
                          size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToBC1(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadETC2RGB8ToBC1(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, true);
 }
 
-void LoadETC2SRGB8A1ToRGBA8(size_t width,
+void LoadETC2SRGB8A1ToRGBA8(const ImageLoadContext &context,
+                            size_t width,
                             size_t height,
                             size_t depth,
                             const uint8_t *input,
@@ -1945,11 +2069,12 @@ void LoadETC2SRGB8A1ToRGBA8(size_t width,
                             size_t outputRowPitch,
                             size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                        outputRowPitch, outputDepthPitch, true);
+    LoadETC2RGB8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                        output, outputRowPitch, outputDepthPitch, true);
 }
 
-void LoadETC2SRGB8A1ToBC1(size_t width,
+void LoadETC2SRGB8A1ToBC1(const ImageLoadContext &context,
+                          size_t width,
                           size_t height,
                           size_t depth,
                           const uint8_t *input,
@@ -1959,11 +2084,12 @@ void LoadETC2SRGB8A1ToBC1(size_t width,
                           size_t outputRowPitch,
                           size_t outputDepthPitch)
 {
-    LoadETC2RGB8ToBC1(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+    LoadETC2RGB8ToBC1(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
                       outputRowPitch, outputDepthPitch, true);
 }
 
-void LoadETC2RGBA8ToRGBA8(size_t width,
+void LoadETC2RGBA8ToRGBA8(const ImageLoadContext &context,
+                          size_t width,
                           size_t height,
                           size_t depth,
                           const uint8_t *input,
@@ -1973,11 +2099,12 @@ void LoadETC2RGBA8ToRGBA8(size_t width,
                           size_t outputRowPitch,
                           size_t outputDepthPitch)
 {
-    LoadETC2RGBA8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                         outputRowPitch, outputDepthPitch, false);
+    LoadETC2RGBA8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                         output, outputRowPitch, outputDepthPitch, false);
 }
 
-void LoadETC2SRGBA8ToSRGBA8(size_t width,
+void LoadETC2SRGBA8ToSRGBA8(const ImageLoadContext &context,
+                            size_t width,
                             size_t height,
                             size_t depth,
                             const uint8_t *input,
@@ -1987,8 +2114,164 @@ void LoadETC2SRGBA8ToSRGBA8(size_t width,
                             size_t outputRowPitch,
                             size_t outputDepthPitch)
 {
-    LoadETC2RGBA8ToRGBA8(width, height, depth, input, inputRowPitch, inputDepthPitch, output,
-                         outputRowPitch, outputDepthPitch, true);
+    LoadETC2RGBA8ToRGBA8(context, width, height, depth, input, inputRowPitch, inputDepthPitch,
+                         output, outputRowPitch, outputDepthPitch, true);
+}
+
+void LoadETC2RGBA8ToBC3(const ImageLoadContext &context,
+                        size_t width,
+                        size_t height,
+                        size_t depth,
+                        const uint8_t *input,
+                        size_t inputRowPitch,
+                        size_t inputDepthPitch,
+                        uint8_t *output,
+                        size_t outputRowPitch,
+                        size_t outputDepthPitch)
+{
+    LoadETC2RGBA8ToBC3(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+                       outputRowPitch, outputDepthPitch, false, false);
+}
+
+void LoadETC2SRGBA8ToBC3(const ImageLoadContext &context,
+                         size_t width,
+                         size_t height,
+                         size_t depth,
+                         const uint8_t *input,
+                         size_t inputRowPitch,
+                         size_t inputDepthPitch,
+                         uint8_t *output,
+                         size_t outputRowPitch,
+                         size_t outputDepthPitch)
+{
+    LoadETC2RGBA8ToBC3(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+                       outputRowPitch, outputDepthPitch, false, false);
+}
+
+void LoadEACR11ToBC4(const ImageLoadContext &context,
+                     size_t width,
+                     size_t height,
+                     size_t depth,
+                     const uint8_t *input,
+                     size_t inputRowPitch,
+                     size_t inputDepthPitch,
+                     uint8_t *output,
+                     size_t outputRowPitch,
+                     size_t outputDepthPitch,
+                     bool isSigned)
+{
+    for (size_t z = 0; z < depth; z++)
+    {
+        for (size_t y = 0; y < height; y += 4)
+        {
+            const ETC2Block *sourceRow =
+                priv::OffsetDataPointer<ETC2Block>(input, y / 4, z, inputRowPitch, inputDepthPitch);
+            uint8_t *destRow = priv::OffsetDataPointer<uint8_t>(output, y / 4, z, outputRowPitch,
+                                                                outputDepthPitch);
+
+            for (size_t x = 0; x < width; x += 4)
+            {
+                const ETC2Block *sourceR11Block = sourceRow + (x / 4);
+                uint8_t *destR11Pixels          = destRow + (x * 2);
+                sourceR11Block->transcodeAsBC4(destR11Pixels, x, y, width, height, isSigned);
+            }
+        }
+    }
+}
+
+void LoadEACRG11ToBC5(const ImageLoadContext &context,
+                      size_t width,
+                      size_t height,
+                      size_t depth,
+                      const uint8_t *input,
+                      size_t inputRowPitch,
+                      size_t inputDepthPitch,
+                      uint8_t *output,
+                      size_t outputRowPitch,
+                      size_t outputDepthPitch,
+                      bool isSigned)
+{
+    for (size_t z = 0; z < depth; z++)
+    {
+        for (size_t y = 0; y < height; y += 4)
+        {
+            const ETC2Block *sourceRow =
+                priv::OffsetDataPointer<ETC2Block>(input, y / 4, z, inputRowPitch, inputDepthPitch);
+            uint8_t *destRow = priv::OffsetDataPointer<uint8_t>(output, y / 4, z, outputRowPitch,
+                                                                outputDepthPitch);
+
+            for (size_t x = 0; x < width; x += 4)
+            {
+                const ETC2Block *sourceR11Block = sourceRow + (x / 2);
+                uint8_t *destR11Pixels          = destRow + (x * 4);
+
+                const ETC2Block *sourceG11Block = sourceR11Block + 1;
+                uint8_t *destG11Pixels          = destR11Pixels + 8;
+                sourceR11Block->transcodeAsBC4(destR11Pixels, x, y, width, height, isSigned);
+                sourceG11Block->transcodeAsBC4(destG11Pixels, x, y, width, height, isSigned);
+            }
+        }
+    }
+}
+
+void LoadEACR11ToBC4(const ImageLoadContext &context,
+                     size_t width,
+                     size_t height,
+                     size_t depth,
+                     const uint8_t *input,
+                     size_t inputRowPitch,
+                     size_t inputDepthPitch,
+                     uint8_t *output,
+                     size_t outputRowPitch,
+                     size_t outputDepthPitch)
+{
+    LoadEACR11ToBC4(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+                    outputRowPitch, outputDepthPitch, false);
+}
+
+void LoadEACR11SToBC4(const ImageLoadContext &context,
+                      size_t width,
+                      size_t height,
+                      size_t depth,
+                      const uint8_t *input,
+                      size_t inputRowPitch,
+                      size_t inputDepthPitch,
+                      uint8_t *output,
+                      size_t outputRowPitch,
+                      size_t outputDepthPitch)
+{
+    LoadEACR11ToBC4(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+                    outputRowPitch, outputDepthPitch, true);
+}
+
+void LoadEACRG11ToBC5(const ImageLoadContext &context,
+                      size_t width,
+                      size_t height,
+                      size_t depth,
+                      const uint8_t *input,
+                      size_t inputRowPitch,
+                      size_t inputDepthPitch,
+                      uint8_t *output,
+                      size_t outputRowPitch,
+                      size_t outputDepthPitch)
+{
+    LoadEACRG11ToBC5(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+                     outputRowPitch, outputDepthPitch, false);
+}
+
+void LoadEACRG11SToBC5(const ImageLoadContext &context,
+                       size_t width,
+                       size_t height,
+                       size_t depth,
+                       const uint8_t *input,
+                       size_t inputRowPitch,
+                       size_t inputDepthPitch,
+                       uint8_t *output,
+                       size_t outputRowPitch,
+                       size_t outputDepthPitch)
+{
+    LoadEACRG11ToBC5(context, width, height, depth, input, inputRowPitch, inputDepthPitch, output,
+                     outputRowPitch, outputDepthPitch, true);
 }
 
 }  // namespace angle
