@@ -50,6 +50,11 @@ bool SameIndexBuffer(const VertexArrayStateGL *a, const gl::Buffer *frontendBuff
     return a->elementArrayBuffer == GetNativeBufferID(frontendBuffer);
 }
 
+bool SameAttribPointer(const VertexAttributeGL &a, const VertexAttribute &b)
+{
+    return a.pointer == b.pointer;
+}
+
 bool IsVertexAttribPointerSupported(size_t attribIndex, const VertexAttribute &attrib)
 {
     return (attribIndex == attrib.bindingIndex && attrib.relativeOffset == 0);
@@ -74,7 +79,7 @@ static angle::Result ValidateStateHelperGetIntegerv(const gl::Context *context,
     {
         WARN() << localName << " (" << localValue << ") != " << driverName << " (" << queryValue
                << ")";
-        // Re-add ASSERT: http://anglebug.com/3900
+        // Re-add ASSERT: http://anglebug.com/42262547
         // ASSERT(false);
     }
 
@@ -96,7 +101,7 @@ static angle::Result ValidateStateHelperGetVertexAttribiv(const gl::Context *con
     {
         WARN() << localName << "[" << index << "] (" << localValue << ") != " << driverName << "["
                << index << "] (" << queryValue << ")";
-        // Re-add ASSERT: http://anglebug.com/3900
+        // Re-add ASSERT: http://anglebug.com/42262547
         // ASSERT(false);
     }
 
@@ -171,7 +176,8 @@ angle::Result VertexArrayGL::updateElementArrayBufferBinding(const gl::Context *
     gl::Buffer *elementArrayBuffer = mState.getElementArrayBuffer();
     if (!SameIndexBuffer(mNativeState, elementArrayBuffer))
     {
-        GLuint elementArrayBufferId = GetNativeBufferID(elementArrayBuffer);
+        GLuint elementArrayBufferId =
+            elementArrayBuffer ? GetNativeBufferID(elementArrayBuffer) : 0;
 
         StateManagerGL *stateManager = GetStateManagerGL(context);
         stateManager->bindBuffer(gl::BufferBinding::ElementArray, elementArrayBufferId);
@@ -192,10 +198,16 @@ angle::Result VertexArrayGL::syncDrawState(const gl::Context *context,
                                            bool primitiveRestartEnabled,
                                            const void **outIndices) const
 {
+    const FunctionsGL *functions = GetFunctionsGL(context);
+
     // Check if any attributes need to be streamed, determines if the index range needs to be
     // computed
-    const gl::AttributesMask &needsStreamingAttribs =
+    gl::AttributesMask needsStreamingAttribs =
         context->getStateCache().getActiveClientAttribsMask();
+    if (nativegl::CanUseClientSideArrays(functions, mVertexArrayID))
+    {
+        needsStreamingAttribs.reset();
+    }
 
     // Determine if an index buffer needs to be streamed and the range of vertices that need to be
     // copied
@@ -268,6 +280,8 @@ angle::Result VertexArrayGL::syncIndexData(const gl::Context *context,
 {
     ASSERT(outIndices);
 
+    const FunctionsGL *functions = GetFunctionsGL(context);
+
     gl::Buffer *elementArrayBuffer = mState.getElementArrayBuffer();
 
     // Need to check the range of indices if attributes need to be streamed
@@ -287,13 +301,18 @@ angle::Result VertexArrayGL::syncIndexData(const gl::Context *context,
         // the draw call
         *outIndices = indices;
     }
+    else if (nativegl::CanUseClientSideArrays(functions, mVertexArrayID))
+    {
+        ASSERT(SameIndexBuffer(mNativeState, nullptr));
+
+        // Use the client index data directly
+        *outIndices = indices;
+    }
     else
     {
-        const FunctionsGL *functions = GetFunctionsGL(context);
         StateManagerGL *stateManager = GetStateManagerGL(context);
 
         // Need to stream the index buffer
-        // TODO: if GLES, nothing needs to be streamed
 
         // Only compute the index range if the attributes also need to be streamed
         if (attributesNeedStreaming)
@@ -489,6 +508,15 @@ angle::Result VertexArrayGL::streamAttributes(
                     // The workaround is only for latest Mac Intel so glMapBufferRange should be
                     // supported
                     ASSERT(CanMapBufferForRead(functions));
+                    // Validate if there is OOB access of the input buffer.
+                    angle::CheckedNumeric<GLint64> inputRequiredSize;
+                    inputRequiredSize = copySize;
+                    inputRequiredSize += static_cast<unsigned int>(binding.getOffset());
+                    ANGLE_CHECK(GetImplAs<ContextGL>(context),
+                                inputRequiredSize.IsValid() && inputRequiredSize.ValueOrDie() <=
+                                                                   bindingBufferPointer->getSize(),
+                                "Failed to map buffer range of the attribute buffer.",
+                                GL_OUT_OF_MEMORY);
                     uint8_t *inputBufferPointer = MapBufferRangeWithFallback(
                         functions, GL_ARRAY_BUFFER, binding.getOffset(), copySize, GL_MAP_READ_BIT);
                     ASSERT(inputBufferPointer);
@@ -646,6 +674,8 @@ angle::Result VertexArrayGL::updateAttribEnabled(const gl::Context *context, siz
 
 angle::Result VertexArrayGL::updateAttribPointer(const gl::Context *context, size_t attribIndex)
 {
+    const angle::FeaturesGL &features = GetFeaturesGL(context);
+    const FunctionsGL *functions      = GetFunctionsGL(context);
 
     const VertexAttribute &attrib = mState.getVertexAttribute(attribIndex);
 
@@ -653,6 +683,8 @@ angle::Result VertexArrayGL::updateAttribPointer(const gl::Context *context, siz
     // of the binding indexed attrib.bindingIndex (unless attribIndex == attrib.bindingIndex).
     const VertexBinding &binding = mState.getVertexBinding(attribIndex);
 
+    const bool canUseClientArrays = nativegl::CanUseClientSideArrays(functions, mVertexArrayID);
+
     // Early return when the vertex attribute isn't using a buffer object:
     // - If we need to stream, defer the attribPointer to the draw call.
     // - Skip the attribute that is disabled and uses a client memory pointer.
@@ -660,7 +692,7 @@ angle::Result VertexArrayGL::updateAttribPointer(const gl::Context *context, siz
     //   client memory pointer either, it must be disabled and shouldn't affect the draw.
     const auto &bindingBuffer = binding.getBuffer();
     gl::Buffer *arrayBuffer   = bindingBuffer.get();
-    if (arrayBuffer == nullptr)
+    if (arrayBuffer == nullptr && !canUseClientArrays)
     {
         // Mark the applied binding isn't using a buffer by setting its buffer to nullptr so that if
         // it starts to use a buffer later, there is no chance that the caching will skip it.
@@ -673,24 +705,45 @@ angle::Result VertexArrayGL::updateAttribPointer(const gl::Context *context, siz
     // We do not need to compare attrib.pointer because when we use a different client memory
     // pointer, we don't need to update mAttributesNeedStreaming by binding.buffer and we won't
     // update attribPointer in this function.
-    if ((SameVertexAttribFormat(mNativeState->attributes[attribIndex], attrib)) &&
+    if (SameVertexAttribFormat(mNativeState->attributes[attribIndex], attrib) &&
         (mNativeState->attributes[attribIndex].bindingIndex == attrib.bindingIndex) &&
-        (SameVertexBuffer(mNativeState->bindings[attribIndex], binding)))
+        SameVertexBuffer(mNativeState->bindings[attribIndex], binding) &&
+        (!canUseClientArrays || SameAttribPointer(mNativeState->attributes[attribIndex], attrib)))
     {
         return angle::Result::Continue;
     }
 
-    // Since ANGLE always uses a non-zero VAO, we cannot use a client memory pointer on it:
-    // [OpenGL ES 3.0.2] Section 2.8 page 24:
-    // An INVALID_OPERATION error is generated when a non-zero vertex array object is bound,
-    // zero is bound to the ARRAY_BUFFER buffer object binding point, and the pointer argument
-    // is not NULL.
-
     StateManagerGL *stateManager = GetStateManagerGL(context);
-    GLuint bufferId              = GetNativeBufferID(arrayBuffer);
-    stateManager->bindBuffer(gl::BufferBinding::Array, bufferId);
-    ANGLE_TRY(callVertexAttribPointer(context, static_cast<GLuint>(attribIndex), attrib,
-                                      binding.getStride(), binding.getOffset()));
+    GLuint bufferId              = 0;
+    if (arrayBuffer != nullptr)
+    {
+        // When ANGLE uses non-zero VAO, we cannot use a client memory pointer on it:
+        // [OpenGL ES 3.0.2] Section 2.8 page 24:
+        // An INVALID_OPERATION error is generated when a non-zero vertex array object is bound,
+        // zero is bound to the ARRAY_BUFFER buffer object binding point, and the pointer argument
+        // is not NULL.
+
+        BufferGL *bufferGL = GetImplAs<BufferGL>(arrayBuffer);
+        bufferId           = bufferGL->getBufferID();
+        stateManager->bindBuffer(gl::BufferBinding::Array, bufferId);
+        if (features.ensureNonEmptyBufferIsBoundForDraw.enabled && bufferGL->getBufferSize() == 0)
+        {
+            constexpr uint32_t data = 0;
+            ANGLE_TRY(bufferGL->setData(context, gl::BufferBinding::Array, &data, sizeof(data),
+                                        gl::BufferUsage::StaticDraw));
+            ASSERT(bufferGL->getBufferSize() > 0);
+        }
+        ANGLE_TRY(callVertexAttribPointer(context, static_cast<GLuint>(attribIndex), attrib,
+                                          binding.getStride(), binding.getOffset()));
+    }
+    else
+    {
+        ASSERT(canUseClientArrays);
+        stateManager->bindBuffer(gl::BufferBinding::Array, 0);
+        ANGLE_TRY(callVertexAttribPointer(context, static_cast<GLuint>(attribIndex), attrib,
+                                          binding.getStride(),
+                                          reinterpret_cast<GLintptr>(attrib.pointer)));
+    }
 
     mNativeState->attributes[attribIndex].format = attrib.format;
 
@@ -698,6 +751,7 @@ angle::Result VertexArrayGL::updateAttribPointer(const gl::Context *context, siz
     // to attribIndex in driver. If attrib.relativeOffset != 0 or attrib.bindingIndex !=
     // attribIndex, they should be set in updateAttribFormat and updateAttribBinding. The cache
     // should be consistent with driver so that we won't miss anything.
+    mNativeState->attributes[attribIndex].pointer        = attrib.pointer;
     mNativeState->attributes[attribIndex].relativeOffset = 0;
     mNativeState->attributes[attribIndex].bindingIndex   = static_cast<GLuint>(attribIndex);
 
@@ -737,13 +791,11 @@ angle::Result VertexArrayGL::callVertexAttribPointer(const gl::Context *context,
 
 bool VertexArrayGL::supportVertexAttribBinding(const gl::Context *context) const
 {
-    const FunctionsGL *functions      = GetFunctionsGL(context);
-    const angle::FeaturesGL &features = GetFeaturesGL(context);
+    const FunctionsGL *functions = GetFunctionsGL(context);
     ASSERT(functions);
     // Vertex attrib bindings are not supported on the default VAO so if we're syncing to the
     // default VAO due to the feature, disable bindings.
-    return (functions->vertexAttribBinding != nullptr) &&
-           !features.syncVertexArraysToDefault.enabled;
+    return (functions->vertexAttribBinding != nullptr) && mVertexArrayID != 0;
 }
 
 angle::Result VertexArrayGL::updateAttribFormat(const gl::Context *context, size_t attribIndex)
@@ -906,13 +958,21 @@ angle::Result VertexArrayGL::syncDirtyBinding(
 {
     // Dependent state changes in buffers can trigger updates with no dirty bits set.
 
-    for (size_t dirtyBit : dirtyBindingBits)
+    for (auto iter = dirtyBindingBits.begin(), endIter = dirtyBindingBits.end(); iter != endIter;
+         ++iter)
     {
+        size_t dirtyBit = *iter;
         switch (dirtyBit)
         {
             case VertexArray::DIRTY_BINDING_BUFFER:
+            case VertexArray::DIRTY_BINDING_STRIDE:
+            case VertexArray::DIRTY_BINDING_OFFSET:
                 ASSERT(supportVertexAttribBinding(context));
                 ANGLE_TRY(updateBindingBuffer(context, bindingIndex));
+                // Clear these bits to avoid repeated processing
+                iter.resetLaterBits(gl::VertexArray::DirtyBindingBits{
+                    VertexArray::DIRTY_BINDING_BUFFER, VertexArray::DIRTY_BINDING_STRIDE,
+                    VertexArray::DIRTY_BINDING_OFFSET});
                 break;
 
             case VertexArray::DIRTY_BINDING_DIVISOR:
@@ -951,10 +1011,23 @@ angle::Result VertexArrayGL::syncState(const gl::Context *context,
     StateManagerGL *stateManager = GetStateManagerGL(context);
     stateManager->bindVertexArray(mVertexArrayID, mNativeState);
 
-    for (size_t dirtyBit : dirtyBits)
+    for (auto iter = dirtyBits.begin(), endIter = dirtyBits.end(); iter != endIter; ++iter)
     {
+        size_t dirtyBit = *iter;
         switch (dirtyBit)
         {
+            case gl::VertexArray::DIRTY_BIT_LOST_OBSERVATION:
+            {
+                // If vertex array was not observing while unbound, we need to check buffer's
+                // internal storage and take action if buffer has changed while not observing.
+                // For now we just simply assume buffer storage has changed and always dirty all
+                // binding points.
+                iter.setLaterBits(
+                    gl::VertexArray::DirtyBits(mState.getBufferBindingMask().to_ulong()
+                                               << gl::VertexArray::DIRTY_BIT_BINDING_0));
+                break;
+            }
+
             case VertexArray::DIRTY_BIT_ELEMENT_ARRAY_BUFFER:
                 ANGLE_TRY(updateElementArrayBufferBinding(context));
                 break;
@@ -1031,7 +1104,7 @@ angle::Result VertexArrayGL::validateState(const gl::Context *context) const
     {
         WARN() << "mNativeState->attributes.size() (" << mNativeState->attributes.size()
                << ") > GL_MAX_VERTEX_ATTRIBS (" << queryValue << ")";
-        // Re-add ASSERT: http://anglebug.com/3900
+        // Re-add ASSERT: http://anglebug.com/42262547
         // ASSERT(false);
     }
 
