@@ -6,14 +6,10 @@
 
 // DeviceCGL.cpp: CGL implementation of egl::Device
 
-#include "common/platform.h"
+#include "libANGLE/renderer/gl/cgl/DeviceCGL.h"
 
-#if defined(ANGLE_PLATFORM_MACOS) || defined(ANGLE_PLATFORM_MACCATALYST)
-
-#    include "libANGLE/renderer/gl/cgl/DeviceCGL.h"
-
-#    include <EGL/eglext.h>
-#    include "libANGLE/renderer/gl/cgl/DisplayCGL.h"
+#include <EGL/eglext.h>
+#include "libANGLE/renderer/gl/cgl/DisplayCGL.h"
 
 namespace rx
 {
@@ -46,16 +42,9 @@ egl::Error DeviceCGL::getAttribute(const egl::Display *display, EGLint attribute
     return egl::NoError();
 }
 
-EGLint DeviceCGL::getType()
-{
-    return 0;
-}
-
 void DeviceCGL::generateExtensions(egl::DeviceExtensions *outExtensions) const
 {
     outExtensions->deviceCGL = true;
 }
 
 }  // namespace rx
-
-#endif  // defined(ANGLE_PLATFORM_MACOS) || defined(ANGLE_PLATFORM_MACCATALYST)
