@@ -35,7 +35,7 @@ class ProgramBinaryTest : public ANGLETest<>
 
     void testSetUp() override
     {
-        mProgram = CompileProgram(essl1_shaders::vs::Simple(), essl1_shaders::fs::Red());
+        mProgram = CompileProgram(essl1_shaders::vs::Texture2D(), essl1_shaders::fs::Texture2D());
         if (mProgram == 0)
         {
             FAIL() << "shader compilation failed.";
@@ -274,7 +274,9 @@ TEST_P(ProgramBinaryTest, ZeroSizedUnlinkedBinary)
 
 // Use this to select which configurations (e.g. which renderer, which GLES major version) these
 // tests should be run against.
-ANGLE_INSTANTIATE_TEST_ES2_AND_ES3(ProgramBinaryTest);
+ANGLE_INSTANTIATE_TEST_ES2_AND_ES3_AND(
+    ProgramBinaryTest,
+    ES3_VULKAN().disable(Feature::EnablePipelineCacheDataCompression));
 
 class ProgramBinaryES3Test : public ProgramBinaryTest
 {
@@ -288,30 +290,52 @@ class ProgramBinaryES3Test : public ProgramBinaryTest
     void testBinaryAndUBOBlockIndexes(bool drawWithProgramFirst);
 };
 
+class ProgramBinaryES31Test : public ANGLETest<>
+{
+  protected:
+    ProgramBinaryES31Test()
+    {
+        setWindowWidth(128);
+        setWindowHeight(128);
+        setConfigRedBits(8);
+        setConfigGreenBits(8);
+        setConfigBlueBits(8);
+        setConfigAlphaBits(8);
+
+        // Test flakiness was noticed when reusing displays.
+        forceNewDisplay();
+    }
+
+    GLint getAvailableProgramBinaryFormatCount() const
+    {
+        GLint formatCount;
+        glGetIntegerv(GL_NUM_PROGRAM_BINARY_FORMATS_OES, &formatCount);
+        return formatCount;
+    }
+};
+
 void ProgramBinaryES3Test::testBinaryAndUBOBlockIndexes(bool drawWithProgramFirst)
 {
     ANGLE_SKIP_TEST_IF(getAvailableProgramBinaryFormatCount() == 0);
 
-    constexpr char kVS[] =
-        "#version 300 es\n"
-        "uniform block {\n"
-        "    float f;\n"
-        "};\n"
-        "in vec4 position;\n"
-        "out vec4 color;\n"
-        "void main() {\n"
-        "    gl_Position = position;\n"
-        "    color = vec4(f, f, f, 1);\n"
-        "}";
+    constexpr char kVS[] = R"(#version 300 es
+uniform block {
+    float f;
+};
+in vec4 position;
+out vec4 color;
+void main() {
+    gl_Position = position;
+    color = vec4(f, f, f, 1);
+})";
 
-    constexpr char kFS[] =
-        "#version 300 es\n"
-        "precision mediump float;\n"
-        "in vec4 color;\n"
-        "out vec4 colorOut;\n"
-        "void main() {\n"
-        "    colorOut = color;\n"
-        "}";
+    constexpr char kFS[] = R"(#version 300 es
+precision mediump float;
+in vec4 color;
+out vec4 colorOut;
+void main() {
+    colorOut = color;
+})";
 
     // Init and draw with the program.
     ANGLE_GL_PROGRAM(program, kVS, kFS);
@@ -320,14 +344,14 @@ void ProgramBinaryES3Test::testBinaryAndUBOBlockIndexes(bool drawWithProgramFirs
     GLuint bindIndex = 2;
 
     GLBuffer ubo;
-    glBindBuffer(GL_UNIFORM_BUFFER, ubo.get());
+    glBindBuffer(GL_UNIFORM_BUFFER, ubo);
     glBufferData(GL_UNIFORM_BUFFER, sizeof(fData), &fData, GL_STATIC_DRAW);
-    glBindBufferRange(GL_UNIFORM_BUFFER, bindIndex, ubo.get(), 0, sizeof(fData));
+    glBindBufferRange(GL_UNIFORM_BUFFER, bindIndex, ubo, 0, sizeof(fData));
 
-    GLint blockIndex = glGetUniformBlockIndex(program.get(), "block");
+    GLint blockIndex = glGetUniformBlockIndex(program, "block");
     ASSERT_NE(-1, blockIndex);
 
-    glUniformBlockBinding(program.get(), blockIndex, bindIndex);
+    glUniformBlockBinding(program, blockIndex, bindIndex);
 
     glClearColor(1.0, 0.0, 0.0, 1.0);
     glClear(GL_COLOR_BUFFER_BIT);
@@ -335,20 +359,20 @@ void ProgramBinaryES3Test::testBinaryAndUBOBlockIndexes(bool drawWithProgramFirs
 
     if (drawWithProgramFirst)
     {
-        drawQuad(program.get(), "position", 0.5f);
+        drawQuad(program, "position", 0.5f);
         ASSERT_GL_NO_ERROR();
         EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::white);
     }
 
     // Read back the binary.
     GLint programLength = 0;
-    glGetProgramiv(program.get(), GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
     ASSERT_GL_NO_ERROR();
 
     GLsizei readLength  = 0;
     GLenum binaryFormat = GL_NONE;
     std::vector<uint8_t> binary(programLength);
-    glGetProgramBinary(program.get(), programLength, &readLength, &binaryFormat, binary.data());
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
     ASSERT_GL_NO_ERROR();
 
     EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
@@ -356,11 +380,14 @@ void ProgramBinaryES3Test::testBinaryAndUBOBlockIndexes(bool drawWithProgramFirs
     // Load a new program with the binary and draw.
     ANGLE_GL_BINARY_ES3_PROGRAM(binaryProgram, binary, binaryFormat);
 
+    // Reconfigure the block binding, which is reset after a call to glProgramBinary.
+    glUniformBlockBinding(binaryProgram, blockIndex, bindIndex);
+
     glClearColor(1.0, 0.0, 0.0, 1.0);
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
 
-    drawQuad(binaryProgram.get(), "position", 0.5f);
+    drawQuad(binaryProgram, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::white);
 }
@@ -371,13 +398,156 @@ TEST_P(ProgramBinaryES3Test, UniformBlockBindingWithDraw)
     testBinaryAndUBOBlockIndexes(true);
 }
 
-// Same as above, but does not do an initial draw with the program. Covers an ANGLE crash.
-// http://anglebug.com/1637
+// Same as UniformBlockBindingWithDraw, but does not do an initial draw with the program. Covers an
+// ANGLE crash.  http://anglebug.com/42260591
 TEST_P(ProgramBinaryES3Test, UniformBlockBindingNoDraw)
 {
     testBinaryAndUBOBlockIndexes(false);
 }
 
+// Same as UniformBlockBindingWithDraw, but specifies the binding in the shader itself.
+TEST_P(ProgramBinaryES31Test, UniformBlockBindingSpecifiedInShader)
+{
+    ANGLE_SKIP_TEST_IF(getAvailableProgramBinaryFormatCount() == 0);
+
+    constexpr char kVS[] = R"(#version 310 es
+layout(binding = 1) uniform block {
+    vec4 v;
+};
+in vec4 position;
+out vec4 color;
+void main() {
+    gl_Position = position;
+    color = v;
+})";
+
+    constexpr char kFS[] = R"(#version 310 es
+precision mediump float;
+in vec4 color;
+out vec4 colorOut;
+void main() {
+    colorOut = color;
+})";
+
+    // Init and draw with the program.
+    ANGLE_GL_PROGRAM(program, kVS, kFS);
+
+    constexpr std::array<float, 4> kBlue = {0, 0, 1, 1};
+    GLBuffer blue;
+    glBindBuffer(GL_UNIFORM_BUFFER, blue);
+    glBufferData(GL_UNIFORM_BUFFER, sizeof(kBlue), kBlue.data(), GL_STATIC_DRAW);
+    glBindBufferBase(GL_UNIFORM_BUFFER, 1, blue);
+
+    GLint blockIndex = glGetUniformBlockIndex(program, "block");
+    ASSERT_NE(-1, blockIndex);
+
+    glClearColor(1.0, 0.0, 0.0, 1.0);
+    glClear(GL_COLOR_BUFFER_BIT);
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
+
+    drawQuad(program, "position", 0.5f);
+    ASSERT_GL_NO_ERROR();
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::blue);
+
+    // Read back the binary.
+    GLint programLength = 0;
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
+    ASSERT_GL_NO_ERROR();
+
+    GLsizei readLength  = 0;
+    GLenum binaryFormat = GL_NONE;
+    std::vector<uint8_t> binary(programLength);
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
+    ASSERT_GL_NO_ERROR();
+
+    EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
+
+    // Load a new program with the binary and draw.
+    ANGLE_GL_BINARY_ES3_PROGRAM(binaryProgram, binary, binaryFormat);
+
+    glClear(GL_COLOR_BUFFER_BIT);
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
+
+    drawQuad(binaryProgram, "position", 0.5f);
+    ASSERT_GL_NO_ERROR();
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::blue);
+}
+
+// Ensure that uniform block bindings are reset to their original value on program binary
+TEST_P(ProgramBinaryES31Test, UniformBlockBindingResetOnReload)
+{
+    ANGLE_SKIP_TEST_IF(getAvailableProgramBinaryFormatCount() == 0);
+
+    constexpr char kVS[] = R"(#version 310 es
+layout(binding = 1) uniform block {
+    vec4 v;
+};
+in vec4 position;
+out vec4 color;
+void main() {
+    gl_Position = position;
+    color = v;
+})";
+
+    constexpr char kFS[] = R"(#version 310 es
+precision mediump float;
+in vec4 color;
+out vec4 colorOut;
+void main() {
+    colorOut = color;
+})";
+
+    // Init and draw with the program.
+    ANGLE_GL_PROGRAM(program, kVS, kFS);
+
+    constexpr std::array<float, 4> kBlue  = {0, 0, 1, 1};
+    constexpr std::array<float, 4> kGreen = {0, 1, 0, 1};
+    GLBuffer blue, green;
+    glBindBuffer(GL_UNIFORM_BUFFER, blue);
+    glBufferData(GL_UNIFORM_BUFFER, sizeof(kBlue), kBlue.data(), GL_STATIC_DRAW);
+    glBindBuffer(GL_UNIFORM_BUFFER, green);
+    glBufferData(GL_UNIFORM_BUFFER, sizeof(kGreen), kGreen.data(), GL_STATIC_DRAW);
+
+    // Bind one buffer to binding 1 and another to binding 2
+    glBindBufferBase(GL_UNIFORM_BUFFER, 1, blue);
+    glBindBufferBase(GL_UNIFORM_BUFFER, 2, green);
+
+    GLint blockIndex = glGetUniformBlockIndex(program, "block");
+    ASSERT_NE(-1, blockIndex);
+
+    // Initially, remap the block to binding 2
+    glUniformBlockBinding(program, blockIndex, 2);
+
+    glClearColor(1.0, 0.0, 0.0, 1.0);
+    glClear(GL_COLOR_BUFFER_BIT);
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
+
+    drawQuad(program, "position", 0.5f);
+    ASSERT_GL_NO_ERROR();
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::green);
+
+    // Read back the binary.
+    GLint programLength = 0;
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
+    ASSERT_GL_NO_ERROR();
+
+    GLsizei readLength  = 0;
+    GLenum binaryFormat = GL_NONE;
+    std::vector<uint8_t> binary(programLength);
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
+    ASSERT_GL_NO_ERROR();
+
+    EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
+
+    // Load a new program with the binary and draw.  On reset, the binding should be reset back to 1
+    ANGLE_GL_BINARY_ES3_PROGRAM(binaryProgram, binary, binaryFormat);
+
+    glClear(GL_COLOR_BUFFER_BIT);
+    drawQuad(binaryProgram, "position", 0.5f);
+    ASSERT_GL_NO_ERROR();
+    EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::blue);
+}
+
 // Test the shaders with arrays-of-struct uniforms are properly saved and restored
 TEST_P(ProgramBinaryES3Test, TestArrayOfStructUniform)
 {
@@ -432,24 +602,24 @@ TEST_P(ProgramBinaryES3Test, TestArrayOfStructUniform)
     // Init and draw with the program.
     ANGLE_GL_PROGRAM(program, kVS, kFS);
 
-    glUseProgram(program.get());
+    glUseProgram(program);
 
-    int location = glGetUniformLocation(program.get(), "u_var[0].m0");
+    int location = glGetUniformLocation(program, "u_var[0].m0");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.15, 0.52, 0.26, 0.35);
-    location = glGetUniformLocation(program.get(), "u_var[0].m1");
+    location = glGetUniformLocation(program, "u_var[0].m1");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.88, 0.09, 0.30, 0.61);
-    location = glGetUniformLocation(program.get(), "u_var[1].m0");
+    location = glGetUniformLocation(program, "u_var[1].m0");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.85, 0.59, 0.33, 0.71);
-    location = glGetUniformLocation(program.get(), "u_var[1].m1");
+    location = glGetUniformLocation(program, "u_var[1].m1");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.62, 0.89, 0.09, 0.99);
-    location = glGetUniformLocation(program.get(), "u_var[2].m0");
+    location = glGetUniformLocation(program, "u_var[2].m0");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.53, 0.89, 0.01, 0.08);
-    location = glGetUniformLocation(program.get(), "u_var[2].m1");
+    location = glGetUniformLocation(program, "u_var[2].m1");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.26, 0.72, 0.60, 0.12);
     ASSERT_GL_NO_ERROR();
@@ -458,19 +628,19 @@ TEST_P(ProgramBinaryES3Test, TestArrayOfStructUniform)
     glClearColor(1.0, 0.0, 0.0, 1.0);
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
-    drawQuad(program.get(), "position", 0.5f);
+    drawQuad(program, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::white);
 
     // Read back the binary.
     GLint programLength = 0;
-    glGetProgramiv(program.get(), GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
     ASSERT_GL_NO_ERROR();
 
     GLsizei readLength  = 0;
     GLenum binaryFormat = GL_NONE;
     std::vector<uint8_t> binary(programLength);
-    glGetProgramBinary(program.get(), programLength, &readLength, &binaryFormat, binary.data());
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
     ASSERT_GL_NO_ERROR();
 
     EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
@@ -478,24 +648,24 @@ TEST_P(ProgramBinaryES3Test, TestArrayOfStructUniform)
     // Load a new program with the binary and draw.
     ANGLE_GL_BINARY_ES3_PROGRAM(binaryProgram, binary, binaryFormat);
 
-    glUseProgram(binaryProgram.get());
+    glUseProgram(binaryProgram);
 
-    location = glGetUniformLocation(binaryProgram.get(), "u_var[0].m0");
+    location = glGetUniformLocation(binaryProgram, "u_var[0].m0");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.15, 0.52, 0.26, 0.35);
-    location = glGetUniformLocation(binaryProgram.get(), "u_var[0].m1");
+    location = glGetUniformLocation(binaryProgram, "u_var[0].m1");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.88, 0.09, 0.30, 0.61);
-    location = glGetUniformLocation(binaryProgram.get(), "u_var[1].m0");
+    location = glGetUniformLocation(binaryProgram, "u_var[1].m0");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.85, 0.59, 0.33, 0.71);
-    location = glGetUniformLocation(binaryProgram.get(), "u_var[1].m1");
+    location = glGetUniformLocation(binaryProgram, "u_var[1].m1");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.62, 0.89, 0.09, 0.99);
-    location = glGetUniformLocation(binaryProgram.get(), "u_var[2].m0");
+    location = glGetUniformLocation(binaryProgram, "u_var[2].m0");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.53, 0.89, 0.01, 0.08);
-    location = glGetUniformLocation(binaryProgram.get(), "u_var[2].m1");
+    location = glGetUniformLocation(binaryProgram, "u_var[2].m1");
     ASSERT_NE(location, -1);
     glUniform4f(location, 0.26, 0.72, 0.60, 0.12);
     ASSERT_GL_NO_ERROR();
@@ -504,7 +674,7 @@ TEST_P(ProgramBinaryES3Test, TestArrayOfStructUniform)
     glClearColor(1.0, 0.0, 0.0, 1.0);
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
-    drawQuad(binaryProgram.get(), "position", 0.5f);
+    drawQuad(binaryProgram, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::white);
 }
@@ -645,10 +815,10 @@ TEST_P(ProgramBinaryES3Test, ActiveUniformShader)
     // Init and draw with the program.
     ANGLE_GL_PROGRAM(program, kVS, kFS);
 
-    GLint valuesLoc = glGetUniformLocation(program.get(), "values");
+    GLint valuesLoc = glGetUniformLocation(program, "values");
     ASSERT_NE(-1, valuesLoc);
 
-    glUseProgram(program.get());
+    glUseProgram(program);
     GLfloat values[2] = {0.5f, 1.0f};
     glUniform1fv(valuesLoc, 2, values);
     ASSERT_GL_NO_ERROR();
@@ -657,19 +827,19 @@ TEST_P(ProgramBinaryES3Test, ActiveUniformShader)
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
 
-    drawQuad(program.get(), "position", 0.5f);
+    drawQuad(program, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::green);
 
     // Read back the binary.
     GLint programLength = 0;
-    glGetProgramiv(program.get(), GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
     ASSERT_GL_NO_ERROR();
 
     GLsizei readLength  = 0;
     GLenum binaryFormat = GL_NONE;
     std::vector<uint8_t> binary(programLength);
-    glGetProgramBinary(program.get(), programLength, &readLength, &binaryFormat, binary.data());
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
     ASSERT_GL_NO_ERROR();
 
     EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
@@ -677,10 +847,10 @@ TEST_P(ProgramBinaryES3Test, ActiveUniformShader)
     // Load a new program with the binary and draw.
     ANGLE_GL_BINARY_ES3_PROGRAM(binaryProgram, binary, binaryFormat);
 
-    valuesLoc = glGetUniformLocation(program.get(), "values");
+    valuesLoc = glGetUniformLocation(program, "values");
     ASSERT_NE(-1, valuesLoc);
 
-    glUseProgram(binaryProgram.get());
+    glUseProgram(binaryProgram);
     GLfloat values2[2] = {0.1f, 1.0f};
     glUniform1fv(valuesLoc, 2, values2);
     ASSERT_GL_NO_ERROR();
@@ -689,7 +859,7 @@ TEST_P(ProgramBinaryES3Test, ActiveUniformShader)
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
 
-    drawQuad(binaryProgram.get(), "position", 0.5f);
+    drawQuad(binaryProgram, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::green);
 }
@@ -697,10 +867,6 @@ TEST_P(ProgramBinaryES3Test, ActiveUniformShader)
 // Test that uses many uniforms in the shaders
 TEST_P(ProgramBinaryES3Test, BinaryWithLargeUniformCount)
 {
-    // Suspecting AMD driver bug - failure seen on bots running on ATI GPU on Windows.
-    // http://anglebug.com/3721
-    ANGLE_SKIP_TEST_IF(IsAMD() && IsOpenGL() && IsWindows());
-
     ANGLE_SKIP_TEST_IF(getAvailableProgramBinaryFormatCount() == 0);
 
     constexpr char kVS[] =
@@ -746,41 +912,41 @@ TEST_P(ProgramBinaryES3Test, BinaryWithLargeUniformCount)
     GLuint bindIndex1   = 2;
 
     GLBuffer ubo0;
-    glBindBuffer(GL_UNIFORM_BUFFER, ubo0.get());
+    glBindBuffer(GL_UNIFORM_BUFFER, ubo0);
     glBufferData(GL_UNIFORM_BUFFER, sizeof(block0Data), &block0Data, GL_STATIC_DRAW);
-    glBindBufferRange(GL_UNIFORM_BUFFER, bindIndex0, ubo0.get(), 0, sizeof(block0Data));
+    glBindBufferRange(GL_UNIFORM_BUFFER, bindIndex0, ubo0, 0, sizeof(block0Data));
     ASSERT_GL_NO_ERROR();
 
     GLBuffer ubo1;
-    glBindBuffer(GL_UNIFORM_BUFFER, ubo1.get());
+    glBindBuffer(GL_UNIFORM_BUFFER, ubo1);
     glBufferData(GL_UNIFORM_BUFFER, sizeof(block1Data), &block1Data, GL_STATIC_DRAW);
-    glBindBufferRange(GL_UNIFORM_BUFFER, bindIndex1, ubo1.get(), 0, sizeof(block1Data));
+    glBindBufferRange(GL_UNIFORM_BUFFER, bindIndex1, ubo1, 0, sizeof(block1Data));
     ASSERT_GL_NO_ERROR();
 
-    GLint block0Index = glGetUniformBlockIndex(program.get(), "block0");
+    GLint block0Index = glGetUniformBlockIndex(program, "block0");
     ASSERT_NE(-1, block0Index);
 
-    GLint block1Index = glGetUniformBlockIndex(program.get(), "block1");
+    GLint block1Index = glGetUniformBlockIndex(program, "block1");
     ASSERT_NE(-1, block1Index);
 
-    glUniformBlockBinding(program.get(), block0Index, bindIndex0);
-    glUniformBlockBinding(program.get(), block1Index, bindIndex1);
+    glUniformBlockBinding(program, block0Index, bindIndex0);
+    glUniformBlockBinding(program, block1Index, bindIndex1);
     ASSERT_GL_NO_ERROR();
 
-    GLint redVSLoc = glGetUniformLocation(program.get(), "redVS");
+    GLint redVSLoc = glGetUniformLocation(program, "redVS");
     ASSERT_NE(-1, redVSLoc);
-    GLint greenVSLoc = glGetUniformLocation(program.get(), "greenVS");
+    GLint greenVSLoc = glGetUniformLocation(program, "greenVS");
     ASSERT_NE(-1, greenVSLoc);
-    GLint blueVSLoc = glGetUniformLocation(program.get(), "blueVS");
+    GLint blueVSLoc = glGetUniformLocation(program, "blueVS");
     ASSERT_NE(-1, blueVSLoc);
-    GLint redFSLoc = glGetUniformLocation(program.get(), "redFS");
+    GLint redFSLoc = glGetUniformLocation(program, "redFS");
     ASSERT_NE(-1, redFSLoc);
-    GLint greenFSLoc = glGetUniformLocation(program.get(), "greenFS");
+    GLint greenFSLoc = glGetUniformLocation(program, "greenFS");
     ASSERT_NE(-1, greenFSLoc);
-    GLint blueFSLoc = glGetUniformLocation(program.get(), "blueFS");
+    GLint blueFSLoc = glGetUniformLocation(program, "blueFS");
     ASSERT_NE(-1, blueFSLoc);
 
-    glUseProgram(program.get());
+    glUseProgram(program);
     glUniform1f(redVSLoc, 0.6f);
     glUniform1f(greenVSLoc, 0.2f);
     glUniform1f(blueVSLoc, 1.1f);
@@ -793,19 +959,19 @@ TEST_P(ProgramBinaryES3Test, BinaryWithLargeUniformCount)
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
 
-    drawQuad(program.get(), "position", 0.5f);
+    drawQuad(program, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::cyan);
 
     // Read back the binary.
     GLint programLength = 0;
-    glGetProgramiv(program.get(), GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH_OES, &programLength);
     ASSERT_GL_NO_ERROR();
 
     GLsizei readLength  = 0;
     GLenum binaryFormat = GL_NONE;
     std::vector<uint8_t> binary(programLength);
-    glGetProgramBinary(program.get(), programLength, &readLength, &binaryFormat, binary.data());
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
     ASSERT_GL_NO_ERROR();
 
     EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
@@ -813,20 +979,23 @@ TEST_P(ProgramBinaryES3Test, BinaryWithLargeUniformCount)
     // Load a new program with the binary and draw.
     ANGLE_GL_BINARY_ES3_PROGRAM(binaryProgram, binary, binaryFormat);
 
-    redVSLoc = glGetUniformLocation(program.get(), "redVS");
+    glUniformBlockBinding(binaryProgram, block0Index, bindIndex0);
+    glUniformBlockBinding(binaryProgram, block1Index, bindIndex1);
+
+    redVSLoc = glGetUniformLocation(binaryProgram, "redVS");
     ASSERT_NE(-1, redVSLoc);
-    greenVSLoc = glGetUniformLocation(program.get(), "greenVS");
+    greenVSLoc = glGetUniformLocation(binaryProgram, "greenVS");
     ASSERT_NE(-1, greenVSLoc);
-    blueVSLoc = glGetUniformLocation(program.get(), "blueVS");
+    blueVSLoc = glGetUniformLocation(binaryProgram, "blueVS");
     ASSERT_NE(-1, blueVSLoc);
-    redFSLoc = glGetUniformLocation(program.get(), "redFS");
+    redFSLoc = glGetUniformLocation(binaryProgram, "redFS");
     ASSERT_NE(-1, redFSLoc);
-    greenFSLoc = glGetUniformLocation(program.get(), "greenFS");
+    greenFSLoc = glGetUniformLocation(binaryProgram, "greenFS");
     ASSERT_NE(-1, greenFSLoc);
-    blueFSLoc = glGetUniformLocation(program.get(), "blueFS");
+    blueFSLoc = glGetUniformLocation(binaryProgram, "blueFS");
     ASSERT_NE(-1, blueFSLoc);
 
-    glUseProgram(binaryProgram.get());
+    glUseProgram(binaryProgram);
     glUniform1f(redVSLoc, 0.2f);
     glUniform1f(greenVSLoc, -0.6f);
     glUniform1f(blueVSLoc, 1.0f);
@@ -839,7 +1008,7 @@ TEST_P(ProgramBinaryES3Test, BinaryWithLargeUniformCount)
     glClear(GL_COLOR_BUFFER_BIT);
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::red);
 
-    drawQuad(binaryProgram.get(), "position", 0.5f);
+    drawQuad(binaryProgram, "position", 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, GLColor::magenta);
 }
@@ -856,10 +1025,10 @@ TEST_P(ProgramBinaryTest, SRGBDecodeWithSamplerAndTexelFetchTest)
                        getClientMajorVersion() < 3);
 
     // These OpenGL drivers appear not to respect the texelFetch exception
-    // http://anglebug.com/4991
+    // http://anglebug.com/42263564
     ANGLE_SKIP_TEST_IF(IsOpenGL() && IsIntel() && IsWindows());
     ANGLE_SKIP_TEST_IF(IsOpenGL() && IsAMD() && IsWindows());
-    ANGLE_SKIP_TEST_IF(IsOpenGL() && (IsNVIDIA() || IsARM64()) && IsOSX());
+    ANGLE_SKIP_TEST_IF(IsOpenGL() && (IsNVIDIA() || IsARM64()) && IsMac());
     ANGLE_SKIP_TEST_IF(IsOpenGLES() && IsNexus5X());
 
     constexpr char kVS[] =
@@ -886,10 +1055,10 @@ TEST_P(ProgramBinaryTest, SRGBDecodeWithSamplerAndTexelFetchTest)
 
     GLProgram program;
     program.makeRaster(kVS, kFS);
-    ASSERT_NE(0u, program.get());
+    ASSERT_NE(0u, program);
 
     GLuint reloadedProgram = glCreateProgram();
-    saveAndLoadProgram(program.get(), reloadedProgram);
+    saveAndLoadProgram(program, reloadedProgram);
 
     GLint textureLocation = glGetUniformLocation(reloadedProgram, "tex");
     ASSERT_NE(-1, textureLocation);
@@ -898,14 +1067,14 @@ TEST_P(ProgramBinaryTest, SRGBDecodeWithSamplerAndTexelFetchTest)
     GLColor srgbColor(13, 54, 133, 255);
 
     GLTexture tex;
-    glBindTexture(GL_TEXTURE_2D, tex.get());
+    glBindTexture(GL_TEXTURE_2D, tex);
     glTexImage2D(GL_TEXTURE_2D, 0, GL_SRGB8_ALPHA8, 1, 1, 0, GL_RGBA, GL_UNSIGNED_BYTE,
                  &linearColor);
     ASSERT_GL_NO_ERROR();
 
     GLSampler sampler;
-    glBindSampler(0, sampler.get());
-    glSamplerParameteri(sampler.get(), GL_TEXTURE_SRGB_DECODE_EXT, GL_DECODE_EXT);
+    glBindSampler(0, sampler);
+    glSamplerParameteri(sampler, GL_TEXTURE_SRGB_DECODE_EXT, GL_DECODE_EXT);
 
     glUseProgram(reloadedProgram);
     glUniform1i(textureLocation, 0);
@@ -915,7 +1084,7 @@ TEST_P(ProgramBinaryTest, SRGBDecodeWithSamplerAndTexelFetchTest)
 
     EXPECT_PIXEL_COLOR_NEAR(0, 0, srgbColor, 1.0);
 
-    glSamplerParameteri(sampler.get(), GL_TEXTURE_SRGB_DECODE_EXT, GL_SKIP_DECODE_EXT);
+    glSamplerParameteri(sampler, GL_TEXTURE_SRGB_DECODE_EXT, GL_SKIP_DECODE_EXT);
     drawQuad(reloadedProgram, "position", 0.5f);
 
     EXPECT_PIXEL_COLOR_NEAR(0, 0, srgbColor, 1.0);
@@ -940,7 +1109,7 @@ TEST_P(ProgramBinaryES3Test, ArrayOfStructContainingArrayOfSamplers)
         "}\n";
 
     ANGLE_GL_PROGRAM(program, essl1_shaders::vs::Simple(), kFS);
-    glUseProgram(program.get());
+    glUseProgram(program);
     GLTexture textures[4];
     GLColor expected = MakeGLColor(32, 64, 96, 255);
     GLubyte data[8]  = {};  // 4 bytes of padding, so that texture can be initialized with 4 bytes
@@ -956,7 +1125,7 @@ TEST_P(ProgramBinaryES3Test, ArrayOfStructContainingArrayOfSamplers)
         std::stringstream uniformName;
         uniformName << "test[" << innerIdx << "].data[" << outerIdx << "]";
         // Then send it as a uniform
-        GLint uniformLocation = glGetUniformLocation(program.get(), uniformName.str().c_str());
+        GLint uniformLocation = glGetUniformLocation(program, uniformName.str().c_str());
         // The uniform should be active.
         EXPECT_NE(uniformLocation, -1);
 
@@ -964,7 +1133,7 @@ TEST_P(ProgramBinaryES3Test, ArrayOfStructContainingArrayOfSamplers)
         ASSERT_GL_NO_ERROR();
     }
 
-    drawQuad(program.get(), essl1_shaders::PositionAttrib(), 0.5f);
+    drawQuad(program, essl1_shaders::PositionAttrib(), 0.5f);
     ASSERT_GL_NO_ERROR();
     EXPECT_PIXEL_COLOR_EQ(0, 0, expected);
 
@@ -1007,38 +1176,11 @@ TEST_P(ProgramBinaryES3Test, ArrayOfStructContainingArrayOfSamplers)
 GTEST_ALLOW_UNINSTANTIATED_PARAMETERIZED_TEST(ProgramBinaryES3Test);
 ANGLE_INSTANTIATE_TEST_ES3(ProgramBinaryES3Test);
 
-class ProgramBinaryES31Test : public ANGLETest<>
-{
-  protected:
-    ProgramBinaryES31Test()
-    {
-        setWindowWidth(128);
-        setWindowHeight(128);
-        setConfigRedBits(8);
-        setConfigGreenBits(8);
-        setConfigBlueBits(8);
-        setConfigAlphaBits(8);
-
-        // Test flakiness was noticed when reusing displays.
-        forceNewDisplay();
-    }
-
-    GLint getAvailableProgramBinaryFormatCount() const
-    {
-        GLint formatCount;
-        glGetIntegerv(GL_NUM_PROGRAM_BINARY_FORMATS_OES, &formatCount);
-        return formatCount;
-    }
-};
-
 // Tests that saving and loading a program attached with computer shader.
 TEST_P(ProgramBinaryES31Test, ProgramBinaryWithComputeShader)
 {
     ANGLE_SKIP_TEST_IF(getAvailableProgramBinaryFormatCount() == 0);
 
-    // http://anglebug.com/4092
-    ANGLE_SKIP_TEST_IF(IsVulkan());
-
     constexpr char kCS[] =
         "#version 310 es\n"
         "layout(local_size_x=4, local_size_y=3, local_size_z=1) in;\n"
@@ -1055,13 +1197,13 @@ TEST_P(ProgramBinaryES31Test, ProgramBinaryWithComputeShader)
 
     // Read back the binary.
     GLint programLength = 0;
-    glGetProgramiv(program.get(), GL_PROGRAM_BINARY_LENGTH, &programLength);
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH, &programLength);
     ASSERT_GL_NO_ERROR();
 
     GLsizei readLength  = 0;
     GLenum binaryFormat = GL_NONE;
     std::vector<uint8_t> binary(programLength);
-    glGetProgramBinary(program.get(), programLength, &readLength, &binaryFormat, binary.data());
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
     ASSERT_GL_NO_ERROR();
 
     EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
@@ -1071,7 +1213,7 @@ TEST_P(ProgramBinaryES31Test, ProgramBinaryWithComputeShader)
     ASSERT_GL_NO_ERROR();
 
     // Dispatch compute with the loaded binary program
-    glUseProgram(binaryProgram.get());
+    glUseProgram(binaryProgram);
     glDispatchCompute(8, 4, 2);
     ASSERT_GL_NO_ERROR();
 }
@@ -1196,13 +1338,13 @@ TEST_P(ProgramBinaryES31Test, ImageTextureBinding)
 
     // Read back the binary.
     GLint programLength = 0;
-    glGetProgramiv(program.get(), GL_PROGRAM_BINARY_LENGTH, &programLength);
+    glGetProgramiv(program, GL_PROGRAM_BINARY_LENGTH, &programLength);
     ASSERT_GL_NO_ERROR();
 
     GLsizei readLength  = 0;
     GLenum binaryFormat = GL_NONE;
     std::vector<uint8_t> binary(programLength);
-    glGetProgramBinary(program.get(), programLength, &readLength, &binaryFormat, binary.data());
+    glGetProgramBinary(program, programLength, &readLength, &binaryFormat, binary.data());
     ASSERT_GL_NO_ERROR();
 
     EXPECT_EQ(static_cast<GLsizei>(programLength), readLength);
@@ -1212,7 +1354,7 @@ TEST_P(ProgramBinaryES31Test, ImageTextureBinding)
     ASSERT_GL_NO_ERROR();
 
     // Dispatch compute with the loaded binary program
-    glUseProgram(binaryProgram.get());
+    glUseProgram(binaryProgram);
     GLTexture texture;
     glBindTexture(GL_TEXTURE_2D, texture);
     glTexStorage2D(GL_TEXTURE_2D, 1, GL_R32UI, 1, 1);
@@ -1302,9 +1444,9 @@ TEST_P(ProgramBinaryTransformFeedbackTest, GetTransformFeedbackVarying)
 
     ANGLE_SKIP_TEST_IF(getAvailableProgramBinaryFormatCount() == 0);
 
-    // http://anglebug.com/3690
+    // http://anglebug.com/42262347
     ANGLE_SKIP_TEST_IF(IsAndroid() && (IsPixel2() || IsPixel2XL()) && IsVulkan());
-    // http://anglebug.com/4092
+    // http://anglebug.com/40096654
     ANGLE_SKIP_TEST_IF(IsAndroid() && IsOpenGLES());
 
     std::vector<uint8_t> binary(0);
@@ -1412,7 +1554,7 @@ class ProgramBinariesAcrossPlatforms : public testing::TestWithParam<PlatformsWi
             EGLWindow::Delete(&eglWindow);
         }
 
-        angle::LoadGLES(eglGetProcAddress);
+        LoadUtilGLES(eglGetProcAddress);
 
         return eglWindow;
     }
