@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2023 Apple Inc. All rights reserved.
  * Copyright (C) 2007-2009 Torch Mobile, Inc.
  * Copyright (C) 2010, 2011 Research In Motion Limited. All rights reserved.
  *
@@ -70,7 +70,7 @@
 #define USE_GLIB 1
 #endif
 
-#if PLATFORM(GTK) || PLATFORM(WPE)
+#if ((PLATFORM(GTK) || PLATFORM(WPE)) && !USE(SKIA))
 #define USE_FREETYPE 1
 #endif
 
@@ -82,10 +82,6 @@
 #define USE_SOUP 1
 #endif
 
-#if PLATFORM(GTK) || PLATFORM(WPE)
-#define USE_WEBP 1
-#endif
-
 #if PLATFORM(COCOA)
 #define USE_CF 1
 #endif
@@ -102,7 +98,7 @@
 #define USE_APPKIT 1
 #endif
 
-#if PLATFORM(MAC)
+#if PLATFORM(MAC) || PLATFORM(IOS) || PLATFORM(VISION)
 #define USE_PASSKIT 1
 #endif
 
@@ -126,10 +122,6 @@
 #define USE_ACCELERATE 1
 #endif
 
-#if OS(WINDOWS)
-#define USE_SYSTEM_MALLOC 1
-#endif
-
 #if CPU(REGISTER64)
 #define USE_JSVALUE64 1
 #else
@@ -153,7 +145,7 @@
 
 /* FIXME: This name should be more specific if it is only for use with CallFrame* */
 /* Use __builtin_frame_address(1) to get CallFrame* */
-#if COMPILER(GCC_COMPATIBLE) && (CPU(ARM64) || CPU(X86_64))
+#if CPU(ARM64) || CPU(X86_64)
 #define USE_BUILTIN_FRAME_ADDRESS 1
 #endif
 
@@ -161,15 +153,11 @@
 #define USE_EXECUTE_ONLY_JIT_WRITE_FUNCTION 1
 #endif
 
-#if PLATFORM(IOS)
-#define USE_PASSKIT 1
-#endif
-
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define USE_QUICK_LOOK 1
 #endif
 
-#if PLATFORM(IOS)
+#if PLATFORM(IOS) || PLATFORM(VISION)
 #define USE_SYSTEM_PREVIEW 1
 #endif
 
@@ -240,12 +228,7 @@
 /* Use GLib's event loop abstraction. Primarily GTK port uses it. */
 #define USE_GLIB_EVENT_LOOP 1
 #elif OS(WINDOWS)
-/* Use Windows message pump abstraction.
- * Even if the port is AppleWin, we use the Windows message pump system for the event loop,
- * so that USE(WINDOWS_EVENT_LOOP) && USE(CF) can be true.
- * And PLATFORM(WIN) and PLATFORM(GTK) are exclusive. If the port is GTK,
- * PLATFORM(WIN) should be false. And in that case, GLib's event loop is used.
- */
+/* Use Windows message pump abstraction. */
 #define USE_WINDOWS_EVENT_LOOP 1
 #elif PLATFORM(COCOA)
 /* OS X and IOS. Use CoreFoundation & GCD abstraction. */
@@ -263,21 +246,12 @@
 #define USE_SOURCE_APPLICATION_AUDIT_DATA 1
 #endif
 
-#if PLATFORM(COCOA) && USE(CA)
-#define USE_IOSURFACE_CANVAS_BACKING_STORE 1
-#endif
-
 /* The override isn't needed on iOS family, as the default behavior is to not sniff. */
 #if PLATFORM(MAC)
 #define USE_CFNETWORK_CONTENT_ENCODING_SNIFFING_OVERRIDE 1
 #endif
 
-#if PLATFORM(MAC) || PLATFORM(WPE) || PLATFORM(GTK)
-/* FIXME: This really needs a descriptive name, this "new theme" was added in 2008. */
-#define USE_NEW_THEME 1
-#endif
-
-#if PLATFORM(IOS) || PLATFORM(MACCATALYST)
+#if PLATFORM(IOS) || PLATFORM(MACCATALYST) || PLATFORM(VISION) || PLATFORM(APPLETV)
 #define USE_UICONTEXTMENU 1
 #endif
 
@@ -293,40 +267,12 @@
 #define USE_DARWIN_REGISTER_MACROS 1
 #endif
 
-#if PLATFORM(COCOA) && !(PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 110000)
-#define USE_CTFONTSHAPEGLYPHS 1
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
+#define USE_NSIMAGE_FOR_SVG_SUPPORT 1
 #endif
 
-#if PLATFORM(COCOA) && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 110000)
-#define USE_CTFONTGETADVANCES_WORKAROUND 1
-#endif
-
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED < 150000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED < 80000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED < 150000)
-#define USE_CTFONTSHAPEGLYPHS_WORKAROUND 1
-#endif
-
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED < 150000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED < 80000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED < 150000)
-#define USE_NON_VARIABLE_SYSTEM_FONT 1
-#endif
-
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000) \
-    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 160000) \
-    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 90000) \
-    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 160000)
-#define USE_CTFONTHASTABLE 1
-#endif
-
-#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 110000) \
-    || (PLATFORM(MACCATALYST) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) \
-    || (PLATFORM(IOS) && PLATFORM(IOS_FAMILY_SIMULATOR) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 140000) \
-    || (PLATFORM(WATCHOS) && PLATFORM(IOS_FAMILY_SIMULATOR) && __WATCH_OS_VERSION_MIN_REQUIRED >= 70000) \
-    || (PLATFORM(APPLETV) && PLATFORM(IOS_FAMILY_SIMULATOR) && __TV_OS_VERSION_MIN_REQUIRED >= 140000)
+// Use OS(MACOS) to pick up JSCOnly ports on Mac.
+#if OS(MACOS) || PLATFORM(MACCATALYST) || (PLATFORM(IOS_FAMILY) && TARGET_OS_SIMULATOR)
 #if USE(APPLE_INTERNAL_SDK)
 /* Always use the macro on internal builds */
 #define USE_PTHREAD_JIT_PERMISSIONS_API 0 
@@ -336,28 +282,153 @@
 #endif
 
 #if PLATFORM(COCOA)
+#define USE_FONT_VARIANT_VIA_FEATURES 1
 #define USE_OPENXR 0
-#define USE_IOSURFACE_FOR_XR_LAYER_DATA 1
 #if !defined(HAVE_WEBXR_INTERNALS) && !HAVE(WEBXR_INTERNALS)
+#if PLATFORM(IOS) && HAVE(ARKIT)
+#define USE_ARKITXR_IOS 1
+#else
 #define USE_EMPTYXR 1
 #endif
 #endif
+#endif
+
+#if PLATFORM(COCOA)
+#define USE_CG_CONTEXT_STROKE_LINE_SEGMENTS_WHEN_STROKING_PATH 1
+#endif
 
 #if PLATFORM(IOS_FAMILY)
 #define USE_SANDBOX_EXTENSIONS_FOR_CACHE_AND_TEMP_DIRECTORY_ACCESS 1
 #endif
 
-#if !defined(USE_LIBPAS_JIT_HEAP) && !USE(SYSTEM_MALLOC)
-#include <bmalloc/BPlatform.h>
-#if BENABLE(LIBPAS) && (OS(DARWIN) || OS(LINUX))
-#define USE_LIBPAS_JIT_HEAP 1
+#if !defined(USE_TZONE_MALLOC)
+#if (CPU(ARM64) || CPU(X86_64)) && OS(DARWIN) && (__SIZEOF_POINTER__ == 8)
+#define USE_TZONE_MALLOC 1
+#define USE_ISO_MALLOC 0
+#else
+#define USE_TZONE_MALLOC 0
 #endif
 #endif
 
-#if PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 120000
-#define USE_VORBIS_AUDIOCOMPONENT_WORKAROUND 1
-#endif
-
 #if !defined(USE_ISO_MALLOC)
 #define USE_ISO_MALLOC 1
 #endif
+
+#if !PLATFORM(WATCHOS)
+#define USE_GLYPH_DISPLAY_LIST_CACHE 1
+#endif
+
+#if PLATFORM(IOS_FAMILY) || (USE(APPLE_INTERNAL_SDK) && PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130300)
+#define USE_RUNNINGBOARD 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 130000)
+#define USE_NSVIEW_SEMANTICCONTEXT 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+#define USE_NSPRESENTATIONSTATE 1
+#endif
+
+#if !defined(USE_MEDIAPARSERD) \
+    && (PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) || PLATFORM(VISION)
+#define USE_MEDIAPARSERD 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED < 130000)
+#define USE_AVIF 1
+#endif
+
+#if PLATFORM(COCOA) && (HAVE(CGSTYLE_CREATE_SHADOW2) || HAVE(CGSTYLE_COLORMATRIX_BLUR))
+#define USE_GRAPHICS_CONTEXT_FILTERS 1
+#endif
+
+#if PLATFORM(COCOA) || USE(GSTREAMER)
+#define USE_CONCATENATED_IMPULSE_RESPONSES 1
+#endif
+
+#if PLATFORM(IOS_FAMILY) && !PLATFORM(APPLETV)
+#define USE_UITOOLBAR_FOR_DATE_PICKER_ACCESSORY_VIEW 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 150000) \
+    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED < 180000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED < 180000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED < 110000)
+// We can delete this when rdar://problem/104370451 is fixed.
+#define USE_CORE_TEXT_OPTICAL_SIZING_WORKAROUND 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 140000) \
+    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED < 170000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED < 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED < 100000)
+#define USE_METAL_ARGUMENT_ACCESS_ENUMS 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 150000) \
+    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED < 180000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED < 180000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED < 110000)
+#define USE_PKSHIPPINGCONTACTEDITINGMODEENABLED 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 140000) \
+    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED < 170000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED < 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED < 100000)
+#define USE_VISION_CPU_ONLY_PROPERTY 1
+#endif
+
+#if !defined(USE_NSSPELLCHECKER_GRAMMAR_CHECKING_POLICY) \
+    && (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000)
+#define USE_NSSPELLCHECKER_GRAMMAR_CHECKING_POLICY 1
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 140000) \
+    || (PLATFORM(IOS_FAMILY) && __IPHONE_OS_VERSION_MAX_ALLOWED < 170000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MAX_ALLOWED < 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MAX_ALLOWED < 100000)
+#define USE_CORE_TEXT_VARIATIONS_CLAMPING_WORKAROUND 1
+#endif
+
+#if PLATFORM(IOS) && !PLATFORM(IOS_FAMILY_SIMULATOR) \
+    && __IPHONE_OS_VERSION_MAX_ALLOWED >= 170400
+#if CPU(ARM64)
+#define USE_INLINE_JIT_PERMISSIONS_API 1
+#endif
+#endif
+
+#if (PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140000) \
+    || ((PLATFORM(IOS) || PLATFORM(MACCATALYST)) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || (PLATFORM(WATCHOS) && __WATCH_OS_VERSION_MIN_REQUIRED >= 100000) \
+    || (PLATFORM(APPLETV) && __TV_OS_VERSION_MIN_REQUIRED >= 170000) \
+    || PLATFORM(VISION)
+#define USE_SANDBOX_VERSION_3 1
+#endif
+
+#if !defined(USE_EXTENSIONKIT) && PLATFORM(IOS) && __IPHONE_OS_VERSION_MAX_ALLOWED >= 170400
+#define USE_EXTENSIONKIT 1
+#endif
+
+#if !defined(USE_BROWSERENGINEKIT) && PLATFORM(IOS) && __has_include(<BrowserEngineKit/BETextInput.h>)
+#define USE_BROWSERENGINEKIT 1
+#endif
+
+#if !defined(USE_LEGACY_EXTENSIONKIT_SPI) && PLATFORM(IOS_SIMULATOR) && __IPHONE_OS_VERSION_MAX_ALLOWED <= 170400
+#define USE_LEGACY_EXTENSIONKIT_SPI 1
+#endif
+
+#if !USE(EXTENSIONKIT_PROCESS_TERMINATION) && PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED >= 180000
+#define USE_EXTENSIONKIT_PROCESS_TERMINATION 1
+#endif
+
+#if !USE(EXIT_XPC_MESSAGE_WORKAROUND) && ((PLATFORM(IOS) && __IPHONE_OS_VERSION_MIN_REQUIRED < 180000) || (PLATFORM(MAC) && __MAC_OS_X_VERSION_MAX_ALLOWED < 150000))
+#define USE_EXIT_XPC_MESSAGE_WORKAROUND 1
+#endif
+
+#if !defined(USE_MODERN_AVCONTENTKEYSESSION) \
+    && ((PLATFORM(MAC) && __MAC_OS_X_VERSION_MIN_REQUIRED >= 140400) \
+    || (PLATFORM(IOS_FAMILY) && !PLATFORM(IOS_FAMILY_SIMULATOR)))
+#define USE_MODERN_AVCONTENTKEYSESSION 1
+#endif
