@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012-2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2012-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -30,10 +30,12 @@
 #include <wtf/text/CString.h>
 #include <wtf/text/WTFString.h>
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN
+
 namespace WTF {
 
-PrintStream::PrintStream() { }
-PrintStream::~PrintStream() { } // Force the vtable to be in this module
+PrintStream::PrintStream() = default;
+PrintStream::~PrintStream() = default; // Force the vtable to be in this module
 
 void PrintStream::printf(const char* format, ...)
 {
@@ -45,14 +47,14 @@ void PrintStream::printf(const char* format, ...)
 
 void PrintStream::printfVariableFormat(const char* format, ...)
 {
-    ALLOW_NONLITERAL_FORMAT_BEGIN
-    IGNORE_GCC_WARNINGS_BEGIN("suggest-attribute=format")
+ALLOW_NONLITERAL_FORMAT_BEGIN
+IGNORE_GCC_WARNINGS_BEGIN("suggest-attribute=format")
     va_list argList;
     va_start(argList, format);
     vprintf(format, argList);
     va_end(argList);
-    IGNORE_GCC_WARNINGS_END
-    ALLOW_NONLITERAL_FORMAT_END
+IGNORE_GCC_WARNINGS_END
+ALLOW_NONLITERAL_FORMAT_END
 }
 
 void PrintStream::flush()
@@ -92,7 +94,7 @@ static void printExpectedCStringHelper(PrintStream& out, const char* type, Expec
 
 void printInternal(PrintStream& out, StringView string)
 {
-    printExpectedCStringHelper(out, "StringView", string.tryGetUtf8());
+    printExpectedCStringHelper(out, "StringView", string.tryGetUTF8());
 }
 
 void printInternal(PrintStream& out, const CString& string)
@@ -102,12 +104,12 @@ void printInternal(PrintStream& out, const CString& string)
 
 void printInternal(PrintStream& out, const String& string)
 {
-    printExpectedCStringHelper(out, "String", string.tryGetUtf8());
+    printExpectedCStringHelper(out, "String", string.tryGetUTF8());
 }
 
 void printInternal(PrintStream& out, const AtomString& string)
 {
-    printExpectedCStringHelper(out, "String", string.string().tryGetUtf8());
+    printExpectedCStringHelper(out, "String", string.string().tryGetUTF8());
 }
 
 void printInternal(PrintStream& out, const StringImpl* string)
@@ -116,7 +118,7 @@ void printInternal(PrintStream& out, const StringImpl* string)
         printInternal(out, "(null StringImpl*)");
         return;
     }
-    printExpectedCStringHelper(out, "StringImpl*", string->tryGetUtf8());
+    printExpectedCStringHelper(out, "StringImpl*", string->tryGetUTF8());
 }
 
 void printInternal(PrintStream& out, bool value)
@@ -144,6 +146,17 @@ void printInternal(PrintStream& out, unsigned char value)
     out.printf("%u", static_cast<unsigned>(value));
 }
 
+void printInternal(PrintStream& out, char16_t value)
+{
+    out.printf("%lc", static_cast<wint_t>(value));
+}
+
+void printInternal(PrintStream& out, char32_t value)
+{
+    // Print each char32_t as an integer.
+    out.printf("%u", static_cast<unsigned>(value));
+}
+
 void printInternal(PrintStream& out, short value)
 {
     out.printf("%d", static_cast<int>(value));
@@ -192,12 +205,20 @@ void printInternal(PrintStream& out, RawHex value)
         return;
     }
 #endif
+#if OS(WINDOWS)
+    out.printf("0x%p", value.ptr());
+#else
     out.printf("%p", value.ptr());
+#endif
 }
 
 void printInternal(PrintStream& out, RawPointer value)
 {
+#if OS(WINDOWS)
+    out.printf("0x%p", value.value());
+#else
     out.printf("%p", value.value());
+#endif
 }
 
 void printInternal(PrintStream& out, FixedWidthDouble value)
@@ -212,3 +233,4 @@ void dumpCharacter(PrintStream& out, char value)
 
 } // namespace WTF
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
