@@ -67,6 +67,19 @@ using GUniquePtr = std::unique_ptr<T, U>;
 FOR_EACH_GLIB_DELETER(WTF_DEFINE_GPTR_DELETER)
 #undef FOR_EACH_GLIB_DELETER
 
+#define WTF_DEFINE_DEPRECATED_GPTR_DELETER(typeName, deleterFunc) \
+    template<> struct GPtrDeleter<typeName> { \
+        void operator()(typeName* ptr) const \
+        { \
+            ALLOW_DEPRECATED_DECLARATIONS_BEGIN; \
+            deleterFunc(ptr); \
+            ALLOW_DEPRECATED_DECLARATIONS_END; \
+        } \
+    };
+
+WTF_DEFINE_DEPRECATED_GPTR_DELETER(GValueArray, g_value_array_free)
+#undef WTF_DEFINE_DEPRECATED_GPTR_DELETER
+
 template <typename T> class GUniqueOutPtr {
     WTF_MAKE_NONCOPYABLE(GUniqueOutPtr);
 public:
