@@ -26,6 +26,7 @@
 #pragma once
 
 #include <wtf/Forward.h>
+#include <wtf/HexNumber.h>
 #include <wtf/Markable.h>
 #include <wtf/OptionSet.h>
 #include <wtf/Ref.h>
@@ -60,6 +61,8 @@ public:
     {
     }
 
+    bool isEmpty() const { return m_text.isEmpty(); }
+
     WTF_EXPORT_PRIVATE TextStream& operator<<(bool);
     WTF_EXPORT_PRIVATE TextStream& operator<<(char);
     WTF_EXPORT_PRIVATE TextStream& operator<<(int);
@@ -74,17 +77,17 @@ public:
     WTF_EXPORT_PRIVATE TextStream& operator<<(const char*);
     WTF_EXPORT_PRIVATE TextStream& operator<<(const void*);
     WTF_EXPORT_PRIVATE TextStream& operator<<(const AtomString&);
+    WTF_EXPORT_PRIVATE TextStream& operator<<(const CString&);
     WTF_EXPORT_PRIVATE TextStream& operator<<(const String&);
     WTF_EXPORT_PRIVATE TextStream& operator<<(ASCIILiteral);
     WTF_EXPORT_PRIVATE TextStream& operator<<(StringView);
+    WTF_EXPORT_PRIVATE TextStream& operator<<(const HexNumberBuffer&);
+    WTF_EXPORT_PRIVATE TextStream& operator<<(const FormattedCSSNumber&);
     // Deprecated. Use the NumberRespectingIntegers FormattingFlag instead.
     WTF_EXPORT_PRIVATE TextStream& operator<<(const FormatNumberRespectingIntegers&);
 
 #if PLATFORM(COCOA)
     WTF_EXPORT_PRIVATE TextStream& operator<<(id);
-#ifdef __OBJC__
-    WTF_EXPORT_PRIVATE TextStream& operator<<(NSArray *);
-#endif
 #endif
 
     OptionSet<Formatting> formattingFlags() const { return m_formattingFlags; }
@@ -110,6 +113,15 @@ public:
         ts.endGroup();
     }
 
+    template<typename T>
+    void dumpProperty(ASCIILiteral name, const T& value)
+    {
+        TextStream& ts = *this;
+        ts.startGroup();
+        ts << name << " "_s << value;
+        ts.endGroup();
+    }
+
     WTF_EXPORT_PRIVATE String release();
     
     WTF_EXPORT_PRIVATE void startGroup();
@@ -232,13 +244,13 @@ TextStream& operator<<(TextStream& ts, const Markable<T, Traits>& item)
     return ts << "unset";
 }
 
-template<typename ItemType, size_t inlineCapacity>
-TextStream& operator<<(TextStream& ts, const Vector<ItemType, inlineCapacity>& vector)
+template<typename SizedContainer>
+TextStream& streamSizedContainer(TextStream& ts, const SizedContainer& sizedContainer)
 {
     ts << "[";
 
     unsigned count = 0;
-    for (const auto& value : vector) {
+    for (const auto& value : sizedContainer) {
         if (count)
             ts << ", ";
         ts << value;
@@ -246,14 +258,38 @@ TextStream& operator<<(TextStream& ts, const Vector<ItemType, inlineCapacity>& v
             break;
     }
 
-    if (count != vector.size())
+    if (count != sizedContainer.size())
         ts << ", ...";
 
     return ts << "]";
 }
 
-template<typename T>
-TextStream& operator<<(TextStream& ts, const WeakPtr<T>& item)
+template<typename ItemType, size_t inlineCapacity>
+TextStream& operator<<(TextStream& ts, const Vector<ItemType, inlineCapacity>& vector)
+{
+    return streamSizedContainer(ts, vector);
+}
+
+template<typename ItemType>
+TextStream& operator<<(TextStream& ts, const FixedVector<ItemType>& vector)
+{
+    return streamSizedContainer(ts, vector);
+}
+
+template<typename ValueArg, typename HashArg, typename TraitsArg, typename TableTraitsArg, ShouldValidateKey shouldValidateKey>
+TextStream& operator<<(TextStream& ts, const HashSet<ValueArg, HashArg, TraitsArg, TableTraitsArg, shouldValidateKey>& set)
+{
+    return streamSizedContainer(ts, set);
+}
+
+template<typename T, size_t size>
+TextStream& operator<<(TextStream& ts, const std::array<T, size>& array)
+{
+    return streamSizedContainer(ts, array);
+}
+
+template<typename T, typename Counter>
+TextStream& operator<<(TextStream& ts, const WeakPtr<T, Counter>& item)
 {
     if (item)
         return ts << *item;
@@ -276,8 +312,17 @@ TextStream& operator<<(TextStream& ts, const Ref<T>& item)
     return ts << item.get();
 }
 
+template<typename T>
+TextStream& operator<<(TextStream& ts, const CheckedPtr<T>& item)
+{
+    if (item)
+        return ts << *item;
+
+    return ts << "null";
+}
+
 template<typename KeyArg, typename MappedArg, typename HashArg, typename KeyTraitsArg, typename MappedTraitsArg>
-TextStream& operator<<(TextStream& ts, const HashMap<KeyArg, MappedArg, HashArg, KeyTraitsArg, MappedTraitsArg>& map)
+TextStream& operator<<(TextStream& ts, const UncheckedKeyHashMap<KeyArg, MappedArg, HashArg, KeyTraitsArg, MappedTraitsArg>& map)
 {
     ts << "{";
 
@@ -296,26 +341,6 @@ TextStream& operator<<(TextStream& ts, const HashMap<KeyArg, MappedArg, HashArg,
     return ts << "}";
 }
 
-template<typename ValueArg, typename HashArg, typename TraitsArg>
-TextStream& operator<<(TextStream& ts, const HashSet<ValueArg, HashArg, TraitsArg>& set)
-{
-    ts << "[";
-
-    unsigned count = 0;
-    for (const auto& item : set) {
-        if (count)
-            ts << ", ";
-        ts << item;
-        if (++count == ts.containerSizeLimit())
-            break;
-    }
-
-    if (count != set.size())
-        ts << ", ...";
-
-    return ts << "]";
-}
-
 template<typename Option>
 TextStream& operator<<(TextStream& ts, const OptionSet<Option>& options)
 {
@@ -330,6 +355,12 @@ TextStream& operator<<(TextStream& ts, const OptionSet<Option>& options)
     return ts << "]";
 }
 
+template<typename T, typename U>
+TextStream& operator<<(TextStream& ts, const std::pair<T, U>& pair)
+{
+    return ts << "[" << pair.first << ", " << pair.second << "]";
+}
+
 template<typename, typename = void, typename = void, typename = void, typename = void, size_t = 0>
 struct supports_text_stream_insertion : std::false_type { };
 
@@ -343,7 +374,7 @@ template<typename ValueArg, typename HashArg, typename TraitsArg>
 struct supports_text_stream_insertion<HashSet<ValueArg, HashArg, TraitsArg>> : supports_text_stream_insertion<ValueArg> { };
 
 template<typename KeyArg, typename MappedArg, typename HashArg, typename KeyTraitsArg, typename MappedTraitsArg>
-struct supports_text_stream_insertion<HashMap<KeyArg, MappedArg, HashArg, KeyTraitsArg, MappedTraitsArg>> : std::conjunction<supports_text_stream_insertion<KeyArg>, supports_text_stream_insertion<MappedArg>> { };
+struct supports_text_stream_insertion<UncheckedKeyHashMap<KeyArg, MappedArg, HashArg, KeyTraitsArg, MappedTraitsArg>> : std::conjunction<supports_text_stream_insertion<KeyArg>, supports_text_stream_insertion<MappedArg>> { };
 
 template<typename T, typename Traits>
 struct supports_text_stream_insertion<Markable<T, Traits>> : supports_text_stream_insertion<T> { };
@@ -354,8 +385,8 @@ struct supports_text_stream_insertion<OptionSet<T>> : supports_text_stream_inser
 template<typename T>
 struct supports_text_stream_insertion<std::optional<T>> : supports_text_stream_insertion<T> { };
 
-template<typename T>
-struct supports_text_stream_insertion<WeakPtr<T>> : supports_text_stream_insertion<T> { };
+template<typename T, typename Counter>
+struct supports_text_stream_insertion<WeakPtr<T, Counter>> : supports_text_stream_insertion<T> { };
 
 template<typename T>
 struct supports_text_stream_insertion<RefPtr<T>> : supports_text_stream_insertion<T> { };
@@ -363,6 +394,12 @@ struct supports_text_stream_insertion<RefPtr<T>> : supports_text_stream_insertio
 template<typename T>
 struct supports_text_stream_insertion<Ref<T>> : supports_text_stream_insertion<T> { };
 
+template<typename T, size_t size>
+struct supports_text_stream_insertion<std::array<T, size>> : supports_text_stream_insertion<T> { };
+
+template<typename T, typename U>
+struct supports_text_stream_insertion<std::pair<T, U>> : std::conjunction<supports_text_stream_insertion<T>, supports_text_stream_insertion<U>> { };
+
 template<typename T>
 struct ValueOrEllipsis {
     explicit ValueOrEllipsis(const T& value)
