@@ -25,9 +25,10 @@
 
 #pragma once
 
+#include "ExceptionOr.h"
 #include <wtf/KeyValuePair.h>
 #include <wtf/Ref.h>
-#include <wtf/RefCounted.h>
+#include <wtf/RefCountedAndCanMakeWeakPtr.h>
 #include <wtf/Vector.h>
 #include <wtf/WeakPtr.h>
 #include <wtf/text/WTFString.h>
@@ -39,12 +40,13 @@ class Clipboard;
 class ClipboardItemDataSource;
 class DeferredPromise;
 class DOMPromise;
+class WeakPtrImplWithEventTargetData;
 class Navigator;
 class PasteboardCustomData;
 class ScriptExecutionContext;
 struct PasteboardItemInfo;
 
-class ClipboardItem : public RefCounted<ClipboardItem> {
+class ClipboardItem : public RefCountedAndCanMakeWeakPtr<ClipboardItem> {
 public:
     ~ClipboardItem();
 
@@ -54,12 +56,13 @@ public:
         PresentationStyle presentationStyle { PresentationStyle::Unspecified };
     };
 
-    static Ref<ClipboardItem> create(Vector<KeyValuePair<String, RefPtr<DOMPromise>>>&&, const Options&);
+    static ExceptionOr<Ref<ClipboardItem>> create(Vector<KeyValuePair<String, Ref<DOMPromise>>>&&, const Options&);
     static Ref<ClipboardItem> create(Clipboard&, const PasteboardItemInfo&);
     static Ref<Blob> blobFromString(ScriptExecutionContext*, const String& stringData, const String& type);
 
     Vector<String> types() const;
     void getType(const String&, Ref<DeferredPromise>&&);
+    static bool supports(const String& type);
 
     void collectDataForWriting(Clipboard& destination, CompletionHandler<void(std::optional<PasteboardCustomData>)>&&);
 
@@ -68,10 +71,10 @@ public:
     Clipboard* clipboard();
 
 private:
-    ClipboardItem(Vector<KeyValuePair<String, RefPtr<DOMPromise>>>&&, const Options&);
+    ClipboardItem(Vector<KeyValuePair<String, Ref<DOMPromise>>>&&, const Options&);
     ClipboardItem(Clipboard&, const PasteboardItemInfo&);
 
-    WeakPtr<Clipboard> m_clipboard;
+    WeakPtr<Clipboard, WeakPtrImplWithEventTargetData> m_clipboard;
     WeakPtr<Navigator> m_navigator;
     std::unique_ptr<ClipboardItemDataSource> m_dataSource;
     PresentationStyle m_presentationStyle { PresentationStyle::Unspecified };
