@@ -31,11 +31,11 @@
 namespace WebCore {
 
 class IDBVersionChangeEvent final : public Event {
-    WTF_MAKE_ISO_ALLOCATED(IDBVersionChangeEvent);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(IDBVersionChangeEvent);
 public:
     static Ref<IDBVersionChangeEvent> create(uint64_t oldVersion, uint64_t newVersion, const AtomString& eventType)
     {
-        return adoptRef(*new IDBVersionChangeEvent(IDBResourceIdentifier::emptyValue(), oldVersion, newVersion, eventType));
+        return adoptRef(*new IDBVersionChangeEvent(std::nullopt, oldVersion, newVersion, eventType));
     }
 
     static Ref<IDBVersionChangeEvent> create(const IDBResourceIdentifier& requestIdentifier, uint64_t oldVersion, uint64_t newVersion, const AtomString& eventType)
@@ -53,7 +53,7 @@ public:
         return adoptRef(*new IDBVersionChangeEvent(type, initializer, isTrusted));
     }
 
-    const IDBResourceIdentifier& requestIdentifier() const { return m_requestIdentifier; }
+    std::optional<IDBResourceIdentifier> requestIdentifier() const { return m_requestIdentifier; }
 
     bool isVersionChangeEvent() const final { return true; }
 
@@ -61,12 +61,10 @@ public:
     std::optional<uint64_t> newVersion() const { return m_newVersion; }
 
 private:
-    IDBVersionChangeEvent(const IDBResourceIdentifier& requestIdentifier, uint64_t oldVersion, uint64_t newVersion, const AtomString& eventType);
+    IDBVersionChangeEvent(std::optional<IDBResourceIdentifier> requestIdentifier, uint64_t oldVersion, uint64_t newVersion, const AtomString& eventType);
     IDBVersionChangeEvent(const AtomString&, const Init&, IsTrusted);
 
-    EventInterface eventInterface() const;
-
-    IDBResourceIdentifier m_requestIdentifier;
+    std::optional<IDBResourceIdentifier> m_requestIdentifier;
     uint64_t m_oldVersion;
     std::optional<uint64_t> m_newVersion;
 };
