@@ -28,35 +28,50 @@
 
 #if ENABLE(WIRELESS_PLAYBACK_TARGET)
 
+#include "ElementInlines.h"
 #include "Event.h"
 #include "EventNames.h"
 #include "HTMLMediaElement.h"
 #include "JSDOMPromiseDeferred.h"
+#include "JSNodeCustom.h"
 #include "Logging.h"
 #include "MediaElementSession.h"
 #include "MediaPlaybackTarget.h"
 #include "RemotePlaybackAvailabilityCallback.h"
-#include <wtf/IsoMallocInlines.h>
+#include "WebCoreOpaqueRootInlines.h"
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(RemotePlayback);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(RemotePlayback);
 
 Ref<RemotePlayback> RemotePlayback::create(HTMLMediaElement& element)
 {
-    return adoptRef(*new RemotePlayback(element));
+    auto remotePlayback = adoptRef(*new RemotePlayback(element));
+    remotePlayback->suspendIfNeeded();
+    return remotePlayback;
 }
 
 RemotePlayback::RemotePlayback(HTMLMediaElement& element)
     : WebCore::ActiveDOMObject(element.scriptExecutionContext())
-    , m_mediaElement(makeWeakPtr(element))
-    , m_eventQueue(MainThreadGenericEventQueue::create(*this))
+#if !RELEASE_LOG_DISABLED
+    , m_logger(element.logger())
+    , m_logIdentifier(element.logIdentifier())
+#endif
+    , m_mediaElement(element)
 {
-    suspendIfNeeded();
 }
 
-RemotePlayback::~RemotePlayback()
+RemotePlayback::~RemotePlayback() = default;
+
+WebCoreOpaqueRoot RemotePlayback::opaqueRootConcurrently() const
 {
+    return root(m_mediaElement.get());
+}
+
+Node* RemotePlayback::ownerNode() const
+{
+    return m_mediaElement.get();
 }
 
 void RemotePlayback::watchAvailability(Ref<RemotePlaybackAvailabilityCallback>&& callback, Ref<DeferredPromise>&& promise)
@@ -67,15 +82,19 @@ void RemotePlayback::watchAvailability(Ref<RemotePlaybackAvailabilityCallback>&&
 
     // 1. Let promise be a new promise->
     // 2. Return promise, and run the following steps below:
-    
-    m_taskQueue.enqueueTask([this, callback = WTFMove(callback), promise = WTFMove(promise)] () mutable {
+
+    auto identifier = LOGIDENTIFIER;
+    ALWAYS_LOG(identifier);
+
+    queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, callback = WTFMove(callback), promise = WTFMove(promise), identifier = identifier] () mutable {
+        if (isContextStopped())
+            return;
+
         // 3. If the disableRemotePlayback attribute is present for the media element, reject the promise with
         //    InvalidStateError and abort all the remaining steps.
-        if (!m_mediaElement
-            || m_mediaElement->hasAttributeWithoutSynchronization(HTMLNames::webkitwirelessvideoplaybackdisabledAttr)
-            || m_mediaElement->hasAttributeWithoutSynchronization(HTMLNames::disableremoteplaybackAttr)) {
-            WTFLogAlways("RemotePlayback::watchAvailability()::task - promise rejected");
-            promise->reject(InvalidStateError);
+        if (!m_mediaElement || m_mediaElement->isWirelessPlaybackTargetDisabled()) {
+            ERROR_LOG(identifier, "promise rejected, remote playback disabled");
+            promise->reject(ExceptionCode::InvalidStateError);
             return;
         }
 
@@ -94,25 +113,29 @@ void RemotePlayback::watchAvailability(Ref<RemotePlaybackAvailabilityCallback>&&
         m_callbackMap.add(callbackId, WTFMove(callback));
 
         // 8. Fulfill promise with the callbackId and run the following steps in parallel:
-        promise->whenSettled([this, protectedThis = makeRefPtr(this), callbackId] {
+        promise->whenSettled([this, protectedThis = Ref { *this }, callbackId] {
             // 8.1 Queue a task to invoke the callback with the current availability for the media element.
-            m_taskQueue.enqueueTask([this, callbackId] {
+            queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, callbackId, available = m_available] {
+                if (isContextStopped())
+                    return;
                 auto foundCallback = m_callbackMap.find(callbackId);
                 if (foundCallback == m_callbackMap.end())
                     return;
 
-                if (updateAvailability() == UpdateResults::Unchanged)
-                    foundCallback->value->handleEvent(m_available);
+                foundCallback->value->handleEvent(available);
             });
 
             // 8.2 Run the algorithm to monitor the list of available remote playback devices.
-            m_mediaElement->remoteHasAvailabilityCallbacksChanged();
+            if (m_mediaElement) {
+                availabilityChanged(m_mediaElement->mediaSession().hasWirelessPlaybackTargets());
+                m_mediaElement->remoteHasAvailabilityCallbacksChanged();
+            }
         });
         promise->resolve<IDLLong>(callbackId);
     });
 }
 
-void RemotePlayback::cancelWatchAvailability(Optional<int32_t> id, Ref<DeferredPromise>&& promise)
+void RemotePlayback::cancelWatchAvailability(std::optional<int32_t> id, Ref<DeferredPromise>&& promise)
 {
     // 6.2.1.5 Stop observing remote playback devices availability
     // https://w3c.github.io/remote-playback/#stop-observing-remote-playback-devices-availability
@@ -121,27 +144,36 @@ void RemotePlayback::cancelWatchAvailability(Optional<int32_t> id, Ref<DeferredP
     // 1. Let promise be a new promise->
     // 2. Return promise, and run the following steps below:
 
-    m_taskQueue.enqueueTask([this, id = WTFMove(id), promise = WTFMove(promise)] {
+    auto identifier = LOGIDENTIFIER;
+    ALWAYS_LOG(identifier);
+
+    queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, id = WTFMove(id), promise = WTFMove(promise), identifier = identifier] {
+        if (isContextStopped())
+            return;
         // 3. If the disableRemotePlayback attribute is present for the media element, reject promise with
         //    InvalidStateError and abort all the remaining steps.
-        if (!m_mediaElement
-            || m_mediaElement->hasAttributeWithoutSynchronization(HTMLNames::webkitwirelessvideoplaybackdisabledAttr)
-            || m_mediaElement->hasAttributeWithoutSynchronization(HTMLNames::disableremoteplaybackAttr)) {
-            promise->reject(InvalidStateError);
+        if (!m_mediaElement || m_mediaElement->isWirelessPlaybackTargetDisabled()) {
+            ERROR_LOG(identifier, "promise rejected, remote playback disabled");
+            promise->reject(ExceptionCode::InvalidStateError);
             return;
         }
 
         // 4. If the parameter id is undefined, clear the set of availability callbacks.
         if (!id)
             m_callbackMap.clear();
-        else {
+        else if (!decltype(m_callbackMap)::isValidKey(*id)) {
+            ERROR_LOG(identifier, "promise rejected, invalid identifier");
+            promise->reject(ExceptionCode::NotFoundError);
+            return;
+        } else {
             // 5. Otherwise, if id matches the callbackId for any entry in the set of availability callbacks,
             //    remove the entry from the set.
             if (auto it = m_callbackMap.find(id.value()) != m_callbackMap.end())
                 m_callbackMap.remove(it);
             // 6. Otherwise, reject promise with NotFoundError and abort all the remaining steps.
             else {
-                promise->reject(NotFoundError);
+                ERROR_LOG(identifier, "promise rejected, no matching callback");
+                promise->reject(ExceptionCode::NotFoundError);
                 return;
             }
         }
@@ -164,13 +196,18 @@ void RemotePlayback::prompt(Ref<DeferredPromise>&& promise)
     // 1. Let promise be a new promise->
     // 2. Return promise, and run the following steps below:
 
-    m_taskQueue.enqueueTask([this, promise = WTFMove(promise), processingUserGesture = UserGestureIndicator::processingUserGesture()] () mutable {
+    auto identifier = LOGIDENTIFIER;
+    ALWAYS_LOG(identifier);
+
+    queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, promise = WTFMove(promise), processingUserGesture = UserGestureIndicator::processingUserGesture(), identifier = identifier] () mutable {
+        if (isContextStopped())
+            return;
+
         // 3. If the disableRemotePlayback attribute is present for the media element, reject the promise with
         //    InvalidStateError and abort all the remaining steps.
-        if (!m_mediaElement
-            || m_mediaElement->hasAttributeWithoutSynchronization(HTMLNames::webkitwirelessvideoplaybackdisabledAttr)
-            || m_mediaElement->hasAttributeWithoutSynchronization(HTMLNames::disableremoteplaybackAttr)) {
-            promise->reject(InvalidStateError);
+        if (!m_mediaElement || m_mediaElement->isWirelessPlaybackTargetDisabled()) {
+            ERROR_LOG(identifier, "promise rejected, remote playback disabled");
+            promise->reject(ExceptionCode::InvalidStateError);
             return;
         }
 
@@ -181,9 +218,10 @@ void RemotePlayback::prompt(Ref<DeferredPromise>&& promise)
 
         // 5. OPTIONALLY, if the user agent knows a priori that showing the UI for this particular media element
         //    is not feasible, reject promise with a NotSupportedError and abort all remaining steps.
-#if !PLATFORM(IOS)
+#if !(PLATFORM(IOS) || PLATFORM(VISION))
         if (m_mediaElement->readyState() < HTMLMediaElementEnums::HAVE_METADATA) {
-            promise->reject(NotSupportedError);
+            ERROR_LOG(identifier, "promise rejected, readyState = ", m_mediaElement->readyState());
+            promise->reject(ExceptionCode::NotSupportedError);
             return;
         }
 #endif
@@ -191,7 +229,8 @@ void RemotePlayback::prompt(Ref<DeferredPromise>&& promise)
         // 6. If the algorithm isn't allowed to show a popup, reject promise with an InvalidAccessError exception
         //    and abort these steps.
         if (!processingUserGesture) {
-            promise->reject(InvalidAccessError);
+            ERROR_LOG(identifier, "promise rejected, user gesture required");
+            promise->reject(ExceptionCode::InvalidAccessError);
             return;
         }
 
@@ -199,8 +238,6 @@ void RemotePlayback::prompt(Ref<DeferredPromise>&& promise)
         //    the list of available remote playback devices, run the steps to monitor the list of available remote
         //    playback devices in parallel.
         // NOTE: Monitoring enabled by adding to m_promptPromises and calling remoteHasAvailabilityCallbacksChanged().
-        //       Meanwhile, just update availability for step 9.
-        updateAvailability();
 
         // 8. If the list of available remote playback devices is empty and will remain so before the request for
         //    user permission is completed, reject promise with a NotFoundError exception and abort all remaining steps.
@@ -209,11 +246,13 @@ void RemotePlayback::prompt(Ref<DeferredPromise>&& promise)
         // 9. If the state is disconnected and availability for the media element is false, reject promise with a
         //    NotSupportedError exception and abort all remaining steps.
         if (m_state == State::Disconnected && !m_available) {
-            promise->reject(NotSupportedError);
+            ERROR_LOG(identifier, "promise rejected, state = ", m_state, ", available = ", m_available);
+            promise->reject(ExceptionCode::NotSupportedError);
             return;
         }
 
         m_promptPromises.append(WTFMove(promise));
+        availabilityChanged(m_mediaElement->mediaSession().hasWirelessPlaybackTargets());
         m_mediaElement->remoteHasAvailabilityCallbacksChanged();
         m_mediaElement->webkitShowPlaybackTargetPicker();
 
@@ -227,7 +266,7 @@ void RemotePlayback::shouldPlayToRemoteTargetChanged(bool shouldPlayToRemoteTarg
     // https://w3c.github.io/remote-playback/#prompt-user-for-changing-remote-playback-statee
     // W3C Editor's Draft 15 July 2016
 
-    LOG(Media, "RemotePlayback::shouldPlayToRemoteTargetChanged(%p), shouldPlay(%d), promise count(%lu)", this, shouldPlayToRemoteTarget, m_promptPromises.size());
+    ALWAYS_LOG(LOGIDENTIFIER, "shouldPlay = ", shouldPlayToRemoteTarget, ", promise count = ", m_promptPromises.size());
 
     // 10. If the user picked a remote playback device device to initiate remote playback with, the user agent
     //     must run the following steps:
@@ -257,7 +296,8 @@ void RemotePlayback::shouldPlayToRemoteTargetChanged(bool shouldPlayToRemoteTarg
     else
         disconnect();
 
-    m_mediaElement->remoteHasAvailabilityCallbacksChanged();
+    if (m_mediaElement)
+        m_mediaElement->remoteHasAvailabilityCallbacksChanged();
 }
 
 void RemotePlayback::setState(State state)
@@ -265,19 +305,21 @@ void RemotePlayback::setState(State state)
     if (m_state == state)
         return;
 
+    ALWAYS_LOG(LOGIDENTIFIER, state);
     m_state = state;
 
-    switch (m_state) {
-    case State::Connected:
-        m_eventQueue->enqueueEvent(Event::create(eventNames().connectEvent, Event::CanBubble::No, Event::IsCancelable::No));
-        break;
-    case State::Connecting:
-        m_eventQueue->enqueueEvent(Event::create(eventNames().connectingEvent, Event::CanBubble::No, Event::IsCancelable::No));
-        break;
-    case State::Disconnected:
-        m_eventQueue->enqueueEvent(Event::create(eventNames().disconnectEvent, Event::CanBubble::No, Event::IsCancelable::No));
-        break;
-    }
+    auto eventName = [](State state) {
+        switch (state) {
+        case State::Connected:
+            return eventNames().connectEvent;
+        case State::Connecting:
+            return eventNames().connectingEvent;
+        case State::Disconnected:
+            return eventNames().disconnectEvent;
+        }
+    };
+
+    queueTaskToDispatchEvent(*this, TaskSource::MediaElement, Event::create(eventName(state), Event::CanBubble::No, Event::IsCancelable::No));
 }
 
 void RemotePlayback::establishConnection()
@@ -306,8 +348,13 @@ void RemotePlayback::disconnect()
     if (m_state == State::Disconnected)
         return;
 
+    ALWAYS_LOG(LOGIDENTIFIER);
+
     // 2. Queue a task to run the following steps:
-    m_taskQueue.enqueueTask([this] {
+    queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this] {
+        if (isContextStopped())
+            return;
+
         // 2.1 Request disconnection of remote from the device. Implementation is user agent specific.
         // NOTE: Implemented by MediaPlayer::setWirelessPlaybackTarget()
         // 2.2 Change the remote's state to disconnected.
@@ -320,18 +367,16 @@ void RemotePlayback::disconnect()
     });
 }
 
-RemotePlayback::UpdateResults RemotePlayback::updateAvailability()
+void RemotePlayback::stop()
 {
-    bool available = m_mediaElement ? m_mediaElement->mediaSession().hasWirelessPlaybackTargets() : false;
-    if (available == m_available)
-        return UpdateResults::Unchanged;
-
-    availabilityChanged(available);
-    return UpdateResults::Changed;
+    m_callbackMap.clear();
 }
 
 void RemotePlayback::playbackTargetPickerWasDismissed()
 {
+    if (m_promptPromises.isEmpty())
+        return;
+
     // 6.2.2 Prompt user for changing remote playback state [Ctd]
     // https://w3c.github.io/remote-playback/#stop-observing-remote-playback-devices-availability
     // W3C Editor's Draft 15 July 2016
@@ -341,8 +386,10 @@ void RemotePlayback::playbackTargetPickerWasDismissed()
     ASSERT(!m_promptPromises.isEmpty());
 
     for (auto& promise : std::exchange(m_promptPromises, { }))
-        promise->reject(NotAllowedError);
-    m_mediaElement->remoteHasAvailabilityCallbacksChanged();
+        promise->reject(ExceptionCode::NotAllowedError);
+
+    if (m_mediaElement)
+        m_mediaElement->remoteHasAvailabilityCallbacksChanged();
 }
 
 void RemotePlayback::isPlayingToRemoteTargetChanged(bool isPlayingToTarget)
@@ -380,16 +427,15 @@ void RemotePlayback::availabilityChanged(bool available)
         return;
     m_available = available;
 
-    m_taskQueue.enqueueTask([this] {
-        // Protect m_callbackMap against mutation while it's being iterated over.
-        Vector<Ref<RemotePlaybackAvailabilityCallback>> callbacks;
-        callbacks.reserveInitialCapacity(m_callbackMap.size());
+    ALWAYS_LOG(LOGIDENTIFIER);
 
-        // Can't use copyValuesToVector() here because Ref<> has a deleted assignment operator.
-        for (auto& callback : m_callbackMap.values())
-            callbacks.uncheckedAppend(callback.copyRef());
-        for (auto& callback : callbacks)
-            callback->handleEvent(m_available);
+    queueTaskKeepingObjectAlive(*this, TaskSource::MediaElement, [this, available] {
+        if (isContextStopped())
+            return;
+
+        // Protect m_callbackMap against mutation while it's being iterated over.
+        for (auto& callback : copyToVector(m_callbackMap.values()))
+            callback->handleEvent(available);
     });
 }
 
@@ -398,16 +444,12 @@ void RemotePlayback::invalidate()
     m_mediaElement = nullptr;
 }
 
-const char* RemotePlayback::activeDOMObjectName() const
+#if !RELEASE_LOG_DISABLED
+WTFLogChannel& RemotePlayback::logChannel() const
 {
-    return "RemotePlayback";
-}
-
-void RemotePlayback::stop()
-{
-    m_taskQueue.close();
-    m_eventQueue->close();
+    return LogMedia;
 }
+#endif
 
 }
 
