@@ -30,15 +30,22 @@
 #include "AudioContext.h"
 #include "AudioContextOptions.h"
 #include "AudioTimestamp.h"
-#include "DOMWindow.h"
+#include "DOMAudioSession.h"
 #include "DocumentInlines.h"
 #include "JSDOMPromiseDeferred.h"
+#include "LocalDOMWindow.h"
 #include "Logging.h"
+#include "MediaSession.h"
+#include "Navigator.h"
+#include "NavigatorAudioSession.h"
+#include "NavigatorMediaSession.h"
+#include "NowPlayingInfo.h"
 #include "PageInlines.h"
 #include "Performance.h"
 #include "PlatformMediaSessionManager.h"
 #include "Quirks.h"
-#include <wtf/IsoMallocInlines.h>
+#include <wtf/MediaTime.h>
+#include <wtf/TZoneMallocInlines.h>
 
 #if ENABLE(MEDIA_STREAM)
 #include "MediaStream.h"
@@ -54,6 +61,10 @@
 #include "MediaElementAudioSourceOptions.h"
 #endif
 
+#if PLATFORM(VISION) && ENABLE(WEBXR)
+#include "Page.h"
+#endif
+
 namespace WebCore {
 
 #define AUDIOCONTEXT_RELEASE_LOG(fmt, ...) RELEASE_LOG(Media, "%p - AudioContext::" fmt, this, ##__VA_ARGS__)
@@ -63,7 +74,7 @@ namespace WebCore {
 constexpr unsigned maxHardwareContexts = 4;
 #endif
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(AudioContext);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(AudioContext);
 
 #if OS(WINDOWS)
 static unsigned hardwareContextCount;
@@ -77,9 +88,13 @@ static std::optional<float>& defaultSampleRateForTesting()
 
 static bool shouldDocumentAllowWebAudioToAutoPlay(const Document& document)
 {
-    if (document.processingUserGestureForMedia() || document.isCapturing())
+    if (document.isCapturing())
         return true;
-    return document.quirks().shouldAutoplayWebAudioForArbitraryUserGesture() && document.topDocument().hasHadUserInteraction();
+    RefPtr mainDocument = document.protectedMainFrameDocument();
+    if (document.quirks().shouldAutoplayWebAudioForArbitraryUserGesture() && mainDocument && mainDocument->hasHadUserInteraction())
+        return true;
+    RefPtr window = document.domWindow();
+    return window && window->hasTransientActivation();
 }
 
 void AudioContext::setDefaultSampleRateForTesting(std::optional<float> sampleRate)
@@ -92,11 +107,11 @@ ExceptionOr<Ref<AudioContext>> AudioContext::create(Document& document, AudioCon
     ASSERT(isMainThread());
 #if OS(WINDOWS)
     if (hardwareContextCount >= maxHardwareContexts)
-        return Exception { QuotaExceededError, "Reached maximum number of hardware contexts on this platform"_s };
+        return Exception { ExceptionCode::QuotaExceededError, "Reached maximum number of hardware contexts on this platform"_s };
 #endif
     
     if (!document.isFullyActive())
-        return Exception { InvalidStateError, "Document is not fully active"_s };
+        return Exception { ExceptionCode::InvalidStateError, "Document is not fully active"_s };
     
     // FIXME: Figure out where latencyHint should go.
 
@@ -104,7 +119,7 @@ ExceptionOr<Ref<AudioContext>> AudioContext::create(Document& document, AudioCon
         contextOptions.sampleRate = *defaultSampleRateForTesting();
 
     if (contextOptions.sampleRate && !isSupportedSampleRate(*contextOptions.sampleRate))
-        return Exception { NotSupportedError, "sampleRate is not in range"_s };
+        return Exception { ExceptionCode::NotSupportedError, "sampleRate is not in range"_s };
     
     auto audioContext = adoptRef(*new AudioContext(document, contextOptions));
     audioContext->suspendIfNeeded();
@@ -113,8 +128,9 @@ ExceptionOr<Ref<AudioContext>> AudioContext::create(Document& document, AudioCon
 
 AudioContext::AudioContext(Document& document, const AudioContextOptions& contextOptions)
     : BaseAudioContext(document)
-    , m_destinationNode(makeUniqueRef<DefaultAudioDestinationNode>(*this, contextOptions.sampleRate))
-    , m_mediaSession(PlatformMediaSession::create(PlatformMediaSessionManager::sharedManager(), *this))
+    , m_destinationNode(makeUniqueRefWithoutRefCountedCheck<DefaultAudioDestinationNode>(*this, contextOptions.sampleRate))
+    , m_mediaSession(PlatformMediaSession::create(PlatformMediaSessionManager::singleton(), *this))
+    , m_currentIdentifier(MediaUniqueIdentifier::generate())
 {
     constructCommon();
 
@@ -135,7 +151,8 @@ AudioContext::AudioContext(Document& document, const AudioContextOptions& contex
 void AudioContext::constructCommon()
 {
     ASSERT(document());
-    if (document()->topDocument().audioPlaybackRequiresUserGesture())
+    RefPtr mainFrameDocument = document()->mainFrameDocument();
+    if (!mainFrameDocument || mainFrameDocument->requiresUserGestureForAudioPlayback())
         addBehaviorRestriction(RequireUserGestureForAudioStartRestriction);
     else
         m_restrictions = NoRestrictions;
@@ -147,7 +164,7 @@ void AudioContext::constructCommon()
 
 AudioContext::~AudioContext()
 {
-    if (auto* document = this->document())
+    if (RefPtr document = this->document())
         document->removeAudioProducer(*this);
 }
 
@@ -173,6 +190,18 @@ double AudioContext::baseLatency()
     return static_cast<double>(destination().framesPerBuffer()) / sampleRate();
 }
 
+double AudioContext::outputLatency()
+{
+    lazyInitialize();
+
+    if (!isPlaying())
+        return 0;
+    if (noiseInjectionPolicies())
+        return 512 / sampleRate(); // A fixed, but reasonable value for most platforms.
+
+    return destination().outputLatency().toDouble();
+}
+
 AudioTimestamp AudioContext::getOutputTimestamp()
 {
     auto position = outputPosition();
@@ -191,7 +220,7 @@ AudioTimestamp AudioContext::getOutputTimestamp()
 void AudioContext::close(DOMPromiseDeferred<void>&& promise)
 {
     if (isStopped()) {
-        promise.reject(InvalidStateError);
+        promise.reject(ExceptionCode::InvalidStateError);
         return;
     }
 
@@ -214,7 +243,7 @@ void AudioContext::close(DOMPromiseDeferred<void>&& promise)
 void AudioContext::suspendRendering(DOMPromiseDeferred<void>&& promise)
 {
     if (isStopped() || state() == State::Closed) {
-        promise.reject(Exception { InvalidStateError, "Context is closed"_s });
+        promise.reject(Exception { ExceptionCode::InvalidStateError, "Context is closed"_s });
         return;
     }
 
@@ -240,7 +269,7 @@ void AudioContext::suspendRendering(DOMPromiseDeferred<void>&& promise)
 void AudioContext::resumeRendering(DOMPromiseDeferred<void>&& promise)
 {
     if (isStopped() || state() == State::Closed) {
-        promise.reject(Exception { InvalidStateError, "Context is closed"_s });
+        promise.reject(Exception { ExceptionCode::InvalidStateError, "Context is closed"_s });
         return;
     }
 
@@ -262,7 +291,7 @@ void AudioContext::resumeRendering(DOMPromiseDeferred<void>&& promise)
         // Since we update the state asynchronously, we may have been interrupted after the
         // call to resume() and before this lambda runs. In this case, we don't want to
         // reset the state to running.
-        bool interrupted = m_mediaSession->state() == PlatformMediaSession::Interrupted;
+        bool interrupted = m_mediaSession->state() == PlatformMediaSession::State::Interrupted;
         setState(interrupted ? State::Interrupted : State::Running);
         if (interrupted)
             addReaction(State::Running, WTFMove(promise));
@@ -293,9 +322,9 @@ void AudioContext::startRendering()
         return;
 
     lazyInitialize();
-    destination().startRendering([this, protectedThis = Ref { *this }, pendingActivity = makePendingActivity(*this)](std::optional<Exception>&& exception) {
+    destination().startRendering([protectedThis = Ref { *this }, pendingActivity = makePendingActivity(*this)](std::optional<Exception>&& exception) {
         if (!exception)
-            setState(State::Running);
+            protectedThis->setState(State::Running);
     });
 }
 
@@ -321,7 +350,7 @@ void AudioContext::lazyInitialize()
 
 bool AudioContext::willPausePlayback()
 {
-    auto* document = this->document();
+    RefPtr document = this->document();
     if (!document)
         return false;
 
@@ -345,10 +374,12 @@ bool AudioContext::willPausePlayback()
 
 MediaProducerMediaStateFlags AudioContext::mediaState() const
 {
-    if (!isStopped() && destination().isPlayingAudio())
-        return MediaProducerMediaState::IsPlayingAudio;
+    return isAudible() ? MediaProducerMediaState::IsPlayingAudio : MediaProducer::IsNotPlaying;
+}
 
-    return MediaProducer::IsNotPlaying;
+bool AudioContext::isAudible() const
+{
+    return !isStopped() && destination().isPlayingAudio();
 }
 
 void AudioContext::mayResumePlayback(bool shouldResume)
@@ -361,19 +392,22 @@ void AudioContext::mayResumePlayback(bool shouldResume)
         return;
     }
 
+    if (m_wasSuspendedByScript)
+        return;
+
     if (!willBeginPlayback())
         return;
 
     lazyInitialize();
 
-    destination().resume([this, protectedThis = Ref { *this }, pendingActivity = makePendingActivity(*this)](std::optional<Exception>&& exception) {
-        setState(exception ? State::Suspended : State::Running);
+    destination().resume([protectedThis = Ref { *this }, pendingActivity = makePendingActivity(*this)](std::optional<Exception>&& exception) {
+        protectedThis->setState(exception ? State::Suspended : State::Running);
     });
 }
 
 bool AudioContext::willBeginPlayback()
 {
-    auto* document = this->document();
+    RefPtr document = this->document();
     if (!document)
         return false;
 
@@ -408,7 +442,7 @@ void AudioContext::suspend(ReasonForSuspension)
     if (isClosed() || m_wasSuspendedByScript)
         return;
 
-    m_mediaSession->beginInterruption(PlatformMediaSession::PlaybackSuspended);
+    m_mediaSession->beginInterruption(PlatformMediaSession::InterruptionType::PlaybackSuspended);
     document()->updateIsPlayingMedia();
 }
 
@@ -417,15 +451,10 @@ void AudioContext::resume()
     if (isClosed() || m_wasSuspendedByScript)
         return;
 
-    m_mediaSession->endInterruption(PlatformMediaSession::MayResumePlaying);
+    m_mediaSession->endInterruption(PlatformMediaSession::EndInterruptionFlags::MayResumePlaying);
     document()->updateIsPlayingMedia();
 }
 
-const char* AudioContext::activeDOMObjectName() const
-{
-    return "AudioContext";
-}
-
 void AudioContext::suspendPlayback()
 {
     if (state() == State::Closed || !isInitialized())
@@ -433,19 +462,156 @@ void AudioContext::suspendPlayback()
 
     lazyInitialize();
 
-    destination().suspend([this, protectedThis = Ref { *this }, pendingActivity = makePendingActivity(*this)](std::optional<Exception>&& exception) {
+    destination().suspend([protectedThis = Ref { *this }, pendingActivity = makePendingActivity(*this)](std::optional<Exception>&& exception) {
         if (exception)
             return;
 
-        bool interrupted = m_mediaSession->state() == PlatformMediaSession::Interrupted;
-        setState(interrupted ? State::Interrupted : State::Suspended);
+        bool interrupted = protectedThis->m_mediaSession->state() == PlatformMediaSession::State::Interrupted;
+        protectedThis->setState(interrupted ? State::Interrupted : State::Suspended);
     });
 }
 
-MediaSessionGroupIdentifier AudioContext::mediaSessionGroupIdentifier() const
+bool AudioContext::canReceiveRemoteControlCommands() const
 {
-    auto* document = downcast<Document>(scriptExecutionContext());
-    return document && document->page() ? document->page()->mediaSessionGroupIdentifier() : MediaSessionGroupIdentifier { };
+#if ENABLE(DOM_AUDIO_SESSION)
+    return isNowPlayingEligible();
+#else
+    return false;
+#endif
+}
+
+void AudioContext::didReceiveRemoteControlCommand(PlatformMediaSession::RemoteControlCommandType command, const PlatformMediaSession::RemoteCommandArgument&)
+{
+    switch (command) {
+    case PlatformMediaSession::RemoteControlCommandType::PlayCommand:
+        mayResumePlayback(true);
+        break;
+    case PlatformMediaSession::RemoteControlCommandType::StopCommand:
+    case PlatformMediaSession::RemoteControlCommandType::PauseCommand:
+        suspendPlayback();
+        break;
+    case PlatformMediaSession::RemoteControlCommandType::TogglePlayPauseCommand:
+        if (state() == State::Interrupted || state() == State::Suspended)
+            mayResumePlayback(true);
+        else
+            suspendPlayback();
+        break;
+    case PlatformMediaSession::RemoteControlCommandType::BeginSeekingBackwardCommand:
+    case PlatformMediaSession::RemoteControlCommandType::BeginSeekingForwardCommand:
+    case PlatformMediaSession::RemoteControlCommandType::EndSeekingBackwardCommand:
+    case PlatformMediaSession::RemoteControlCommandType::EndSeekingForwardCommand:
+    case PlatformMediaSession::RemoteControlCommandType::BeginScrubbingCommand:
+    case PlatformMediaSession::RemoteControlCommandType::EndScrubbingCommand:
+    case PlatformMediaSession::RemoteControlCommandType::SkipForwardCommand:
+    case PlatformMediaSession::RemoteControlCommandType::SkipBackwardCommand:
+    case PlatformMediaSession::RemoteControlCommandType::SeekToPlaybackPositionCommand:
+    default:
+        ASSERT_NOT_REACHED();
+    }
+}
+
+std::optional<MediaSessionGroupIdentifier> AudioContext::mediaSessionGroupIdentifier() const
+{
+    RefPtr document = downcast<Document>(scriptExecutionContext());
+    return document && document->page() ? document->page()->mediaSessionGroupIdentifier() : std::nullopt;
+}
+
+static bool hasPlayBackAudioSession(Document* document)
+{
+#if ENABLE(DOM_AUDIO_SESSION)
+    RefPtr window = document ? document->domWindow() : nullptr;
+
+    RefPtr navigator = window ? window->optionalNavigator() : nullptr;
+    RefPtr audioSession = navigator ? NavigatorAudioSession::audioSession(*navigator) : nullptr;
+
+    auto audioSessionType = audioSession ? audioSession->type() : DOMAudioSessionType::Auto;
+    return audioSessionType == DOMAudioSessionType::Playback || audioSessionType == DOMAudioSessionType::PlayAndRecord;
+#else
+    UNUSED_PARAM(document);
+    return false;
+#endif
+}
+
+bool AudioContext::isNowPlayingEligible() const
+{
+    if (!destination().isConnected() || m_wasSuspendedByScript)
+        return false;
+
+    RefPtr document = this->document();
+    if (!document)
+        return false;
+
+    RefPtr page = document->page();
+    if (page && page->mediaPlaybackIsSuspended())
+        return false;
+
+    return hasPlayBackAudioSession(document.get());
+}
+
+std::optional<NowPlayingInfo> AudioContext::nowPlayingInfo() const
+{
+    if (!isNowPlayingEligible())
+        return { };
+
+    RefPtr document = this->document();
+    RefPtr page = document ? document->page() : nullptr;
+    RefPtr window = document ? document->domWindow() : nullptr;
+    if (!page || !window)
+        return { };
+
+    NowPlayingInfo nowPlayingInfo {
+        {
+            { },
+            { },
+            { },
+            { },
+            { }
+        },
+        MediaPlayer::invalidTime(),
+        MediaPlayer::invalidTime(),
+        1.0,
+        false,
+        m_currentIdentifier,
+        isPlaying(),
+        !page->isVisibleAndActive(),
+        false
+    };
+
+    if (page->usesEphemeralSession() && !document->settings().allowPrivacySensitiveOperationsInNonPersistentDataStores())
+        return nowPlayingInfo;
+
+#if ENABLE(MEDIA_SESSION)
+    if (RefPtr mediaSession = NavigatorMediaSession::mediaSessionIfExists(window->protectedNavigator()))
+        mediaSession->updateNowPlayingInfo(nowPlayingInfo);
+#endif
+
+    if (nowPlayingInfo.metadata.title.isEmpty()) {
+        RegistrableDomain domain { document->securityOrigin().data() };
+        if (!domain.isEmpty())
+            nowPlayingInfo.metadata.title = domain.string();
+    }
+
+    return nowPlayingInfo;
+}
+
+WeakPtr<PlatformMediaSession> AudioContext::selectBestMediaSession(const Vector<WeakPtr<PlatformMediaSession>>& sessions, PlatformMediaSession::PlaybackControlsPurpose purpose)
+{
+    if (purpose != PlatformMediaSession::PlaybackControlsPurpose::NowPlaying)
+        return nullptr;
+
+    WeakPtr<PlatformMediaSession> audibleSession;
+    for (auto& session : sessions) {
+        if (!isNowPlayingEligible())
+            continue;
+
+        if (!audibleSession)
+            audibleSession = session;
+
+        if (session->isPlaying())
+            return session;
+    }
+
+    return audibleSession;
 }
 
 bool AudioContext::isSuspended() const
@@ -480,30 +646,66 @@ void AudioContext::isPlayingAudioDidChange()
     // Make sure to call Document::updateIsPlayingMedia() on the main thread, since
     // we could be on the audio I/O thread here and the call into WebCore could block.
     callOnMainThread([protectedThis = Ref { *this }] {
-        if (auto* document = protectedThis->document())
+        if (RefPtr document = protectedThis->document())
             document->updateIsPlayingMedia();
     });
 }
 
 bool AudioContext::shouldOverrideBackgroundPlaybackRestriction(PlatformMediaSession::InterruptionType interruption) const
 {
-    return m_canOverrideBackgroundPlaybackRestriction && interruption == PlatformMediaSession::InterruptionType::EnteringBackground && !destination().isConnected();
+    if (interruption != PlatformMediaSession::InterruptionType::EnteringBackground)
+        return false;
+
+    if (m_canOverrideBackgroundPlaybackRestriction && !destination().isConnected())
+        return true;
+
+    RefPtr document = this->document();
+    if (!document)
+        return false;
+
+#if PLATFORM(VISION) && ENABLE(WEBXR)
+    RefPtr page = document->page();
+    if (page && page->hasActiveImmersiveSession())
+        return true;
+#endif
+
+    return hasPlayBackAudioSession(document.get());
 }
 
 void AudioContext::defaultDestinationWillBecomeConnected()
 {
     // We might need to interrupt if we previously overrode a background interruption.
-    if (!PlatformMediaSessionManager::sharedManager().isApplicationInBackground() || m_mediaSession->state() == PlatformMediaSession::Interrupted)
+    if (!PlatformMediaSessionManager::singleton().isApplicationInBackground() || m_mediaSession->state() == PlatformMediaSession::State::Interrupted) {
+        PlatformMediaSessionManager::updateNowPlayingInfoIfNecessary();
         return;
+    }
 
     // We end the overriden interruption (if any) to get the right count of interruptions and start a new interruption.
     m_mediaSession->endInterruption(PlatformMediaSession::EndInterruptionFlags::NoFlags);
 
     m_canOverrideBackgroundPlaybackRestriction = false;
-    m_mediaSession->beginInterruption(PlatformMediaSession::EnteringBackground);
+    m_mediaSession->beginInterruption(PlatformMediaSession::InterruptionType::EnteringBackground);
     m_canOverrideBackgroundPlaybackRestriction = true;
 }
 
+void AudioContext::isActiveNowPlayingSessionChanged()
+{
+    if (RefPtr document = this->document()) {
+        if (RefPtr page = document->protectedPage())
+            page->hasActiveNowPlayingSessionChanged();
+    }
+}
+
+ProcessID AudioContext::presentingApplicationPID() const
+{
+    if (RefPtr document = this->document()) {
+        if (RefPtr page = document->protectedPage())
+            return page->presentingApplicationPID();
+    }
+
+    return { };
+}
+
 #if !RELEASE_LOG_DISABLED
 const Logger& AudioContext::logger() const
 {
