@@ -35,6 +35,7 @@
 #include <wtf/glib/RunLoopSourcePriority.h>
 
 namespace WebCore {
+DEFINE_ALLOCATOR_WITH_HEAP_IDENTIFIER(AccessibilityAtspi);
 
 AccessibilityAtspi& AccessibilityAtspi::singleton()
 {
@@ -50,11 +51,16 @@ AccessibilityAtspi::AccessibilityAtspi()
     m_cacheClearTimer.setPriority(RunLoopSourcePriority::ReleaseUnusedResourcesTimer);
 }
 
-void AccessibilityAtspi::connect(const String& busAddress)
+void AccessibilityAtspi::connect(const String& busAddress, const String& busName)
 {
     if (busAddress.isEmpty())
         return;
 
+    RELEASE_ASSERT(g_dbus_is_name(busName.utf8().data()));
+    RELEASE_ASSERT(!g_dbus_is_unique_name(busName.utf8().data()));
+
+    m_busName = busName;
+
     m_isConnecting = true;
     g_dbus_connection_new_for_address(busAddress.utf8().data(),
         static_cast<GDBusConnectionFlags>(G_DBUS_CONNECTION_FLAGS_AUTHENTICATION_CLIENT | G_DBUS_CONNECTION_FLAGS_MESSAGE_BUS_CONNECTION), nullptr, nullptr,
@@ -69,10 +75,24 @@ void AccessibilityAtspi::connect(const String& busAddress)
 
 void AccessibilityAtspi::didConnect(GRefPtr<GDBusConnection>&& connection)
 {
-    m_isConnecting = false;
     m_connection = WTFMove(connection);
-    if (!m_connection)
+    if (!m_connection) {
+        m_isConnecting = false;
         return;
+    }
+
+    RELEASE_ASSERT(g_dbus_is_name(m_busName.utf8().data()));
+    g_bus_own_name_on_connection(m_connection.get(), m_busName.utf8().data(), G_BUS_NAME_OWNER_FLAGS_DO_NOT_QUEUE,
+        [](GDBusConnection*, const char*, gpointer userData) {
+            auto& atspi = *static_cast<AccessibilityAtspi*>(userData);
+            atspi.didOwnName();
+        },
+        nullptr, this, nullptr);
+}
+
+void AccessibilityAtspi::didOwnName()
+{
+    m_isConnecting = false;
 
     for (auto& pendingRegistration : m_pendingRootRegistrations)
         registerRoot(pendingRegistration.root, WTFMove(pendingRegistration.interfaces), WTFMove(pendingRegistration.completionHandler));
@@ -127,6 +147,7 @@ void AccessibilityAtspi::initializeRegistry()
     }, this);
 }
 
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_BEGIN // GLib port
 static GUniquePtr<char*> eventConvertingDetailToNonCamelCase(const char* eventName)
 {
     GUniquePtr<char*> event(g_strsplit(eventName, ":", 3));
@@ -173,6 +194,7 @@ static bool eventIsSubtype(char** needle, char** haystack)
 
     return true;
 }
+WTF_ALLOW_UNSAFE_BUFFER_USAGE_END
 
 void AccessibilityAtspi::removeEventListener(const char* dbusName, const char* eventName)
 {
@@ -288,15 +310,12 @@ void AccessibilityAtspi::registerRoot(AccessibilityRootAtspi& rootObject, Vector
     }
 
     ensureCache();
-    String path = makeString("/org/a11y/webkit/accessible/", createVersion4UUIDString().replace('-', '_'));
-    Vector<unsigned, 3> registeredObjects;
-    registeredObjects.reserveInitialCapacity(interfaces.size());
-    for (const auto& interface : interfaces) {
-        auto id = g_dbus_connection_register_object(m_connection.get(), path.utf8().data(), interface.first, interface.second, &rootObject, nullptr, nullptr);
-        registeredObjects.uncheckedAppend(id);
-    }
+    String path = makeString("/org/a11y/webkit/accessible/"_s, makeStringByReplacingAll(createVersion4UUIDString(), '-', '_'));
+    auto registeredObjects = WTF::map<3>(interfaces, [&](auto& interface) -> unsigned {
+        return g_dbus_connection_register_object(m_connection.get(), path.utf8().data(), interface.first, interface.second, &rootObject, nullptr, nullptr);
+    });
     m_rootObjects.add(&rootObject, WTFMove(registeredObjects));
-    String reference = makeString(uniqueName(), ':', path);
+    String reference = makeString(m_busName, ':', path);
     rootObject.setPath(WTFMove(path));
     completionHandler(reference);
 }
@@ -332,13 +351,10 @@ String AccessibilityAtspi::registerObject(AccessibilityObjectAtspi& atspiObject,
         return { };
 
     ensureCache();
-    String path = makeString("/org/a11y/atspi/accessible/", createVersion4UUIDString().replace('-', '_'));
-    Vector<unsigned, 7> registeredObjects;
-    registeredObjects.reserveInitialCapacity(interfaces.size());
-    for (const auto& interface : interfaces) {
-        auto id = g_dbus_connection_register_object(m_connection.get(), path.utf8().data(), interface.first, interface.second, &atspiObject, nullptr, nullptr);
-        registeredObjects.uncheckedAppend(id);
-    }
+    String path = makeString("/org/a11y/atspi/accessible/"_s, makeStringByReplacingAll(createVersion4UUIDString(), '-', '_'));
+    auto registeredObjects = WTF::map<7>(interfaces, [&](auto& interface) -> unsigned {
+        return g_dbus_connection_register_object(m_connection.get(), path.utf8().data(), interface.first, interface.second, &atspiObject, nullptr, nullptr);
+    });
     m_atspiObjects.add(&atspiObject, WTFMove(registeredObjects));
 
     m_cacheUpdateList.add(&atspiObject);
@@ -382,13 +398,10 @@ String AccessibilityAtspi::registerHyperlink(AccessibilityObjectAtspi& atspiObje
     if (!m_connection)
         return { };
 
-    String path = makeString("/org/a11y/atspi/accessible/", createVersion4UUIDString().replace('-', '_'));
-    Vector<unsigned, 1> registeredObjects;
-    registeredObjects.reserveInitialCapacity(interfaces.size());
-    for (const auto& interface : interfaces) {
-        auto id = g_dbus_connection_register_object(m_connection.get(), path.utf8().data(), interface.first, interface.second, &atspiObject, nullptr, nullptr);
-        registeredObjects.uncheckedAppend(id);
-    }
+    String path = makeString("/org/a11y/atspi/accessible/"_s, makeStringByReplacingAll(createVersion4UUIDString(), '-', '_'));
+    auto registeredObjects = WTF::map<1>(interfaces, [&](auto& interface) -> unsigned {
+        return g_dbus_connection_register_object(m_connection.get(), path.utf8().data(), interface.first, interface.second, &atspiObject, nullptr, nullptr);
+    });
     m_atspiHyperlinks.add(&atspiObject, WTFMove(registeredObjects));
 
     return path;
@@ -540,6 +553,25 @@ void AccessibilityAtspi::valueChanged(AccessibilityObjectAtspi& atspiObject, dou
         g_variant_new("(siiva{sv})", "accessible-value", 0, 0, g_variant_new_double(value), nullptr), nullptr);
 }
 
+void AccessibilityAtspi::activeDescendantChanged(AccessibilityObjectAtspi& atspiObject)
+{
+#if ENABLE(DEVELOPER_MODE)
+    notifyActiveDescendantChanged(atspiObject);
+#endif
+
+    if (!m_connection)
+        return;
+
+    if (!shouldEmitSignal("Object", "ActiveDescendantChanged"))
+        return;
+
+    auto* activeDescendant = atspiObject.activeDescendant();
+    ASSERT(activeDescendant);
+
+    g_dbus_connection_emit_signal(m_connection.get(), nullptr, atspiObject.path().utf8().data(), "org.a11y.atspi.Event.Object", "ActiveDescendantChanged",
+        g_variant_new("(siiva{sv})", "", activeDescendant->indexInParent(), 0, g_variant_new("(so)", uniqueName(), activeDescendant->path().utf8().data()), nullptr), nullptr);
+}
+
 void AccessibilityAtspi::selectionChanged(AccessibilityObjectAtspi& atspiObject)
 {
 #if ENABLE(DEVELOPER_MODE)
@@ -585,20 +617,17 @@ static constexpr std::pair<AccessibilityRole, RoleNameEntry> roleNames[] = {
     { AccessibilityRole::ApplicationAlert, { "notification", N_("notification") } },
     { AccessibilityRole::ApplicationAlertDialog, { "alert", N_("alert") } },
     { AccessibilityRole::ApplicationDialog, { "dialog", N_("dialog") } },
-    { AccessibilityRole::ApplicationGroup, { "grouping", N_("grouping") } },
     { AccessibilityRole::ApplicationLog, { "log", N_("log") } },
     { AccessibilityRole::ApplicationMarquee, { "marquee", N_("marquee") } },
     { AccessibilityRole::ApplicationStatus, { "statusbar", N_("statusbar") } },
-    { AccessibilityRole::ApplicationTextGroup, { "section", N_("section") } },
     { AccessibilityRole::ApplicationTimer, { "timer", N_("timer") } },
     { AccessibilityRole::Audio, { "audio", N_("audio") } },
     { AccessibilityRole::Blockquote, { "block quote", N_("block quote") } },
-    { AccessibilityRole::BusyIndicator, { "progress bar", N_("progress bar") } },
     { AccessibilityRole::Button, { "push button", N_("push button") } },
     { AccessibilityRole::Canvas, { "canvas", N_("canvas") } },
     { AccessibilityRole::Caption, { "caption", N_("caption") } },
     { AccessibilityRole::Cell, { "table cell", N_("table cell") } },
-    { AccessibilityRole::CheckBox, { "check box", N_("check box") } },
+    { AccessibilityRole::Checkbox, { "check box", N_("check box") } },
     { AccessibilityRole::ColorWell, { "push button", N_("push button") } },
     { AccessibilityRole::ColumnHeader, { "column header", N_("column header") } },
     { AccessibilityRole::ComboBox, { "combo box", N_("combo box") } },
@@ -608,7 +637,6 @@ static constexpr std::pair<AccessibilityRole, RoleNameEntry> roleNames[] = {
     { AccessibilityRole::DescriptionListDetail, { "description value", N_("description value") } },
     { AccessibilityRole::DescriptionListTerm, { "description term", N_("description term") } },
     { AccessibilityRole::Directory, { "directory pane", N_("directory pane") } },
-    { AccessibilityRole::Div, { "section", N_("section") } },
     { AccessibilityRole::Document, { "document frame", N_("document frame") } },
     { AccessibilityRole::DocumentArticle, { "article", N_("article") } },
     { AccessibilityRole::DocumentMath, { "math", N_("math") } },
@@ -618,12 +646,13 @@ static constexpr std::pair<AccessibilityRole, RoleNameEntry> roleNames[] = {
     { AccessibilityRole::Footer, { "footer", N_("footer") } },
     { AccessibilityRole::Footnote, { "footnote", N_("footnote") } },
     { AccessibilityRole::Form, { "form", N_("form") } },
+    { AccessibilityRole::Generic, { "section", N_("section") } },
     { AccessibilityRole::GraphicsDocument, { "document frame", N_("document frame") } },
     { AccessibilityRole::GraphicsObject, { "panel", N_("panel") } },
     { AccessibilityRole::GraphicsSymbol, { "image", N_("image") } },
     { AccessibilityRole::Grid, { "table", N_("table") } },
     { AccessibilityRole::GridCell, { "table cell", N_("table cell") } },
-    { AccessibilityRole::Group, { "panel", N_("panel") } },
+    { AccessibilityRole::Group, { "grouping", N_("grouping") } },
     { AccessibilityRole::Heading, { "heading", N_("heading") } },
     { AccessibilityRole::HorizontalRule, { "separator", N_("separator") } },
     { AccessibilityRole::Inline, { "text", N_("text") } },
@@ -651,14 +680,12 @@ static constexpr std::pair<AccessibilityRole, RoleNameEntry> roleNames[] = {
     { AccessibilityRole::MathElement, { "math", N_("math") } },
     { AccessibilityRole::Menu, { "menu", N_("menu") } },
     { AccessibilityRole::MenuBar, { "menu bar", N_("menu bar") } },
-    { AccessibilityRole::MenuButton, { "menu item", N_("menu item") } },
     { AccessibilityRole::MenuItem, { "menu item", N_("menu item") } },
     { AccessibilityRole::MenuItemCheckbox, { "check menu item", N_("check menu item") } },
     { AccessibilityRole::MenuItemRadio, { "radio menu item", N_("radio menu item") } },
     { AccessibilityRole::MenuListPopup, { "menu", N_("menu") } },
     { AccessibilityRole::MenuListOption, { "menu item", N_("menu item") } },
     { AccessibilityRole::Meter, { "level bar", N_("level bar") } },
-    { AccessibilityRole::Outline, { "tree", N_("tree") } },
     { AccessibilityRole::Paragraph, { "paragraph", N_("paragraph") } },
     { AccessibilityRole::PopUpButton, { "combo box", N_("combo box") } },
     { AccessibilityRole::Pre, { "section", N_("section") } },
@@ -672,7 +699,6 @@ static constexpr std::pair<AccessibilityRole, RoleNameEntry> roleNames[] = {
     { AccessibilityRole::SearchField, { "entry", N_("entry") } },
     { AccessibilityRole::Slider, { "slider", N_("slider") } },
     { AccessibilityRole::SpinButton, { "spin button", N_("spin button") } },
-    { AccessibilityRole::SplitGroup, { "split pane", N_("split pane") } },
     { AccessibilityRole::Splitter, { "separator", N_("separator") } },
     { AccessibilityRole::StaticText, { "text", N_("text") } },
     { AccessibilityRole::Subscript, { "subscript", N_("subscript") } },
@@ -702,7 +728,6 @@ static constexpr std::pair<AccessibilityRole, RoleNameEntry> roleNames[] = {
     { AccessibilityRole::Video, { "video", N_("video") } },
     { AccessibilityRole::WebArea, { "document web", N_("document web") } },
     { AccessibilityRole::WebCoreLink, { "link", N_("link") } },
-    { AccessibilityRole::Window, { "window", N_("window") } }
 };
 
 const char* AccessibilityAtspi::localizedRoleName(AccessibilityRole role)
@@ -835,11 +860,20 @@ void AccessibilityAtspi::removeNotificationObserver(void* context)
     m_notificationObservers.remove(context);
 }
 
-void AccessibilityAtspi::notifyStateChanged(AccessibilityObjectAtspi& atspiObject, const char* name, bool value) const
+void AccessibilityAtspi::notify(AccessibilityObjectAtspi& atspiObject, const char* name, NotificationObserverParameter parameter) const
 {
     if (m_notificationObservers.isEmpty())
         return;
 
+    for (auto* context : copyToVector(m_notificationObservers.keys())) {
+        auto it = m_notificationObservers.find(context);
+        ASSERT(it != m_notificationObservers.end());
+        it->value(atspiObject, name, parameter);
+    }
+}
+
+void AccessibilityAtspi::notifyStateChanged(AccessibilityObjectAtspi& atspiObject, const char* name, bool value) const
+{
     auto notificationName = [&](const char* name) -> const char* {
         if (!g_strcmp0(name, "checked"))
             return "CheckedStateChanged";
@@ -870,39 +904,38 @@ void AccessibilityAtspi::notifyStateChanged(AccessibilityObjectAtspi& atspiObjec
     const char* notification = notificationName(name);
     if (!notification)
         return;
+    
+    notify(atspiObject, notification, value);
+}
 
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, notification, value);
+void AccessibilityAtspi::notifyActiveDescendantChanged(AccessibilityObjectAtspi& atspiObject) const
+{
+    notify(atspiObject, "AXActiveElementChanged", nullptr);
 }
 
 void AccessibilityAtspi::notifySelectionChanged(AccessibilityObjectAtspi& atspiObject) const
-{
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, "AXSelectedChildrenChanged", nullptr);
+{   
+    notify(atspiObject, "AXSelectedChildrenChanged", nullptr);
 }
 
 void AccessibilityAtspi::notifyMenuSelectionChanged(AccessibilityObjectAtspi& atspiObject) const
 {
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, "AXMenuItemSelected", nullptr);
+    notify(atspiObject, "AXMenuItemSelected", nullptr);
 }
 
 void AccessibilityAtspi::notifyTextChanged(AccessibilityObjectAtspi& atspiObject) const
 {
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, "AXTextChanged", nullptr);
+    notify(atspiObject, "AXTextChanged", nullptr);
 }
 
 void AccessibilityAtspi::notifyTextCaretMoved(AccessibilityObjectAtspi& atspiObject, unsigned caretOffset) const
 {
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, "AXTextCaretMoved", caretOffset);
+    notify(atspiObject, "AXTextCaretMoved", caretOffset);
 }
 
 void AccessibilityAtspi::notifyValueChanged(AccessibilityObjectAtspi& atspiObject) const
 {
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, "AXValueChanged", nullptr);
+    notify(atspiObject, "AXValueChanged", nullptr);
 }
 
 void AccessibilityAtspi::notifyLoadEvent(AccessibilityObjectAtspi& atspiObject, const CString& event) const
@@ -910,8 +943,7 @@ void AccessibilityAtspi::notifyLoadEvent(AccessibilityObjectAtspi& atspiObject,
     if (event != "LoadComplete")
         return;
 
-    for (const auto& observer : m_notificationObservers.values())
-        observer(atspiObject, "AXLoadComplete", nullptr);
+    notify(atspiObject, "AXLoadComplete", nullptr);
 }
 
 #endif
