@@ -27,25 +27,31 @@
 #include "DocumentTimeline.h"
 
 #include "AnimationEventBase.h"
+#include "AnimationTimelinesController.h"
+#include "CSSProperty.h"
 #include "CSSTransition.h"
 #include "CustomAnimationOptions.h"
 #include "CustomEffect.h"
 #include "CustomEffectCallback.h"
-#include "DeclarativeAnimation.h"
 #include "Document.h"
-#include "DocumentTimelinesController.h"
 #include "EventNames.h"
 #include "GraphicsLayer.h"
 #include "KeyframeEffect.h"
 #include "KeyframeEffectStack.h"
+#include "Logging.h"
 #include "Node.h"
 #include "Page.h"
 #include "RenderBoxModelObject.h"
 #include "RenderElement.h"
 #include "RenderLayer.h"
 #include "RenderLayerBacking.h"
+#include "StyleOriginatedAnimation.h"
 #include "WebAnimationTypes.h"
 
+#if ENABLE(THREADED_ANIMATION_RESOLUTION)
+#include "AcceleratedEffectStackUpdater.h"
+#endif
+
 namespace WebCore {
 
 Ref<DocumentTimeline> DocumentTimeline::create(Document& document)
@@ -59,22 +65,16 @@ Ref<DocumentTimeline> DocumentTimeline::create(Document& document, DocumentTimel
 }
 
 DocumentTimeline::DocumentTimeline(Document& document, Seconds originTime)
-    : AnimationTimeline()
-    , m_tickScheduleTimer(*this, &DocumentTimeline::scheduleAnimationResolution)
+    : m_tickScheduleTimer(*this, &DocumentTimeline::scheduleAnimationResolution)
     , m_document(document)
     , m_originTime(originTime)
 {
-    if (auto* controller = this->controller())
-        controller->addTimeline(*this);
+    document.ensureTimelinesController().addTimeline(*this);
 }
 
-DocumentTimeline::~DocumentTimeline()
-{
-    if (auto* controller = this->controller())
-        controller->removeTimeline(*this);
-}
+DocumentTimeline::~DocumentTimeline() = default;
 
-DocumentTimelinesController* DocumentTimeline::controller() const
+AnimationTimelinesController* DocumentTimeline::controller() const
 {
     if (m_document)
         return &m_document->ensureTimelinesController();
@@ -83,16 +83,10 @@ DocumentTimelinesController* DocumentTimeline::controller() const
 
 void DocumentTimeline::detachFromDocument()
 {
-    Ref<DocumentTimeline> protectedThis(*this);
-    if (auto* controller = this->controller())
-        controller->removeTimeline(*this);
+    AnimationTimeline::detachFromDocument();
 
     m_pendingAnimationEvents.clear();
 
-    auto& animationsToRemove = m_animations;
-    while (!animationsToRemove.isEmpty())
-        animationsToRemove.first()->remove();
-
     clearTickScheduleTimer();
     m_document = nullptr;
 }
@@ -107,8 +101,7 @@ Seconds DocumentTimeline::animationInterval() const
 
 void DocumentTimeline::suspendAnimations()
 {
-    for (const auto& animation : m_animations)
-        animation->setSuspended(true);
+    AnimationTimeline::suspendAnimations();
 
     applyPendingAcceleratedAnimations();
 
@@ -117,17 +110,11 @@ void DocumentTimeline::suspendAnimations()
 
 void DocumentTimeline::resumeAnimations()
 {
-    for (const auto& animation : m_animations)
-        animation->setSuspended(false);
+    AnimationTimeline::resumeAnimations();
 
     scheduleAnimationResolution();
 }
 
-bool DocumentTimeline::animationsAreSuspended() const
-{
-    return controller() && controller()->animationsAreSuspended();
-}
-
 unsigned DocumentTimeline::numberOfActiveAnimationsForTesting() const
 {
     unsigned count = 0;
@@ -138,7 +125,7 @@ unsigned DocumentTimeline::numberOfActiveAnimationsForTesting() const
     return count;
 }
 
-std::optional<Seconds> DocumentTimeline::currentTime()
+std::optional<WebAnimationTime> DocumentTimeline::currentTime()
 {
     if (auto* controller = this->controller()) {
         if (auto currentTime = controller->currentTime())
@@ -151,7 +138,8 @@ std::optional<Seconds> DocumentTimeline::currentTime()
 void DocumentTimeline::animationTimingDidChange(WebAnimation& animation)
 {
     AnimationTimeline::animationTimingDidChange(animation);
-    scheduleAnimationResolution();
+    if (!animation.isEffectInvalidationSuspended())
+        scheduleAnimationResolution();
 }
 
 void DocumentTimeline::removeAnimation(WebAnimation& animation)
@@ -167,8 +155,11 @@ void DocumentTimeline::scheduleAnimationResolution()
     if (animationsAreSuspended() || m_animationResolutionScheduled || !m_document || !m_document->page())
         return;
 
+    bool havePendingActivity = shouldRunUpdateAnimationsAndSendEventsIgnoringSuspensionState();
+    LOG_WITH_STREAM(Animations, stream << "DocumentTimeline " << this << " scheduleAnimationResolution - havePendingActivity " << havePendingActivity);
+
     // We need some relevant animations or pending events to proceed.
-    if (!shouldRunUpdateAnimationsAndSendEventsIgnoringSuspensionState())
+    if (!havePendingActivity)
         return;
 
     m_document->page()->scheduleRenderingUpdate(RenderingUpdateStep::Animations);
@@ -185,14 +176,14 @@ bool DocumentTimeline::shouldRunUpdateAnimationsAndSendEventsIgnoringSuspensionS
     return !m_animations.isEmpty() || !m_pendingAnimationEvents.isEmpty() || !m_acceleratedAnimationsPendingRunningStateChange.isEmpty();
 }
 
-DocumentTimeline::ShouldUpdateAnimationsAndSendEvents DocumentTimeline::documentWillUpdateAnimationsAndSendEvents()
+AnimationTimeline::ShouldUpdateAnimationsAndSendEvents DocumentTimeline::documentWillUpdateAnimationsAndSendEvents()
 {
     // Updating animations and sending events may invalidate the timing of some animations, so we must set the m_animationResolutionScheduled
     // flag to false prior to running that procedure to allow animation with timing model updates to schedule updates.
     bool wasAnimationResolutionScheduled = std::exchange(m_animationResolutionScheduled, false);
 
     if (!wasAnimationResolutionScheduled || animationsAreSuspended() || !shouldRunUpdateAnimationsAndSendEventsIgnoringSuspensionState())
-        return DocumentTimeline::ShouldUpdateAnimationsAndSendEvents::No;
+        return AnimationTimeline::ShouldUpdateAnimationsAndSendEvents::No;
 
     m_numberOfAnimationTimelineInvalidationsForTesting++;
 
@@ -201,7 +192,7 @@ DocumentTimeline::ShouldUpdateAnimationsAndSendEvents DocumentTimeline::document
     // this procedure is running should not schedule animation resolution until the event queue has been cleared.
     m_shouldScheduleAnimationResolutionForNewPendingEvents = false;
 
-    return DocumentTimeline::ShouldUpdateAnimationsAndSendEvents::Yes;
+    return AnimationTimeline::ShouldUpdateAnimationsAndSendEvents::Yes;
 }
 
 void DocumentTimeline::documentDidUpdateAnimationsAndSendEvents()
@@ -212,6 +203,11 @@ void DocumentTimeline::documentDidUpdateAnimationsAndSendEvents()
         scheduleNextTick();
 }
 
+void DocumentTimeline::styleOriginatedAnimationsWereCreated()
+{
+    scheduleAnimationResolution();
+}
+
 bool DocumentTimeline::animationCanBeRemoved(WebAnimation& animation)
 {
     // https://drafts.csswg.org/web-animations/#removing-replaced-animations
@@ -227,24 +223,31 @@ bool DocumentTimeline::animationCanBeRemoved(WebAnimation& animation)
         return false;
 
     // - has an associated animation effect whose target element is a descendant of doc, and
-    auto* effect = animation.effect();
-    if (!is<KeyframeEffect>(effect))
+    auto* keyframeEffect = dynamicDowncast<KeyframeEffect>(animation.effect());
+    if (!keyframeEffect)
         return false;
 
-    auto* keyframeEffect = downcast<KeyframeEffect>(effect);
     auto target = keyframeEffect->targetStyleable();
     if (!target || !target->element.isDescendantOf(*m_document))
         return false;
 
+IGNORE_GCC_WARNINGS_BEGIN("dangling-reference")
     auto& style = [&]() -> const RenderStyle& {
         if (auto* renderer = target->renderer())
             return renderer->style();
         return RenderStyle::defaultStyle();
     }();
+IGNORE_GCC_WARNINGS_END
 
-    HashSet<CSSPropertyID> propertiesToMatch;
-    for (auto cssProperty : keyframeEffect->animatedProperties())
-        propertiesToMatch.add(CSSProperty::resolveDirectionAwareProperty(cssProperty, style.direction(), style.writingMode()));
+    auto resolvedProperty = [&] (AnimatableCSSProperty property) -> AnimatableCSSProperty {
+        if (std::holds_alternative<CSSPropertyID>(property))
+            return CSSProperty::resolveDirectionAwareProperty(std::get<CSSPropertyID>(property), style.writingMode());
+        return property;
+    };
+
+    UncheckedKeyHashSet<AnimatableCSSProperty> propertiesToMatch;
+    for (auto property : keyframeEffect->animatedProperties())
+        propertiesToMatch.add(resolvedProperty(property));
 
     auto protectedAnimations = [&]() -> Vector<RefPtr<WebAnimation>> {
         if (auto* effectStack = target->keyframeEffectStack()) {
@@ -260,12 +263,9 @@ bool DocumentTimeline::animationCanBeRemoved(WebAnimation& animation)
             break;
 
         if (animationWithHigherCompositeOrder && animationWithHigherCompositeOrder->isReplaceable()) {
-            auto* effectWithHigherCompositeOrder = animationWithHigherCompositeOrder->effect();
-            if (is<KeyframeEffect>(effectWithHigherCompositeOrder)) {
-                auto* keyframeEffectWithHigherCompositeOrder = downcast<KeyframeEffect>(effectWithHigherCompositeOrder);
-                for (auto cssProperty : keyframeEffectWithHigherCompositeOrder->animatedProperties()) {
-                    auto resolvedProperty = CSSProperty::resolveDirectionAwareProperty(cssProperty, style.direction(), style.writingMode());
-                    if (propertiesToMatch.remove(resolvedProperty) && propertiesToMatch.isEmpty())
+            if (auto* keyframeEffectWithHigherCompositeOrder = dynamicDowncast<KeyframeEffect>(animationWithHigherCompositeOrder->effect())) {
+                for (auto property : keyframeEffectWithHigherCompositeOrder->animatedProperties()) {
+                    if (propertiesToMatch.remove(resolvedProperty(property)) && propertiesToMatch.isEmpty())
                         break;
                 }
             }
@@ -278,44 +278,60 @@ bool DocumentTimeline::animationCanBeRemoved(WebAnimation& animation)
 void DocumentTimeline::removeReplacedAnimations()
 {
     // https://drafts.csswg.org/web-animations/#removing-replaced-animations
-
-    Vector<RefPtr<WebAnimation>> animationsToRemove;
+    auto& eventNames = WebCore::eventNames();
+    Vector<Ref<WebAnimation>> animationsToRemove;
 
     // When asked to remove replaced animations for a Document, doc, then for every animation, animation
-    for (auto& animation : m_allAnimations) {
-        if (animation && animationCanBeRemoved(*animation)) {
-            // perform the following steps:
-            // 1. Set animation's replace state to removed.
-            animation->setReplaceState(WebAnimation::ReplaceState::Removed);
-            // 2. Create an AnimationPlaybackEvent, removeEvent.
-            // 3. Set removeEvent's type attribute to remove.
-            // 4. Set removeEvent's currentTime attribute to the current time of animation.
-            // 5. Set removeEvent's timelineTime attribute to the current time of the timeline with which animation is associated.
-            // 6. If animation has a document for timing, then append removeEvent to its document for timing's pending animation
-            //    event queue along with its target, animation. For the scheduled event time, use the result of applying the procedure
-            //    to convert timeline time to origin-relative time to the current time of the timeline with which animation is associated.
-            //    Otherwise, queue a task to dispatch removeEvent at animation. The task source for this task is the DOM manipulation task source.
-            animation->enqueueAnimationPlaybackEvent(eventNames().removeEvent, animation->currentTime(), currentTime());
+    for (auto& animation : m_animations) {
+        if (!animationCanBeRemoved(animation))
+            continue;
 
-            animationsToRemove.append(animation.get());
+        // perform the following steps:
+        // 1. Set animation's replace state to removed.
+        animation->setReplaceState(WebAnimation::ReplaceState::Removed);
+        // 2. Create an AnimationPlaybackEvent, removeEvent.
+        // 3. Set removeEvent's type attribute to remove.
+        // 4. Set removeEvent's currentTime attribute to the current time of animation.
+        // 5. Set removeEvent's timelineTime attribute to the current time of the timeline with which animation is associated.
+        // 6. If animation has a document for timing, then append removeEvent to its document for timing's pending animation
+        //    event queue along with its target, animation. For the scheduled event time, use the result of applying the procedure
+        //    to convert timeline time to origin-relative time to the current time of the timeline with which animation is associated.
+        //    Otherwise, queue a task to dispatch removeEvent at animation. The task source for this task is the DOM manipulation task source.
+        if (animation->hasEventListeners(eventNames.removeEvent)) {
+            auto scheduledTime = [&]() -> std::optional<Seconds> {
+                if (auto* documentTimeline = dynamicDowncast<DocumentTimeline>(animation->timeline())) {
+                    if (auto currentTime = documentTimeline->currentTime()) {
+                        ASSERT(currentTime->time());
+                        return documentTimeline->convertTimelineTimeToOriginRelativeTime(*currentTime->time());
+                    }
+                }
+                return std::nullopt;
+            }();
+            auto animationCurrentTime = [&]() -> std::optional<Seconds> {
+                if (auto animationTime = animation->currentTime())
+                    return animationTime->time();
+                return std::nullopt;
+            }();
+            animation->enqueueAnimationPlaybackEvent(eventNames.removeEvent, animationCurrentTime, scheduledTime);
         }
+
+        animationsToRemove.append(animation.get());
     }
 
     for (auto& animation : animationsToRemove) {
         if (auto* timeline = animation->timeline())
-            timeline->removeAnimation(*animation);
+            timeline->removeAnimation(animation);
     }
 }
 
-void DocumentTimeline::transitionDidComplete(RefPtr<CSSTransition> transition)
+void DocumentTimeline::transitionDidComplete(Ref<CSSTransition>&& transition)
 {
-    ASSERT(transition);
-    removeAnimation(*transition);
-    if (is<KeyframeEffect>(transition->effect())) {
-        if (auto styleable = downcast<KeyframeEffect>(transition->effect())->targetStyleable()) {
+    removeAnimation(transition.get());
+    if (auto* keyframeEffect = dynamicDowncast<KeyframeEffect>(transition->effect())) {
+        if (auto styleable = keyframeEffect->targetStyleable()) {
             auto property = transition->property();
             if (styleable->hasRunningTransitionForProperty(property))
-                styleable->ensureCompletedTransitionsByProperty().set(property, transition);
+                styleable->ensureCompletedTransitionsByProperty().set(property, WTFMove(transition));
         }
     }
 }
@@ -342,6 +358,8 @@ void DocumentTimeline::scheduleNextTick()
         return std::nullopt;
     };
 
+    auto animationInterval = this->animationInterval();
+    
     for (const auto& animation : m_animations) {
         if (!animation->isRelevant())
             continue;
@@ -355,7 +373,7 @@ void DocumentTimeline::scheduleNextTick()
             // Now let's get the time until any animation with this animation's frame rate would tick.
             // If that time is longer than what we previously computed without accounting for the frame
             // rate, we use this time instead since our animation wouldn't tick anyway since the
-            // DocumentTimelinesController would ignore it. Doing this ensures that we don't schedule
+            // AnimationTimelinesController would ignore it. Doing this ensures that we don't schedule
             // updates that wouldn't actually yield any work and guarantees that in a page with only
             // animations as the source for scheduling updates, updates are only scheduled at the minimal
             // frame rate.
@@ -364,15 +382,19 @@ void DocumentTimeline::scheduleNextTick()
                 animationTimeToNextRequiredTick = *timeToNextPossibleTickAccountingForFrameRate - nextTickTimeEpsilon;
         }
 
-        if (animationTimeToNextRequiredTick < animationInterval()) {
+        if (animationTimeToNextRequiredTick < animationInterval) {
+            LOG_WITH_STREAM(Animations, stream << "DocumentTimeline " << this << " scheduleNextTick - scheduleAnimationResolution now");
             scheduleAnimationResolution();
             return;
         }
         scheduleDelay = std::min(scheduleDelay, animationTimeToNextRequiredTick);
     }
 
-    if (scheduleDelay < Seconds::infinity())
+    if (scheduleDelay < Seconds::infinity()) {
+        scheduleDelay = std::max(scheduleDelay - animationInterval, 0_s);
+        LOG_WITH_STREAM(Animations, stream << "DocumentTimeline " << this << " scheduleNextTick - scheduleDelay " << scheduleDelay);
         m_tickScheduleTimer.startOneShot(scheduleDelay);
+    }
 }
 
 void DocumentTimeline::animationAcceleratedRunningStateDidChange(WebAnimation& animation)
@@ -387,35 +409,45 @@ void DocumentTimeline::animationAcceleratedRunningStateDidChange(WebAnimation& a
 
 void DocumentTimeline::applyPendingAcceleratedAnimations()
 {
-    auto acceleratedAnimationsPendingRunningStateChange = m_acceleratedAnimationsPendingRunningStateChange;
-    m_acceleratedAnimationsPendingRunningStateChange.clear();
+#if ENABLE(THREADED_ANIMATION_RESOLUTION)
+    if (m_document && m_document->settings().threadedAnimationResolutionEnabled()) {
+        m_acceleratedAnimationsPendingRunningStateChange.clear();
+        if (CheckedPtr timelinesController = m_document->timelinesController()) {
+            if (auto* acceleratedEffectStackUpdater = timelinesController->existingAcceleratedEffectStackUpdater())
+                acceleratedEffectStackUpdater->updateEffectStacks();
+        }
+        return;
+    }
+#endif
 
-    // Animations may fail to run accelerated for reasons private to GraphicsLayerCA. If that happens, and the animation
-    // in question targets a transform-related property, we must prevent all other transform-related animations for this
-    // element to run accelerated since we can't run some transform-related animations accelerated, and some not. To do
-    // this, we keep a list of all KeyframeEffectStack objects containing an effect that failed to start a transform-related
-    // animation so that we can return any transform-related accelerated animation to run non-accelerated.
-    HashSet<KeyframeEffectStack*> effectStacksContainingEffectThatFailedToRunAcceleratedTransformRelatedAnimation;
+    auto acceleratedAnimationsPendingRunningStateChange = std::exchange(m_acceleratedAnimationsPendingRunningStateChange, { });
+
+    UncheckedKeyHashSet<KeyframeEffectStack*> keyframeEffectStacksToUpdate;
 
     bool hasForcedLayout = false;
     for (auto& animation : acceleratedAnimationsPendingRunningStateChange) {
-        auto* effect = animation->effect();
-        if (!is<KeyframeEffect>(effect))
-            continue;
+        if (auto* keyframeEffect = dynamicDowncast<KeyframeEffect>(animation->effect())) {
+            if (!hasForcedLayout)
+                hasForcedLayout |= keyframeEffect->forceLayoutIfNeeded();
 
-        auto& keyframeEffect = downcast<KeyframeEffect>(*effect);
-        if (!hasForcedLayout)
-            hasForcedLayout |= keyframeEffect.forceLayoutIfNeeded();
-        auto pendingAccelerationActionResult = keyframeEffect.applyPendingAcceleratedActions();
-        if (pendingAccelerationActionResult.contains(AcceleratedActionApplicationResult::TransformRelatedAnimationCannotBeAccelerated)) {
-            ASSERT(keyframeEffect.targetStyleable());
-            ASSERT(keyframeEffect.targetStyleable()->keyframeEffectStack());
-            effectStacksContainingEffectThatFailedToRunAcceleratedTransformRelatedAnimation.add(keyframeEffect.targetStyleable()->keyframeEffectStack());
+            // If the animation is no longer relevant, apply the pending accelerations in isolation.
+            if (!animation->isRelevant()) {
+                keyframeEffect->applyPendingAcceleratedActions();
+                continue;
+            }
+
+            // Otherwise, this accelerated animation exists in the context of an effect stack and we must
+            // update all effects in that stack to ensure their sort order is respected.
+            if (auto target = keyframeEffect->targetStyleable()) {
+                auto* keyframeEffectStack = target ? target->keyframeEffectStack() : nullptr;
+                if (keyframeEffectStack)
+                    keyframeEffectStacksToUpdate.add(keyframeEffectStack);
+            }
         }
     }
 
-    for (auto& effectStack : effectStacksContainingEffectThatFailedToRunAcceleratedTransformRelatedAnimation)
-        effectStack->stopAcceleratingTransformRelatedProperties(UseAcceleratedAction::No);
+    for (auto* keyframeEffectStack : keyframeEffectStacksToUpdate)
+        keyframeEffectStack->applyPendingAcceleratedActions();
 }
 
 void DocumentTimeline::enqueueAnimationEvent(AnimationEventBase& event)
@@ -425,6 +457,13 @@ void DocumentTimeline::enqueueAnimationEvent(AnimationEventBase& event)
         scheduleAnimationResolution();
 }
 
+bool DocumentTimeline::hasPendingAnimationEventForAnimation(const WebAnimation& animation) const
+{
+    return m_pendingAnimationEvents.containsIf([&](auto& event) {
+        return event->animation() == &animation;
+    });
+}
+
 AnimationEvents DocumentTimeline::prepareForPendingAnimationEventsDispatch()
 {
     m_shouldScheduleAnimationResolutionForNewPendingEvents = true;
@@ -433,11 +472,12 @@ AnimationEvents DocumentTimeline::prepareForPendingAnimationEventsDispatch()
 
 Vector<std::pair<String, double>> DocumentTimeline::acceleratedAnimationsForElement(Element& element) const
 {
+    ASSERT(m_document);
     auto* renderer = element.renderer();
     if (renderer && renderer->isComposited()) {
         auto* compositedRenderer = downcast<RenderBoxModelObject>(renderer);
         if (auto* graphicsLayer = compositedRenderer->layer()->backing()->graphicsLayer())
-            return graphicsLayer->acceleratedAnimationsForTesting();
+            return graphicsLayer->acceleratedAnimationsForTesting(m_document->settings());
     }
     return { };
 }
@@ -450,9 +490,9 @@ unsigned DocumentTimeline::numberOfAnimationTimelineInvalidationsForTesting() co
 ExceptionOr<Ref<WebAnimation>> DocumentTimeline::animate(Ref<CustomEffectCallback>&& callback, std::optional<std::variant<double, CustomAnimationOptions>>&& options)
 {
     if (!m_document)
-        return Exception { InvalidStateError };
+        return Exception { ExceptionCode::InvalidStateError };
 
-    String id = "";
+    String id = emptyString();
     std::variant<FramesPerSecond, AnimationFrameRatePreset> frameRate = AnimationFrameRatePreset::Auto;
     std::optional<std::variant<double, EffectTiming>> customEffectOptions;
 
@@ -469,12 +509,12 @@ ExceptionOr<Ref<WebAnimation>> DocumentTimeline::animate(Ref<CustomEffectCallbac
         customEffectOptions = customEffectOptionsVariant;
     }
 
-    auto customEffectResult = CustomEffect::create(WTFMove(callback), WTFMove(customEffectOptions));
+    auto customEffectResult = CustomEffect::create(*m_document, WTFMove(callback), WTFMove(customEffectOptions));
     if (customEffectResult.hasException())
         return customEffectResult.releaseException();
 
     auto animation = WebAnimation::create(*document(), &customEffectResult.returnValue().get());
-    animation->setId(id);
+    animation->setId(WTFMove(id));
     animation->setBindingsFrameRate(WTFMove(frameRate));
 
     auto animationPlayResult = animation->play();
@@ -491,4 +531,10 @@ std::optional<FramesPerSecond> DocumentTimeline::maximumFrameRate() const
     return m_document->page()->displayNominalFramesPerSecond();
 }
 
+Seconds DocumentTimeline::convertTimelineTimeToOriginRelativeTime(Seconds timelineTime) const
+{
+    // https://drafts.csswg.org/web-animations-1/#ref-for-timeline-time-to-origin-relative-time
+    return timelineTime + m_originTime;
+}
+
 } // namespace WebCore
