@@ -62,7 +62,7 @@ struct ReverseSuffixTreeVertex {
     ReverseSuffixTreeEdges edges;
     uint32_t nodeId;
 };
-typedef HashMap<HashableActionList, ReverseSuffixTreeVertex, HashableActionListHash, HashableActionListHashTraits> ReverseSuffixTreeRoots;
+typedef UncheckedKeyHashMap<HashableActionList, ReverseSuffixTreeVertex, HashableActionListHash, HashableActionListHashTraits> ReverseSuffixTreeRoots;
 
 #if CONTENT_EXTENSIONS_PERFORMANCE_REPORTING
 static size_t recursiveMemoryUsed(const PrefixTreeVertex& vertex)
@@ -87,19 +87,19 @@ size_t CombinedURLFilters::memoryUsed() const
     return sizeof(CombinedURLFilters)
         + m_alphabet.memoryUsed()
         + recursiveMemoryUsed(*m_prefixTreeRoot.get())
-        + sizeof(HashMap<PrefixTreeVertex*, ActionList>)
+        + sizeof(UncheckedKeyHashMap<PrefixTreeVertex*, ActionList>)
         + m_actions.capacity() * (sizeof(PrefixTreeVertex*) + sizeof(ActionList))
         + actionsSize;
 }
 #endif
     
 #if CONTENT_EXTENSIONS_STATE_MACHINE_DEBUGGING
-static String prefixTreeVertexToString(const PrefixTreeVertex& vertex, const HashMap<const PrefixTreeVertex*, ActionList>& actions, unsigned depth)
+static String prefixTreeVertexToString(const PrefixTreeVertex& vertex, const UncheckedKeyHashMap<const PrefixTreeVertex*, ActionList>& actions, unsigned depth)
 {
     StringBuilder builder;
     while (depth--)
-        builder.append("  ");
-    builder.append("vertex actions: ");
+        builder.append("  "_s);
+    builder.append("vertex actions: "_s);
 
     auto actionsSlot = actions.find(&vertex);
     if (actionsSlot != actions.end()) {
@@ -110,14 +110,14 @@ static String prefixTreeVertexToString(const PrefixTreeVertex& vertex, const Has
     return builder.toString();
 }
 
-static void recursivePrint(const PrefixTreeVertex& vertex, const HashMap<const PrefixTreeVertex*, ActionList>& actions, unsigned depth)
+static void recursivePrint(const PrefixTreeVertex& vertex, const UncheckedKeyHashMap<const PrefixTreeVertex*, ActionList>& actions, unsigned depth)
 {
     dataLogF("%s", prefixTreeVertexToString(vertex, actions, depth).utf8().data());
     for (const auto& edge : vertex.edges) {
         StringBuilder builder;
         for (unsigned i = 0; i < depth * 2; ++i)
             builder.append(' ');
-        builder.append("vertex edge: ", edge.term->toString(), '\n');
+        builder.append("vertex edge: "_s, edge.term->toString(), '\n');
         dataLogF("%s", builder.toString().utf8().data());
         ASSERT(edge.child);
         recursivePrint(*edge.child.get(), actions, depth + 1);
@@ -186,7 +186,7 @@ struct ActiveSubtree {
     unsigned edgeIndex;
 };
 
-static void generateInfixUnsuitableForReverseSuffixTree(NFA& nfa, Vector<ActiveSubtree>& stack, const HashMap<const PrefixTreeVertex*, ActionList>& actions)
+static void generateInfixUnsuitableForReverseSuffixTree(NFA& nfa, Vector<ActiveSubtree>& stack, const UncheckedKeyHashMap<const PrefixTreeVertex*, ActionList>& actions)
 {
     // To avoid conflicts, we use the reverse suffix tree for subtrees that do not merge
     // in the prefix tree.
@@ -242,7 +242,7 @@ static void generateInfixUnsuitableForReverseSuffixTree(NFA& nfa, Vector<ActiveS
     }
 }
 
-static void generateSuffixWithReverseSuffixTree(NFA& nfa, Vector<ActiveSubtree>& stack, const HashMap<const PrefixTreeVertex*, ActionList>& actions, ReverseSuffixTreeRoots& reverseSuffixTreeRoots)
+static void generateSuffixWithReverseSuffixTree(NFA& nfa, Vector<ActiveSubtree>& stack, const UncheckedKeyHashMap<const PrefixTreeVertex*, ActionList>& actions, ReverseSuffixTreeRoots& reverseSuffixTreeRoots)
 {
     ActiveSubtree& leafSubtree = stack.last();
     ASSERT_WITH_MESSAGE(!leafSubtree.nfaNode.isValid(), "The leaf should never be generated by the code above, it should always be inserted into the prefix tree.");
@@ -333,7 +333,7 @@ static void clearReverseSuffixTree(ReverseSuffixTreeRoots& reverseSuffixTreeRoot
     reverseSuffixTreeRoots.clear();
 }
 
-static void generateNFAForSubtree(NFA& nfa, ImmutableCharNFANodeBuilder&& subtreeRoot, PrefixTreeVertex& root, const HashMap<const PrefixTreeVertex*, ActionList>& actions, size_t maxNFASize)
+static void generateNFAForSubtree(NFA& nfa, ImmutableCharNFANodeBuilder&& subtreeRoot, PrefixTreeVertex& root, const UncheckedKeyHashMap<const PrefixTreeVertex*, ActionList>& actions, size_t maxNFASize)
 {
     // This recurses the subtree of the prefix tree.
     // For each edge that has fixed length (no quantifiers like ?, *, or +) it generates the nfa graph,
