@@ -35,8 +35,10 @@
 #include "ContentExtensionRule.h"
 #include "ContentExtensionsBackend.h"
 #include "ContentExtensionsDebugging.h"
+#include "ProcessWarming.h"
 #include <wtf/Expected.h>
 #include <wtf/JSONValues.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/WTFString.h>
 
 namespace WebCore::ContentExtensions {
@@ -58,7 +60,7 @@ static Expected<Vector<String>, std::error_code> getStringList(const JSON::Array
         String string = value->asString();
         if (string.isEmpty())
             return makeUnexpected(ContentExtensionError::JSONInvalidConditionList);
-        strings.uncheckedAppend(string);
+        strings.append(string);
     }
     return strings;
 }
@@ -96,9 +98,9 @@ static Expected<Vector<String>, std::error_code> getDomainList(const JSON::Array
         for (auto& pair : escapeTable)
             domain = makeStringByReplacingAll(domain, pair.first, pair.second);
 
-        const char* protocolRegex = "[a-z][a-z+.-]*:\\/\\/";
-        const char* allowSubdomainsRegex = "([^/]*\\.)*";
-        regexes.uncheckedAppend(makeString(protocolRegex, allowSubdomains ? allowSubdomainsRegex : "", domain, "[:/]"));
+        constexpr auto protocolRegex = "[a-z][a-z+.-]*:\\/\\/"_s;
+        constexpr auto allowSubdomainsRegex = "([^/]*\\.)*"_s;
+        regexes.append(makeString(protocolRegex, allowSubdomains ? allowSubdomainsRegex : ""_s, domain, "[:/]"_s));
     }
     return regexes;
 }
@@ -197,6 +199,9 @@ static Expected<Trigger, std::error_code> loadTrigger(const JSON::Object& ruleOb
     if (auto error = checkCondition("if-frame-url"_s, getStringList, ActionCondition::IfFrameURL))
         return makeUnexpected(error);
 
+    if (auto error = checkCondition("unless-frame-url"_s, getStringList, ActionCondition::UnlessFrameURL))
+        return makeUnexpected(error);
+
     trigger.checkValidity();
     return trigger;
 }
@@ -204,22 +209,19 @@ static Expected<Trigger, std::error_code> loadTrigger(const JSON::Object& ruleOb
 bool isValidCSSSelector(const String& selector)
 {
     ASSERT(isMainThread());
-    initializeCommonAtomStrings();
-    QualifiedName::init();
+    ProcessWarming::initializeNames();
 
     // This explicitly does not use the CSSParserContext created in contentExtensionCSSParserContext because
     // we want to use quirks mode in parsing, but automatic mode when actually applying the content blocker styles.
     // FIXME: rdar://105733691 (Parse/apply content blocker style sheets in both standards and quirks mode lazily).
     WebCore::CSSParserContext context(HTMLQuirksMode);
-    context.hasPseudoClassEnabled = true;
     CSSParser parser(context);
-    return !!parser.parseSelector(selector);
+    return !!parser.parseSelectorList(selector);
 }
 
 WebCore::CSSParserContext contentExtensionCSSParserContext()
 {
     WebCore::CSSParserContext context(HTMLStandardMode);
-    context.hasPseudoClassEnabled = true;
     return context;
 }
 
@@ -310,7 +312,7 @@ static Expected<Vector<ContentExtensionRule>, std::error_code> loadEncodedRules(
             continue;
         if (!rule->has_value())
             return makeUnexpected(rule->error());
-        ruleList.uncheckedAppend(WTFMove(rule->value()));
+        ruleList.append(WTFMove(rule->value()));
     }
 
     return ruleList;
