@@ -25,34 +25,38 @@
 
 #pragma once
 
-#if ENABLE(CSS_TYPED_OM)
-
 #include "CSSNumericValue.h"
 #include "CSSTransformComponent.h"
 
 namespace WebCore {
 
+class CSSFunctionValue;
+
 template<typename> class ExceptionOr;
 
 class CSSSkew : public CSSTransformComponent {
-    WTF_MAKE_ISO_ALLOCATED(CSSSkew);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(CSSSkew);
 public:
     static ExceptionOr<Ref<CSSSkew>> create(Ref<CSSNumericValue>, Ref<CSSNumericValue>);
-    
+    static ExceptionOr<Ref<CSSSkew>> create(Ref<const CSSFunctionValue>);
+
     const CSSNumericValue& ax() const { return m_ax.get(); }
     const CSSNumericValue& ay() const { return m_ay.get(); }
     
-    void setAx(Ref<CSSNumericValue> ax) { m_ax = WTFMove(ax); }
-    void setAy(Ref<CSSNumericValue> ay) { m_ay = WTFMove(ay); }
+    ExceptionOr<void> setAx(Ref<CSSNumericValue>);
+    ExceptionOr<void> setAy(Ref<CSSNumericValue>);
 
     void serialize(StringBuilder&) const final;
     ExceptionOr<Ref<DOMMatrix>> toMatrix() final;
-    
+    void setIs2D(bool) final { };
+
     CSSTransformType getType() const final { return CSSTransformType::Skew; }
 
+    RefPtr<CSSValue> toCSSValue() const final;
+
 private:
     CSSSkew(Ref<CSSNumericValue> ax, Ref<CSSNumericValue> ay);
-    
+
     Ref<CSSNumericValue> m_ax;
     Ref<CSSNumericValue> m_ay;
 };
@@ -62,5 +66,3 @@ private:
 SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::CSSSkew)
     static bool isType(const WebCore::CSSTransformComponent& transform) { return transform.getType() == WebCore::CSSTransformType::Skew; }
 SPECIALIZE_TYPE_TRAITS_END()
-
-#endif
