@@ -44,9 +44,11 @@ private:
     void doUnapply() override;
 
 #ifndef NDEBUG
-    void getNodesInCommand(HashSet<Ref<Node>>&) override;
+    void getNodesInCommand(NodeSet&) override;
 #endif
 
+    Ref<Element> protectedElement() const { return m_element; }
+
     Ref<Element> m_element;
     QualifiedName m_attribute;
     AtomString m_value;
