@@ -1,5 +1,6 @@
 /*
- * Copyright (C) 2004-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2004-2022 Apple Inc. All rights reserved.
+ * Copyright (C) 2014 Google Inc. All rights reserved.
  * Portions Copyright (c) 2011 Motorola Mobility, Inc.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,8 +37,9 @@
 #include "HTMLHtmlElement.h"
 #include "HTMLNames.h"
 #include "InlineIteratorBox.h"
-#include "InlineIteratorLine.h"
+#include "InlineIteratorLineBox.h"
 #include "InlineRunAndOffset.h"
+#include "LineSelection.h"
 #include "Logging.h"
 #include "Range.h"
 #include "RenderBlockFlow.h"
@@ -47,6 +49,7 @@
 #include "VisibleUnits.h"
 #include <stdio.h>
 #include <wtf/text/CString.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/TextStream.h>
 
 namespace WebCore {
@@ -125,130 +128,130 @@ Position VisiblePosition::leftVisuallyDistinctCandidate() const
     InlineIterator::LineLogicalOrderCache orderCache;
 
     while (true) {
-        auto [run, offset] = p.inlineRunAndOffset(m_affinity, primaryDirection);
-        if (!run)
+        auto [box, offset] = p.inlineBoxAndOffset(m_affinity, primaryDirection);
+        if (!box)
             return primaryDirection == TextDirection::LTR ? previousVisuallyDistinctCandidate(m_deepPosition) : nextVisuallyDistinctCandidate(m_deepPosition);
 
-        auto* renderer = &run->renderer();
+        CheckedPtr renderer = &box->renderer();
 
         while (true) {
-            if ((renderer->isReplacedOrInlineBlock() || renderer->isBR()) && offset == run->rightmostCaretOffset())
-                return run->isLeftToRightDirection() ? previousVisuallyDistinctCandidate(m_deepPosition) : nextVisuallyDistinctCandidate(m_deepPosition);
+            if ((renderer->isReplacedOrAtomicInline() || renderer->isBR()) && offset == box->rightmostCaretOffset())
+                return box->isLeftToRightDirection() ? previousVisuallyDistinctCandidate(m_deepPosition) : nextVisuallyDistinctCandidate(m_deepPosition);
 
             if (!renderer->node()) {
-                run.traversePreviousOnLine();
-                if (!run)
+                box.traverseLineLeftwardOnLine();
+                if (!box)
                     return primaryDirection == TextDirection::LTR ? previousVisuallyDistinctCandidate(m_deepPosition) : nextVisuallyDistinctCandidate(m_deepPosition);
-                renderer = &run->renderer();
-                offset = run->rightmostCaretOffset();
+                renderer = &box->renderer();
+                offset = box->rightmostCaretOffset();
                 continue;
             }
 
             // Note that this may underflow the (unsigned) offset. This is fine and handled below.
-            offset = run->isLeftToRightDirection() ? renderer->previousOffset(offset) : renderer->nextOffset(offset);
+            offset = box->isLeftToRightDirection() ? renderer->previousOffset(offset) : renderer->nextOffset(offset);
 
-            auto caretMinOffset = run->minimumCaretOffset();
-            auto caretMaxOffset = run->maximumCaretOffset();
+            auto caretMinOffset = box->minimumCaretOffset();
+            auto caretMaxOffset = box->maximumCaretOffset();
 
             if (offset > caretMinOffset && offset < caretMaxOffset)
                 break;
 
-            if (offset != run->leftmostCaretOffset()) {
+            if (offset != box->leftmostCaretOffset()) {
                 // Overshot to the left.
-                auto previousRun = run->previousOnLineIgnoringLineBreak();
-                if (!previousRun) {
+                auto previousBox = box->nextLineLeftwardOnLineIgnoringLineBreak();
+                if (!previousBox) {
                     Position positionOnLeft = primaryDirection == TextDirection::LTR ? previousVisuallyDistinctCandidate(m_deepPosition) : nextVisuallyDistinctCandidate(m_deepPosition);
-                    auto runOnLeft = positionOnLeft.inlineRunAndOffset(m_affinity, primaryDirection).run;
-                    if (runOnLeft && runOnLeft->line() == run->line())
+                    auto boxOnLeft = positionOnLeft.inlineBoxAndOffset(m_affinity, primaryDirection).box;
+                    if (boxOnLeft && boxOnLeft->lineBox() == box->lineBox())
                         return Position();
                     return positionOnLeft;
                 }
 
                 // Reposition at the other logical position corresponding to our edge's visual position and go for another round.
-                run = previousRun;
-                renderer = &run->renderer();
-                offset = previousRun->rightmostCaretOffset();
+                box = previousBox;
+                renderer = &box->renderer();
+                offset = previousBox->rightmostCaretOffset();
                 continue;
             }
 
-            unsigned char level = run->bidiLevel();
-            auto previousRun = run->previousOnLine();
+            unsigned char level = box->bidiLevel();
+            auto previousBox = box->nextLineLeftwardOnLine();
 
-            if (run->direction() == primaryDirection) {
-                if (!previousRun) {
+            if (box->direction() == primaryDirection) {
+                if (!previousBox) {
                     auto logicalStart = primaryDirection == TextDirection::LTR
-                        ? InlineIterator::firstLeafOnLineInLogicalOrderWithNode(run->line(), orderCache)
-                        : InlineIterator::lastLeafOnLineInLogicalOrderWithNode(run->line(), orderCache);
+                        ? InlineIterator::firstLeafOnLineInLogicalOrderWithNode(box->lineBox(), orderCache)
+                        : InlineIterator::lastLeafOnLineInLogicalOrderWithNode(box->lineBox(), orderCache);
                     if (logicalStart) {
-                        run = logicalStart;
-                        renderer = &run->renderer();
-                        offset = primaryDirection == TextDirection::LTR ? run->minimumCaretOffset() : run->maximumCaretOffset();
+                        box = logicalStart;
+                        renderer = &box->renderer();
+                        offset = primaryDirection == TextDirection::LTR ? box->minimumCaretOffset() : box->maximumCaretOffset();
                     }
                     break;
                 }
-                if (previousRun->bidiLevel() >= level)
+                if (previousBox->bidiLevel() >= level)
                     break;
 
-                level = previousRun->bidiLevel();
+                level = previousBox->bidiLevel();
 
-                auto nextRun = run;
+                auto nextBox = box;
                 do {
-                    nextRun.traverseNextOnLine();
-                } while (nextRun && nextRun->bidiLevel() > level);
+                    nextBox.traverseLineRightwardOnLine();
+                } while (nextBox && nextBox->bidiLevel() > level);
 
-                if (nextRun && nextRun->bidiLevel() == level)
+                if (nextBox && nextBox->bidiLevel() == level)
                     break;
 
-                run = previousRun;
-                renderer = &run->renderer();
-                offset = run->rightmostCaretOffset();
-                if (run->direction() == primaryDirection)
+                box = previousBox;
+                renderer = &box->renderer();
+                offset = box->rightmostCaretOffset();
+                if (box->direction() == primaryDirection)
                     break;
                 continue;
             }
 
-            while (previousRun && !previousRun->renderer().node())
-                previousRun.traversePreviousOnLine();
+            while (previousBox && !previousBox->renderer().node())
+                previousBox.traverseLineLeftwardOnLine();
 
-            if (previousRun) {
-                run = previousRun;
-                renderer = &run->renderer();
-                offset = run->rightmostCaretOffset();
-                if (run->bidiLevel() > level) {
+            if (previousBox) {
+                box = previousBox;
+                renderer = &box->renderer();
+                offset = box->rightmostCaretOffset();
+                if (box->bidiLevel() > level) {
                     do {
-                        previousRun = previousRun.traversePreviousOnLine();
-                    } while (previousRun && previousRun->bidiLevel() > level);
+                        previousBox = previousBox.traverseLineLeftwardOnLine();
+                    } while (previousBox && previousBox->bidiLevel() > level);
 
-                    if (!previousRun || previousRun->bidiLevel() < level)
+                    if (!previousBox || previousBox->bidiLevel() < level)
                         continue;
                 }
             } else {
-                // Trailing edge of a secondary run. Set to the leading edge of the entire run.
+                // Trailing edge of a secondary box. Set to the leading edge of the entire box.
                 while (true) {
-                    while (auto nextRun = run->nextOnLine()) {
-                        if (nextRun->bidiLevel() < level)
+                    while (auto nextBox = box->nextLineRightwardOnLine()) {
+                        if (nextBox->bidiLevel() < level)
                             break;
-                        run = nextRun;
+                        box = nextBox;
                     }
-                    if (run->bidiLevel() == level)
+                    if (box->bidiLevel() == level)
                         break;
-                    level = run->bidiLevel();
-                    while (auto previousRun = run->previousOnLine()) {
-                        if (previousRun->bidiLevel() < level)
+                    level = box->bidiLevel();
+                    while (auto previousBox = box->nextLineLeftwardOnLine()) {
+                        if (previousBox->bidiLevel() < level)
                             break;
-                        run = previousRun;
+                        box = previousBox;
                     }
-                    if (run->bidiLevel() == level)
+                    if (box->bidiLevel() == level)
                         break;
-                    level = run->bidiLevel();
+                    level = box->bidiLevel();
                 }
-                renderer = &run->renderer();
-                offset = primaryDirection == TextDirection::LTR ? run->minimumCaretOffset() : run->maximumCaretOffset();
+                renderer = &box->renderer();
+                offset = primaryDirection == TextDirection::LTR ? box->minimumCaretOffset() : box->maximumCaretOffset();
             }
             break;
         }
 
-        p = makeDeprecatedLegacyPosition(renderer->node(), offset);
+        p = makeDeprecatedLegacyPosition(renderer->protectedNode().get(), offset);
 
         if ((p.isCandidate() && p.downstream() != downstreamStart) || p.atStartOfTree() || p.atEndOfTree())
             return p;
@@ -275,8 +278,7 @@ VisiblePosition VisiblePosition::left(bool stayInEditableContent, bool* reachedB
     if (!stayInEditableContent)
         return left;
 
-    // FIXME: This may need to do something different from "before".
-    return honorEditingBoundaryAtOrBefore(left, reachedBoundary);
+    return directionOfEnclosingBlock(left.deepEquivalent()) == TextDirection::LTR ? honorEditingBoundaryAtOrBefore(left, reachedBoundary) : honorEditingBoundaryAtOrAfter(left, reachedBoundary);
 }
 
 Position VisiblePosition::rightVisuallyDistinctCandidate() const
@@ -291,134 +293,134 @@ Position VisiblePosition::rightVisuallyDistinctCandidate() const
     InlineIterator::LineLogicalOrderCache orderCache;
 
     while (true) {
-        auto [run, offset] = p.inlineRunAndOffset(m_affinity, primaryDirection);
-        if (!run)
+        auto [box, offset] = p.inlineBoxAndOffset(m_affinity, primaryDirection);
+        if (!box)
             return primaryDirection == TextDirection::LTR ? nextVisuallyDistinctCandidate(m_deepPosition) : previousVisuallyDistinctCandidate(m_deepPosition);
 
-        auto* renderer = &run->renderer();
+        CheckedPtr renderer = &box->renderer();
 
         while (true) {
-            if ((renderer->isReplacedOrInlineBlock() || renderer->isBR()) && offset == run->leftmostCaretOffset())
-                return run->isLeftToRightDirection() ? nextVisuallyDistinctCandidate(m_deepPosition) : previousVisuallyDistinctCandidate(m_deepPosition);
+            if ((renderer->isReplacedOrAtomicInline() || renderer->isBR()) && offset == box->leftmostCaretOffset())
+                return box->isLeftToRightDirection() ? nextVisuallyDistinctCandidate(m_deepPosition) : previousVisuallyDistinctCandidate(m_deepPosition);
 
             if (!renderer->node()) {
-                run.traverseNextOnLine();
-                if (!run)
+                box.traverseLineRightwardOnLine();
+                if (!box)
                     return primaryDirection == TextDirection::LTR ? nextVisuallyDistinctCandidate(m_deepPosition) : previousVisuallyDistinctCandidate(m_deepPosition);
-                renderer = &run->renderer();
-                offset = run->leftmostCaretOffset();
+                renderer = &box->renderer();
+                offset = box->leftmostCaretOffset();
                 continue;
             }
 
             // Note that this may underflow the (unsigned) offset. This is fine and handled below.
-            offset = run->isLeftToRightDirection() ? renderer->nextOffset(offset) : renderer->previousOffset(offset);
+            offset = box->isLeftToRightDirection() ? renderer->nextOffset(offset) : renderer->previousOffset(offset);
 
-            auto caretMinOffset = run->minimumCaretOffset();
-            auto caretMaxOffset = run->maximumCaretOffset();
+            auto caretMinOffset = box->minimumCaretOffset();
+            auto caretMaxOffset = box->maximumCaretOffset();
 
             if (offset > caretMinOffset && offset < caretMaxOffset)
                 break;
 
-            if (offset != run->rightmostCaretOffset()) {
+            if (offset != box->rightmostCaretOffset()) {
                 // Overshot to the right.
-                auto nextRun = run->nextOnLineIgnoringLineBreak();
-                if (!nextRun) {
+                auto nextBox = box->nextLineRightwardOnLineIgnoringLineBreak();
+                if (!nextBox) {
                     Position positionOnRight = primaryDirection == TextDirection::LTR ? nextVisuallyDistinctCandidate(m_deepPosition) : previousVisuallyDistinctCandidate(m_deepPosition);
-                    auto runOnRight = positionOnRight.inlineRunAndOffset(m_affinity, primaryDirection).run;
-                    if (runOnRight && runOnRight->line() == run->line())
+                    auto boxOnRight = positionOnRight.inlineBoxAndOffset(m_affinity, primaryDirection).box;
+                    if (boxOnRight && boxOnRight->lineBox() == box->lineBox())
                         return Position();
                     return positionOnRight;
                 }
 
                 // Reposition at the other logical position corresponding to our edge's visual position and go for another round.
-                run = nextRun;
-                renderer = &run->renderer();
-                offset = nextRun->leftmostCaretOffset();
+                box = nextBox;
+                renderer = &box->renderer();
+                offset = nextBox->leftmostCaretOffset();
                 continue;
             }
 
-            unsigned char level = run->bidiLevel();
-            auto nextRun = run->nextOnLine();
+            unsigned char level = box->bidiLevel();
+            auto nextBox = box->nextLineRightwardOnLine();
 
-            if (run->direction() == primaryDirection) {
-                if (!nextRun) {
+            if (box->direction() == primaryDirection) {
+                if (!nextBox) {
                     auto logicalEnd = primaryDirection == TextDirection::LTR
-                        ? InlineIterator::lastLeafOnLineInLogicalOrderWithNode(run->line(), orderCache)
-                        : InlineIterator::firstLeafOnLineInLogicalOrderWithNode(run->line(), orderCache);
+                        ? InlineIterator::lastLeafOnLineInLogicalOrderWithNode(box->lineBox(), orderCache)
+                        : InlineIterator::firstLeafOnLineInLogicalOrderWithNode(box->lineBox(), orderCache);
 
                     if (logicalEnd) {
-                        run = logicalEnd;
-                        renderer = &run->renderer();
-                        offset = primaryDirection == TextDirection::LTR ? run->maximumCaretOffset() : run->minimumCaretOffset();
+                        box = logicalEnd;
+                        renderer = &box->renderer();
+                        offset = primaryDirection == TextDirection::LTR ? box->maximumCaretOffset() : box->minimumCaretOffset();
                     }
                     break;
                 }
 
-                if (nextRun->bidiLevel() >= level)
+                if (nextBox->bidiLevel() >= level)
                     break;
 
-                level = nextRun->bidiLevel();
+                level = nextBox->bidiLevel();
 
-                auto previousRun = run;
+                auto previousBox = box;
                 do {
-                    previousRun.traversePreviousOnLine();
-                } while (previousRun && previousRun->bidiLevel() > level);
+                    previousBox.traverseLineLeftwardOnLine();
+                } while (previousBox && previousBox->bidiLevel() > level);
 
-                if (previousRun && previousRun->bidiLevel() == level) // For example, abc FED 123 ^ CBA
+                if (previousBox && previousBox->bidiLevel() == level) // For example, abc FED 123 ^ CBA
                     break;
 
                 // For example, abc 123 ^ CBA or 123 ^ CBA abc
-                run = nextRun;
-                renderer = &run->renderer();
-                offset = run->leftmostCaretOffset();
-                if (run->direction() == primaryDirection)
+                box = nextBox;
+                renderer = &box->renderer();
+                offset = box->leftmostCaretOffset();
+                if (box->direction() == primaryDirection)
                     break;
                 continue;
             }
 
-            while (nextRun && !nextRun->renderer().node())
-                nextRun.traverseNextOnLine();
+            while (nextBox && !nextBox->renderer().node())
+                nextBox.traverseLineRightwardOnLine();
 
-            if (nextRun) {
-                run = nextRun;
-                renderer = &run->renderer();
-                offset = run->leftmostCaretOffset();
+            if (nextBox) {
+                box = nextBox;
+                renderer = &box->renderer();
+                offset = box->leftmostCaretOffset();
 
-                if (run->bidiLevel() > level) {
+                if (box->bidiLevel() > level) {
                     do {
-                        nextRun.traverseNextOnLine();
-                    } while (nextRun && nextRun->bidiLevel() > level);
+                        nextBox.traverseLineRightwardOnLine();
+                    } while (nextBox && nextBox->bidiLevel() > level);
 
-                    if (!nextRun || nextRun->bidiLevel() < level)
+                    if (!nextBox || nextBox->bidiLevel() < level)
                         continue;
                 }
             } else {
-                // Trailing edge of a secondary run. Set to the leading edge of the entire run.
+                // Trailing edge of a secondary box. Set to the leading edge of the entire box.
                 while (true) {
-                    while (auto previousRun = run->previousOnLine()) {
-                        if (previousRun->bidiLevel() < level)
+                    while (auto previousBox = box->nextLineLeftwardOnLine()) {
+                        if (previousBox->bidiLevel() < level)
                             break;
-                        run = previousRun;
+                        box = previousBox;
                     }
-                    if (run->bidiLevel() == level)
+                    if (box->bidiLevel() == level)
                         break;
-                    level = run->bidiLevel();
-                    while (auto nextRun = run->nextOnLine()) {
-                        if (nextRun->bidiLevel() < level)
+                    level = box->bidiLevel();
+                    while (auto nextBox = box->nextLineRightwardOnLine()) {
+                        if (nextBox->bidiLevel() < level)
                             break;
-                        run = nextRun;
+                        box = nextBox;
                     }
-                    if (run->bidiLevel() == level)
+                    if (box->bidiLevel() == level)
                         break;
-                    level = run->bidiLevel();
+                    level = box->bidiLevel();
                 }
-                renderer = &run->renderer();
-                offset = primaryDirection == TextDirection::LTR ? run->maximumCaretOffset() : run->minimumCaretOffset();
+                renderer = &box->renderer();
+                offset = primaryDirection == TextDirection::LTR ? box->maximumCaretOffset() : box->minimumCaretOffset();
             }
             break;
         }
 
-        p = makeDeprecatedLegacyPosition(renderer->node(), offset);
+        p = makeDeprecatedLegacyPosition(renderer->protectedNode().get(), offset);
 
         if ((p.isCandidate() && p.downstream() != downstreamStart) || p.atStartOfTree() || p.atEndOfTree())
             return p;
@@ -445,8 +447,7 @@ VisiblePosition VisiblePosition::right(bool stayInEditableContent, bool* reached
     if (!stayInEditableContent)
         return right;
 
-    // FIXME: This may need to do something different from "after".
-    return honorEditingBoundaryAtOrAfter(right, reachedBoundary);
+    return directionOfEnclosingBlock(right.deepEquivalent()) == TextDirection::LTR ? honorEditingBoundaryAtOrAfter(right, reachedBoundary) : honorEditingBoundaryAtOrBefore(right, reachedBoundary);
 }
 
 VisiblePosition VisiblePosition::honorEditingBoundaryAtOrBefore(const VisiblePosition& position, bool* reachedBoundary) const
@@ -456,10 +457,10 @@ VisiblePosition VisiblePosition::honorEditingBoundaryAtOrBefore(const VisiblePos
     if (position.isNull())
         return position;
     
-    auto* highestRoot = highestEditableRoot(deepEquivalent());
+    RefPtr highestRoot = highestEditableRoot(deepEquivalent());
     
     // Return empty position if pos is not somewhere inside the editable region containing this position
-    if (highestRoot && !position.deepEquivalent().deprecatedNode()->isDescendantOf(*highestRoot)) {
+    if (highestRoot && !position.deepEquivalent().protectedDeprecatedNode()->isDescendantOf(*highestRoot)) {
         if (reachedBoundary)
             *reachedBoundary = true;
         return VisiblePosition();
@@ -483,7 +484,7 @@ VisiblePosition VisiblePosition::honorEditingBoundaryAtOrBefore(const VisiblePos
     }
 
     // Return the last position before pos that is in the same editable region as this position
-    return lastEditablePositionBeforePositionInRoot(position.deepEquivalent(), highestRoot);
+    return lastEditablePositionBeforePositionInRoot(position.deepEquivalent(), highestRoot.get());
 }
 
 VisiblePosition VisiblePosition::honorEditingBoundaryAtOrAfter(const VisiblePosition& otherPosition, bool* reachedBoundary) const
@@ -493,10 +494,10 @@ VisiblePosition VisiblePosition::honorEditingBoundaryAtOrAfter(const VisiblePosi
     if (otherPosition.isNull())
         return otherPosition;
     
-    auto* highestRoot = highestEditableRoot(deepEquivalent());
+    RefPtr highestRoot = highestEditableRoot(deepEquivalent());
     
     // Return empty position if otherPosition is not somewhere inside the editable region containing this position
-    if (highestRoot && !otherPosition.deepEquivalent().deprecatedNode()->isDescendantOf(*highestRoot)) {
+    if (highestRoot && !otherPosition.deepEquivalent().protectedDeprecatedNode()->isDescendantOf(*highestRoot)) {
         if (reachedBoundary)
             *reachedBoundary = true;
         return VisiblePosition();
@@ -520,7 +521,7 @@ VisiblePosition VisiblePosition::honorEditingBoundaryAtOrAfter(const VisiblePosi
     }
 
     // Return the next position after pos that is in the same editable region as this position
-    return firstEditablePositionAfterPositionInRoot(otherPosition.deepEquivalent(), highestRoot);
+    return firstEditablePositionAfterPositionInRoot(otherPosition.deepEquivalent(), highestRoot.get());
 }
 
 static Position canonicalizeCandidate(const Position& candidate)
@@ -552,7 +553,7 @@ Position VisiblePosition::canonicalPosition(const Position& passedPosition)
     ASSERT(position.document());
     position.document()->updateLayoutIgnorePendingStylesheets();
 
-    Node* node = position.containerNode();
+    RefPtr node = position.containerNode();
 
     Position candidate = position.upstream();
     if (candidate.isCandidate())
@@ -565,18 +566,18 @@ Position VisiblePosition::canonicalPosition(const Position& passedPosition)
     // blocks or enter new ones), we search forward and backward until we find one.
     Position next = canonicalizeCandidate(nextCandidate(position));
     Position prev = canonicalizeCandidate(previousCandidate(position));
-    Node* nextNode = next.deprecatedNode();
-    Node* prevNode = prev.deprecatedNode();
+    auto nextNode = next.protectedDeprecatedNode();
+    auto prevNode = prev.protectedDeprecatedNode();
 
     // The new position must be in the same editable element. Enforce that first.
     // Unless the descent is from a non-editable html element to an editable body.
     if (is<HTMLHtmlElement>(node) && !node->hasEditableStyle()) {
-        auto* body = node->document().bodyOrFrameset();
+        RefPtr body = node->document().bodyOrFrameset();
         if (body && body->hasEditableStyle())
             return next.isNotNull() ? next : prev;
     }
 
-    Node* editingRoot = editableRootForPosition(position);
+    RefPtr editingRoot = editableRootForPosition(position);
         
     // If the html element is editable, descending into its body will look like a descent 
     // from non-editable to editable content since rootEditableElement() always stops at the body.
@@ -595,21 +596,21 @@ Position VisiblePosition::canonicalPosition(const Position& passedPosition)
         return Position();
 
     // The new position should be in the same block flow element. Favor that.
-    Element* originalBlock = deprecatedEnclosingBlockFlowElement(node);
-    bool nextIsOutsideOriginalBlock = !nextNode->isDescendantOf(originalBlock) && nextNode != originalBlock;
-    bool prevIsOutsideOriginalBlock = !prevNode->isDescendantOf(originalBlock) && prevNode != originalBlock;
+    RefPtr originalBlock = deprecatedEnclosingBlockFlowElement(node.get());
+    bool nextIsOutsideOriginalBlock = !nextNode->isDescendantOf(originalBlock.get()) && nextNode != originalBlock;
+    bool prevIsOutsideOriginalBlock = !prevNode->isDescendantOf(originalBlock.get()) && prevNode != originalBlock;
     if (nextIsOutsideOriginalBlock && !prevIsOutsideOriginalBlock)
         return prev;
         
     return next;
 }
 
-UChar32 VisiblePosition::characterAfter() const
+char32_t VisiblePosition::characterAfter() const
 {
     // We canonicalize to the first of two equivalent candidates, but the second of the two candidates
     // is the one that will be inside the text node containing the character after this visible position.
     Position pos = m_deepPosition.downstream();
-    if (!pos.containerNode() || !pos.containerNode()->isTextNode())
+    if (!is<Text>(pos.containerNode()))
         return 0;
     switch (pos.anchorType()) {
     case Position::PositionIsAfterChildren:
@@ -621,38 +622,38 @@ UChar32 VisiblePosition::characterAfter() const
         break;
     }
     unsigned offset = static_cast<unsigned>(pos.offsetInContainerNode());
-    Text* textNode = pos.containerText();
+    RefPtr textNode = pos.containerText();
     unsigned length = textNode->length();
     if (offset >= length)
         return 0;
 
-    UChar32 ch;
+    char32_t ch;
     U16_NEXT(textNode->data(), offset, length, ch);
     return ch;
 }
 
-InlineRunAndOffset VisiblePosition::inlineRunAndOffset() const
+InlineBoxAndOffset VisiblePosition::inlineBoxAndOffset() const
 {
-    return m_deepPosition.inlineRunAndOffset(m_affinity);
+    return m_deepPosition.inlineBoxAndOffset(m_affinity);
 }
 
-InlineRunAndOffset VisiblePosition::inlineRunAndOffset(TextDirection primaryDirection) const
+InlineBoxAndOffset VisiblePosition::inlineBoxAndOffset(TextDirection primaryDirection) const
 {
-    return m_deepPosition.inlineRunAndOffset(m_affinity, primaryDirection);
+    return m_deepPosition.inlineBoxAndOffset(m_affinity, primaryDirection);
 }
 
 auto VisiblePosition::localCaretRect() const -> LocalCaretRect
 {
-    auto node = m_deepPosition.anchorNode();
+    RefPtr node = m_deepPosition.anchorNode();
     if (!node)
         return { };
 
-    auto runAndOffset = inlineRunAndOffset();
-    auto renderer = runAndOffset.run ? &runAndOffset.run->renderer() : node->renderer();
+    auto boxAndOffset = inlineBoxAndOffset();
+    CheckedPtr renderer = boxAndOffset.box ? &boxAndOffset.box->renderer() : node->renderer();
     if (!renderer)
         return { };
 
-    return { computeLocalCaretRect(*renderer, runAndOffset), const_cast<RenderObject*>(renderer) };
+    return { computeLocalCaretRect(*renderer, boxAndOffset), const_cast<RenderObject*>(renderer.get()) };
 }
 
 IntRect VisiblePosition::absoluteCaretBounds(bool* insideFixed) const
@@ -664,16 +665,12 @@ IntRect VisiblePosition::absoluteCaretBounds(bool* insideFixed) const
 
 FloatRect VisiblePosition::absoluteSelectionBoundsForLine() const
 {
-    auto run = inlineRunAndOffset().run;
-    if (!run)
+    auto box = inlineBoxAndOffset().box;
+    if (!box)
         return { };
 
-    auto line = run->line();
-    auto localRect = FloatRect { FloatPoint { line->contentLogicalLeft(), line->selectionTop() }, FloatPoint { line->contentLogicalRight(), line->selectionBottom() } };
-    if (!line->isHorizontal())
-        localRect = localRect.transposedRect();
-    line->containingBlock().flipForWritingMode(localRect);
-    return line->containingBlock().localToAbsoluteQuad(localRect).boundingBox();
+    auto line = box->lineBox();
+    return line->formattingContextRoot().localToAbsoluteQuad(FloatRect { LineSelection::physicalRect(*line) }).boundingBox();
 }
 
 int VisiblePosition::lineDirectionPointForBlockDirectionNavigation() const
@@ -685,21 +682,22 @@ int VisiblePosition::lineDirectionPointForBlockDirectionNavigation() const
     // This ignores transforms on purpose, for now. Vertical navigation is done
     // without consulting transforms, so that 'up' in transformed text is 'up'
     // relative to the text, not absolute 'up'.
-    auto caretPoint = localRect.renderer->localToAbsolute(localRect.rect.location());
-    RenderObject* containingBlock = localRect.renderer->containingBlock();
+    CheckedPtr renderer = localRect.renderer;
+    auto caretPoint = renderer->localToAbsolute(localRect.rect.location());
+    CheckedPtr<RenderObject> containingBlock = renderer->containingBlock();
     if (!containingBlock)
-        containingBlock = localRect.renderer; // Just use ourselves to determine the writing mode if we have no containing block.
+        containingBlock = WTFMove(renderer); // Just use ourselves to determine the writing mode if we have no containing block.
     return containingBlock->isHorizontalWritingMode() ? caretPoint.x() : caretPoint.y();
 }
 
 #if ENABLE(TREE_DEBUGGING)
 
-void VisiblePosition::debugPosition(const char* msg) const
+void VisiblePosition::debugPosition(ASCIILiteral msg) const
 {
     if (isNull())
-        fprintf(stderr, "Position [%s]: null\n", msg);
+        SAFE_FPRINTF(stderr, "Position [%s]: null\n", msg);
     else {
-        fprintf(stderr, "Position [%s]: %s, ", msg, m_deepPosition.deprecatedNode()->nodeName().utf8().data());
+        SAFE_FPRINTF(stderr, "Position [%s]: %s, ", msg, m_deepPosition.deprecatedNode()->nodeName().utf8());
         m_deepPosition.showAnchorTypeAndOffset();
     }
 }
@@ -714,7 +712,11 @@ void VisiblePosition::showTreeForThis() const
     m_deepPosition.showTreeForThis();
 }
 
-#endif
+String VisiblePositionRange::debugDescription() const
+{
+    return makeString("start: "_s, start.debugDescription(), ", end: "_s, end.debugDescription());
+}
+#endif // ENABLE(TREE_DEBUGGING)
 
 // FIXME: Maybe this should be deprecated too, like the underlying function?
 Element* enclosingBlockFlowElement(const VisiblePosition& visiblePosition)
@@ -722,7 +724,7 @@ Element* enclosingBlockFlowElement(const VisiblePosition& visiblePosition)
     if (visiblePosition.isNull())
         return nullptr;
 
-    return deprecatedEnclosingBlockFlowElement(visiblePosition.deepEquivalent().deprecatedNode());
+    return deprecatedEnclosingBlockFlowElement(visiblePosition.deepEquivalent().protectedDeprecatedNode().get());
 }
 
 bool isFirstVisiblePositionInNode(const VisiblePosition& visiblePosition, const Node* node)
@@ -730,11 +732,11 @@ bool isFirstVisiblePositionInNode(const VisiblePosition& visiblePosition, const
     if (visiblePosition.isNull())
         return false;
 
-    if (!visiblePosition.deepEquivalent().containerNode()->isDescendantOf(node))
+    if (!visiblePosition.deepEquivalent().protectedContainerNode()->isDescendantOf(node))
         return false;
 
     VisiblePosition previous = visiblePosition.previous();
-    return previous.isNull() || !previous.deepEquivalent().deprecatedNode()->isDescendantOf(node);
+    return previous.isNull() || !previous.deepEquivalent().protectedDeprecatedNode()->isDescendantOf(node);
 }
 
 bool isLastVisiblePositionInNode(const VisiblePosition& visiblePosition, const Node* node)
@@ -742,16 +744,16 @@ bool isLastVisiblePositionInNode(const VisiblePosition& visiblePosition, const N
     if (visiblePosition.isNull())
         return false;
 
-    if (!visiblePosition.deepEquivalent().containerNode()->isDescendantOf(node))
+    if (!visiblePosition.deepEquivalent().protectedContainerNode()->isDescendantOf(node))
         return false;
 
     VisiblePosition next = visiblePosition.next();
-    return next.isNull() || !next.deepEquivalent().deprecatedNode()->isDescendantOf(node);
+    return next.isNull() || !next.deepEquivalent().protectedDeprecatedNode()->isDescendantOf(node);
 }
 
 bool areVisiblePositionsInSameTreeScope(const VisiblePosition& a, const VisiblePosition& b)
 {
-    return connectedInSameTreeScope(a.deepEquivalent().anchorNode(), b.deepEquivalent().anchorNode());
+    return connectedInSameTreeScope(a.deepEquivalent().protectedAnchorNode().get(), b.deepEquivalent().protectedAnchorNode().get());
 }
 
 bool VisiblePosition::equals(const VisiblePosition& other) const
@@ -805,7 +807,7 @@ VisiblePositionRange makeVisiblePositionRange(const std::optional<SimpleRange>&
     return { makeContainerOffsetPosition(range->start), makeContainerOffsetPosition(range->end) };
 }
 
-PartialOrdering documentOrder(const VisiblePosition& a, const VisiblePosition& b)
+std::partial_ordering documentOrder(const VisiblePosition& a, const VisiblePosition& b)
 {
     // FIXME: Should two positions with different affinity be considered equivalent or not?
     return treeOrder<ComposedTree>(a.deepEquivalent(), b.deepEquivalent());
@@ -833,10 +835,12 @@ Node* commonInclusiveAncestor(const VisiblePositionRange& range)
 
 VisiblePosition midpoint(const VisiblePositionRange& range)
 {
-    auto rootNode = commonInclusiveAncestor(range);
+    RefPtr rootNode = commonInclusiveAncestor(range);
     if (!rootNode)
         return { };
-    auto rootContainerNode = rootNode->isContainerNode() ? downcast<ContainerNode>(rootNode) : rootNode->parentNode();
+    RefPtr rootContainerNode = dynamicDowncast<ContainerNode>(rootNode);
+    if (!rootContainerNode)
+        rootContainerNode = rootNode->parentNode();
     if (!rootContainerNode)
         return { };
     auto scope = makeRangeSelectingNodeContents(*rootContainerNode);
