@@ -61,14 +61,14 @@ public:
     WEBCORE_EXPORT VisiblePosition left(bool stayInEditableContent = false, bool* reachedBoundary = nullptr) const;
     WEBCORE_EXPORT VisiblePosition right(bool stayInEditableContent = false, bool* reachedBoundary = nullptr) const;
 
-    WEBCORE_EXPORT UChar32 characterAfter() const;
-    UChar32 characterBefore() const { return previous().characterAfter(); }
+    WEBCORE_EXPORT char32_t characterAfter() const;
+    char32_t characterBefore() const { return previous().characterAfter(); }
 
     // FIXME: This does not handle [table, 0] correctly.
     Element* rootEditableElement() const { return m_deepPosition.isNotNull() ? m_deepPosition.deprecatedNode()->rootEditableElement() : 0; }
 
-    InlineRunAndOffset inlineRunAndOffset() const;
-    InlineRunAndOffset inlineRunAndOffset(TextDirection primaryDirection) const;
+    InlineBoxAndOffset inlineBoxAndOffset() const;
+    InlineBoxAndOffset inlineBoxAndOffset(TextDirection primaryDirection) const;
 
     struct LocalCaretRect {
         LayoutRect rect;
@@ -90,7 +90,7 @@ public:
     bool equals(const VisiblePosition&) const;
 
 #if ENABLE(TREE_DEBUGGING)
-    void debugPosition(const char* msg = "") const;
+    void debugPosition(ASCIILiteral msg = ""_s) const;
     String debugDescription() const;
     void showTreeForThis() const;
 #endif
@@ -106,9 +106,8 @@ private:
 };
 
 bool operator==(const VisiblePosition&, const VisiblePosition&);
-bool operator!=(const VisiblePosition&, const VisiblePosition&);
 
-WEBCORE_EXPORT PartialOrdering documentOrder(const VisiblePosition&, const VisiblePosition&);
+WEBCORE_EXPORT std::partial_ordering documentOrder(const VisiblePosition&, const VisiblePosition&);
 bool operator<(const VisiblePosition&, const VisiblePosition&);
 bool operator>(const VisiblePosition&, const VisiblePosition&);
 bool operator<=(const VisiblePosition&, const VisiblePosition&);
@@ -133,6 +132,9 @@ struct VisiblePositionRange {
     VisiblePosition end;
 
     bool isNull() const { return start.isNull() || end.isNull(); }
+#if ENABLE(TREE_DEBUGGING)
+    String debugDescription() const;
+#endif
 };
 
 WEBCORE_EXPORT std::optional<SimpleRange> makeSimpleRange(const VisiblePositionRange&);
@@ -153,11 +155,6 @@ inline bool operator==(const VisiblePosition& a, const VisiblePosition& b)
     return a.deepEquivalent() == b.deepEquivalent();
 }
 
-inline bool operator!=(const VisiblePosition& a, const VisiblePosition& b)
-{
-    return !(a == b);
-}
-
 inline bool operator<(const VisiblePosition& a, const VisiblePosition& b)
 {
     return is_lt(documentOrder(a, b));
