@@ -40,21 +40,27 @@
 #include "Event.h"
 #include "EventNames.h"
 #include "EventSender.h"
-#include "Frame.h"
 #include "FrameLoader.h"
-#include "FrameLoaderClient.h"
 #include "FrameTree.h"
-#include "FrameView.h"
 #include "HTMLAnchorElement.h"
+#include "HTMLDocumentParser.h"
 #include "HTMLNames.h"
 #include "HTMLParserIdioms.h"
+#include "IdTargetObserver.h"
+#include "JSRequestPriority.h"
+#include "LocalFrame.h"
+#include "LocalFrameLoaderClient.h"
+#include "LocalFrameView.h"
 #include "Logging.h"
-#include "MediaList.h"
 #include "MediaQueryEvaluator.h"
 #include "MediaQueryParser.h"
+#include "MediaQueryParserContext.h"
 #include "MouseEvent.h"
+#include "NodeName.h"
+#include "Page.h"
 #include "ParsedContentType.h"
 #include "RenderStyle.h"
+#include "RequestPriority.h"
 #include "SecurityOrigin.h"
 #include "Settings.h"
 #include "StyleInheritedData.h"
@@ -62,28 +68,49 @@
 #include "StyleScope.h"
 #include "StyleSheetContents.h"
 #include "SubresourceIntegrity.h"
-#include <wtf/IsoMallocInlines.h>
+#include "TreeScopeInlines.h"
 #include <wtf/Ref.h>
 #include <wtf/Scope.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/text/MakeString.h>
 #include <wtf/text/TextStream.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(HTMLLinkElement);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(HTMLLinkElement);
 
 using namespace HTMLNames;
 
 static LinkEventSender& linkLoadEventSender()
 {
-    static NeverDestroyed<LinkEventSender> sharedLoadEventSender(eventNames().loadEvent);
+    static NeverDestroyed<LinkEventSender> sharedLoadEventSender;
     return sharedLoadEventSender;
 }
 
-static LinkEventSender& linkErrorEventSender()
+class ExpectIdTargetObserver final : public IdTargetObserver {
+    WTF_MAKE_TZONE_ALLOCATED(ExpectIdTargetObserver);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(ExpectIdTargetObserver);
+public:
+    ExpectIdTargetObserver(const AtomString& id, HTMLLinkElement&);
+
+    void idTargetChanged(Element&) override;
+
+private:
+    WeakPtr<HTMLLinkElement, WeakPtrImplWithEventTargetData> m_element;
+};
+
+WTF_MAKE_TZONE_ALLOCATED_IMPL(ExpectIdTargetObserver);
+
+ExpectIdTargetObserver::ExpectIdTargetObserver(const AtomString& id, HTMLLinkElement& element)
+    : IdTargetObserver(element.treeScope().idTargetObserverRegistry(), id)
+    , m_element(element)
 {
-    static NeverDestroyed<LinkEventSender> sharedErrorEventSender(eventNames().errorEvent);
-    return sharedErrorEventSender;
+}
+
+void ExpectIdTargetObserver::idTargetChanged(Element& element)
+{
+    if (m_element)
+        m_element->processInternalResourceLink(&element);
 }
 
 inline HTMLLinkElement::HTMLLinkElement(const QualifiedName& tagName, Document& document, bool createdByParser)
@@ -95,7 +122,7 @@ inline HTMLLinkElement::HTMLLinkElement(const QualifiedName& tagName, Document&
     , m_loadedResource(false)
     , m_isHandlingBeforeLoad(false)
     , m_allowPrefetchLoadAndErrorForTesting(false)
-    , m_pendingSheetType(Unknown)
+    , m_pendingSheetType(PendingSheetType::Unknown)
 {
     ASSERT(hasTagName(linkTag));
 }
@@ -117,7 +144,6 @@ HTMLLinkElement::~HTMLLinkElement()
         m_styleScope->removeStyleSheetCandidateNode(*this);
 
     linkLoadEventSender().cancelEvent(*this);
-    linkErrorEventSender().cancelEvent(*this);
 }
 
 void HTMLLinkElement::setDisabledState(bool disabled)
@@ -140,7 +166,7 @@ void HTMLLinkElement::setDisabledState(bool disabled)
 
         // Check #2: An alternate sheet becomes enabled while it is still loading.
         if (m_relAttribute.isAlternate && m_disabledState == EnabledViaScript)
-            addPendingSheet(ActiveSheet);
+            addPendingSheet(PendingSheetType::Active);
 
         // Check #3: A main sheet becomes enabled while it was still loading and
         // after it was disabled via script. It takes really terrible code to make this
@@ -148,7 +174,7 @@ void HTMLLinkElement::setDisabledState(bool disabled)
         // virtualplastic.net, which manages to do about 12 enable/disables on only 3
         // sheets. :)
         if (!m_relAttribute.isAlternate && m_disabledState == EnabledViaScript && oldDisabledState == Disabled)
-            addPendingSheet(ActiveSheet);
+            addPendingSheet(PendingSheetType::Active);
 
         // If the sheet is already loading just bail.
         return;
@@ -160,62 +186,90 @@ void HTMLLinkElement::setDisabledState(bool disabled)
     else {
         ASSERT(m_styleScope);
         m_styleScope->didChangeActiveStyleSheetCandidates();
+        if (m_sheet)
+            clearSheet();
     }
 }
 
-void HTMLLinkElement::parseAttribute(const QualifiedName& name, const AtomString& value)
+void HTMLLinkElement::attributeChanged(const QualifiedName& name, const AtomString& oldValue, const AtomString& newValue, AttributeModificationReason attributeModificationReason)
 {
-    if (name == relAttr) {
-        auto parsedRel = LinkRelAttribute(document(), value);
+    switch (name.nodeName()) {
+    case AttributeNames::relAttr: {
+        auto parsedRel = LinkRelAttribute(document(), newValue);
         auto didMutateRel = parsedRel != m_relAttribute;
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+        auto wasSpatialBackdrop = m_relAttribute.isSpatialBackdrop;
+#endif
         m_relAttribute = WTFMove(parsedRel);
         if (m_relList)
-            m_relList->associatedAttributeValueChanged(value);
+            m_relList->associatedAttributeValueChanged();
         if (didMutateRel)
             process();
-        return;
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+        if (wasSpatialBackdrop && !m_relAttribute.isSpatialBackdrop)
+            document().spatialBackdropLinkElementChanged();
+#endif
+        break;
     }
-    if (name == hrefAttr) {
+    case AttributeNames::hrefAttr: {
         URL url = getNonEmptyURLAttribute(hrefAttr);
         if (url == m_url)
             return;
         m_url = WTFMove(url);
         process();
-        return;
+        break;
     }
-    if (name == typeAttr) {
-        if (value == m_type)
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+    case AttributeNames::environmentmapAttr: {
+        URL environmentMapURL = getNonEmptyURLAttribute(environmentmapAttr);
+        if (environmentMapURL == m_environmentMapURL)
             return;
-        m_type = value;
+        m_environmentMapURL = WTFMove(environmentMapURL);
         process();
-        return;
+        break;
     }
-    if (name == sizesAttr) {
+#endif
+    case AttributeNames::typeAttr:
+        if (newValue == m_type)
+            return;
+        m_type = newValue;
+        process();
+        break;
+    case AttributeNames::sizesAttr:
         if (m_sizes)
-            m_sizes->associatedAttributeValueChanged(value);
+            m_sizes->associatedAttributeValueChanged();
         process();
-        return;
-    }
-    if (name == mediaAttr) {
-        auto media = value.string().convertToASCIILowercase();
+        break;
+    case AttributeNames::blockingAttr:
+        blocking().associatedAttributeValueChanged();
+        if (blocking().contains("render"_s)) {
+            processInternalResourceLink();
+            if (m_loading && mediaAttributeMatches() && !isAlternate())
+                potentiallyBlockRendering();
+        } else if (!isImplicitlyPotentiallyRenderBlocking())
+            unblockRendering();
+        break;
+    case AttributeNames::mediaAttr: {
+        auto media = newValue.string().convertToASCIILowercase();
         if (media == m_media)
             return;
         m_media = WTFMove(media);
         process();
         if (m_sheet && !isDisabled())
             m_styleScope->didChangeActiveStyleSheetCandidates();
-        return;
+        break;
     }
-    if (name == disabledAttr) {
-        setDisabledState(!value.isNull());
-        return;
-    }
-    if (name == titleAttr) {
+    case AttributeNames::disabledAttr:
+        setDisabledState(!newValue.isNull());
+        break;
+    case AttributeNames::titleAttr:
         if (m_sheet && !isInShadowTree())
-            m_sheet->setTitle(value);
-        return;
+            m_sheet->setTitle(newValue);
+        break;
+    default:
+        HTMLElement::attributeChanged(name, oldValue, newValue, attributeModificationReason);
+        break;
     }
-    HTMLElement::parseAttribute(name, value);
 }
 
 bool HTMLLinkElement::shouldLoadLink()
@@ -263,6 +317,16 @@ void HTMLLinkElement::process()
     if (m_isHandlingBeforeLoad)
         return;
 
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+    if (m_relAttribute.isSpatialBackdrop)
+        document().spatialBackdropLinkElementChanged();
+#endif
+
+    processInternalResourceLink();
+    if (m_relAttribute.isInternalResourceLink)
+        return;
+
+    Ref document = this->document();
     LinkLoadParameters params {
         m_relAttribute,
         m_url,
@@ -274,9 +338,10 @@ void HTMLLinkElement::process()
         attributeWithoutSynchronization(imagesizesAttr),
         nonce(),
         referrerPolicy(),
+        fetchPriority(),
     };
 
-    m_linkLoader.loadLink(params, document());
+    m_linkLoader.loadLink(params, document);
 
     bool treatAsStyleSheet = false;
     if (m_relAttribute.isStyleSheet) {
@@ -286,19 +351,18 @@ void HTMLLinkElement::process()
             treatAsStyleSheet = equalLettersIgnoringASCIICase(parsedContentType->mimeType(), "text/css"_s);
     }
     if (!treatAsStyleSheet)
-        treatAsStyleSheet = document().settings().treatsAnyTextCSSLinkAsStylesheet() && m_type.containsIgnoringASCIICase("text/css"_s);
+        treatAsStyleSheet = document->settings().treatsAnyTextCSSLinkAsStylesheet() && m_type.containsIgnoringASCIICase("text/css"_s);
 
     LOG_WITH_STREAM(StyleSheets, stream << "HTMLLinkElement " << this << " process() - treatAsStyleSheet " << treatAsStyleSheet);
 
-    if (m_disabledState != Disabled && treatAsStyleSheet && document().frame() && m_url.isValid()) {
+    if (m_disabledState != Disabled && treatAsStyleSheet && document->frame() && m_url.isValid()) {
         String charset = attributeWithoutSynchronization(charsetAttr);
         if (!PAL::TextEncoding { charset }.isValid())
-            charset = document().charset();
+            charset = document->charset();
 
-        if (m_cachedSheet) {
+        if (CachedResourceHandle cachedSheet = std::exchange(m_cachedSheet, nullptr)) {
             removePendingSheet();
-            m_cachedSheet->removeClient(*this);
-            m_cachedSheet = nullptr;
+            cachedSheet->removeClient(*this);
         }
 
         {
@@ -314,7 +378,12 @@ void HTMLLinkElement::process()
         // Don't hold up render tree construction and script execution on stylesheets
         // that are not needed for the rendering at the moment.
         bool isActive = mediaAttributeMatches() && !isAlternate();
-        addPendingSheet(isActive ? ActiveSheet : InactiveSheet);
+        addPendingSheet(isActive ? PendingSheetType::Active : PendingSheetType::Inactive);
+
+        if (isActive)
+            potentiallyBlockRendering();
+        else
+            unblockRendering();
 
         // Load stylesheets that are not needed for the rendering immediately with low priority.
         std::optional<ResourceLoadPriority> priority;
@@ -326,31 +395,35 @@ void HTMLLinkElement::process()
         ResourceLoaderOptions options = CachedResourceLoader::defaultCachedResourceOptions();
         options.nonce = nonce();
         options.sameOriginDataURLFlag = SameOriginDataURLFlag::Set;
-        if (document().contentSecurityPolicy()->allowStyleWithNonce(options.nonce))
+        if (document->checkedContentSecurityPolicy()->allowStyleWithNonce(options.nonce))
             options.contentSecurityPolicyImposition = ContentSecurityPolicyImposition::SkipPolicyCheck;
         options.integrity = m_integrityMetadataForPendingSheetRequest;
         options.referrerPolicy = params.referrerPolicy;
+        options.fetchPriority = fetchPriority();
 
-        auto request = createPotentialAccessControlRequest(m_url, WTFMove(options), document(), crossOrigin());
+        auto request = createPotentialAccessControlRequest(m_url, WTFMove(options), document, crossOrigin());
         request.setPriority(WTFMove(priority));
         request.setCharset(WTFMove(charset));
         request.setInitiator(*this);
 
         ASSERT_WITH_SECURITY_IMPLICATION(!m_cachedSheet);
-        m_cachedSheet = document().cachedResourceLoader().requestCSSStyleSheet(WTFMove(request)).value_or(nullptr);
+        m_cachedSheet = document->protectedCachedResourceLoader()->requestCSSStyleSheet(WTFMove(request)).value_or(nullptr);
 
-        if (m_cachedSheet)
-            m_cachedSheet->addClient(*this);
+        if (CachedResourceHandle cachedSheet = m_cachedSheet)
+            cachedSheet->addClient(*this);
         else {
             // The request may have been denied if (for example) the stylesheet is local and the document is remote.
             m_loading = false;
             sheetLoaded();
             notifyLoadedSheetAndAllCriticalSubresources(true);
+            unblockRendering();
         }
 
         return;
     }
 
+    unblockRendering();
+
     if (m_sheet) {
         // we no longer contain a stylesheet, e.g. perhaps rel or type was changed
         clearSheet();
@@ -360,7 +433,7 @@ void HTMLLinkElement::process()
 
 #if ENABLE(APPLICATION_MANIFEST)
     if (isApplicationManifest()) {
-        if (RefPtr loader = document().loader())
+        if (RefPtr loader = document->loader())
             loader->loadApplicationManifest({ });
         return;
     }
@@ -375,6 +448,80 @@ void HTMLLinkElement::clearSheet()
     m_sheet = nullptr;
 }
 
+// https://html.spec.whatwg.org/multipage/links.html#process-internal-resource-link
+void HTMLLinkElement::processInternalResourceLink(Element* element)
+{
+    if (document().wasRemovedLastRefCalled())
+        return;
+
+    if (!m_relAttribute.isInternalResourceLink) {
+        return;
+    }
+
+    if (!equalIgnoringFragmentIdentifier(m_url, document().url())) {
+        unblockRendering();
+        return;
+    }
+
+    RefPtr<Element> indicatedElement;
+    // If the change originated from a specific element, then we can just check if that's
+    // the right one instead doing a tree search using the name
+    if (element) {
+        auto elementMatchesLinkId = [&](StringView id) {
+            if (element->getIdAttribute() == id)
+                return true;
+            RefPtr anchorElement = dynamicDowncast<HTMLAnchorElement>(element);
+            if (anchorElement && document().isMatchingAnchor(*anchorElement, m_url.fragmentIdentifier()))
+                return true;
+            return false;
+        };
+
+        if (element->isConnected() && (elementMatchesLinkId(m_url.fragmentIdentifier()) || elementMatchesLinkId(PAL::decodeURLEscapeSequences(m_url.fragmentIdentifier()))))
+            indicatedElement = element;
+    } else {
+        indicatedElement = document().findAnchor(m_url.fragmentIdentifier());
+        if (!indicatedElement)
+            indicatedElement = document().findAnchor(PAL::decodeURLEscapeSequences(m_url.fragmentIdentifier()));
+    }
+
+    // Don't match if indicatedElement "is on a stack of open elements of an HTML parser whose associated Document is doc"
+    if (RefPtr parser = document().htmlDocumentParser(); parser && indicatedElement) {
+        if (parser->isOnStackOfOpenElements(*indicatedElement))
+            indicatedElement = nullptr;
+    }
+
+    if (document().readyState() == Document::ReadyState::Loading && isConnected() && mediaAttributeMatches() && !indicatedElement) {
+        potentiallyBlockRendering();
+        if (!m_expectIdTargetObserver)
+            m_expectIdTargetObserver = makeUnique<ExpectIdTargetObserver>(makeAtomString(m_url.fragmentIdentifier()), *this);
+    } else
+        unblockRendering();
+}
+
+// https://html.spec.whatwg.org/multipage/urls-and-fetching.html#blocking-attributes
+void HTMLLinkElement::potentiallyBlockRendering()
+{
+    bool explicitRenderBlocking = m_blockingList && m_blockingList->contains("render"_s);
+    if (explicitRenderBlocking || isImplicitlyPotentiallyRenderBlocking()) {
+        document().blockRenderingOn(*this, explicitRenderBlocking ? Document::ImplicitRenderBlocking::No : Document::ImplicitRenderBlocking::Yes);
+        m_isRenderBlocking = true;
+    }
+}
+
+void HTMLLinkElement::unblockRendering()
+{
+    if (m_isRenderBlocking) {
+        document().unblockRenderingOn(*this);
+        m_isRenderBlocking = false;
+    }
+}
+
+// https://html.spec.whatwg.org/multipage/links.html#link-type-stylesheet
+bool HTMLLinkElement::isImplicitlyPotentiallyRenderBlocking() const
+{
+    return m_relAttribute.isStyleSheet && m_createdByParser;
+}
+
 Node::InsertedIntoAncestorResult HTMLLinkElement::insertedIntoAncestor(InsertionType insertionType, ContainerNode& parentOfInsertedTree)
 {
     HTMLElement::insertedIntoAncestor(insertionType, parentOfInsertedTree);
@@ -403,6 +550,11 @@ void HTMLLinkElement::removedFromAncestor(RemovalType removalType, ContainerNode
 
     bool wasLoading = styleSheetIsLoading();
 
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+    if (m_relAttribute.isSpatialBackdrop)
+        oldParentOfRemovedTree.document().spatialBackdropLinkElementChanged();
+#endif
+
     if (m_sheet)
         clearSheet();
 
@@ -413,6 +565,9 @@ void HTMLLinkElement::removedFromAncestor(RemovalType removalType, ContainerNode
         m_styleScope->removeStyleSheetCandidateNode(*this);
         m_styleScope = nullptr;
     }
+
+    processInternalResourceLink();
+    unblockRendering();
 }
 
 void HTMLLinkElement::finishParsingChildren()
@@ -423,13 +578,13 @@ void HTMLLinkElement::finishParsingChildren()
 
 void HTMLLinkElement::initializeStyleSheet(Ref<StyleSheetContents>&& styleSheet, const CachedCSSStyleSheet& cachedStyleSheet, MediaQueryParserContext context)
 {
-    // FIXME: originClean should be turned to false except if fetch mode is CORS.
-    std::optional<bool> originClean;
-    if (cachedStyleSheet.options().mode == FetchOptions::Mode::Cors)
-        originClean = cachedStyleSheet.isCORSSameOrigin();
+    if (m_sheet) {
+        ASSERT(m_sheet->ownerNode() == this);
+        m_sheet->clearOwnerNode();
+    }
 
-    m_sheet = CSSStyleSheet::create(WTFMove(styleSheet), *this, originClean);
-    m_sheet->setMediaQueries(MediaQuerySet::create(m_media, context));
+    m_sheet = CSSStyleSheet::create(WTFMove(styleSheet), *this, cachedStyleSheet.isCORSSameOrigin());
+    m_sheet->setMediaQueries(MQ::MediaQueryParser::parse(m_media, context));
     if (!isInShadowTree())
         m_sheet->setTitle(title());
 
@@ -437,8 +592,9 @@ void HTMLLinkElement::initializeStyleSheet(Ref<StyleSheetContents>&& styleSheet,
         m_sheet->contents().setAsOpaque();
 }
 
-void HTMLLinkElement::setCSSStyleSheet(const String& href, const URL& baseURL, const String& charset, const CachedCSSStyleSheet* cachedStyleSheet)
+void HTMLLinkElement::setCSSStyleSheet(const String& href, const URL& baseURL, ASCIILiteral charset, const CachedCSSStyleSheet* cachedStyleSheet)
 {
+    unblockRendering();
     if (!isConnected()) {
         ASSERT(!m_sheet);
         return;
@@ -451,7 +607,7 @@ void HTMLLinkElement::setCSSStyleSheet(const String& href, const URL& baseURL, c
     Ref<HTMLLinkElement> protectedThis(*this);
 
     if (!cachedStyleSheet->errorOccurred() && !matchIntegrityMetadata(*cachedStyleSheet, m_integrityMetadataForPendingSheetRequest)) {
-        document().addConsoleMessage(MessageSource::Security, MessageLevel::Error, makeString("Cannot load stylesheet ", integrityMismatchDescription(*cachedStyleSheet, m_integrityMetadataForPendingSheetRequest)));
+        document().addConsoleMessage(MessageSource::Security, MessageLevel::Error, makeString("Cannot load stylesheet "_s, integrityMismatchDescription(*cachedStyleSheet, m_integrityMetadataForPendingSheetRequest)));
 
         m_loading = false;
         sheetLoaded();
@@ -465,7 +621,7 @@ void HTMLLinkElement::setCSSStyleSheet(const String& href, const URL& baseURL, c
     if (auto restoredSheet = const_cast<CachedCSSStyleSheet*>(cachedStyleSheet)->restoreParsedStyleSheet(parserContext, cachePolicy, frame->loader())) {
         ASSERT(restoredSheet->isCacheable());
         ASSERT(!restoredSheet->isLoading());
-        initializeStyleSheet(restoredSheet.releaseNonNull(), *cachedStyleSheet, MediaQueryParserContext(document()));
+        initializeStyleSheet(restoredSheet.releaseNonNull(), *cachedStyleSheet, MediaQueryParserContext(parserContext));
 
         m_loading = false;
         sheetLoaded();
@@ -474,7 +630,7 @@ void HTMLLinkElement::setCSSStyleSheet(const String& href, const URL& baseURL, c
     }
 
     auto styleSheet = StyleSheetContents::create(href, parserContext);
-    initializeStyleSheet(styleSheet.copyRef(), *cachedStyleSheet, MediaQueryParserContext(document()));
+    initializeStyleSheet(styleSheet.copyRef(), *cachedStyleSheet, MediaQueryParserContext(parserContext));
 
     // FIXME: Set the visibility option based on m_sheet being clean or not.
     // Best approach might be to set it on the style sheet content itself or its context parser otherwise.
@@ -505,7 +661,7 @@ bool HTMLLinkElement::styleSheetIsLoading() const
 DOMTokenList& HTMLLinkElement::sizes()
 {
     if (!m_sizes)
-        m_sizes = makeUnique<DOMTokenList>(*this, sizesAttr);
+        m_sizes = makeUniqueWithoutRefCountedCheck<DOMTokenList>(*this, sizesAttr);
     return *m_sizes;
 }
 
@@ -517,22 +673,24 @@ bool HTMLLinkElement::mediaAttributeMatches() const
     std::optional<RenderStyle> documentStyle;
     if (document().hasLivingRenderTree())
         documentStyle = Style::resolveForDocument(document());
-    auto media = MediaQuerySet::create(m_media, MediaQueryParserContext(document()));
+    auto mediaQueryList = MQ::MediaQueryParser::parse(m_media, { document() });
     LOG(MediaQueries, "HTMLLinkElement::mediaAttributeMatches");
-    return MediaQueryEvaluator(document().frame()->view()->mediaType(), document(), documentStyle ? &*documentStyle : nullptr).evaluate(media.get());
+
+    MQ::MediaQueryEvaluator evaluator(document().frame()->view()->mediaType(), document(), documentStyle ? &*documentStyle : nullptr);
+    return evaluator.evaluate(mediaQueryList);
 }
 
 void HTMLLinkElement::linkLoaded()
 {
     m_loadedResource = true;
     if (!m_relAttribute.isLinkPrefetch || m_allowPrefetchLoadAndErrorForTesting)
-        linkLoadEventSender().dispatchEventSoon(*this);
+        linkLoadEventSender().dispatchEventSoon(*this, eventNames().loadEvent);
 }
 
 void HTMLLinkElement::linkLoadingErrored()
 {
     if (!m_relAttribute.isLinkPrefetch || m_allowPrefetchLoadAndErrorForTesting)
-        linkErrorEventSender().dispatchEventSoon(*this);
+        linkLoadEventSender().dispatchEventSoon(*this, eventNames().errorEvent);
 }
 
 bool HTMLLinkElement::sheetLoaded()
@@ -549,40 +707,54 @@ void HTMLLinkElement::dispatchPendingLoadEvents(Page* page)
     linkLoadEventSender().dispatchPendingEvents(page);
 }
 
-void HTMLLinkElement::dispatchPendingEvent(LinkEventSender* eventSender)
+void HTMLLinkElement::dispatchPendingEvent(LinkEventSender* eventSender, const AtomString& eventType)
 {
-    ASSERT_UNUSED(eventSender, eventSender == &linkLoadEventSender() || eventSender == &linkErrorEventSender());
-    if (m_loadedResource)
-        dispatchEvent(Event::create(eventNames().loadEvent, Event::CanBubble::No, Event::IsCancelable::No));
-    else
-        dispatchEvent(Event::create(eventNames().errorEvent, Event::CanBubble::No, Event::IsCancelable::No));
+    ASSERT_UNUSED(eventSender, eventSender == &linkLoadEventSender());
+    dispatchEvent(Event::create(eventType, Event::CanBubble::No, Event::IsCancelable::No));
 }
 
 DOMTokenList& HTMLLinkElement::relList()
 {
     if (!m_relList) 
-        m_relList = makeUnique<DOMTokenList>(*this, HTMLNames::relAttr, [](Document& document, StringView token) {
+        m_relList = makeUniqueWithoutRefCountedCheck<DOMTokenList>(*this, HTMLNames::relAttr, [](Document& document, StringView token) {
             return LinkRelAttribute::isSupported(document, token);
         });
     return *m_relList;
 }
 
+// https://html.spec.whatwg.org/multipage/semantics.html#dom-link-blocking
+DOMTokenList& HTMLLinkElement::blocking()
+{
+    if (!m_blockingList) {
+        m_blockingList = makeUniqueWithoutRefCountedCheck<DOMTokenList>(*this, HTMLNames::blockingAttr, [](Document&, StringView token) {
+            if (equalLettersIgnoringASCIICase(token, "render"_s))
+                return true;
+            return false;
+        });
+    }
+    return *m_blockingList;
+}
+
 void HTMLLinkElement::notifyLoadedSheetAndAllCriticalSubresources(bool errorOccurred)
 {
     m_loadedResource = !errorOccurred;
-    linkLoadEventSender().dispatchEventSoon(*this);
+    linkLoadEventSender().dispatchEventSoon(*this, m_loadedResource ? eventNames().loadEvent : eventNames().errorEvent);
 }
 
 void HTMLLinkElement::startLoadingDynamicSheet()
 {
     // We don't support multiple active sheets.
-    ASSERT(m_pendingSheetType < ActiveSheet);
-    addPendingSheet(ActiveSheet);
+    ASSERT(m_pendingSheetType < PendingSheetType::Active);
+    addPendingSheet(PendingSheetType::Active);
 }
 
 bool HTMLLinkElement::isURLAttribute(const Attribute& attribute) const
 {
-    return attribute.name().localName() == hrefAttr || HTMLElement::isURLAttribute(attribute);
+    return attribute.name().localName() == hrefAttr
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+    || attribute.name().localName() == environmentmapAttr
+#endif
+    || HTMLElement::isURLAttribute(attribute);
 }
 
 URL HTMLLinkElement::href() const
@@ -595,6 +767,13 @@ const AtomString& HTMLLinkElement::rel() const
     return attributeWithoutSynchronization(relAttr);
 }
 
+#if ENABLE(WEB_PAGE_SPATIAL_BACKDROP)
+URL HTMLLinkElement::environmentMap() const
+{
+    return document().completeURL(attributeWithoutSynchronization(environmentmapAttr));
+}
+#endif
+
 AtomString HTMLLinkElement::target() const
 {
     return attributeWithoutSynchronization(targetAttr);
@@ -610,17 +789,25 @@ std::optional<LinkIconType> HTMLLinkElement::iconType() const
     return m_relAttribute.iconType;
 }
 
+static bool mayFetchResource(LinkRelAttribute relAttribute)
+{
+    // https://html.spec.whatwg.org/multipage/links.html#linkTypes
+    return relAttribute.isStyleSheet
+        || relAttribute.isLinkModulePreload
+        || relAttribute.isLinkPreload
+#if ENABLE(APPLICATION_MANIFEST)
+        || relAttribute.isApplicationManifest
+#endif
+        || !!relAttribute.iconType;
+}
+
 void HTMLLinkElement::addSubresourceAttributeURLs(ListHashSet<URL>& urls) const
 {
     HTMLElement::addSubresourceAttributeURLs(urls);
 
-    // Favicons are handled by a special case in LegacyWebArchive::create()
-    if (m_relAttribute.iconType)
+    if (!mayFetchResource(m_relAttribute))
         return;
 
-    if (!m_relAttribute.isStyleSheet)
-        return;
-    
     // Append the URL of this link element.
     addSubresourceURL(urls, href());
 
@@ -638,7 +825,7 @@ void HTMLLinkElement::addPendingSheet(PendingSheetType type)
         return;
     m_pendingSheetType = type;
 
-    if (m_pendingSheetType == InactiveSheet)
+    if (m_pendingSheetType == PendingSheetType::Inactive)
         return;
     ASSERT(m_styleScope);
     m_styleScope->addPendingSheet(*this);
@@ -647,13 +834,13 @@ void HTMLLinkElement::addPendingSheet(PendingSheetType type)
 void HTMLLinkElement::removePendingSheet()
 {
     PendingSheetType type = m_pendingSheetType;
-    m_pendingSheetType = Unknown;
+    m_pendingSheetType = PendingSheetType::Unknown;
 
-    if (type == Unknown)
+    if (type == PendingSheetType::Unknown)
         return;
 
     ASSERT(m_styleScope);
-    if (type == InactiveSheet) {
+    if (type == PendingSheetType::Inactive) {
         // Document just needs to know about the sheet for exposure through document.styleSheets
         m_styleScope->didChangeActiveStyleSheetCandidates();
         return;
@@ -674,9 +861,7 @@ String HTMLLinkElement::referrerPolicyForBindings() const
 
 ReferrerPolicy HTMLLinkElement::referrerPolicy() const
 {
-    if (document().settings().referrerPolicyAttributeEnabled())
-        return parseReferrerPolicy(attributeWithoutSynchronization(referrerpolicyAttr), ReferrerPolicySource::ReferrerPolicyAttribute).value_or(ReferrerPolicy::EmptyString);
-    return ReferrerPolicy::EmptyString;
+    return parseReferrerPolicy(attributeWithoutSynchronization(referrerpolicyAttr), ReferrerPolicySource::ReferrerPolicyAttribute).value_or(ReferrerPolicy::EmptyString);
 }
 
 String HTMLLinkElement::debugDescription() const
@@ -684,4 +869,19 @@ String HTMLLinkElement::debugDescription() const
     return makeString(HTMLElement::debugDescription(), ' ', type(), ' ', href().string());
 }
 
+void HTMLLinkElement::setFetchPriorityForBindings(const AtomString& value)
+{
+    setAttributeWithoutSynchronization(fetchpriorityAttr, value);
+}
+
+String HTMLLinkElement::fetchPriorityForBindings() const
+{
+    return convertEnumerationToString(fetchPriority());
+}
+
+RequestPriority HTMLLinkElement::fetchPriority() const
+{
+    return parseEnumerationFromString<RequestPriority>(attributeWithoutSynchronization(fetchpriorityAttr)).value_or(RequestPriority::Auto);
+}
+
 } // namespace WebCore
