@@ -27,17 +27,20 @@
 #include "config.h"
 #include "PublicURLManager.h"
 
-#include "ScriptExecutionContext.h"
+#include "ContextDestructionObserverInlines.h"
 #include "SecurityOrigin.h"
 #include "URLRegistry.h"
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/URL.h>
 #include <wtf/text/StringHash.h>
 
 namespace WebCore {
 
-std::unique_ptr<PublicURLManager> PublicURLManager::create(ScriptExecutionContext* context)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(PublicURLManager);
+
+Ref<PublicURLManager> PublicURLManager::create(ScriptExecutionContext* context)
 {
-    auto publicURLManager = makeUnique<PublicURLManager>(context);
+    Ref publicURLManager = adoptRef(*new PublicURLManager(context));
     publicURLManager->suspendIfNeeded();
     return publicURLManager;
 }
@@ -60,7 +63,7 @@ void PublicURLManager::revoke(const URL& url)
     if (m_isStopped || !scriptExecutionContext())
         return;
 
-    auto* contextOrigin = scriptExecutionContext()->securityOrigin();
+    RefPtr contextOrigin = scriptExecutionContext()->securityOrigin();
     if (!contextOrigin)
         return;
 
@@ -69,7 +72,7 @@ void PublicURLManager::revoke(const URL& url)
         return;
 
     URLRegistry::forEach([&](auto& registry) {
-        registry.unregisterURL(url);
+        registry.unregisterURL(url, scriptExecutionContext()->topOrigin().data());
     });
 }
 
@@ -79,16 +82,11 @@ void PublicURLManager::stop()
         return;
 
     m_isStopped = true;
-    if (auto* context = scriptExecutionContext()) {
+    if (RefPtr context = scriptExecutionContext()) {
         URLRegistry::forEach([&](auto& registry) {
             registry.unregisterURLsForContext(*context);
         });
     }
 }
 
-const char* PublicURLManager::activeDOMObjectName() const
-{
-    return "PublicURLManager";
-}
-
 } // namespace WebCore
