@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2010 Google Inc. All rights reserved.
+ * Copyright (C) 2010-2013 Google Inc. All rights reserved.
  * Copyright (C) 2011-2020 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -36,17 +36,18 @@
 #include "Chrome.h"
 #include "ChromeClient.h"
 #include "DOMFormData.h"
-#include "DeprecatedGlobalSettings.h"
 #include "Editor.h"
-#include "ElementInlines.h"
 #include "ElementRareData.h"
+#include "EventLoop.h"
 #include "EventNames.h"
-#include "Frame.h"
 #include "FrameSelection.h"
+#include "HTMLDataListElement.h"
 #include "HTMLInputElement.h"
 #include "HTMLNames.h"
+#include "HTMLOptionElement.h"
 #include "HTMLParserIdioms.h"
 #include "KeyboardEvent.h"
+#include "LocalFrame.h"
 #include "LocalizedStrings.h"
 #include "NodeRenderStyle.h"
 #include "Page.h"
@@ -57,22 +58,20 @@
 #include "RenderTheme.h"
 #include "ScriptDisallowedScope.h"
 #include "Settings.h"
-#include "ShadowPseudoIds.h"
 #include "ShadowRoot.h"
 #include "TextControlInnerElements.h"
 #include "TextEvent.h"
 #include "TextIterator.h"
 #include "TextNodeTraversal.h"
+#include "TypedElementDescendantIteratorInlines.h"
+#include "UserAgentParts.h"
 #include "UserTypingGestureIndicator.h"
-#include "WheelEvent.h"
-
-#if ENABLE(DATALIST_ELEMENT)
-#include "HTMLDataListElement.h"
-#include "HTMLOptionElement.h"
-#endif
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(TextFieldInputType);
+
 using namespace HTMLNames;
 
 TextFieldInputType::TextFieldInputType(Type type, HTMLInputElement& element)
@@ -83,11 +82,9 @@ TextFieldInputType::TextFieldInputType(Type type, HTMLInputElement& element)
 
 TextFieldInputType::~TextFieldInputType()
 {
-    if (m_innerSpinButton)
-        m_innerSpinButton->removeSpinButtonOwner();
-#if ENABLE(DATALIST_ELEMENT)
     closeSuggestions();
-#endif
+    if (RefPtr suggestionPicker = m_suggestionPicker)
+        suggestionPicker->detach();
 }
 
 bool TextFieldInputType::isKeyboardFocusable(KeyboardEvent*) const
@@ -110,9 +107,9 @@ bool TextFieldInputType::isEmptyValue() const
 {
     auto innerText = innerTextElement();
     if (!innerText) {
-        // Since we always create the shadow subtree if a value is set, we know
-        // that the value is empty.
-        return true;
+        if (element()->shadowRoot())
+            return true; // Shadow tree is empty
+        return visibleValue().isEmpty();
     }
 
     for (Text* text = TextNodeTraversal::firstWithin(*innerText); text; text = TextNodeTraversal::next(*text, innerText.get())) {
@@ -176,34 +173,37 @@ void TextFieldInputType::setValue(const String& sanitizedValue, bool valueChange
         break;
     }
 
-    // FIXME: Why do we do this when eventBehavior == DispatchNoEvent
-    if (!input->focused() || eventBehavior == DispatchNoEvent)
+    if (!input->focused())
         input->setTextAsOfLastFormControlChangeEvent(sanitizedValue);
 
     if (UserTypingGestureIndicator::processingUserTypingGesture())
         didSetValueByUserEdit();
 }
 
-#if ENABLE(DATALIST_ELEMENT)
 void TextFieldInputType::handleClickEvent(MouseEvent&)
 {
-    if (element()->focused() && element()->list())
+    if (element()->focused() && element()->hasDataList())
         displaySuggestions(DataListSuggestionActivationType::ControlClicked);
 }
+
+void TextFieldInputType::showPicker()
+{
+#if !PLATFORM(IOS_FAMILY)
+    if (element()->hasDataList())
+        displaySuggestions(DataListSuggestionActivationType::ControlClicked);
 #endif
+}
 
 auto TextFieldInputType::handleKeydownEvent(KeyboardEvent& event) -> ShouldCallBaseEventHandler
 {
     ASSERT(element());
     if (!element()->focused())
         return ShouldCallBaseEventHandler::Yes;
-#if ENABLE(DATALIST_ELEMENT)
     const String& key = event.keyIdentifier();
-    if (m_suggestionPicker && (key == "Enter"_s || key == "Up"_s || key == "Down"_s)) {
-        m_suggestionPicker->handleKeydownWithIdentifier(key);
+    if (RefPtr suggestionPicker = m_suggestionPicker; suggestionPicker && (key == "Enter"_s || key == "Up"_s || key == "Down"_s || key == "U+001B"_s)) {
+        suggestionPicker->handleKeydownWithIdentifier(key);
         event.setDefaultHandled();
     }
-#endif
     RefPtr frame = element()->document().frame();
     if (frame && frame->editor().doTextFieldCommandFromEvent(*element(), &event))
         event.setDefaultHandled();
@@ -215,10 +215,8 @@ void TextFieldInputType::handleKeydownEventForSpinButton(KeyboardEvent& event)
     ASSERT(element());
     if (!element()->isMutable())
         return;
-#if ENABLE(DATALIST_ELEMENT)
     if (m_suggestionPicker)
         return;
-#endif
     const String& key = event.keyIdentifier();
     if (key == "Up"_s)
         spinButtonStepUp();
@@ -233,12 +231,6 @@ void TextFieldInputType::forwardEvent(Event& event)
 {
     ASSERT(element());
 
-    if (m_innerSpinButton) {
-        m_innerSpinButton->forwardEvent(event);
-        if (event.defaultHandled())
-            return;
-    }
-
     auto& eventNames = WebCore::eventNames();
     bool isFocusEvent = event.type() == eventNames.focusEvent;
     bool isBlurEvent = event.type() == eventNames.blurEvent;
@@ -269,9 +261,7 @@ void TextFieldInputType::elementDidBlur()
     ScrollOffset scrollOffset(isLeftToRightDirection ? 0 : innerLayerScrollable->scrollWidth(), 0);
     innerLayerScrollable->scrollToOffset(scrollOffset);
 
-#if ENABLE(DATALIST_ELEMENT)
     closeSuggestions();
-#endif
 }
 
 void TextFieldInputType::handleFocusEvent(Node* oldFocusedNode, FocusDirection)
@@ -280,10 +270,6 @@ void TextFieldInputType::handleFocusEvent(Node* oldFocusedNode, FocusDirection)
     ASSERT_UNUSED(oldFocusedNode, oldFocusedNode != element());
     if (RefPtr frame = element()->document().frame()) {
         frame->editor().textFieldDidBeginEditing(*element());
-#if ENABLE(DATALIST_ELEMENT)
-        if (shouldOnlyShowDataListDropdownButtonWhenFocusedOrEdited() && element()->list() && m_dataListDropdownIndicator)
-            m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, suggestions().size() ? CSSValueBlock : CSSValueNone, true);
-#endif
     }
 }
 
@@ -292,22 +278,19 @@ void TextFieldInputType::handleBlurEvent()
     InputType::handleBlurEvent();
     ASSERT(element());
     element()->endEditing();
-#if ENABLE(DATALIST_ELEMENT)
-    if (shouldOnlyShowDataListDropdownButtonWhenFocusedOrEdited() && element()->list() && m_dataListDropdownIndicator)
-        m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, CSSValueNone, true);
-#endif
 }
 
 bool TextFieldInputType::shouldSubmitImplicitly(Event& event)
 {
-    return (event.type() == eventNames().textInputEvent && is<TextEvent>(event) && downcast<TextEvent>(event).data() == "\n"_s)
+    auto* textEvent = dynamicDowncast<TextEvent>(event);
+    return (event.type() == eventNames().textInputEvent && textEvent && textEvent->data() == "\n"_s)
         || InputType::shouldSubmitImplicitly(event);
 }
 
 RenderPtr<RenderElement> TextFieldInputType::createInputRenderer(RenderStyle&& style)
 {
     ASSERT(element());
-    return createRenderer<RenderTextControlSingleLine>(*element(), WTFMove(style));
+    return createRenderer<RenderTextControlSingleLine>(RenderObject::Type::TextControlSingleLine, *element(), WTFMove(style));
 }
 
 bool TextFieldInputType::needsContainer() const
@@ -340,51 +323,49 @@ void TextFieldInputType::createShadowSubtree()
     ASSERT(!m_capsLockIndicator);
     ASSERT(!m_autoFillButton);
 
-    Document& document = element()->document();
+    Ref document = element()->document();
     bool shouldHaveSpinButton = this->shouldHaveSpinButton();
     bool shouldHaveCapsLockIndicator = this->shouldHaveCapsLockIndicator();
     bool shouldDrawAutoFillButton = this->shouldDrawAutoFillButton();
-#if ENABLE(DATALIST_ELEMENT)
-    bool hasDataList = element()->list();
-#endif
-    bool createsContainer = shouldHaveSpinButton || shouldHaveCapsLockIndicator || shouldDrawAutoFillButton
-#if ENABLE(DATALIST_ELEMENT)
-        || hasDataList
-#endif
-        || needsContainer();
+    bool hasDataList = element()->hasDataList();
+    bool createsContainer = shouldHaveSpinButton || shouldHaveCapsLockIndicator || shouldDrawAutoFillButton || hasDataList || needsContainer();
 
-    m_innerText = TextControlInnerTextElement::create(document, element()->isInnerTextElementEditable());
+    Ref innerText = TextControlInnerTextElement::create(document, element()->isInnerTextElementEditable());
+    m_innerText = innerText.copyRef();
 
-    ScriptDisallowedScope::EventAllowedScope eventAllowedScope { *element()->userAgentShadowRoot() };
+    Ref shadowRoot = *element()->userAgentShadowRoot();
+    ScriptDisallowedScope::EventAllowedScope eventAllowedScope { shadowRoot };
     if (!createsContainer) {
-        element()->userAgentShadowRoot()->appendChild(ContainerNode::ChildChange::Source::Parser, *m_innerText);
+        shadowRoot->appendChild(ContainerNode::ChildChange::Source::Parser, innerText);
         updatePlaceholderText();
+        updateInnerTextValue();
         return;
     }
 
-    createContainer();
+    createContainer(PreserveSelectionRange::No);
     updatePlaceholderText();
+    updateInnerTextValue();
 
     if (shouldHaveSpinButton) {
-        m_innerSpinButton = SpinButtonElement::create(document, *this);
-        m_container->appendChild(ContainerNode::ChildChange::Source::Parser, *m_innerSpinButton);
+        Ref innerSpinButton = SpinButtonElement::create(document, *this);
+        m_innerSpinButton = innerSpinButton.copyRef();
+        RefPtr { m_container }->appendChild(ContainerNode::ChildChange::Source::Parser, innerSpinButton);
     }
 
     if (shouldHaveCapsLockIndicator) {
-        m_capsLockIndicator = HTMLDivElement::create(document);
-        m_container->appendChild(ContainerNode::ChildChange::Source::Parser, *m_capsLockIndicator);
+        Ref capsLockIndicator = HTMLDivElement::create(document);
+        m_capsLockIndicator = capsLockIndicator.copyRef();
+        RefPtr { m_container }->appendChild(ContainerNode::ChildChange::Source::Parser, capsLockIndicator);
 
-        m_capsLockIndicator->setPseudo(ShadowPseudoIds::webkitCapsLockIndicator());
+        capsLockIndicator->setUserAgentPart(UserAgentParts::webkitCapsLockIndicator());
 
         bool shouldDrawCapsLockIndicator = this->shouldDrawCapsLockIndicator();
-        m_capsLockIndicator->setInlineStyleProperty(CSSPropertyDisplay, shouldDrawCapsLockIndicator ? CSSValueBlock : CSSValueNone, true);
+        capsLockIndicator->setInlineStyleProperty(CSSPropertyDisplay, shouldDrawCapsLockIndicator ? CSSValueBlock : CSSValueNone, IsImportant::Yes);
     }
 
     updateAutoFillButton();
 
-#if ENABLE(DATALIST_ELEMENT)
     dataListMayHaveChanged();
-#endif
 }
 
 HTMLElement* TextFieldInputType::containerElement() const
@@ -417,9 +398,9 @@ HTMLElement* TextFieldInputType::placeholderElement() const
     return m_placeholder.get();
 }
 
-void TextFieldInputType::destroyShadowSubtree()
+void TextFieldInputType::removeShadowSubtree()
 {
-    InputType::destroyShadowSubtree();
+    InputType::removeShadowSubtree();
     m_innerText = nullptr;
     m_placeholder = nullptr;
     m_innerBlock = nullptr;
@@ -428,16 +409,14 @@ void TextFieldInputType::destroyShadowSubtree()
     m_innerSpinButton = nullptr;
     m_capsLockIndicator = nullptr;
     m_autoFillButton = nullptr;
-#if ENABLE(DATALIST_ELEMENT)
     m_dataListDropdownIndicator = nullptr;
-#endif
     m_container = nullptr;
 }
 
 void TextFieldInputType::attributeChanged(const QualifiedName& name)
 {
     if (name == valueAttr || name == placeholderAttr) {
-        if (element())
+        if (element() && element()->shadowRoot())
             updateInnerTextValue();
     }
     InputType::attributeChanged(name);
@@ -475,8 +454,6 @@ bool TextFieldInputType::shouldUseInputMethod() const
     return true;
 }
 
-#if ENABLE(DATALIST_ELEMENT)
-
 void TextFieldInputType::createDataListDropdownIndicator()
 {
     ASSERT(!m_dataListDropdownIndicator);
@@ -488,30 +465,16 @@ void TextFieldInputType::createDataListDropdownIndicator()
     ScriptDisallowedScope::EventAllowedScope allowedScope(*m_container);
     m_dataListDropdownIndicator = DataListButtonElement::create(element()->document(), *this);
     m_container->appendChild(*m_dataListDropdownIndicator);
-    m_dataListDropdownIndicator->setPseudo(ShadowPseudoIds::webkitListButton());
-    m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, CSSValueNone, true);
+    m_dataListDropdownIndicator->setUserAgentPart(UserAgentParts::webkitListButton());
+    m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, CSSValueNone, IsImportant::Yes);
 }
 
-bool TextFieldInputType::shouldOnlyShowDataListDropdownButtonWhenFocusedOrEdited() const
-{
-#if PLATFORM(IOS_FAMILY)
-#if ENABLE(IOS_FORM_CONTROL_REFRESH)
-    return !element()->document().settings().iOSFormControlRefreshEnabled();
-#else
-    return true;
-#endif
-#else
-    return false;
-#endif
-}
-
-#endif // ENABLE(DATALIST_ELEMENT)
-
 static String limitLength(const String& string, unsigned maxLength)
 {
-    unsigned newLength = std::min(maxLength, string.length());
-    if (newLength == string.length())
+    if (LIKELY(string.length() <= maxLength))
         return string;
+
+    unsigned newLength = maxLength;
     if (newLength > 0 && U16_IS_LEAD(string[newLength - 1]))
         --newLength;
     return string.left(newLength);
@@ -560,15 +523,15 @@ static AtomString autoFillButtonTypeToAutoFillButtonPseudoClassName(AutoFillButt
 {
     switch (autoFillButtonType) {
     case AutoFillButtonType::Contacts:
-        return ShadowPseudoIds::webkitContactsAutoFillButton();
+        return UserAgentParts::webkitContactsAutoFillButton();
     case AutoFillButtonType::Credentials:
-        return ShadowPseudoIds::webkitCredentialsAutoFillButton();
+        return UserAgentParts::webkitCredentialsAutoFillButton();
     case AutoFillButtonType::StrongPassword:
-        return ShadowPseudoIds::webkitStrongPasswordAutoFillButton();
+        return UserAgentParts::webkitStrongPasswordAutoFillButton();
     case AutoFillButtonType::CreditCard:
-        return ShadowPseudoIds::webkitCreditCardAutoFillButton();
+        return UserAgentParts::webkitCreditCardAutoFillButton();
     case AutoFillButtonType::Loading:
-        return ShadowPseudoIds::webkitLoadingAutoFillButton();
+        return UserAgentParts::internalLoadingAutoFillButton();
     case AutoFillButtonType::None:
         ASSERT_NOT_REACHED();
         return emptyAtom();
@@ -579,22 +542,29 @@ static AtomString autoFillButtonTypeToAutoFillButtonPseudoClassName(AutoFillButt
 
 static bool isAutoFillButtonTypeChanged(const AtomString& attribute, AutoFillButtonType autoFillButtonType)
 {
-    if (attribute == ShadowPseudoIds::webkitContactsAutoFillButton() && autoFillButtonType != AutoFillButtonType::Contacts)
+    if (attribute == UserAgentParts::webkitContactsAutoFillButton() && autoFillButtonType != AutoFillButtonType::Contacts)
         return true;
-    if (attribute == ShadowPseudoIds::webkitCredentialsAutoFillButton() && autoFillButtonType != AutoFillButtonType::Credentials)
+    if (attribute == UserAgentParts::webkitCredentialsAutoFillButton() && autoFillButtonType != AutoFillButtonType::Credentials)
         return true;
-    if (attribute == ShadowPseudoIds::webkitStrongPasswordAutoFillButton() && autoFillButtonType != AutoFillButtonType::StrongPassword)
+    if (attribute == UserAgentParts::webkitStrongPasswordAutoFillButton() && autoFillButtonType != AutoFillButtonType::StrongPassword)
         return true;
-    if (attribute == ShadowPseudoIds::webkitCreditCardAutoFillButton() && autoFillButtonType != AutoFillButtonType::CreditCard)
+    if (attribute == UserAgentParts::webkitCreditCardAutoFillButton() && autoFillButtonType != AutoFillButtonType::CreditCard)
         return true;
-    if (attribute == ShadowPseudoIds::webkitLoadingAutoFillButton() && autoFillButtonType != AutoFillButtonType::Loading)
+    if (attribute == UserAgentParts::internalLoadingAutoFillButton() && autoFillButtonType != AutoFillButtonType::Loading)
         return true;
     return false;
 }
 
 String TextFieldInputType::sanitizeValue(const String& proposedValue) const
 {
-    return limitLength(proposedValue.removeCharacters(isHTMLLineBreak), HTMLInputElement::maxEffectiveLength);
+    if (LIKELY(!containsHTMLLineBreak(proposedValue)))
+        return limitLength(proposedValue, HTMLInputElement::maxEffectiveLength);
+
+    // Passing a lambda instead of a function name helps the compiler inline isHTMLLineBreak.
+    auto proposedValueWithoutLineBreaks = proposedValue.removeCharacters([](auto character) {
+        return isHTMLLineBreak(character);
+    });
+    return limitLength(WTFMove(proposedValueWithoutLineBreaks), HTMLInputElement::maxEffectiveLength);
 }
 
 void TextFieldInputType::handleBeforeTextInsertedEvent(BeforeTextInsertedEvent& event)
@@ -641,11 +611,7 @@ void TextFieldInputType::handleBeforeTextInsertedEvent(BeforeTextInsertedEvent&
 
 bool TextFieldInputType::shouldRespectListAttribute()
 {
-#if ENABLE(DATALIST_ELEMENT)
-    return DeprecatedGlobalSettings::dataListElementEnabled();
-#else
-    return InputType::themeSupportsDataListUI(this);
-#endif
+    return element() && element()->document().settings().dataListElementEnabled();
 }
 
 void TextFieldInputType::updatePlaceholderText()
@@ -660,26 +626,30 @@ void TextFieldInputType::updatePlaceholderText()
 
     String placeholderText = element()->placeholder();
     if (placeholderText.isEmpty()) {
-        if (m_placeholder) {
-            m_placeholder->parentNode()->removeChild(*m_placeholder);
-            m_placeholder = nullptr;
-        }
+        if (RefPtr placeholder = std::exchange(m_placeholder, nullptr))
+            placeholder->remove();
         return;
     }
     if (!m_placeholder) {
-        m_placeholder = TextControlPlaceholderElement::create(element()->document());
-        element()->userAgentShadowRoot()->insertBefore(*m_placeholder, m_container ? m_container.get() : innerTextElement().get());
+        Ref placeholder = TextControlPlaceholderElement::create(element()->protectedDocument());
+        m_placeholder = placeholder.copyRef();
+        if (RefPtr container = m_container)
+            element()->protectedUserAgentShadowRoot()->insertBefore(placeholder, container);
+        else
+            element()->protectedUserAgentShadowRoot()->insertBefore(placeholder, innerTextElement());
     }
-    m_placeholder->setInnerText(WTFMove(placeholderText));
+    RefPtr { m_placeholder }->setInnerText(WTFMove(placeholderText));
 }
 
 bool TextFieldInputType::appendFormData(DOMFormData& formData) const
 {
     InputType::appendFormData(formData);
     ASSERT(element());
-    auto& dirnameAttrValue = element()->attributeWithoutSynchronization(dirnameAttr);
-    if (!dirnameAttrValue.isNull())
-        formData.append(dirnameAttrValue, element()->directionForFormData());
+    // FIXME: should type=number be TextFieldInputType to begin with?
+    if (element()->isNumberField())
+        return true;
+    if (auto& dirname = element()->attributeWithoutSynchronization(dirnameAttr); !dirname.isNull())
+        formData.append(dirname, element()->directionForFormData());
     return true;
 }
 
@@ -721,13 +691,8 @@ void TextFieldInputType::didSetValueByUserEdit()
         return;
     if (RefPtr frame = element()->document().frame())
         frame->editor().textDidChangeInTextField(*element());
-#if ENABLE(DATALIST_ELEMENT)
-    if (shouldOnlyShowDataListDropdownButtonWhenFocusedOrEdited() && element()->list() && m_dataListDropdownIndicator)
-        m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, suggestions().size() ? CSSValueBlock : CSSValueNone, true);
-
-    if (element()->list())
+    if (element()->hasDataList())
         displaySuggestions(DataListSuggestionActivationType::TextChanged);
-#endif
 }
 
 void TextFieldInputType::spinButtonStepDown()
@@ -765,12 +730,6 @@ bool TextFieldInputType::shouldSpinButtonRespondToMouseEvents() const
     return element()->isMutable();
 }
 
-bool TextFieldInputType::shouldSpinButtonRespondToWheelEvents() const
-{
-    ASSERT(element());
-    return shouldSpinButtonRespondToMouseEvents() && element()->focused();
-}
-
 bool TextFieldInputType::shouldDrawCapsLockIndicator() const
 {
     ASSERT(element());
@@ -780,10 +739,10 @@ bool TextFieldInputType::shouldDrawCapsLockIndicator() const
     if (!element()->isMutable())
         return false;
 
-    if (element()->hasAutoFillStrongPasswordButton())
+    if (element()->hasAutofillStrongPasswordButton())
         return false;
 
-    RefPtr<Frame> frame = element()->document().frame();
+    RefPtr frame { element()->document().frame() };
     if (!frame)
         return false;
 
@@ -799,41 +758,69 @@ void TextFieldInputType::capsLockStateMayHaveChanged()
         return;
 
     bool shouldDrawCapsLockIndicator = this->shouldDrawCapsLockIndicator();
-    m_capsLockIndicator->setInlineStyleProperty(CSSPropertyDisplay, shouldDrawCapsLockIndicator ? CSSValueBlock : CSSValueNone, true);
+    m_capsLockIndicator->setInlineStyleProperty(CSSPropertyDisplay, shouldDrawCapsLockIndicator ? CSSValueBlock : CSSValueNone, IsImportant::Yes);
 }
 
 bool TextFieldInputType::shouldDrawAutoFillButton() const
 {
     ASSERT(element());
-    return element()->isMutable() && element()->autoFillButtonType() != AutoFillButtonType::None;
+    return element()->isMutable() && element()->autofillButtonType() != AutoFillButtonType::None;
 }
 
 void TextFieldInputType::autoFillButtonElementWasClicked()
 {
-    ASSERT(element());
-    Page* page = element()->document().page();
+    RefPtr element = this->element();
+    ASSERT(element);
+    Page* page = element->document().page();
     if (!page)
         return;
 
-    page->chrome().client().handleAutoFillButtonClick(*element());
+    auto event = Event::create(eventNames().webkitautofillrequestEvent, Event::CanBubble::No, Event::IsCancelable::No);
+    event->setIsAutofillEvent();
+    element->dispatchEvent(WTFMove(event));
+
+    page->chrome().client().handleAutoFillButtonClick(*element);
 }
 
-void TextFieldInputType::createContainer()
+void TextFieldInputType::createContainer(PreserveSelectionRange preserveSelection)
 {
     ASSERT(!m_container);
     ASSERT(element());
 
     static MainThreadNeverDestroyed<const AtomString> webkitTextfieldDecorationContainerName("-webkit-textfield-decoration-container"_s);
 
-    ScriptDisallowedScope::EventAllowedScope allowedScope(*element()->userAgentShadowRoot());
+    Ref shadowRoot = *element()->userAgentShadowRoot();
+    ScriptDisallowedScope::EventAllowedScope allowedScope(shadowRoot);
 
-    m_container = TextControlInnerContainer::create(element()->document());
-    element()->userAgentShadowRoot()->appendChild(*m_container);
-    m_container->setPseudo(ShadowPseudoIds::webkitTextfieldDecorationContainer());
+    Ref document = element()->document();
+    // FIXME: <https://webkit.org/b/245977> Suppress selectionchange events during subtree modification.
+    std::optional<std::tuple<unsigned, unsigned, TextFieldSelectionDirection>> selectionState;
+    if (preserveSelection == PreserveSelectionRange::Yes && enclosingTextFormControl(document->selection().selection().start()) == element())
+        selectionState = { element()->selectionStart(), element()->selectionEnd(), element()->computeSelectionDirection() };
 
-    m_innerBlock = TextControlInnerElement::create(element()->document());
-    m_container->appendChild(*m_innerBlock);
-    m_innerBlock->appendChild(*m_innerText);
+    Ref container = TextControlInnerContainer::create(document);
+    m_container = container.copyRef();
+    shadowRoot->appendChild(container);
+    container->setUserAgentPart(UserAgentParts::webkitTextfieldDecorationContainer());
+
+    Ref innerBlock = TextControlInnerElement::create(document);
+    m_innerBlock = innerBlock.copyRef();
+    m_container->appendChild(innerBlock);
+    innerBlock->appendChild(*m_innerText.copyRef());
+
+    if (selectionState) {
+        document->checkedEventLoop()->queueTask(TaskSource::DOMManipulation, [selectionState = *selectionState, element = WeakPtr { element() }] {
+            if (!element || !element->focused())
+                return;
+
+            auto selection = element->document().selection().selection();
+            if (selection.start().deprecatedNode() != element->userAgentShadowRoot())
+                return;
+
+            auto& [selectionStart, selectionEnd, selectionDirection] = selectionState;
+            element->setSelectionRange(selectionStart, selectionEnd, selectionDirection);
+        });
+    }
 }
 
 void TextFieldInputType::createAutoFillButton(AutoFillButtonType autoFillButtonType)
@@ -847,7 +834,7 @@ void TextFieldInputType::createAutoFillButton(AutoFillButtonType autoFillButtonT
     m_autoFillButton = AutoFillButtonElement::create(element()->document(), *this);
     m_container->appendChild(*m_autoFillButton);
 
-    m_autoFillButton->setPseudo(autoFillButtonTypeToAutoFillButtonPseudoClassName(autoFillButtonType));
+    m_autoFillButton->setUserAgentPart(autoFillButtonTypeToAutoFillButtonPseudoClassName(autoFillButtonType));
     m_autoFillButton->setAttributeWithoutSynchronization(roleAttr, HTMLNames::buttonTag->localName());
     m_autoFillButton->setAttributeWithoutSynchronization(aria_labelAttr, AtomString { autoFillButtonTypeToAccessibilityLabel(autoFillButtonType) });
     m_autoFillButton->setTextContent(autoFillButtonTypeToAutoFillButtonText(autoFillButtonType));
@@ -866,27 +853,25 @@ void TextFieldInputType::updateAutoFillButton()
         if (!m_container)
             createContainer();
 
-        AutoFillButtonType autoFillButtonType = element()->autoFillButtonType();
+        AutoFillButtonType autoFillButtonType = element()->autofillButtonType();
         if (!m_autoFillButton)
             createAutoFillButton(autoFillButtonType);
 
-        const AtomString& attribute = m_autoFillButton->attributeWithoutSynchronization(pseudoAttr);
+        const AtomString& attribute = m_autoFillButton->userAgentPart();
         bool shouldUpdateAutoFillButtonType = isAutoFillButtonTypeChanged(attribute, autoFillButtonType);
         if (shouldUpdateAutoFillButtonType) {
-            m_autoFillButton->setPseudo(autoFillButtonTypeToAutoFillButtonPseudoClassName(autoFillButtonType));
+            m_autoFillButton->setUserAgentPart(autoFillButtonTypeToAutoFillButtonPseudoClassName(autoFillButtonType));
             m_autoFillButton->setAttributeWithoutSynchronization(aria_labelAttr, AtomString { autoFillButtonTypeToAccessibilityLabel(autoFillButtonType) });
             m_autoFillButton->setTextContent(autoFillButtonTypeToAutoFillButtonText(autoFillButtonType));
         }
-        m_autoFillButton->setInlineStyleProperty(CSSPropertyDisplay, CSSValueBlock, true);
+        m_autoFillButton->setInlineStyleProperty(CSSPropertyDisplay, CSSValueBlock, IsImportant::Yes);
         return;
     }
     
     if (m_autoFillButton)
-        m_autoFillButton->setInlineStyleProperty(CSSPropertyDisplay, CSSValueNone, true);        
+        m_autoFillButton->setInlineStyleProperty(CSSPropertyDisplay, CSSValueNone, IsImportant::Yes);
 }
 
-#if ENABLE(DATALIST_ELEMENT)
-
 void TextFieldInputType::dataListMayHaveChanged()
 {
     if (!hasCreatedShadowSubtree())
@@ -898,8 +883,9 @@ void TextFieldInputType::dataListMayHaveChanged()
         createDataListDropdownIndicator();
     if (!element())
         return;
-    if (!shouldOnlyShowDataListDropdownButtonWhenFocusedOrEdited())
-        m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, element()->list() ? CSSValueBlock : CSSValueNone, true);
+    m_dataListDropdownIndicator->setInlineStyleProperty(CSSPropertyDisplay, element()->list() ? CSSValueBlock : CSSValueNone, IsImportant::Yes);
+    if (element()->hasDataList() && element()->focused())
+        displaySuggestions(DataListSuggestionActivationType::DataListMayHaveChanged);
 }
 
 HTMLElement* TextFieldInputType::dataListButtonElement() const
@@ -910,7 +896,7 @@ HTMLElement* TextFieldInputType::dataListButtonElement() const
 void TextFieldInputType::dataListButtonElementWasClicked()
 {
     Ref<HTMLInputElement> input(*element());
-    if (input->list()) {
+    if (input->hasDataList()) {
         m_isFocusingWithDataListDropdown = true;
         unsigned max = visibleValue().length();
         input->setSelectionRange(max, max);
@@ -973,7 +959,8 @@ void TextFieldInputType::didSelectDataListOption(const String& selectedOption)
 void TextFieldInputType::didCloseSuggestions()
 {
     m_cachedSuggestions = { };
-    m_suggestionPicker = nullptr;
+    if (RefPtr suggestionPicker = std::exchange(m_suggestionPicker, nullptr))
+        suggestionPicker->detach();
     if (element()->renderer())
         element()->renderer()->repaint();
 }
@@ -983,22 +970,20 @@ void TextFieldInputType::displaySuggestions(DataListSuggestionActivationType typ
     if (element()->isDisabledFormControl() || !element()->renderer())
         return;
 
-    if (!UserGestureIndicator::processingUserGesture() && type != DataListSuggestionActivationType::TextChanged)
+    if (!UserGestureIndicator::processingUserGesture() && !(type == DataListSuggestionActivationType::TextChanged || type == DataListSuggestionActivationType::DataListMayHaveChanged))
         return;
 
     if (!m_suggestionPicker && suggestions().size() > 0)
         m_suggestionPicker = chrome()->createDataListSuggestionPicker(*this);
 
-    if (!m_suggestionPicker)
-        return;
-
-    m_suggestionPicker->displayWithActivationType(type);
+    if (RefPtr suggestionPicker = m_suggestionPicker)
+        suggestionPicker->displayWithActivationType(type);
 }
 
 void TextFieldInputType::closeSuggestions()
 {
-    if (m_suggestionPicker)
-        m_suggestionPicker->close();
+    if (RefPtr suggestionPicker = m_suggestionPicker)
+        suggestionPicker->close();
 }
 
 bool TextFieldInputType::isPresentingAttachedView() const
@@ -1011,6 +996,4 @@ bool TextFieldInputType::isFocusingWithDataListDropdown() const
     return m_isFocusingWithDataListDropdown;
 }
 
-#endif
-
 } // namespace WebCore
