@@ -28,8 +28,8 @@
 
 #if ENABLE(WEBGL)
 
+#include "WebCoreOpaqueRoot.h"
 #include "WebGLCompressedTextureS3TC.h"
-#include "WebGLContextGroup.h"
 #include "WebGLDebugRendererInfo.h"
 #include "WebGLDebugShaders.h"
 #include "WebGLLoseContext.h"
@@ -37,55 +37,85 @@
 
 namespace WebCore {
 
-void WebGLObject::setObject(PlatformGLObject object)
+WebGLObject::WebGLObject(WebGLRenderingContextBase& context, PlatformGLObject object)
+    : m_context(context.createRefForContextObject())
+    , m_object(object)
 {
-    ASSERT(!m_object);
-    ASSERT(!m_deleted);
-    m_object = object;
 }
 
-void WebGLObject::deleteObject(GraphicsContextGLOpenGL* context3d)
+WebGLObject::~WebGLObject() = default;
+
+WebGLRenderingContextBase* WebGLObject::context() const
+{
+    return m_context.get();
+}
+
+Lock& WebGLObject::objectGraphLockForContext()
+{
+    // Should not call this if the object or context has been deleted.
+    ASSERT(m_context);
+    return m_context->objectGraphLock();
+}
+
+GraphicsContextGL* WebGLObject::graphicsContextGL() const
+{
+    return m_context ? m_context->graphicsContextGL() : nullptr;
+}
+
+void WebGLObject::runDestructor()
+{
+    auto& lock = objectGraphLockForContext();
+    if (lock.isHeld()) {
+        // Destruction of WebGLObjects can happen in chains triggered from GC.
+        // The lock must be held only once, at the beginning of the chain.
+        auto locker = AbstractLocker(NoLockingNecessary);
+        deleteObject(locker, nullptr);
+    } else {
+        Locker locker { lock };
+        deleteObject(locker, nullptr);
+    }
+}
+
+void WebGLObject::deleteObject(const AbstractLocker& locker, GraphicsContextGL* context3d)
 {
     m_deleted = true;
     if (!m_object)
         return;
 
-    if (!hasGroupOrContext())
+    if (!m_context)
         return;
 
-    // ANGLE correctly handles object deletion with WebGL semantics.
-    // For other GL implementations, delay deletion until we know
-    // the object is not attached anywhere.
-#if USE(ANGLE)
-    if (!m_calledDelete) {
-        m_calledDelete = true;
-#else
     if (!m_attachmentCount) {
-#endif
         if (!context3d)
-            context3d = getAGraphicsContextGL();
+            context3d = graphicsContextGL();
 
         if (context3d)
-            deleteObjectImpl(context3d, m_object);
+            deleteObjectImpl(locker, context3d, m_object);
     }
 
     if (!m_attachmentCount)
         m_object = 0;
 }
 
-void WebGLObject::detach()
-{
-    m_attachmentCount = 0; // Make sure OpenGL resource is deleted.
-}
-
-void WebGLObject::onDetached(GraphicsContextGLOpenGL* context3d)
+void WebGLObject::onDetached(const AbstractLocker& locker, GraphicsContextGL* context3d)
 {
+    ASSERT(m_attachmentCount); // FIXME: handle attachment with WebGLAttachmentPoint RAII object and remove the ifs.
     if (m_attachmentCount)
         --m_attachmentCount;
     if (m_deleted)
-        deleteObject(context3d);
+        deleteObject(locker, context3d);
 }
 
+bool WebGLObject::validate(const WebGLRenderingContextBase& context) const
+{
+    return &context == m_context;
 }
 
+WebCoreOpaqueRoot root(WebGLObject* object)
+{
+    return WebCoreOpaqueRoot { object };
+}
+
+} // namespace WebCore
+
 #endif // ENABLE(WEBGL)
