@@ -33,43 +33,44 @@
 #if ENABLE(VIDEO)
 
 #include "DOMTokenList.h"
-#include "ElementChildIterator.h"
+#include "ElementChildIteratorInlines.h"
 #include "EventHandler.h"
 #include "EventLoop.h"
 #include "EventNames.h"
-#include "Frame.h"
 #include "FullscreenManager.h"
 #include "GraphicsContext.h"
 #include "HTMLVideoElement.h"
 #include "ImageBuffer.h"
+#include "LocalFrame.h"
 #include "LocalizedStrings.h"
 #include "Logging.h"
 #include "PODInterval.h"
 #include "Page.h"
 #include "PageGroup.h"
 #include "RenderLayer.h"
+#include "RenderLayerInlines.h"
 #include "RenderVideo.h"
 #include "RenderView.h"
 #include "Settings.h"
-#include "ShadowPseudoIds.h"
 #include "ShadowRoot.h"
 #include "StyleProperties.h"
 #include "TextTrackCueGeneric.h"
 #include "TextTrackList.h"
+#include "UserAgentParts.h"
 #include "VTTRegionList.h"
-#include <wtf/IsoMallocInlines.h>
 #include <wtf/Language.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
-WTF_MAKE_ISO_ALLOCATED_IMPL(MediaControlTextTrackContainerElement);
+WTF_MAKE_TZONE_OR_ISO_ALLOCATED_IMPL(MediaControlTextTrackContainerElement);
 
 using namespace HTMLNames;
 
 Ref<MediaControlTextTrackContainerElement> MediaControlTextTrackContainerElement::create(Document& document, HTMLMediaElement& mediaElement)
 {
     auto element = adoptRef(*new MediaControlTextTrackContainerElement(document, mediaElement));
-    element->setPseudo(ShadowPseudoIds::webkitMediaTextTrackContainer());
+    element->setUserAgentPart(UserAgentParts::webkitMediaTextTrackContainer());
     element->hide();
     return element;
 }
@@ -82,7 +83,7 @@ MediaControlTextTrackContainerElement::MediaControlTextTrackContainerElement(Doc
 
 RenderPtr<RenderElement> MediaControlTextTrackContainerElement::createElementRenderer(RenderStyle&& style, const RenderTreePosition&)
 {
-    return createRenderer<RenderBlockFlow>(*this, WTFMove(style));
+    return createRenderer<RenderBlockFlow>(RenderObject::Type::BlockFlow, *this, WTFMove(style));
 }
 
 static bool compareCueIntervalForDisplay(const CueInterval& one, const CueInterval& two)
@@ -98,11 +99,13 @@ void MediaControlTextTrackContainerElement::updateDisplay()
     // 1. If the media element is an audio element, or is another playback
     // mechanism with no rendering area, abort these steps. There is nothing to
     // render.
-    if (!m_mediaElement || !m_mediaElement->isVideo() || m_videoDisplaySize.size().isEmpty())
+    if (!m_mediaElement || m_videoDisplaySize.size().isEmpty())
         return;
 
     // 2. Let video be the media element or other playback mechanism.
-    HTMLVideoElement& video = downcast<HTMLVideoElement>(*m_mediaElement);
+    RefPtr video = dynamicDowncast<HTMLVideoElement>(*m_mediaElement);
+    if (!video)
+        return;
 
     // 3. Let output be an empty list of absolutely positioned CSS block boxes.
 
@@ -111,7 +114,7 @@ void MediaControlTextTrackContainerElement::updateDisplay()
     // cover the same region as the user interface.
 
     // 5. If the last time these rules were run, the user agent was not exposing
-    // a user interface for video, but now it is, let reset be true. Otherwise,
+    // a user interface for video, but now it is, optionally let reset be true. Otherwise,
     // let reset be false.
 
     // There is nothing to be done explicitly for 4th and 5th steps, as
@@ -125,16 +128,25 @@ void MediaControlTextTrackContainerElement::updateDisplay()
     // 7. Let cues be an empty list of text track cues.
     // 8. For each track track in tracks, append to cues all the cues from
     // track's list of cues that have their text track cue active flag set.
-    CueList activeCues = video.currentlyActiveCues();
+    CueList activeCues = video->currentlyActiveCues();
 
-    // 9. If reset is false, then, for each text track cue cue in cues: if cue's
-    // text track cue display state has a set of CSS boxes, then add those boxes
-    // to output, and remove cue from cues.
+    // 9. Let regions be an empty list of WebVTT regions.
+
+    // 10. For each track track in tracks, append to regions all the regions with an
+    // identifier from track’s list of regions.
+
+    // Steps 9 and 10 are unneccesary because we will initialize each region
+    // individually in text track cue order.
 
     // There is nothing explicitly to be done here, as all the caching occurs
     // within the TextTrackCue instance itself. If parameters of the cue change,
     // the display tree is cleared.
 
+    // 11. If reset is false, then, for each WebVTT region in regions let regionNode
+    // be a WebVTT region object.
+
+    // Steps 12 and 13 are performed in VTTRegion
+
     // If the number of CSS boxes in the output is less than the number of cues
     // we wish to render (e.g., we are adding another cue in a set of roll-up
     // cues), remove all the existing CSS boxes representing the cues and re-add
@@ -145,7 +157,7 @@ void MediaControlTextTrackContainerElement::updateDisplay()
         removeChildren();
 
     activeCues.removeAllMatching([] (CueInterval& cueInterval) {
-        RefPtr<TextTrackCue> cue = cueInterval.data();
+        RefPtr cue = cueInterval.data();
         return !cue->track()
             || !cue->track()->isRendered()
             || cue->track()->mode() == TextTrack::Mode::Disabled
@@ -164,11 +176,15 @@ void MediaControlTextTrackContainerElement::updateDisplay()
         // following substeps:
         for (auto& interval : activeCues) {
             auto cue = interval.data();
-            cue->setFontSize(m_fontSize, m_videoDisplaySize.size(), m_fontSizeIsImportant);
-            if (is<VTTCue>(*cue))
-                processActiveVTTCue(downcast<VTTCue>(*cue));
+
+            if (cue->track()->isSpoken())
+                continue;
+
+            cue->setFontSize(m_fontSize, m_fontSizeIsImportant);
+            if (RefPtr vttCue = dynamicDowncast<VTTCue>(*cue))
+                processActiveVTTCue(*vttCue);
             else {
-                auto displayBox = cue->getDisplayTree(m_videoDisplaySize.size(), m_fontSize);
+                auto displayBox = cue->getDisplayTree();
                 if (displayBox->hasChildNodes() && !contains(displayBox.get()))
                     appendChild(*displayBox);
             }
@@ -203,7 +219,6 @@ void MediaControlTextTrackContainerElement::updateTextTrackRepresentationImageIf
 void MediaControlTextTrackContainerElement::processActiveVTTCue(VTTCue& cue)
 {
     DEBUG_LOG(LOGIDENTIFIER, "adding and positioning cue: \"", cue.text(), "\", start=", cue.startTime(), ", end=", cue.endTime());
-    Ref<TextTrackCueBox> displayBox = *cue.getDisplayTree(m_videoDisplaySize.size(), m_fontSize);
 
     if (auto region = cue.track()->regions()->getRegionById(cue.regionId())) {
         // Let region be the WebVTT region whose region identifier
@@ -213,11 +228,12 @@ void MediaControlTextTrackContainerElement::processActiveVTTCue(VTTCue& cue)
         if (!contains(regionNode.ptr()))
             appendChild(region->getDisplayTree());
 
-        region->appendTextTrackCueBox(WTFMove(displayBox));
+        region->appendTextTrackCueBox(*cue.getDisplayTree());
     } else {
         // If cue has an empty text track cue region identifier or there is no
         // WebVTT region whose region identifier is identical to cue's text
         // track cue region identifier, run the following substeps:
+        Ref<TextTrackCueBox> displayBox = *cue.getDisplayTree();
         if (displayBox->hasChildNodes() && !contains(displayBox.ptr())) {
             // Note: the display tree of a cue is removed when the active flag of the cue is unset.
             appendChild(displayBox);
@@ -233,14 +249,17 @@ void MediaControlTextTrackContainerElement::updateActiveCuesFontSize()
     if (!m_mediaElement)
         return;
 
-    float smallestDimension = std::min(m_videoDisplaySize.size().height(), m_videoDisplaySize.size().width());
     float fontScale = document().page()->group().ensureCaptionPreferences().captionFontSizeScaleAndImportance(m_fontSizeIsImportant);
-    m_fontSize = lroundf(smallestDimension * fontScale);
+
+    // Caption fonts are defined as |size vh| units, so there's no need to
+    // scale by display size. Since |vh| is a decimal percentage, multiply
+    // the scale factor by 100 to achive the final font size.
+    m_fontSize = lroundf(100 * fontScale);
 
     for (auto& activeCue : m_mediaElement->currentlyActiveCues()) {
-        RefPtr<TextTrackCue> cue = activeCue.data();
+        RefPtr cue = activeCue.data();
         if (cue->isRenderable())
-            cue->setFontSize(m_fontSize, m_videoDisplaySize.size(), m_fontSizeIsImportant);
+            cue->setFontSize(m_fontSize, m_fontSizeIsImportant);
     }
 }
 
@@ -257,9 +276,9 @@ void MediaControlTextTrackContainerElement::updateTextStrokeStyle()
     // FIXME: Since it is possible to have more than one text track enabled, the following code may not find the correct language.
     // The default UI only allows a user to enable one track at a time, so it should be OK for now, but we should consider doing
     // this differently, see <https://bugs.webkit.org/show_bug.cgi?id=169875>.
-    if (auto* tracks = m_mediaElement->textTracks()) {
+    if (RefPtr tracks = m_mediaElement->textTracks()) {
         for (unsigned i = 0; i < tracks->length(); ++i) {
-            auto track = tracks->item(i);
+            RefPtr track = tracks->item(i);
             if (track && track->mode() == TextTrack::Mode::Showing) {
                 language = track->validBCP47Language();
                 break;
@@ -272,7 +291,7 @@ void MediaControlTextTrackContainerElement::updateTextStrokeStyle()
 
     // FIXME: find a way to set this property in the stylesheet like the other user style preferences, see <https://bugs.webkit.org/show_bug.cgi?id=169874>.
     if (document().page()->group().ensureCaptionPreferences().captionStrokeWidthForFont(m_fontSize, language, strokeWidth, important))
-        setInlineStyleProperty(CSSPropertyStrokeWidth, strokeWidth, CSSUnitType::CSS_PX, important);
+        setInlineStyleProperty(CSSPropertyStrokeWidth, strokeWidth, CSSUnitType::CSS_PX, important ? IsImportant::Yes : IsImportant::No);
 }
 
 void MediaControlTextTrackContainerElement::updateTextTrackRepresentationIfNeeded()
@@ -294,7 +313,7 @@ void MediaControlTextTrackContainerElement::updateTextTrackRepresentationIfNeede
     if (!m_textTrackRepresentation) {
         ALWAYS_LOG(LOGIDENTIFIER);
 
-        m_textTrackRepresentation = TextTrackRepresentation::create(*this);
+        m_textTrackRepresentation = TextTrackRepresentation::create(*this, *m_mediaElement);
         if (document().page())
             m_textTrackRepresentation->setContentScale(document().page()->deviceScaleFactor());
         m_mediaElement->setTextTrackRepresentation(m_textTrackRepresentation.get());
@@ -333,6 +352,12 @@ void MediaControlTextTrackContainerElement::updateTextTrackStyle()
     removeInlineStyleProperty(CSSPropertyTop);
 }
 
+void MediaControlTextTrackContainerElement::requiresTextTrackRepresentationChanged()
+{
+    updateTextTrackRepresentationIfNeeded();
+    updateSizes(ForceUpdate::Yes);
+}
+
 void MediaControlTextTrackContainerElement::enteredFullscreen()
 {
     updateTextTrackRepresentationIfNeeded();
@@ -360,9 +385,10 @@ bool MediaControlTextTrackContainerElement::updateVideoDisplaySize()
         videoBox.setWidth(videoBox.width() * deviceScaleFactor);
         videoBox.setHeight(videoBox.height() * deviceScaleFactor);
     } else {
-        if (!is<RenderVideo>(m_mediaElement->renderer()))
+        if (auto* renderVideo = dynamicDowncast<RenderVideo>(m_mediaElement->renderer()))
+            videoBox = renderVideo->videoBox();
+        else
             return false;
-        videoBox = downcast<RenderVideo>(*m_mediaElement->renderer()).videoBox();
     }
 
     if (m_videoDisplaySize == videoBox)
@@ -393,16 +419,16 @@ void MediaControlTextTrackContainerElement::updateSizes(ForceUpdate force)
     });
 }
 
-RefPtr<Image> MediaControlTextTrackContainerElement::createTextTrackRepresentationImage()
+RefPtr<NativeImage> MediaControlTextTrackContainerElement::createTextTrackRepresentationImage()
 {
     if (!hasChildNodes())
         return nullptr;
 
-    RefPtr<Frame> frame = document().frame();
+    RefPtr frame = document().frame();
     if (!frame)
         return nullptr;
 
-    document().updateLayout();
+    protectedDocument()->updateLayout();
 
     auto* renderer = this->renderer();
     if (!renderer)
@@ -420,7 +446,7 @@ RefPtr<Image> MediaControlTextTrackContainerElement::createTextTrackRepresentati
     IntRect paintingRect = IntRect(IntPoint(), layer->size());
 
     // FIXME (149422): This buffer should not be unconditionally unaccelerated.
-    auto buffer = ImageBuffer::create(paintingRect.size(), RenderingMode::Unaccelerated, deviceScaleFactor, DestinationColorSpace::SRGB(), PixelFormat::BGRA8);
+    auto buffer = ImageBuffer::create(paintingRect.size(), RenderingMode::Unaccelerated, RenderingPurpose::Unspecified, deviceScaleFactor, DestinationColorSpace::SRGB(), ImageBufferPixelFormat::BGRA8);
     if (!buffer)
         return nullptr;
 
@@ -428,7 +454,7 @@ RefPtr<Image> MediaControlTextTrackContainerElement::createTextTrackRepresentati
     paintFlags.add(RenderLayer::PaintLayerFlag::TemporaryClipRects);
     layer->paint(buffer->context(), paintingRect, LayoutSize(), { PaintBehavior::FlattenCompositingLayers, PaintBehavior::Snapshotting }, nullptr, paintFlags);
 
-    return ImageBuffer::sinkIntoImage(WTFMove(buffer));
+    return ImageBuffer::sinkIntoNativeImage(WTFMove(buffer));
 }
 
 void MediaControlTextTrackContainerElement::textTrackRepresentationBoundsChanged(const IntRect&)
@@ -466,7 +492,7 @@ const Logger& MediaControlTextTrackContainerElement::logger() const
     return *m_logger;
 }
 
-const void* MediaControlTextTrackContainerElement::logIdentifier() const
+uint64_t MediaControlTextTrackContainerElement::logIdentifier() const
 {
     if (!m_logIdentifier && m_mediaElement)
         m_logIdentifier = m_mediaElement->logIdentifier();
