@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2017-2019 Apple Inc. All rights reserved.
+ * Copyright (C) 2017-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -34,24 +34,19 @@
 #include "CanvasRenderingContext2D.h"
 #include "DOMMatrix2DInit.h"
 #include "DOMPointInit.h"
-#include "Document.h"
-#include "Element.h"
 #include "EventLoop.h"
-#include "Frame.h"
-#include "FrameDestructionObserverInlines.h"
 #include "HTMLCanvasElement.h"
 #include "HTMLImageElement.h"
 #include "HTMLVideoElement.h"
 #include "ImageBitmap.h"
 #include "ImageBitmapRenderingContext.h"
 #include "ImageData.h"
-#include "InspectorDOMAgent.h"
 #include "InspectorInstrumentation.h"
 #include "InspectorShaderProgram.h"
 #include "InstrumentingAgents.h"
 #include "JSExecState.h"
-#include "OffscreenCanvas.h"
 #include "Path2D.h"
+#include "PlaceholderRenderingContext.h"
 #include "StringAdaptors.h"
 #include "WebGL2RenderingContext.h"
 #include "WebGLBuffer.h"
@@ -81,23 +76,30 @@
 #include <wtf/HashSet.h>
 #include <wtf/Lock.h>
 #include <wtf/RefPtr.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/Vector.h>
 #include <wtf/text/WTFString.h>
 
+
+#if ENABLE(OFFSCREEN_CANVAS)
+#include "OffscreenCanvas.h"
+#include "OffscreenCanvasRenderingContext2D.h"
+#endif
 namespace WebCore {
 
 using namespace Inspector;
 
-InspectorCanvasAgent::InspectorCanvasAgent(PageAgentContext& context)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(InspectorCanvasAgent);
+
+InspectorCanvasAgent::InspectorCanvasAgent(WebAgentContext& context)
     : InspectorAgentBase("Canvas"_s, context)
     , m_frontendDispatcher(makeUnique<Inspector::CanvasFrontendDispatcher>(context.frontendRouter))
     , m_backendDispatcher(Inspector::CanvasBackendDispatcher::create(context.backendDispatcher, this))
     , m_injectedScriptManager(context.injectedScriptManager)
-    , m_inspectedPage(context.inspectedPage)
     , m_canvasDestroyedTimer(*this, &InspectorCanvasAgent::canvasDestroyedTimerFired)
 #if ENABLE(WEBGL)
     , m_programDestroyedTimer(*this, &InspectorCanvasAgent::programDestroyedTimerFired)
-#endif // ENABLE(WEBGL)
+#endif
 {
 }
 
@@ -117,27 +119,50 @@ void InspectorCanvasAgent::discardAgent()
     reset();
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::enable()
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::enable()
 {
-    if (m_instrumentingAgents.enabledCanvasAgent() == this)
-        return { };
+    if (enabled())
+        return makeUnexpected("Canvas domain already enabled"_s);
+
+    internalEnable();
+
+    return { };
+}
+
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::disable()
+{
+    internalDisable();
+
+    return { };
+}
+
+bool InspectorCanvasAgent::enabled() const
+{
+    return m_instrumentingAgents.enabledCanvasAgent() == this;
+}
+
+void InspectorCanvasAgent::internalEnable()
+{
+    ASSERT(!enabled());
 
     m_instrumentingAgents.setEnabledCanvasAgent(this);
 
-    const auto existsInCurrentPage = [&] (ScriptExecutionContext* scriptExecutionContext) {
-        if (!is<Document>(scriptExecutionContext))
-            return false;
-
-        // FIXME: <https://webkit.org/b/168475> Web Inspector: Correctly display iframe's WebSockets
-        auto* document = downcast<Document>(scriptExecutionContext);
-        return document->page() == &m_inspectedPage;
-    };
-
     {
         Locker locker { CanvasRenderingContext::instancesLock() };
         for (auto* context : CanvasRenderingContext::instances()) {
+            if (!is<CanvasRenderingContext2D>(context)
+                && !is<ImageBitmapRenderingContext>(context)
+#if ENABLE(OFFSCREEN_CANVAS)
+                && !is<OffscreenCanvasRenderingContext2D>(context)
+#endif
+#if ENABLE(WEBGL)
+                && !is<WebGLRenderingContext>(context)
+                && !is<WebGL2RenderingContext>(context)
+#endif
+            )
+                continue;
 
-            if (existsInCurrentPage(context->canvasBase().scriptExecutionContext()))
+            if (matchesCurrentContext(context->canvasBase().scriptExecutionContext()))
                 bindCanvas(*context, false);
         }
     }
@@ -146,85 +171,34 @@ Protocol::ErrorStringOr<void> InspectorCanvasAgent::enable()
     {
         Locker locker { WebGLProgram::instancesLock() };
         for (auto& [program, contextWebGLBase] : WebGLProgram::instances()) {
-            if (contextWebGLBase && existsInCurrentPage(contextWebGLBase->canvasBase().scriptExecutionContext()))
+            if (contextWebGLBase && matchesCurrentContext(contextWebGLBase->canvasBase().scriptExecutionContext()))
                 didCreateWebGLProgram(*contextWebGLBase, *program);
         }
     }
-#endif // ENABLE(WEBGL)
-
-    return { };
+#endif
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::disable()
+void InspectorCanvasAgent::internalDisable()
 {
     m_instrumentingAgents.setEnabledCanvasAgent(nullptr);
 
     reset();
 
     m_recordingAutoCaptureFrameCount = std::nullopt;
-
-    return { };
 }
 
-Protocol::ErrorStringOr<Protocol::DOM::NodeId> InspectorCanvasAgent::requestNode(const Protocol::Canvas::CanvasId& canvasId)
+Inspector::Protocol::ErrorStringOr<String> InspectorCanvasAgent::requestContent(const Inspector::Protocol::Canvas::CanvasId& canvasId)
 {
-    Protocol::ErrorString errorString;
-
+    Inspector::Protocol::ErrorString errorString;
     auto inspectorCanvas = assertInspectorCanvas(errorString, canvasId);
     if (!inspectorCanvas)
         return makeUnexpected(errorString);
-
-    auto* node = inspectorCanvas->canvasElement();
-    if (!node)
-        makeUnexpected("Missing element of canvas for given canvasId"_s);
-
-    // FIXME: <https://webkit.org/b/213499> Web Inspector: allow DOM nodes to be instrumented at any point, regardless of whether the main document has also been instrumented
-    int documentNodeId = m_instrumentingAgents.persistentDOMAgent()->boundNodeId(&node->document());
-    if (!documentNodeId)
-        makeUnexpected("Document must have been requested"_s);
-
-    return m_instrumentingAgents.persistentDOMAgent()->pushNodeToFrontend(errorString, documentNodeId, node);
+    return inspectorCanvas->getContentAsDataURL();
 }
 
-Protocol::ErrorStringOr<String> InspectorCanvasAgent::requestContent(const Protocol::Canvas::CanvasId& canvasId)
+Inspector::Protocol::ErrorStringOr<Ref<Inspector::Protocol::Runtime::RemoteObject>> InspectorCanvasAgent::resolveContext(const Inspector::Protocol::Canvas::CanvasId& canvasId, const String& objectGroup)
 {
-    Protocol::ErrorString errorString;
-
-    auto inspectorCanvas = assertInspectorCanvas(errorString, canvasId);
-    if (!inspectorCanvas)
-        return makeUnexpected(errorString);
-
-    auto result = inspectorCanvas->getCanvasContentAsDataURL(errorString);
-    if (!result)
-        return makeUnexpected(errorString);
-
-    return result;
-}
-
-Protocol::ErrorStringOr<Ref<JSON::ArrayOf<Protocol::DOM::NodeId>>> InspectorCanvasAgent::requestClientNodes(const Protocol::Canvas::CanvasId& canvasId)
-{
-    Protocol::ErrorString errorString;
-
-    auto* domAgent = m_instrumentingAgents.persistentDOMAgent();
-    if (!domAgent)
-        return makeUnexpected("DOM domain must be enabled"_s);
-
-    auto inspectorCanvas = assertInspectorCanvas(errorString, canvasId);
-    if (!inspectorCanvas)
-        return makeUnexpected(errorString);
-
-    auto clientNodeIds = JSON::ArrayOf<Protocol::DOM::NodeId>::create();
-    for (auto& clientNode : inspectorCanvas->clientNodes()) {
-        // FIXME: <https://webkit.org/b/213499> Web Inspector: allow DOM nodes to be instrumented at any point, regardless of whether the main document has also been instrumented
-        if (auto documentNodeId = domAgent->boundNodeId(&clientNode->document()))
-            clientNodeIds->addItem(domAgent->pushNodeToFrontend(errorString, documentNodeId, clientNode));
-    }
-    return clientNodeIds;
-}
-
-Protocol::ErrorStringOr<Ref<Protocol::Runtime::RemoteObject>> InspectorCanvasAgent::resolveContext(const Protocol::Canvas::CanvasId& canvasId, const String& objectGroup)
-{
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorCanvas = assertInspectorCanvas(errorString, canvasId);
     if (!inspectorCanvas)
@@ -248,7 +222,7 @@ Protocol::ErrorStringOr<Ref<Protocol::Runtime::RemoteObject>> InspectorCanvasAge
     return result.releaseNonNull();
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::setRecordingAutoCaptureFrameCount(int count)
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::setRecordingAutoCaptureFrameCount(int count)
 {
     if (count > 0)
         m_recordingAutoCaptureFrameCount = count;
@@ -257,21 +231,19 @@ Protocol::ErrorStringOr<void> InspectorCanvasAgent::setRecordingAutoCaptureFrame
     return { };
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::startRecording(const Protocol::Canvas::CanvasId& canvasId, std::optional<int>&& frameCount, std::optional<int>&& memoryLimit)
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::startRecording(const Inspector::Protocol::Canvas::CanvasId& canvasId, std::optional<int>&& frameCount, std::optional<int>&& memoryLimit)
 {
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorCanvas = assertInspectorCanvas(errorString, canvasId);
     if (!inspectorCanvas)
         return makeUnexpected(errorString);
 
-    auto* context = inspectorCanvas->canvasContext();
+    auto& context = inspectorCanvas->canvasContext();
 
     // FIXME: <https://webkit.org/b/201651> Web Inspector: Canvas: support canvas recordings for WebGPUDevice
-    if (!context)
-        return makeUnexpected("Not supported"_s);
 
-    if (context->hasActiveInspectorCanvasCallTracer())
+    if (context.hasActiveInspectorCanvasCallTracer())
         return makeUnexpected("Already recording canvas"_s);
 
     RecordingOptions recordingOptions;
@@ -279,37 +251,36 @@ Protocol::ErrorStringOr<void> InspectorCanvasAgent::startRecording(const Protoco
         recordingOptions.frameCount = *frameCount;
     if (memoryLimit)
         recordingOptions.memoryLimit = *memoryLimit;
-    startRecording(*inspectorCanvas, Protocol::Recording::Initiator::Frontend, WTFMove(recordingOptions));
+    startRecording(*inspectorCanvas, Inspector::Protocol::Recording::Initiator::Frontend, WTFMove(recordingOptions));
 
     return { };
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::stopRecording(const Protocol::Canvas::CanvasId& canvasId)
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::stopRecording(const Inspector::Protocol::Canvas::CanvasId& canvasId)
 {
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorCanvas = assertInspectorCanvas(errorString, canvasId);
     if (!inspectorCanvas)
         return makeUnexpected(errorString);
 
-    auto* context = inspectorCanvas->canvasContext();
+    auto& context = inspectorCanvas->canvasContext();
 
     // FIXME: <https://webkit.org/b/201651> Web Inspector: Canvas: support canvas recordings for WebGPUDevice
-    if (!context)
-        return makeUnexpected("Not supported"_s);
 
-    if (!context->hasActiveInspectorCanvasCallTracer())
+    if (!context.hasActiveInspectorCanvasCallTracer())
         return makeUnexpected("Not recording canvas"_s);
 
-    didFinishRecordingCanvasFrame(*context, true);
+    didFinishRecordingCanvasFrame(context, true);
 
     return { };
 }
 
 #if ENABLE(WEBGL)
-Protocol::ErrorStringOr<String> InspectorCanvasAgent::requestShaderSource(const Protocol::Canvas::ProgramId& programId, Protocol::Canvas::ShaderType shaderType)
+
+Inspector::Protocol::ErrorStringOr<String> InspectorCanvasAgent::requestShaderSource(const Inspector::Protocol::Canvas::ProgramId& programId, Inspector::Protocol::Canvas::ShaderType shaderType)
 {
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorProgram = assertInspectorProgram(errorString, programId);
     if (!inspectorProgram)
@@ -322,9 +293,9 @@ Protocol::ErrorStringOr<String> InspectorCanvasAgent::requestShaderSource(const
     return source;
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::updateShader(const Protocol::Canvas::ProgramId& programId, Protocol::Canvas::ShaderType shaderType, const String& source)
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::updateShader(const Inspector::Protocol::Canvas::ProgramId& programId, Inspector::Protocol::Canvas::ShaderType shaderType, const String& source)
 {
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorProgram = assertInspectorProgram(errorString, programId);
     if (!inspectorProgram)
@@ -336,9 +307,9 @@ Protocol::ErrorStringOr<void> InspectorCanvasAgent::updateShader(const Protocol:
     return { };
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::setShaderProgramDisabled(const Protocol::Canvas::ProgramId& programId, bool disabled)
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::setShaderProgramDisabled(const Inspector::Protocol::Canvas::ProgramId& programId, bool disabled)
 {
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorProgram = assertInspectorProgram(errorString, programId);
     if (!inspectorProgram)
@@ -349,9 +320,9 @@ Protocol::ErrorStringOr<void> InspectorCanvasAgent::setShaderProgramDisabled(con
     return { };
 }
 
-Protocol::ErrorStringOr<void> InspectorCanvasAgent::setShaderProgramHighlighted(const Protocol::Canvas::ProgramId& programId, bool highlighted)
+Inspector::Protocol::ErrorStringOr<void> InspectorCanvasAgent::setShaderProgramHighlighted(const Inspector::Protocol::Canvas::ProgramId& programId, bool highlighted)
 {
-    Protocol::ErrorString errorString;
+    Inspector::Protocol::ErrorString errorString;
 
     auto inspectorProgram = assertInspectorProgram(errorString, programId);
     if (!inspectorProgram)
@@ -361,43 +332,9 @@ Protocol::ErrorStringOr<void> InspectorCanvasAgent::setShaderProgramHighlighted(
 
     return { };
 }
+
 #endif // ENABLE(WEBGL)
 
-void InspectorCanvasAgent::frameNavigated(Frame& frame)
-{
-    if (frame.isMainFrame()) {
-        reset();
-        return;
-    }
-
-    Vector<InspectorCanvas*> inspectorCanvases;
-    for (auto& inspectorCanvas : m_identifierToInspectorCanvas.values()) {
-        if (auto* canvasElement = inspectorCanvas->canvasElement()) {
-            if (canvasElement->document().frame() == &frame)
-                inspectorCanvases.append(inspectorCanvas.get());
-        }
-    }
-
-    for (auto* inspectorCanvas : inspectorCanvases)
-        unbindCanvas(*inspectorCanvas);
-}
-
-void InspectorCanvasAgent::didChangeCSSCanvasClientNodes(CanvasBase& canvasBase)
-{
-    auto* context = canvasBase.renderingContext();
-    if (!context) {
-        ASSERT_NOT_REACHED();
-        return;
-    }
-
-    auto inspectorCanvas = findInspectorCanvas(*context);
-    ASSERT(inspectorCanvas);
-    if (!inspectorCanvas)
-        return;
-
-    m_frontendDispatcher->clientNodesChanged(inspectorCanvas->identifier());
-}
-
 void InspectorCanvasAgent::didCreateCanvasRenderingContext(CanvasRenderingContext& context)
 {
     if (findInspectorCanvas(context)) {
@@ -410,10 +347,25 @@ void InspectorCanvasAgent::didCreateCanvasRenderingContext(CanvasRenderingContex
     if (m_recordingAutoCaptureFrameCount) {
         RecordingOptions recordingOptions;
         recordingOptions.frameCount = m_recordingAutoCaptureFrameCount.value();
-        startRecording(inspectorCanvas, Protocol::Recording::Initiator::AutoCapture, WTFMove(recordingOptions));
+        startRecording(inspectorCanvas, Inspector::Protocol::Recording::Initiator::AutoCapture, WTFMove(recordingOptions));
     }
 }
 
+void InspectorCanvasAgent::didChangeCanvasSize(CanvasRenderingContext& context)
+{
+    RefPtr<InspectorCanvas> inspectorCanvas;
+
+    if (!inspectorCanvas)
+        inspectorCanvas = findInspectorCanvas(context);
+
+    ASSERT(inspectorCanvas);
+    if (!inspectorCanvas)
+        return;
+
+    const auto& size = inspectorCanvas->canvasContext().canvasBase().size();
+    m_frontendDispatcher->canvasSizeChanged(inspectorCanvas->identifier(), size.width(), size.height());
+}
+
 void InspectorCanvasAgent::didChangeCanvasMemory(CanvasRenderingContext& context)
 {
     RefPtr<InspectorCanvas> inspectorCanvas;
@@ -425,13 +377,10 @@ void InspectorCanvasAgent::didChangeCanvasMemory(CanvasRenderingContext& context
     if (!inspectorCanvas)
         return;
 
-    // FIXME: <https://webkit.org/b/180833> Web Inspector: support OffscreenCanvas for Canvas related operations
-
-    if (auto* node = inspectorCanvas->canvasElement())
-        m_frontendDispatcher->canvasMemoryChanged(inspectorCanvas->identifier(), node->memoryCost());
+    m_frontendDispatcher->canvasMemoryChanged(inspectorCanvas->identifier(), inspectorCanvas->canvasContext().canvasBase().memoryCost());
 }
 
-void InspectorCanvasAgent::canvasChanged(CanvasBase& canvasBase, const std::optional<FloatRect>&)
+void InspectorCanvasAgent::canvasChanged(CanvasBase& canvasBase, const FloatRect&)
 {
     auto* context = canvasBase.renderingContext();
     if (!context)
@@ -512,7 +461,7 @@ void InspectorCanvasAgent::consoleStartRecordingCanvas(CanvasRenderingContext& c
         if (JSC::JSValue optionName = options->get(&exec, JSC::Identifier::fromString(vm, "name"_s)))
             recordingOptions.name = optionName.toWTFString(&exec);
     }
-    startRecording(*inspectorCanvas, Protocol::Recording::Initiator::Console, WTFMove(recordingOptions));
+    startRecording(*inspectorCanvas, Inspector::Protocol::Recording::Initiator::Console, WTFMove(recordingOptions));
 }
 
 void InspectorCanvasAgent::consoleStopRecordingCanvas(CanvasRenderingContext& context)
@@ -521,6 +470,7 @@ void InspectorCanvasAgent::consoleStopRecordingCanvas(CanvasRenderingContext& co
 }
 
 #if ENABLE(WEBGL)
+
 void InspectorCanvasAgent::didEnableExtension(WebGLRenderingContextBase& context, const String& extension)
 {
     auto inspectorCanvas = findInspectorCanvas(context);
@@ -572,6 +522,7 @@ bool InspectorCanvasAgent::isWebGLProgramHighlighted(WebGLProgram& program)
 
     return inspectorProgram->highlighted();
 }
+
 #endif // ENABLE(WEBGL)
 
 #define PROCESS_ARGUMENT_DEFINITION(ArgumentType) \
@@ -607,12 +558,11 @@ void InspectorCanvasAgent::recordAction(CanvasRenderingContext& canvasRenderingC
                     if (!inspectorCanvas)
                         continue;
 
-                    auto* canvasRenderingContext = inspectorCanvas->canvasContext();
-                    ASSERT(canvasRenderingContext);
+                    auto& canvasRenderingContext = inspectorCanvas->canvasContext();
                     // FIXME: <https://webkit.org/b/201651> Web Inspector: Canvas: support canvas recordings for WebGPUDevice
 
-                    if (canvasRenderingContext->hasActiveInspectorCanvasCallTracer())
-                        canvasAgent.didFinishRecordingCanvasFrame(*canvasRenderingContext);
+                    if (canvasRenderingContext.hasActiveInspectorCanvasCallTracer())
+                        canvasAgent.didFinishRecordingCanvasFrame(canvasRenderingContext);
                 }
 
                 canvasAgent.m_recordingCanvasIdentifiers.clear();
@@ -628,24 +578,24 @@ void InspectorCanvasAgent::recordAction(CanvasRenderingContext& canvasRenderingC
         didFinishRecordingCanvasFrame(canvasRenderingContext, true);
 }
 
-void InspectorCanvasAgent::startRecording(InspectorCanvas& inspectorCanvas, Protocol::Recording::Initiator initiator, RecordingOptions&& recordingOptions)
+void InspectorCanvasAgent::startRecording(InspectorCanvas& inspectorCanvas, Inspector::Protocol::Recording::Initiator initiator, RecordingOptions&& recordingOptions)
 {
-    auto* context = inspectorCanvas.canvasContext();
-    ASSERT(context);
+    auto& context = inspectorCanvas.canvasContext();
     // FIXME: <https://webkit.org/b/201651> Web Inspector: Canvas: support canvas recordings for WebGPUDevice
 
     if (!is<CanvasRenderingContext2D>(context)
         && !is<ImageBitmapRenderingContext>(context)
+#if ENABLE(OFFSCREEN_CANVAS)
+        && !is<OffscreenCanvasRenderingContext2D>(context)
+#endif
 #if ENABLE(WEBGL)
         && !is<WebGLRenderingContext>(context)
-#endif // ENABLE(WEBGL)
-#if ENABLE(WEBGL2)
         && !is<WebGL2RenderingContext>(context)
-#endif // ENABLE(WEBGL2)
+#endif
     )
         return;
 
-    if (context->hasActiveInspectorCanvasCallTracer())
+    if (context.hasActiveInspectorCanvasCallTracer())
         return;
 
     inspectorCanvas.resetRecordingData();
@@ -655,7 +605,7 @@ void InspectorCanvasAgent::startRecording(InspectorCanvas& inspectorCanvas, Prot
         inspectorCanvas.setBufferLimit(recordingOptions.memoryLimit.value());
     if (recordingOptions.name)
         inspectorCanvas.setRecordingName(recordingOptions.name.value());
-    context->setHasActiveInspectorCanvasCallTracer(true);
+    context.setHasActiveInspectorCanvasCallTracer(true);
 
     m_frontendDispatcher->recordingStarted(inspectorCanvas.identifier(), initiator);
 }
@@ -672,6 +622,7 @@ void InspectorCanvasAgent::canvasDestroyedTimerFired()
 }
 
 #if ENABLE(WEBGL)
+
 void InspectorCanvasAgent::programDestroyedTimerFired()
 {
     if (!m_removedProgramIdentifiers.size())
@@ -682,14 +633,13 @@ void InspectorCanvasAgent::programDestroyedTimerFired()
 
     m_removedProgramIdentifiers.clear();
 }
-#endif // ENABLE(WEBGL)
+
+#endif
 
 void InspectorCanvasAgent::reset()
 {
-    for (auto& inspectorCanvas : m_identifierToInspectorCanvas.values()) {
-        if (auto* context = inspectorCanvas->canvasContext())
-            context->canvasBase().removeObserver(*this);
-    }
+    for (auto& inspectorCanvas : m_identifierToInspectorCanvas.values())
+        inspectorCanvas->canvasContext().canvasBase().removeObserver(*this);
 
     m_identifierToInspectorCanvas.clear();
     m_removedCanvasIdentifiers.clear();
@@ -701,7 +651,7 @@ void InspectorCanvasAgent::reset()
     m_removedProgramIdentifiers.clear();
     if (m_programDestroyedTimer.isActive())
         m_programDestroyedTimer.stop();
-#endif // ENABLE(WEBGL)
+#endif
 
     m_recordingCanvasIdentifiers.clear();
 }
@@ -732,19 +682,19 @@ InspectorCanvas& InspectorCanvasAgent::bindCanvas(CanvasRenderingContext& contex
 
 void InspectorCanvasAgent::unbindCanvas(InspectorCanvas& inspectorCanvas)
 {
+    didFinishRecordingCanvasFrame(inspectorCanvas.canvasContext(), true);
+
 #if ENABLE(WEBGL)
     Vector<InspectorShaderProgram*> programsToRemove;
     for (auto& inspectorProgram : m_identifierToInspectorProgram.values()) {
         if (&inspectorProgram->canvas() == &inspectorCanvas)
             programsToRemove.append(inspectorProgram.get());
     }
-
     for (auto* inspectorProgram : programsToRemove)
         unbindProgram(*inspectorProgram);
-#endif // ENABLE(WEBGL)
+#endif
 
-    if (auto* context = inspectorCanvas.canvasContext())
-        context->canvasBase().removeObserver(*this);
+    inspectorCanvas.canvasContext().canvasBase().removeObserver(*this);
 
     String identifier = inspectorCanvas.identifier();
     m_identifierToInspectorCanvas.remove(identifier);
@@ -758,7 +708,7 @@ void InspectorCanvasAgent::unbindCanvas(InspectorCanvas& inspectorCanvas)
         m_canvasDestroyedTimer.startOneShot(0_s);
 }
 
-RefPtr<InspectorCanvas> InspectorCanvasAgent::assertInspectorCanvas(Protocol::ErrorString& errorString, const String& canvasId)
+RefPtr<InspectorCanvas> InspectorCanvasAgent::assertInspectorCanvas(Inspector::Protocol::ErrorString& errorString, const String& canvasId)
 {
     auto inspectorCanvas = m_identifierToInspectorCanvas.get(canvasId);
     if (!inspectorCanvas) {
@@ -771,13 +721,14 @@ RefPtr<InspectorCanvas> InspectorCanvasAgent::assertInspectorCanvas(Protocol::Er
 RefPtr<InspectorCanvas> InspectorCanvasAgent::findInspectorCanvas(CanvasRenderingContext& context)
 {
     for (auto& inspectorCanvas : m_identifierToInspectorCanvas.values()) {
-        if (inspectorCanvas->canvasContext() == &context)
+        if (&inspectorCanvas->canvasContext() == &context)
             return inspectorCanvas;
     }
     return nullptr;
 }
 
 #if ENABLE(WEBGL)
+
 void InspectorCanvasAgent::unbindProgram(InspectorShaderProgram& inspectorProgram)
 {
     String identifier = inspectorProgram.identifier();
@@ -792,7 +743,7 @@ void InspectorCanvasAgent::unbindProgram(InspectorShaderProgram& inspectorProgra
         m_programDestroyedTimer.startOneShot(0_s);
 }
 
-RefPtr<InspectorShaderProgram> InspectorCanvasAgent::assertInspectorProgram(Protocol::ErrorString& errorString, const String& programId)
+RefPtr<InspectorShaderProgram> InspectorCanvasAgent::assertInspectorProgram(Inspector::Protocol::ErrorString& errorString, const String& programId)
 {
     auto inspectorProgram = m_identifierToInspectorProgram.get(programId);
     if (!inspectorProgram) {
@@ -805,11 +756,12 @@ RefPtr<InspectorShaderProgram> InspectorCanvasAgent::assertInspectorProgram(Prot
 RefPtr<InspectorShaderProgram> InspectorCanvasAgent::findInspectorProgram(WebGLProgram& program)
 {
     for (auto& inspectorProgram : m_identifierToInspectorProgram.values()) {
-        if (inspectorProgram->program() == &program)
+        if (&inspectorProgram->program() == &program)
             return inspectorProgram;
     }
     return nullptr;
 }
+
 #endif // ENABLE(WEBGL)
 
 } // namespace WebCore
