@@ -30,10 +30,13 @@
 #include "ContentSecurityPolicy.h"
 #include "SecurityOriginData.h"
 #include <pal/text/TextEncoding.h>
+#include <wtf/TZoneMallocInlines.h>
 #include <wtf/URL.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(ContentSecurityPolicySource);
+
 ContentSecurityPolicySource::ContentSecurityPolicySource(const ContentSecurityPolicy& policy, const String& scheme, const String& host, std::optional<uint16_t> port, const String& path, bool hostHasWildcard, bool portHasWildcard, IsSelfSource isSelfSource)
     : m_policy(policy)
     , m_scheme(scheme)
@@ -93,8 +96,11 @@ static bool wildcardMatches(StringView host, const String& hostWithWildcard)
 bool ContentSecurityPolicySource::hostMatches(const URL& url) const
 {
     auto host = url.host();
-    if (m_hostHasWildcard)
+    if (m_hostHasWildcard) {
+        if (m_host.isEmpty())
+            return true;
         return wildcardMatches(host, m_host);
+    }
     return equalIgnoringASCIICase(host, m_host);
 }
 
@@ -140,7 +146,7 @@ bool ContentSecurityPolicySource::portMatches(const URL& url) const
 
 bool ContentSecurityPolicySource::isSchemeOnly() const
 {
-    return m_host.isEmpty();
+    return m_host.isEmpty() && !m_hostHasWildcard;
 }
 
 ContentSecurityPolicySource::operator SecurityOriginData() const
