@@ -36,9 +36,12 @@
 #include "DenormalDisabler.h"
 #include <algorithm>
 #include <wtf/MathExtras.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebCore {
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(DynamicsCompressorKernel);
+
 using namespace AudioUtilities;
 
 // Metering hits peaks instantly, but releases this fast (in seconds).
@@ -60,9 +63,9 @@ void DynamicsCompressorKernel::setNumberOfChannels(unsigned numberOfChannels)
     if (m_preDelayBuffers.size() == numberOfChannels)
         return;
 
-    m_preDelayBuffers.clear();
-    for (unsigned i = 0; i < numberOfChannels; ++i)
-        m_preDelayBuffers.append(makeUnique<AudioFloatArray>(MaxPreDelayFrames));
+    m_preDelayBuffers = Vector<std::unique_ptr<AudioFloatArray>>(numberOfChannels, [](size_t) {
+        return makeUnique<AudioFloatArray>(MaxPreDelayFrames);
+    });
 }
 
 void DynamicsCompressorKernel::setPreDelayTime(float preDelayTime)
@@ -185,11 +188,9 @@ float DynamicsCompressorKernel::updateStaticCurveParameters(float dbThreshold, f
     return m_K;
 }
 
-void DynamicsCompressorKernel::process(const float* sourceChannels[],
-                                       float* destinationChannels[],
-                                       unsigned numberOfChannels,
+void DynamicsCompressorKernel::process(std::span<std::span<const float>> sourceChannels,
+                                       std::span<std::span<float>> destinationChannels,
                                        unsigned framesToProcess,
-
                                        float dbThreshold,
                                        float dbKnee,
                                        float ratio,
@@ -205,7 +206,7 @@ void DynamicsCompressorKernel::process(const float* sourceChannels[],
                                        float releaseZone4
                                        )
 {
-    ASSERT(m_preDelayBuffers.size() == numberOfChannels);
+    ASSERT(m_preDelayBuffers.size() == sourceChannels.size());
 
     float sampleRate = this->sampleRate();
 
@@ -357,8 +358,8 @@ void DynamicsCompressorKernel::process(const float* sourceChannels[],
                 float compressorInput = 0;
 
                 // Predelay signal, computing compression amount from un-delayed version.
-                for (unsigned i = 0; i < numberOfChannels; ++i) {
-                    float* delayBuffer = m_preDelayBuffers[i]->data();
+                for (unsigned i = 0; i < sourceChannels.size(); ++i) {
+                    auto delayBuffer = m_preDelayBuffers[i]->span();
                     float undelayedSource = sourceChannels[i][frameIndex];
                     delayBuffer[preDelayWriteIndex] = undelayedSource;
 
@@ -423,8 +424,8 @@ void DynamicsCompressorKernel::process(const float* sourceChannels[],
                     m_meteringGain += (dbRealGain - m_meteringGain) * m_meteringReleaseK;
 
                 // Apply final gain.
-                for (unsigned i = 0; i < numberOfChannels; ++i) {
-                    float* delayBuffer = m_preDelayBuffers[i]->data();
+                for (unsigned i = 0; i < destinationChannels.size(); ++i) {
+                    auto delayBuffer = m_preDelayBuffers[i]->span();
                     destinationChannels[i][frameIndex] = delayBuffer[preDelayReadIndex] * totalGain;
                 }
 
