@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2021 Apple Inc. All rights reserved.
+ * Copyright (C) 2003-2024 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -36,8 +36,15 @@
 #include <wtf/OptionSet.h>
 #include <wtf/Ref.h>
 #include <wtf/StdLibExtras.h>
+#include <wtf/TZoneMalloc.h>
 #include <wtf/ThreadSafeRefCounted.h>
 
+#if USE(SKIA)
+WTF_IGNORE_WARNINGS_IN_THIRD_PARTY_CODE_BEGIN
+#include <skia/core/SkColor.h>
+WTF_IGNORE_WARNINGS_IN_THIRD_PARTY_CODE_END
+#endif
+
 #if USE(CG)
 typedef struct CGColor* CGColorRef;
 #endif
@@ -48,13 +55,27 @@ typedef struct _GdkRGBA GdkRGBA;
 
 namespace WebCore {
 
+struct OutOfLineColorDataForIPC {
+    ColorSpace colorSpace;
+    float c1;
+    float c2;
+    float c3;
+    float alpha;
+};
+
+struct ColorDataForIPC {
+    bool isSemantic;
+    bool usesFunctionSerialization;
+    std::variant<PackedColor::RGBA, OutOfLineColorDataForIPC> data;
+};
+
 // Able to represent:
 //    - Special "invalid color" state, treated as transparent black but distinguishable
 //    - 4x 8-bit (0-255) sRGBA, stored inline, no allocation
 //    - 4x float color components + color space, stored in a reference counted sub-object
 
 class Color {
-    WTF_MAKE_FAST_ALLOCATED;
+    WTF_MAKE_TZONE_ALLOCATED(Color);
 public:
     enum class Flags {
         Semantic                        = 1 << 0,
@@ -63,9 +84,10 @@ public:
 
     Color() = default;
 
-    Color(SRGBA<uint8_t>, OptionSet<Flags> = { });
+    WEBCORE_EXPORT Color(SRGBA<uint8_t>, OptionSet<Flags> = { });
     Color(std::optional<SRGBA<uint8_t>>, OptionSet<Flags> = { });
-    
+    WEBCORE_EXPORT Color(std::optional<ColorDataForIPC>&&);
+
     template<typename ColorType, typename std::enable_if_t<IsColorTypeWithComponentType<ColorType, float>>* = nullptr>
     Color(const ColorType&, OptionSet<Flags> = { });
     
@@ -75,6 +97,7 @@ public:
     explicit Color(WTF::HashTableEmptyValueType);
     explicit Color(WTF::HashTableDeletedValueType);
     bool isHashTableDeletedValue() const;
+    bool isHashTableEmptyValue() const;
 
     WEBCORE_EXPORT Color(const Color&);
     WEBCORE_EXPORT Color(Color&&);
@@ -84,13 +107,12 @@ public:
 
     ~Color();
 
-    unsigned hash() const;
-
     bool isValid() const;
     bool isSemantic() const;
     bool usesColorFunctionSerialization() const;
 
     ColorSpace colorSpace() const;
+    WEBCORE_EXPORT std::optional<ColorDataForIPC> data() const;
 
     bool isOpaque() const { return isOutOfLine() ? asOutOfLine().resolvedAlpha() == 1.0 : asInline().resolved().alpha == 255; }
     bool isVisible() const { return isOutOfLine() ? asOutOfLine().resolvedAlpha() > 0.0 : asInline().resolved().alpha > 0; }
@@ -108,6 +130,10 @@ public:
     // or precision of ColorType is smaller than the current underlying type.
     template<typename ColorType> ColorType toColorTypeLossy() const;
 
+    // This acts just like toColorTypeLossy(), but will carry forward missing components
+    // from the underlying type into any analogous components in ColorType.
+    template<typename ColorType> ColorType toColorTypeLossyCarryingForwardMissing() const;
+
     ColorComponents<float, 4> toResolvedColorComponentsInColorSpace(ColorSpace) const;
     ColorComponents<float, 4> toResolvedColorComponentsInColorSpace(const DestinationColorSpace&) const;
 
@@ -129,7 +155,8 @@ public:
 
     Color semanticColor() const;
 
-    // Returns the underlying color if its type is SRGBA<uint8_t>.
+    // Returns the underlying color if its type is inline.
+    std::optional<PackedColor::RGBA> tryGetAsPackedInline() const;
     std::optional<SRGBA<uint8_t>> tryGetAsSRGBABytes() const;
 
 #if PLATFORM(GTK)
@@ -137,6 +164,14 @@ public:
     operator GdkRGBA() const;
 #endif
 
+#if USE(SKIA)
+    Color(const SkColor&);
+    WEBCORE_EXPORT operator SkColor() const;
+
+    Color(const SkColor4f&);
+    operator SkColor4f() const;
+#endif
+
 #if USE(CG)
     WEBCORE_EXPORT static Color createAndPreserveColorSpace(CGColorRef, OptionSet<Flags> = { });
 #endif
@@ -155,6 +190,8 @@ public:
     static constexpr auto green = SRGBA<uint8_t> { 0, 255, 0 };
     static constexpr auto darkGreen = SRGBA<uint8_t> { 0, 128, 0 };
     static constexpr auto orange = SRGBA<uint8_t> { 255, 128, 0 };
+    static constexpr auto purple = SRGBA<uint8_t> { 128, 0, 255 };
+    static constexpr auto gold = SRGBA<uint8_t> { 255, 215, 0 };
 
     static bool isBlackColor(const Color&);
     static bool isWhiteColor(const Color&);
@@ -165,9 +202,6 @@ public:
     friend bool outOfLineComponentsEqual(const Color&, const Color&);
     friend bool outOfLineComponentsEqualIgnoringSemanticColor(const Color&, const Color&);
 
-    template<class Encoder> void encode(Encoder&) const;
-    template<class Decoder> static std::optional<Color> decode(Decoder&);
-
     // Returns the underlying color converted to pre-resolved 8-bit sRGBA, useful for debugging purposes.
     struct DebugRGBA {
         unsigned red;
@@ -180,11 +214,14 @@ public:
     String debugDescription() const;
 
 private:
+    friend void add(Hasher&, const Color&);
+
     class OutOfLineComponents : public ThreadSafeRefCounted<OutOfLineComponents> {
+        WTF_MAKE_FAST_COMPACT_ALLOCATED;
     public:
-        static Ref<OutOfLineComponents> create(ColorComponents<float, 4> components)
+        static Ref<OutOfLineComponents> create(ColorComponents<float, 4>&& components)
         {
-            return adoptRef(*new OutOfLineComponents(components));
+            return adoptRef(*new OutOfLineComponents(WTFMove(components)));
         }
 
         float unresolvedAlpha() const { return m_components[3]; }
@@ -193,8 +230,8 @@ private:
         ColorComponents<float, 4> resolvedComponents() const { return resolveColorComponents(m_components); }
 
     private:
-        OutOfLineComponents(ColorComponents<float, 4> components)
-            : m_components(components)
+        OutOfLineComponents(ColorComponents<float, 4>&& components)
+            : m_components(WTFMove(components))
         {
         }
 
@@ -227,7 +264,7 @@ private:
     PackedColor::RGBA asPackedInline() const;
 
     const OutOfLineComponents& asOutOfLine() const;
-    Ref<OutOfLineComponents> asOutOfLineRef() const;
+    Ref<OutOfLineComponents> protectedAsOutOfLine() const;
 
 #if CPU(ADDRESS64)
     static constexpr unsigned maxNumberOfBitsInPointer = 48;
@@ -257,8 +294,15 @@ private:
     uint64_t m_colorAndFlags { invalidColorAndFlags };
 };
 
+inline void add(Hasher& hasher, const Color& color)
+{
+    if (color.isOutOfLine())
+        add(hasher, color.asOutOfLine().unresolvedComponents(), color.colorSpace(), color.flags());
+    else
+        add(hasher, color.asPackedInline().value, color.flags());
+}
+
 bool operator==(const Color&, const Color&);
-bool operator!=(const Color&, const Color&);
 
 // One or both must be out of line colors.
 bool outOfLineComponentsEqual(const Color&, const Color&);
@@ -279,11 +323,6 @@ inline bool operator==(const Color& a, const Color& b)
     return a.m_colorAndFlags == b.m_colorAndFlags;
 }
 
-inline bool operator!=(const Color& a, const Color& b)
-{
-    return !(a == b);
-}
-
 inline bool outOfLineComponentsEqual(const Color& a, const Color& b)
 {
     if (a.isOutOfLine() && b.isOutOfLine())
@@ -359,19 +398,17 @@ inline bool Color::isHashTableDeletedValue() const
     return flags().contains(FlagsIncludingPrivate::HashTableDeletedValue);
 }
 
+inline bool Color::isHashTableEmptyValue() const
+{
+    return flags().contains(FlagsIncludingPrivate::HashTableEmptyValue);
+}
+
 inline Color::~Color()
 {
     if (isOutOfLine())
         asOutOfLine().deref();
 }
 
-inline unsigned Color::hash() const
-{
-    if (isOutOfLine())
-        return computeHash(asOutOfLine().unresolvedComponents(), colorSpace(), flags().toRaw());
-    return computeHash(asPackedInline().value, flags().toRaw());
-}
-
 inline bool Color::isValid() const
 {
     return flags().contains(FlagsIncludingPrivate::Valid);
@@ -406,6 +443,13 @@ template<typename ColorType> ColorType Color::toColorTypeLossy() const
     });
 }
 
+template<typename ColorType> ColorType Color::toColorTypeLossyCarryingForwardMissing() const
+{
+    return callOnUnderlyingType([] (const auto& underlyingColor) {
+        return convertColorCarryingForwardMissing<ColorType>(underlyingColor);
+    });
+}
+
 inline Color Color::invertedColorWithAlpha(std::optional<float> alpha) const
 {
     return alpha ? invertedColorWithAlpha(alpha.value()) : *this;
@@ -447,7 +491,7 @@ inline const Color::OutOfLineComponents& Color::asOutOfLine() const
     return decodedOutOfLineComponents(m_colorAndFlags);
 }
 
-inline Ref<Color::OutOfLineComponents> Color::asOutOfLineRef() const
+inline Ref<Color::OutOfLineComponents> Color::protectedAsOutOfLine() const
 {
     ASSERT(isOutOfLine());
     return decodedOutOfLineComponents(m_colorAndFlags);
@@ -465,6 +509,13 @@ inline PackedColor::RGBA Color::asPackedInline() const
     return decodedPackedInlineColor(m_colorAndFlags);
 }
 
+inline std::optional<PackedColor::RGBA> Color::tryGetAsPackedInline() const
+{
+    if (isInline())
+        return asPackedInline();
+    return std::nullopt;
+}
+
 inline std::optional<SRGBA<uint8_t>> Color::tryGetAsSRGBABytes() const
 {
     if (isInline())
@@ -495,9 +546,9 @@ inline uint64_t Color::encodedPackedInlineColor(PackedColor::RGBA color)
 inline uint64_t Color::encodedOutOfLineComponents(Ref<OutOfLineComponents>&& outOfLineComponents)
 {
 #if CPU(ADDRESS64)
-    return bitwise_cast<uint64_t>(&outOfLineComponents.leakRef());
+    return std::bit_cast<uint64_t>(&outOfLineComponents.leakRef());
 #else
-    return bitwise_cast<uint32_t>(&outOfLineComponents.leakRef());
+    return std::bit_cast<uint32_t>(&outOfLineComponents.leakRef());
 #endif
 }
 
@@ -524,9 +575,9 @@ inline PackedColor::RGBA Color::decodedPackedInlineColor(uint64_t value)
 inline Color::OutOfLineComponents& Color::decodedOutOfLineComponents(uint64_t value)
 {
 #if CPU(ADDRESS64)
-    return *bitwise_cast<OutOfLineComponents*>(value & colorValueMask);
+    return *std::bit_cast<OutOfLineComponents*>(value & colorValueMask);
 #else
-    return *bitwise_cast<OutOfLineComponents*>(static_cast<uint32_t>(value & colorValueMask));
+    return *std::bit_cast<OutOfLineComponents*>(static_cast<uint32_t>(value & colorValueMask));
 #endif
 }
 
@@ -544,95 +595,6 @@ inline void Color::setOutOfLineComponents(Ref<OutOfLineComponents>&& color, Colo
     ASSERT(isOutOfLine());
 }
 
-template<class Encoder> void Color::encode(Encoder& encoder) const
-{
-    if (!isValid()) {
-        encoder << false;
-        return;
-    }
-    encoder << true;
-
-    encoder << flags().contains(FlagsIncludingPrivate::Semantic);
-    encoder << flags().contains(FlagsIncludingPrivate::UseColorFunctionSerialization);
-    
-    if (isOutOfLine()) {
-        encoder << true;
-        encoder << colorSpace();
-
-        auto& outOfLineComponents = asOutOfLine();
-        auto [c1, c2, c3, alpha] = outOfLineComponents.unresolvedComponents();
-        encoder << c1;
-        encoder << c2;
-        encoder << c3;
-        encoder << alpha;
-        return;
-    }
-    encoder << false;
-
-    encoder << asPackedInline().value;
-}
-
-template<class Decoder> std::optional<Color> Color::decode(Decoder& decoder)
-{
-    bool isValid;
-    if (!decoder.decode(isValid))
-        return std::nullopt;
-
-    if (!isValid)
-        return Color { };
-
-    OptionSet<Flags> flags;
-
-    bool isSemantic;
-    if (!decoder.decode(isSemantic))
-        return std::nullopt;
-
-    if (isSemantic)
-        flags.add(Flags::Semantic);
-
-    bool usesColorFunctionSerialization;
-    if (!decoder.decode(usesColorFunctionSerialization))
-        return std::nullopt;
-
-    if (usesColorFunctionSerialization)
-        flags.add(Flags::UseColorFunctionSerialization);
-
-    bool isOutOfLine;
-    if (!decoder.decode(isOutOfLine))
-        return std::nullopt;
-
-    if (isOutOfLine) {
-        ColorSpace colorSpace;
-        if (!decoder.decode(colorSpace))
-            return std::nullopt;
-        float c1;
-        if (!decoder.decode(c1))
-            return std::nullopt;
-        float c2;
-        if (!decoder.decode(c2))
-            return std::nullopt;
-        float c3;
-        if (!decoder.decode(c3))
-            return std::nullopt;
-        float alpha;
-        if (!decoder.decode(alpha))
-            return std::nullopt;
-        return Color { OutOfLineComponents::create({ c1, c2, c3, alpha }), colorSpace, flags };
-    }
-
-    uint32_t value;
-    if (!decoder.decode(value))
-        return std::nullopt;
-
-    return Color { asSRGBA(PackedColor::RGBA { value }), flags };
-}
-
-inline void add(Hasher& hasher, const Color& color)
-{
-    // FIXME: We don't want to hash a hash; do better.
-    add(hasher, color.hash());
-}
-
 } // namespace WebCore
 
 namespace WTF {
