@@ -26,35 +26,65 @@
 #include "DisplayCaptureManager.h"
 #include "GRefPtrGStreamer.h"
 #include "GStreamerCaptureDevice.h"
+#include "GStreamerCapturer.h"
 #include "GStreamerVideoCapturer.h"
+#include "RealtimeMediaSourceCenter.h"
 #include "RealtimeMediaSourceFactory.h"
+#include <wtf/Forward.h>
+
+#include <wtf/Noncopyable.h>
 
 namespace WebCore {
 
 using NodeAndFD = GStreamerVideoCapturer::NodeAndFD;
 
-class GStreamerCaptureDeviceManager : public CaptureDeviceManager {
+void teardownGStreamerCaptureDeviceManagers();
+
+class GStreamerCaptureDeviceManager : public CaptureDeviceManager, public RealtimeMediaSourceCenterObserver {
+    WTF_MAKE_NONCOPYABLE(GStreamerCaptureDeviceManager)
 public:
+    GStreamerCaptureDeviceManager();
     ~GStreamerCaptureDeviceManager();
     std::optional<GStreamerCaptureDevice> gstreamerDeviceWithUID(const String&);
 
+    const Vector<CaptureDevice>& speakerDevices();
     const Vector<CaptureDevice>& captureDevices() final;
-    virtual CaptureDevice::DeviceType deviceType() = 0;
+    virtual const OptionSet<CaptureDevice::DeviceType> deviceTypes() const = 0;
+
+    // RealtimeMediaSourceCenterObserver interface.
+    void devicesChanged() final;
+    void deviceWillBeRemoved(const String& persistentId) final;
+
+    void registerCapturer(RefPtr<GStreamerCapturer>&&);
+    void unregisterCapturer(const GStreamerCapturer&);
+    void stopCapturing(const String& persistentId);
+
+    void teardown();
 
 private:
     void addDevice(GRefPtr<GstDevice>&&);
+    void removeDevice(GRefPtr<GstDevice>&&);
+    void stopMonitor();
     void refreshCaptureDevices();
 
     GRefPtr<GstDeviceMonitor> m_deviceMonitor;
     Vector<GStreamerCaptureDevice> m_gstreamerDevices;
     Vector<CaptureDevice> m_devices;
+    Vector<RefPtr<GStreamerCapturer>> m_capturers;
+    bool m_isTearingDown { false };
+    Vector<CaptureDevice> m_speakerDevices;
 };
 
 class GStreamerAudioCaptureDeviceManager final : public GStreamerCaptureDeviceManager {
     friend class NeverDestroyed<GStreamerAudioCaptureDeviceManager>;
 public:
     static GStreamerAudioCaptureDeviceManager& singleton();
-    CaptureDevice::DeviceType deviceType() final { return CaptureDevice::DeviceType::Microphone; }
+    const OptionSet<CaptureDevice::DeviceType> deviceTypes() const final { return { CaptureDevice::DeviceType::Microphone, CaptureDevice::DeviceType::Speaker }; }
+
+    // ref() and deref() do nothing because this object is a singleton.
+    void ref() const final { }
+    void deref() const final { }
+
 private:
     GStreamerAudioCaptureDeviceManager() = default;
 };
@@ -63,8 +93,14 @@ class GStreamerVideoCaptureDeviceManager final : public GStreamerCaptureDeviceMa
     friend class NeverDestroyed<GStreamerVideoCaptureDeviceManager>;
 public:
     static GStreamerVideoCaptureDeviceManager& singleton();
+    const OptionSet<CaptureDevice::DeviceType> deviceTypes() const final { return { CaptureDevice::DeviceType::Camera }; }
+
+    // ref() and deref() do nothing because this object is a singleton.
+    void ref() const final { }
+    void deref() const final { }
+
     static VideoCaptureFactory& videoFactory();
-    CaptureDevice::DeviceType deviceType() final { return CaptureDevice::DeviceType::Camera; }
+
 private:
     GStreamerVideoCaptureDeviceManager() = default;
 };
@@ -75,7 +111,7 @@ public:
     static GStreamerDisplayCaptureDeviceManager& singleton();
     const Vector<CaptureDevice>& captureDevices() final { return m_devices; };
     void computeCaptureDevices(CompletionHandler<void()>&&) final;
-    CaptureSourceOrError createDisplayCaptureSource(const CaptureDevice&, String&&, const MediaConstraints*);
+    CaptureSourceOrError createDisplayCaptureSource(const CaptureDevice&, MediaDeviceHashSalts&&, const MediaConstraints*);
 
     enum PipeWireOutputType {
         Monitor = 1 << 0,
@@ -84,6 +120,9 @@ public:
 
     void stopSource(const String& persistentID);
 
+    // DisplayCaptureManager interface
+    bool requiresCaptureDevicesEnumeration() const final { return true; }
+
 protected:
     void notifyResponse(GVariant* parameters) { m_currentResponseCallback(parameters); }
 
