@@ -27,11 +27,6 @@
 
 #include "ProtectionSpace.h"
 
-#if USE(CFURLCONNECTION)
-#include "AuthenticationCF.h"
-#include <CFNetwork/CFURLProtectionSpacePriv.h>
-#endif
-
 namespace WebCore {
  
 // Need to enforce empty, non-null strings due to the pickiness of the String == String operator
@@ -71,13 +66,20 @@ bool ProtectionSpaceBase::isPasswordBased() const
     case AuthenticationScheme::NTLM:
     case AuthenticationScheme::Negotiate:
     case AuthenticationScheme::OAuth:
+#if PLATFORM(COCOA)
+    case AuthenticationScheme::XMobileMeAuthToken:
+    case AuthenticationScheme::PrivateAccessToken:
+    case AuthenticationScheme::OAuthBearerToken:
+#endif
 #if USE(GLIB)
     case AuthenticationScheme::ClientCertificatePINRequested:
 #endif
         return true;
     case AuthenticationScheme::ClientCertificateRequested:
     case AuthenticationScheme::ServerTrustEvaluationRequested:
+#if !PLATFORM(COCOA)
     case AuthenticationScheme::Unknown:
+#endif
         return false;
     }
 
