@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2020 Igalia, S.L.
+ * Copyright (C) 2020-2023 Apple Inc. All rights reserved.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -53,8 +54,6 @@ OpenXRDevice::OpenXRDevice(XrInstance instance, XrSystemId system, Ref<WorkQueue
 {
 }
 
-OpenXRDevice::~OpenXRDevice() = default;
-
 void OpenXRDevice::initialize(CompletionHandler<void()>&& callback)
 {
     ASSERT(isMainThread());
@@ -102,14 +101,13 @@ void OpenXRDevice::initializeTrackingAndRendering(const WebCore::SecurityOriginD
         RETURN_IF_FAILED(result, "xrGetOpenGLGraphicsRequirementsKHR", m_instance);
 
         m_graphicsBinding = createStructure<XrGraphicsBindingEGLMNDX, XR_TYPE_GRAPHICS_BINDING_EGL_MNDX>();
-        m_egl = GLContextEGL::createSharingContext(PlatformDisplay::sharedDisplay());
+        m_egl = GLContext::createSharing(PlatformDisplay::sharedDisplay());
         if (!m_egl) {
             LOG(XR, "Failed to create EGL context");
             return;
         }
 
-        auto& context = static_cast<GLContext&>(*m_egl);
-        context.makeContextCurrent();
+        m_egl->makeContextCurrent();
 
         GraphicsContextGLAttributes attributes;
         attributes.depth = false;
@@ -123,7 +121,7 @@ void OpenXRDevice::initializeTrackingAndRendering(const WebCore::SecurityOriginD
         }
 
         m_graphicsBinding.display = PlatformDisplay::sharedDisplay().eglDisplay();
-        m_graphicsBinding.context = context.platformContext();
+        m_graphicsBinding.context = m_egl->platformContext();
         m_graphicsBinding.config = m_egl->config();
         m_graphicsBinding.getProcAddress = m_extensions.methods().getProcAddressFunc;
 
@@ -170,7 +168,7 @@ void OpenXRDevice::initializeReferenceSpace(PlatformXR::ReferenceSpaceType space
         updateStageParameters();
 }
 
-void OpenXRDevice::requestFrame(RequestFrameCallback&& callback)
+void OpenXRDevice::requestFrame(std::optional<RequestData>&&, RequestFrameCallback&& callback)
 {
     m_queue.dispatch([this, protectedThis = Ref { *this }, callback = WTFMove(callback)]() mutable {
         pollEvents();
@@ -191,7 +189,7 @@ void OpenXRDevice::requestFrame(RequestFrameCallback&& callback)
         result = xrBeginFrame(m_session, &frameBeginInfo);
         RETURN_IF_FAILED(result, "xrBeginFrame", m_instance);
 
-        Device::FrameData frameData;
+        FrameData frameData;
         frameData.predictedDisplayTime = m_frameState.predictedDisplayTime;
         frameData.shouldRender = m_frameState.shouldRender;
         frameData.stageParameters = m_stageParameters;
@@ -234,8 +232,10 @@ void OpenXRDevice::requestFrame(RequestFrameCallback&& callback)
 
             for (auto& layer : m_layers) {
                 auto layerData = layer.value->startFrame();
-                if (layerData)
-                    frameData.layers.add(layer.key, *layerData);
+                if (layerData) {
+                    auto layerDataRef = makeUniqueRef<PlatformXR::FrameData::LayerData>(*layerData);
+                    frameData.layers.add(layer.key, WTFMove(layerDataRef));
+                }
             }
 
             if (m_input)
@@ -368,14 +368,14 @@ void OpenXRDevice::collectSupportedSessionModes()
     auto result = xrEnumerateViewConfigurations(m_instance, m_systemId, 0, &viewConfigurationCount, nullptr);
     RETURN_IF_FAILED(result, "xrEnumerateViewConfigurations", m_instance);
 
-    XrViewConfigurationType viewConfigurations[viewConfigurationCount];
-    result = xrEnumerateViewConfigurations(m_instance, m_systemId, viewConfigurationCount, &viewConfigurationCount, viewConfigurations);
+    Vector<XrViewConfigurationType> viewConfigurations(viewConfigurationCount);
+    result = xrEnumerateViewConfigurations(m_instance, m_systemId, viewConfigurationCount, &viewConfigurationCount, viewConfigurations.data());
     RETURN_IF_FAILED(result, "xrEnumerateViewConfigurations", m_instance);
 
     FeatureList features = collectSupportedFeatures();
-    for (uint32_t i = 0; i < viewConfigurationCount; ++i) {
+    for (auto& viewConfiguration : viewConfigurations) {
         auto viewConfigurationProperties = createStructure<XrViewConfigurationProperties, XR_TYPE_VIEW_CONFIGURATION_PROPERTIES>();
-        result = xrGetViewConfigurationProperties(m_instance, m_systemId, viewConfigurations[i], &viewConfigurationProperties);
+        result = xrGetViewConfigurationProperties(m_instance, m_systemId, viewConfiguration, &viewConfigurationProperties);
         if (result != XR_SUCCESS) {
             LOG(XR, "xrGetViewConfigurationProperties(): error %s\n", resultToString(result, m_instance).utf8().data());
             continue;
@@ -500,8 +500,9 @@ void OpenXRDevice::endSession()
         return;
 
     // Notify did end event
-    callOnMainThread([this, weakThis = WeakPtr { *this }]() {
-        if (!weakThis)
+    callOnMainThread([this, weakThis = ThreadSafeWeakPtr { *this }]() {
+        auto protectedThis = weakThis.get();
+        if (!protectedThis)
             return;
         if (m_trackingAndRenderingClient)
             m_trackingAndRenderingClient->sessionDidEnd();
@@ -589,8 +590,9 @@ void OpenXRDevice::updateInteractionProfile()
 
     didNotifyInputInitialization = true;
     auto inputSources = m_input->collectInputSources(m_frameState);
-    callOnMainThread([this, weakThis = WeakPtr { *this }, inputSources = WTFMove(inputSources)]() mutable {
-        if (!weakThis)
+    callOnMainThread([this, weakThis = ThreadSafeWeakPtr { *this }, inputSources = WTFMove(inputSources)]() mutable {
+        auto protectedThis = weakThis.get();
+        if (!protectedThis)
             return;
         if (m_trackingAndRenderingClient)
             m_trackingAndRenderingClient->sessionDidInitializeInputSources(WTFMove(inputSources));
