@@ -1,7 +1,8 @@
 /*
  * This file is part of the select element renderer in WebCore.
  *
- * Copyright (C) 2006, 2007, 2009, 2014 Apple Inc. All rights reserved.
+ * Copyright (C) 2006-2024 Apple Inc. All rights reserved.
+ * Copyright (C) 2014 Google Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -35,48 +36,54 @@
 
 namespace WebCore {
 
+class HTMLOptGroupElement;
+class HTMLOptionElement;
 class HTMLSelectElement;
 
 class RenderListBox final : public RenderBlockFlow, public ScrollableArea {
-    WTF_MAKE_ISO_ALLOCATED(RenderListBox);
+    WTF_MAKE_TZONE_OR_ISO_ALLOCATED(RenderListBox);
+    WTF_OVERRIDE_DELETE_FOR_CHECKED_PTR(RenderListBox);
 public:
     RenderListBox(HTMLSelectElement&, RenderStyle&&);
     virtual ~RenderListBox();
 
+    // CheckedPtr interface
+    uint32_t checkedPtrCount() const final { return CanMakeCheckedPtr::checkedPtrCount(); }
+    uint32_t checkedPtrCountWithoutThreadCheck() const final { return CanMakeCheckedPtr::checkedPtrCountWithoutThreadCheck(); }
+    void incrementCheckedPtrCount() const final { CanMakeCheckedPtr::incrementCheckedPtrCount(); }
+    void decrementCheckedPtrCount() const final { CanMakeCheckedPtr::decrementCheckedPtrCount(); }
+
     HTMLSelectElement& selectElement() const;
 
     void selectionChanged();
 
     void setOptionsChanged(bool changed) { m_optionsChanged = changed; }
 
-    int listIndexAtOffset(const LayoutSize&);
-    LayoutRect itemBoundingBoxRect(const LayoutPoint&, int index);
+    int listIndexAtOffset(const LayoutSize&) const;
+    LayoutRect itemBoundingBoxRect(const LayoutPoint&, int index) const;
+
+    std::optional<LayoutRect> localBoundsOfOption(const HTMLOptionElement&) const;
+    std::optional<LayoutRect> localBoundsOfOptGroup(const HTMLOptGroupElement&) const;
 
     bool scrollToRevealElementAtListIndex(int index);
     bool listIndexIsVisible(int index);
 
     int scrollToward(const IntPoint&); // Returns the new index or -1 if no scroll occurred
 
-    int size() const;
+    unsigned size() const;
 
     bool scroll(ScrollDirection, ScrollGranularity, unsigned stepCount = 1, Element** stopElement = nullptr, RenderBox* startBox = nullptr, const IntPoint& wheelEventAbsolutePoint = IntPoint()) override;
-
-    bool scrolledToTop() const final;
-    bool scrolledToBottom() const final;
-    bool scrolledToLeft() const final;
-    bool scrolledToRight() const final;
-    
-    bool isVisibleToHitTesting() const final { return visibleToHitTesting(); };
+    std::optional<FrameIdentifier> rootFrameID() const final;
 
 private:
+    bool isVisibleToHitTesting() const final;
+
     void willBeDestroyed() override;
 
     void element() const = delete;
 
     ASCIILiteral renderName() const override { return "RenderListBox"_s; }
 
-    bool isListBox() const override { return true; }
-
     void updateFromElement() override;
     bool hasControlClip() const override { return true; }
     void paintObject(PaintInfo&, const LayoutPoint&) override;
@@ -86,6 +93,8 @@ private:
 
     bool logicalScroll(ScrollLogicalDirection, ScrollGranularity, unsigned stepCount = 1, Element** stopElement = nullptr) override;
 
+    void styleDidChange(StyleDifference, const RenderStyle* oldStyle) override;
+
     void computeIntrinsicLogicalWidths(LayoutUnit& minLogicalWidth, LayoutUnit& maxLogicalWidth) const override;
     void computePreferredLogicalWidths() override;
     LayoutUnit baselinePosition(FontBaseline, bool firstLine, LineDirectionMode, LinePositionMode = PositionOnContainingLine) const override;
@@ -93,7 +102,7 @@ private:
 
     void layout() override;
 
-    void addFocusRingRects(Vector<LayoutRect>&, const LayoutPoint& additionalOffset, const RenderLayerModelObject* paintContainer = nullptr) override;
+    void addFocusRingRects(Vector<LayoutRect>&, const LayoutPoint& additionalOffset, const RenderLayerModelObject* paintContainer = nullptr) const override;
 
     bool canBeProgramaticallyScrolled() const override { return true; }
     void autoscroll(const IntPoint&) override;
@@ -103,6 +112,7 @@ private:
     void panScroll(const IntPoint&) override;
 
     int verticalScrollbarWidth() const override;
+    int horizontalScrollbarHeight() const override;
     int scrollLeft() const override;
     int scrollTop() const override;
     int scrollWidth() const override;
@@ -110,6 +120,9 @@ private:
     void setScrollLeft(int, const ScrollPositionChangeOptions&) override;
     void setScrollTop(int, const ScrollPositionChangeOptions&) override;
 
+    int logicalScrollTop() const;
+    void setLogicalScrollTop(int);
+
     bool nodeAtPoint(const HitTestRequest&, HitTestResult&, const HitTestLocation& locationInContainer, const LayoutPoint& accumulatedOffset, HitTestAction) override;
 
     // ScrollableArea interface.
@@ -130,7 +143,8 @@ private:
     IntRect convertFromContainingViewToScrollbar(const Scrollbar&, const IntRect&) const final;
     IntPoint convertFromScrollbarToContainingView(const Scrollbar&, const IntPoint&) const final;
     IntPoint convertFromContainingViewToScrollbar(const Scrollbar&, const IntPoint&) const final;
-    Scrollbar* verticalScrollbar() const final { return m_vBar.get(); }
+    Scrollbar* verticalScrollbar() const final;
+    Scrollbar* horizontalScrollbar() const final;
     IntSize contentsSize() const final;
     IntSize visibleSize() const final { return IntSize(width(), height()); }
     IntPoint lastKnownMousePositionInView() const final;
@@ -147,54 +161,69 @@ private:
     String debugDescription() const final;
     void didStartScrollAnimation() final;
 
+    bool useDarkAppearance() const final;
+
     // NOTE: This should only be called by the overridden setScrollOffset from ScrollableArea.
-    void scrollTo(int newOffset);
+    void scrollTo(const ScrollPosition&);
+
+    void scrollToPosition(int positionIndex);
 
     using PaintFunction = Function<void(PaintInfo&, const LayoutPoint&, int listItemIndex)>;
     void paintItem(PaintInfo&, const LayoutPoint&, const PaintFunction&);
 
-    void setHasVerticalScrollbar(bool hasScrollbar);
-    Ref<Scrollbar> createScrollbar();
+    void setHasScrollbar(ScrollbarOrientation);
+    Ref<Scrollbar> createScrollbar(ScrollbarOrientation);
     void destroyScrollbar();
-    
-    int maximumNumberOfItemsThatFitInPaddingBottomArea() const;
 
-    int numberOfVisibleItemsInPaddingTop() const;
-    int numberOfVisibleItemsInPaddingBottom() const;
+    int maximumNumberOfItemsThatFitInPaddingAfterArea() const;
 
-    void computeFirstIndexesVisibleInPaddingTopBottomAreas();
+    int numberOfVisibleItemsInPaddingBefore() const;
+    int numberOfVisibleItemsInPaddingAfter() const;
 
-    LayoutUnit itemHeight() const;
-    void valueChanged(unsigned listIndex);
+    void computeFirstIndexesVisibleInPaddingBeforeAfterAreas();
 
-    enum class ConsiderPadding { Yes, No };
+    LayoutUnit itemLogicalHeight() const;
+
+    enum class ConsiderPadding : bool { No, Yes };
     int numVisibleItems(ConsiderPadding = ConsiderPadding::No) const;
     int numItems() const;
-    LayoutUnit listHeight() const;
-    void paintScrollbar(PaintInfo&, const LayoutPoint&);
+    LayoutUnit listLogicalHeight() const;
+
+    std::optional<int> optionRowIndex(const HTMLOptionElement&) const;
+
+    float deviceScaleFactor() const final;
+
+    LayoutRect rectForScrollbar(const Scrollbar&) const;
+
+    void paintScrollbar(PaintInfo&, const LayoutPoint&, Scrollbar&);
     void paintItemForeground(PaintInfo&, const LayoutPoint&, int listIndex);
     void paintItemBackground(PaintInfo&, const LayoutPoint&, int listIndex);
     void scrollToRevealSelection();
 
+    ScrollbarOrientation scrollbarOrientationForWritingMode() const;
+
     bool shouldPlaceVerticalScrollbarOnLeft() const final { return RenderBlockFlow::shouldPlaceVerticalScrollbarOnLeft(); }
 
+    int indexOffset() const;
+
     bool m_optionsChanged { true };
     bool m_scrollToRevealSelectionAfterLayout { false };
     bool m_inAutoscroll { false };
-    int m_optionsWidth { 0 };
+    int m_optionsLogicalWidth { 0 };
 
-    RefPtr<Scrollbar> m_vBar;
+    RefPtr<Scrollbar> m_scrollbar;
 
-    int m_indexOffset { 0 };
+    // Note: This is based on item index rather than a pixel offset.
+    ScrollPosition m_scrollPosition;
 
-    std::optional<int> m_indexOfFirstVisibleItemInsidePaddingTopArea;
-    std::optional<int> m_indexOfFirstVisibleItemInsidePaddingBottomArea;
+    std::optional<int> m_indexOfFirstVisibleItemInsidePaddingBeforeArea;
+    std::optional<int> m_indexOfFirstVisibleItemInsidePaddingAfterArea;
 
 };
 
 } // namepace WebCore
 
 SPECIALIZE_TYPE_TRAITS_BEGIN(WebCore::RenderListBox)
-    static bool isType(const WebCore::RenderObject& widget) { return widget.isListBox(); }
+    static bool isType(const WebCore::RenderObject& renderer) { return renderer.isRenderListBox(); }
     static bool isType(const WebCore::ScrollableArea& area) { return area.isListBox(); }
 SPECIALIZE_TYPE_TRAITS_END()
