@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2015-2020 Apple Inc. All rights reserved.
+ * Copyright (C) 2015-2023 Apple Inc. All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
  * modification, are permitted provided that the following conditions
@@ -26,11 +26,37 @@
 #include "config.h"
 #include "RenderStyleConstants.h"
 
+#include "CSSPrimitiveValueMappings.h"
 #include "TabSize.h"
 #include <wtf/text/TextStream.h>
 
 namespace WebCore {
 
+
+bool alwaysPageBreak(BreakBetween between)
+{
+    return between >= BreakBetween::Page;
+}
+
+CSSBoxType transformBoxToCSSBoxType(TransformBox transformBox)
+{
+    switch (transformBox) {
+    case TransformBox::StrokeBox:
+        return CSSBoxType::StrokeBox;
+    case TransformBox::ContentBox:
+        return CSSBoxType::ContentBox;
+    case TransformBox::BorderBox:
+        return CSSBoxType::BorderBox;
+    case TransformBox::FillBox:
+        return CSSBoxType::FillBox;
+    case TransformBox::ViewBox:
+        return CSSBoxType::ViewBox;
+    default:
+        ASSERT_NOT_REACHED();
+        return CSSBoxType::BorderBox;
+    }
+}
+
 TextStream& operator<<(TextStream& ts, AnimationFillMode fillMode)
 {
     switch (fillMode) {
@@ -51,42 +77,6 @@ TextStream& operator<<(TextStream& ts, AnimationPlayState playState)
     return ts;
 }
 
-#if ENABLE(APPLE_PAY)
-TextStream& operator<<(TextStream& ts, ApplePayButtonStyle buttonStyle)
-{
-    switch (buttonStyle) {
-    case ApplePayButtonStyle::White: ts << "white"; break;
-    case ApplePayButtonStyle::WhiteOutline: ts << "white-outline"; break;
-    case ApplePayButtonStyle::Black: ts << "black"; break;
-    }
-    return ts;
-}
-
-TextStream& operator<<(TextStream& ts, ApplePayButtonType playState)
-{
-    switch (playState) {
-    case ApplePayButtonType::Plain: ts << "plain"; break;
-    case ApplePayButtonType::Buy: ts << "buy"; break;
-    case ApplePayButtonType::SetUp: ts << "setup"; break;
-    case ApplePayButtonType::Donate: ts << "donate"; break;
-    case ApplePayButtonType::CheckOut: ts << "checkout"; break;
-    case ApplePayButtonType::Book: ts << "book"; break;
-    case ApplePayButtonType::Subscribe: ts << "subscribe"; break;
-#if ENABLE(APPLE_PAY_NEW_BUTTON_TYPES)
-    case ApplePayButtonType::Reload: ts << "reload"; break;
-    case ApplePayButtonType::AddMoney: ts << "add-money"; break;
-    case ApplePayButtonType::TopUp: ts << "top-up"; break;
-    case ApplePayButtonType::Order: ts << "order"; break;
-    case ApplePayButtonType::Rent: ts << "rent"; break;
-    case ApplePayButtonType::Support: ts << "support"; break;
-    case ApplePayButtonType::Contribute: ts << "contribute"; break;
-    case ApplePayButtonType::Tip: ts << "tip"; break;
-#endif
-    }
-    return ts;
-}
-#endif
-
 TextStream& operator<<(TextStream& ts, AspectRatioType aspectRatioType)
 {
     switch (aspectRatioType) {
@@ -117,6 +107,37 @@ TextStream& operator<<(TextStream& ts, BackfaceVisibility visibility)
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, BlockStepAlign blockStepAlign)
+{
+    switch (blockStepAlign) {
+    case BlockStepAlign::Auto: ts << "auto"; break;
+    case BlockStepAlign::Center: ts << "center"; break;
+    case BlockStepAlign::Start: ts << "start"; break;
+    case BlockStepAlign::End: ts << "end"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, BlockStepInsert blockStepInsert)
+{
+    switch (blockStepInsert) {
+    case BlockStepInsert::MarginBox: ts << "margin-box"; break;
+    case BlockStepInsert::PaddingBox: ts << "padding-box"; break;
+    case BlockStepInsert::ContentBox: ts << "content-box"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, BlockStepRound blockStepRound)
+{
+    switch (blockStepRound) {
+    case BlockStepRound::Up: ts << "up"; break;
+    case BlockStepRound::Down: ts << "down"; break;
+    case BlockStepRound::Nearest: ts << "nearest"; break;
+    }
+    return ts;
+}
+
 TextStream& operator<<(TextStream& ts, BorderCollapse collapse)
 {
     switch (collapse) {
@@ -126,15 +147,6 @@ TextStream& operator<<(TextStream& ts, BorderCollapse collapse)
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, BorderFit borderFit)
-{
-    switch (borderFit) {
-    case BorderFit::Border: ts << "border"; break;
-    case BorderFit::Lines: ts << "lines"; break;
-    }
-    return ts;
-}
-
 TextStream& operator<<(TextStream& ts, BorderStyle borderStyle)
 {
     switch (borderStyle) {
@@ -268,8 +280,6 @@ TextStream& operator<<(TextStream& ts, CaptionSide side)
     switch (side) {
     case CaptionSide::Top: ts << "top"; break;
     case CaptionSide::Bottom: ts << "bottom"; break;
-    case CaptionSide::Left: ts << "left"; break;
-    case CaptionSide::Right: ts << "right"; break;
     }
     return ts;
 }
@@ -280,11 +290,24 @@ TextStream& operator<<(TextStream& ts, Clear clear)
     case Clear::None: ts << "none"; break;
     case Clear::Left: ts << "left"; break;
     case Clear::Right: ts << "right"; break;
+    case Clear::InlineStart : ts << "inline-start"; break;
+    case Clear::InlineEnd : ts << "inline-end"; break;
     case Clear::Both: ts << "both"; break;
     }
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, UsedClear clear)
+{
+    switch (clear) {
+    case UsedClear::None: ts << "none"; break;
+    case UsedClear::Left: ts << "left"; break;
+    case UsedClear::Right: ts << "right"; break;
+    case UsedClear::Both: ts << "both"; break;
+    }
+    return ts;
+}
+
 #if ENABLE(DARK_MODE_CSS)
 TextStream& operator<<(TextStream& ts, ColorScheme colorScheme)
 {
@@ -363,6 +386,16 @@ TextStream& operator<<(TextStream& ts, ContentPosition position)
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, ContentVisibility contentVisibility)
+{
+    switch (contentVisibility) {
+    case ContentVisibility::Visible: ts << "visible"; break;
+    case ContentVisibility::Auto: ts << "auto"; break;
+    case ContentVisibility::Hidden: ts << "hidden"; break;
+    }
+    return ts;
+}
+
 TextStream& operator<<(TextStream& ts, CursorType cursor)
 {
     switch (cursor) {
@@ -437,13 +470,15 @@ TextStream& operator<<(TextStream& ts, DisplayType display)
     case DisplayType::Box: ts << "box"; break;
     case DisplayType::InlineBox: ts << "inline-box"; break;
     case DisplayType::Flex: ts << "flex"; break;
-    case DisplayType::WebKitFlex: ts << "web-kit-flex"; break;
     case DisplayType::InlineFlex: ts << "inline-flex"; break;
-    case DisplayType::WebKitInlineFlex: ts << "web-kit-inline-flex"; break;
     case DisplayType::Contents: ts << "contents"; break;
     case DisplayType::Grid: ts << "grid"; break;
     case DisplayType::InlineGrid: ts << "inline-grid"; break;
     case DisplayType::FlowRoot: ts << "flow-root"; break;
+    case DisplayType::Ruby: ts << "ruby"; break;
+    case DisplayType::RubyBlock: ts << "block ruby"; break;
+    case DisplayType::RubyBase: ts << "ruby-base"; break;
+    case DisplayType::RubyAnnotation: ts << "ruby-text"; break;
     case DisplayType::None: ts << "none"; break;
     }
     return ts;
@@ -474,6 +509,24 @@ TextStream& operator<<(TextStream& ts, EventListenerRegionType listenerType)
     switch (listenerType) {
     case EventListenerRegionType::Wheel: ts << "wheel"; break;
     case EventListenerRegionType::NonPassiveWheel: ts << "active wheel"; break;
+    case EventListenerRegionType::MouseClick: ts << "mouse click"; break;
+    case EventListenerRegionType::TouchStart: ts << "touch start"; break;
+    case EventListenerRegionType::NonPassiveTouchStart: ts << "active touch start"; break;
+    case EventListenerRegionType::TouchEnd: ts << "touch end"; break;
+    case EventListenerRegionType::NonPassiveTouchEnd: ts << "active touch end"; break;
+    case EventListenerRegionType::TouchCancel: ts << "touch cancel"; break;
+    case EventListenerRegionType::NonPassiveTouchCancel: ts << "active touch cancel"; break;
+    case EventListenerRegionType::TouchMove: ts << "touch move"; break;
+    case EventListenerRegionType::NonPassiveTouchMove: ts << "active touch move"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, FieldSizing sizing)
+{
+    switch (sizing) {
+    case FieldSizing::Fixed: ts << "fixed"; break;
+    case FieldSizing::Content: ts << "content"; break;
     }
     return ts;
 }
@@ -491,10 +544,12 @@ TextStream& operator<<(TextStream& ts, FillAttachment attachment)
 TextStream& operator<<(TextStream& ts, FillBox fill)
 {
     switch (fill) {
-    case FillBox::Border: ts << "border"; break;
-    case FillBox::Padding: ts << "padding"; break;
-    case FillBox::Content: ts << "content"; break;
+    case FillBox::BorderBox: ts << "border-box"; break;
+    case FillBox::PaddingBox: ts << "padding-box"; break;
+    case FillBox::ContentBox: ts << "content-box"; break;
+    case FillBox::BorderArea: ts << "border-area"; break;
     case FillBox::Text: ts << "text"; break;
+    case FillBox::NoClip: ts << "no-clip"; break;
     }
     return ts;
 }
@@ -546,9 +601,21 @@ TextStream& operator<<(TextStream& ts, FlexWrap flexWrap)
 TextStream& operator<<(TextStream& ts, Float floating)
 {
     switch (floating) {
-    case Float::No: ts << "none"; break;
+    case Float::None: ts << "none"; break;
     case Float::Left: ts << "left"; break;
     case Float::Right: ts << "right"; break;
+    case Float::InlineStart: ts << "inline-start"; break;
+    case Float::InlineEnd: ts << "inline-end"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, UsedFloat floating)
+{
+    switch (floating) {
+    case UsedFloat::None: ts << "none"; break;
+    case UsedFloat::Left: ts << "left"; break;
+    case UsedFloat::Right: ts << "right"; break;
     }
     return ts;
 }
@@ -567,7 +634,6 @@ TextStream& operator<<(TextStream& ts, GridAutoFlow gridAutoFlow)
 TextStream& operator<<(TextStream& ts, HangingPunctuation punctuation)
 {
     switch (punctuation) {
-    case HangingPunctuation::None: ts << "none"; break;
     case HangingPunctuation::First: ts << "first"; break;
     case HangingPunctuation::Last: ts << "last"; break;
     case HangingPunctuation::AllowEnd: ts << "allow-end"; break;
@@ -635,6 +701,7 @@ TextStream& operator<<(TextStream& ts, ItemPosition position)
     case ItemPosition::FlexEnd: ts << "flex-end"; break;
     case ItemPosition::Left: ts << "left"; break;
     case ItemPosition::Right: ts << "right"; break;
+    case ItemPosition::AnchorCenter: ts << "anchor-center"; break;
     }
     return ts;
 }
@@ -689,101 +756,13 @@ TextStream& operator<<(TextStream& ts, ListStylePosition position)
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, ListStyleType styleType)
+TextStream& operator<<(TextStream& ts, MarginTrimType marginTrimType)
 {
-    switch (styleType) {
-    case ListStyleType::Disc: ts << "disc"; break;
-    case ListStyleType::Circle: ts << "circle"; break;
-    case ListStyleType::Square: ts << "square"; break;
-    case ListStyleType::Decimal: ts << "decimal"; break;
-    case ListStyleType::DecimalLeadingZero: ts << "decimal-leading-zero"; break;
-    case ListStyleType::ArabicIndic: ts << "arabic-indic"; break;
-    case ListStyleType::Binary: ts << "binary"; break;
-    case ListStyleType::Bengali: ts << "bengali"; break;
-    case ListStyleType::Cambodian: ts << "cambodian"; break;
-    case ListStyleType::Khmer: ts << "khmer"; break;
-    case ListStyleType::Devanagari: ts << "devanagari"; break;
-    case ListStyleType::Gujarati: ts << "gujarati"; break;
-    case ListStyleType::Gurmukhi: ts << "gurmukhi"; break;
-    case ListStyleType::Kannada: ts << "kannada"; break;
-    case ListStyleType::LowerHexadecimal: ts << "lower-hexadecimal"; break;
-    case ListStyleType::Lao: ts << "lao"; break;
-    case ListStyleType::Malayalam: ts << "malayalam"; break;
-    case ListStyleType::Mongolian: ts << "mongolian"; break;
-    case ListStyleType::Myanmar: ts << "myanmar"; break;
-    case ListStyleType::Octal: ts << "octal"; break;
-    case ListStyleType::Oriya: ts << "oriya"; break;
-    case ListStyleType::Persian: ts << "persian"; break;
-    case ListStyleType::Urdu: ts << "urdu"; break;
-    case ListStyleType::Telugu: ts << "telugu"; break;
-    case ListStyleType::Tibetan: ts << "tibetan"; break;
-    case ListStyleType::Thai: ts << "thai"; break;
-    case ListStyleType::UpperHexadecimal: ts << "upper-hexadecimal"; break;
-    case ListStyleType::LowerRoman: ts << "lower-roman"; break;
-    case ListStyleType::UpperRoman: ts << "upper-roman"; break;
-    case ListStyleType::LowerGreek: ts << "lower-greek"; break;
-    case ListStyleType::LowerAlpha: ts << "lower-alpha"; break;
-    case ListStyleType::LowerLatin: ts << "lower-latin"; break;
-    case ListStyleType::UpperAlpha: ts << "upper-alpha"; break;
-    case ListStyleType::UpperLatin: ts << "upper-latin"; break;
-    case ListStyleType::Afar: ts << "afar"; break;
-    case ListStyleType::EthiopicHalehameAaEt: ts << "ethiopic-halehame-aa-et"; break;
-    case ListStyleType::EthiopicHalehameAaEr: ts << "ethiopic-halehame-aa-er"; break;
-    case ListStyleType::Amharic: ts << "amharic"; break;
-    case ListStyleType::EthiopicHalehameAmEt: ts << "ethiopic-halehame-am-et"; break;
-    case ListStyleType::AmharicAbegede: ts << "amharic-abegede"; break;
-    case ListStyleType::EthiopicAbegedeAmEt: ts << "ethiopic-abegede-am-et"; break;
-    case ListStyleType::CjkEarthlyBranch: ts << "cjk-earthly-branch"; break;
-    case ListStyleType::CjkHeavenlyStem: ts << "cjk-heavenly-stem"; break;
-    case ListStyleType::Ethiopic: ts << "ethiopic"; break;
-    case ListStyleType::EthiopicHalehameGez: ts << "ethiopic-halehame-gez"; break;
-    case ListStyleType::EthiopicAbegede: ts << "ethiopic-abegede"; break;
-    case ListStyleType::EthiopicAbegedeGez: ts << "ethiopic-abegede-gez"; break;
-    case ListStyleType::HangulConsonant: ts << "hangul-consonant"; break;
-    case ListStyleType::Hangul: ts << "hangul"; break;
-    case ListStyleType::LowerNorwegian: ts << "lower-norwegian"; break;
-    case ListStyleType::Oromo: ts << "oromo"; break;
-    case ListStyleType::EthiopicHalehameOmEt: ts << "ethiopic-halehame-om-et"; break;
-    case ListStyleType::Sidama: ts << "sidama"; break;
-    case ListStyleType::EthiopicHalehameSidEt: ts << "ethiopic-halehame-sid-et"; break;
-    case ListStyleType::Somali: ts << "somali"; break;
-    case ListStyleType::EthiopicHalehameSoEt: ts << "ethiopic-halehame-so-et"; break;
-    case ListStyleType::Tigre: ts << "tigre"; break;
-    case ListStyleType::EthiopicHalehameTig: ts << "ethiopic-halehame-tig"; break;
-    case ListStyleType::TigrinyaEr: ts << "tigrinya-er"; break;
-    case ListStyleType::EthiopicHalehameTiEr: ts << "ethiopic-halehame-ti-er"; break;
-    case ListStyleType::TigrinyaErAbegede: ts << "tigrinya-er-abegede"; break;
-    case ListStyleType::EthiopicAbegedeTiEr: ts << "ethiopic-abegede-ti-er"; break;
-    case ListStyleType::TigrinyaEt: ts << "tigrinya-et"; break;
-    case ListStyleType::EthiopicHalehameTiEt: ts << "ethiopic-halehame-ti-et"; break;
-    case ListStyleType::TigrinyaEtAbegede: ts << "tigrinya-et-abegede"; break;
-    case ListStyleType::EthiopicAbegedeTiEt: ts << "ethiopic-abegede-ti-et"; break;
-    case ListStyleType::UpperGreek: ts << "upper-greek"; break;
-    case ListStyleType::UpperNorwegian: ts << "upper-norwegian"; break;
-    case ListStyleType::Asterisks: ts << "asterisks"; break;
-    case ListStyleType::Footnotes: ts << "footnotes"; break;
-    case ListStyleType::Hebrew: ts << "hebrew"; break;
-    case ListStyleType::Armenian: ts << "armenian"; break;
-    case ListStyleType::LowerArmenian: ts << "lower-armenian"; break;
-    case ListStyleType::UpperArmenian: ts << "upper-armenian"; break;
-    case ListStyleType::Georgian: ts << "georgian"; break;
-    case ListStyleType::CJKIdeographic: ts << "cjk-ideographic"; break;
-    case ListStyleType::Hiragana: ts << "hiragana"; break;
-    case ListStyleType::Katakana: ts << "katakana"; break;
-    case ListStyleType::HiraganaIroha: ts << "hiragana-iroha"; break;
-    case ListStyleType::KatakanaIroha: ts << "katakana-iroha"; break;
-    case ListStyleType::None: ts << "none"; break;
-    case ListStyleType::String: ts << "string"; break;
-    }
-    return ts;
-}
-
-TextStream& operator<<(TextStream& ts, MarginCollapse collapse)
-{
-    switch (collapse) {
-    case MarginCollapse::Collapse: ts << "collapse"; break;
-    case MarginCollapse::Separate: ts << "separate"; break;
-    case MarginCollapse::Discard: ts << "discard"; break;
+    switch (marginTrimType) {
+    case MarginTrimType::BlockStart: ts << "block-start"; break;
+    case MarginTrimType::BlockEnd: ts << "block-end"; break;
+    case MarginTrimType::InlineStart: ts << "inline-start"; break;
+    case MarginTrimType::InlineEnd: ts << "inline-end"; break;
     }
     return ts;
 }
@@ -813,11 +792,12 @@ TextStream& operator<<(TextStream& ts, MarqueeDirection marqueeDirection)
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, MaskSourceType maskSource)
+TextStream& operator<<(TextStream& ts, MaskMode maskMode)
 {
-    switch (maskSource) {
-    case MaskSourceType::Alpha: ts << "alpha"; break;
-    case MaskSourceType::Luminance: ts << "luminance"; break;
+    switch (maskMode) {
+    case MaskMode::Alpha: ts << "alpha"; break;
+    case MaskMode::Luminance: ts << "luminance"; break;
+    case MaskMode::MatchSource: ts << "match-source"; break;
     }
 
     return ts;
@@ -862,6 +842,7 @@ TextStream& operator<<(TextStream& ts, Overflow overflow)
     case Overflow::Auto: ts << "auto"; break;
     case Overflow::PagedX: ts << "paged-x"; break;
     case Overflow::PagedY: ts << "paged-y"; break;
+    case Overflow::Clip: ts << "clip"; break;
     }
     return ts;
 }
@@ -880,7 +861,8 @@ TextStream& operator<<(TextStream& ts, OverflowWrap overflowWrap)
 {
     switch (overflowWrap) {
     case OverflowWrap::Normal: ts << "normal"; break;
-    case OverflowWrap::Break: ts << "break"; break;
+    case OverflowWrap::BreakWord: ts << "break-word"; break;
+    case OverflowWrap::Anywhere: ts << "anywhere"; break;
     }
     return ts;
 }
@@ -944,18 +926,28 @@ TextStream& operator<<(TextStream& ts, PseudoId pseudoId)
     case PseudoId::None: ts << "none"; break;
     case PseudoId::FirstLine: ts << "first-line"; break;
     case PseudoId::FirstLetter: ts << "first-letter"; break;
+    case PseudoId::GrammarError: ts << "grammar-error"; break;
     case PseudoId::Highlight: ts << "highlight"; break;
+    case PseudoId::InternalWritingSuggestions: ts << "-internal-writing-suggestions"; break;
     case PseudoId::Marker: ts << "marker"; break;
+    case PseudoId::Backdrop: ts << "backdrop"; break;
     case PseudoId::Before: ts << "before"; break;
     case PseudoId::After: ts << "after"; break;
     case PseudoId::Selection: ts << "selection"; break;
-    case PseudoId::Scrollbar: ts << "scrollbar"; break;
-    case PseudoId::ScrollbarThumb: ts << "scrollbar-thumb"; break;
-    case PseudoId::ScrollbarButton: ts << "scrollbar-button"; break;
-    case PseudoId::ScrollbarTrack: ts << "scrollbar-track"; break;
-    case PseudoId::ScrollbarTrackPiece: ts << "scrollbar-trackpiece"; break;
-    case PseudoId::ScrollbarCorner: ts << "scrollbar-corner"; break;
-    case PseudoId::Resizer: ts << "resizer"; break;
+    case PseudoId::SpellingError: ts << "spelling-error"; break;
+    case PseudoId::TargetText: ts << "target-text"; break;
+    case PseudoId::ViewTransition: ts << "view-transition"; break;
+    case PseudoId::ViewTransitionGroup: ts << "view-transition-group"; break;
+    case PseudoId::ViewTransitionImagePair: ts << "view-transition-image-pair"; break;
+    case PseudoId::ViewTransitionOld: ts << "view-transition-old"; break;
+    case PseudoId::ViewTransitionNew: ts << "view-transition-new"; break;
+    case PseudoId::WebKitResizer: ts << "-webkit-resizer"; break;
+    case PseudoId::WebKitScrollbar: ts << "-webkit-scrollbar"; break;
+    case PseudoId::WebKitScrollbarThumb: ts << "-webkit-scrollbar-thumb"; break;
+    case PseudoId::WebKitScrollbarButton: ts << "-webkit-scrollbar-button"; break;
+    case PseudoId::WebKitScrollbarTrack: ts << "-webkit-scrollbar-track"; break;
+    case PseudoId::WebKitScrollbarTrackPiece: ts << "-webkit-scrollbar-trackpiece"; break;
+    case PseudoId::WebKitScrollbarCorner: ts << "-webkit-scrollbar-corner"; break;
     default:
         ts << "other";
         break;
@@ -992,6 +984,8 @@ TextStream& operator<<(TextStream& ts, Resize resize)
     case Resize::Both: ts << "both"; break;
     case Resize::Horizontal: ts << "horizontal"; break;
     case Resize::Vertical: ts << "vertical"; break;
+    case Resize::Block: ts << "block"; break;
+    case Resize::Inline: ts << "inline"; break;
     }
     return ts;
 }
@@ -999,14 +993,34 @@ TextStream& operator<<(TextStream& ts, Resize resize)
 TextStream& operator<<(TextStream& ts, RubyPosition position)
 {
     switch (position) {
-    case RubyPosition::Before: ts << "before"; break;
-    case RubyPosition::After: ts << "after"; break;
+    case RubyPosition::Over: ts << "over"; break;
+    case RubyPosition::Under: ts << "under"; break;
     case RubyPosition::InterCharacter: ts << "inter-character"; break;
+    case RubyPosition::LegacyInterCharacter: ts << "legacy inter-character"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, RubyAlign alignment)
+{
+    switch (alignment) {
+    case RubyAlign::Start: ts << "start"; break;
+    case RubyAlign::Center: ts << "center"; break;
+    case RubyAlign::SpaceBetween: ts << "space-between"; break;
+    case RubyAlign::SpaceAround: ts << "space-around"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, RubyOverhang overhang)
+{
+    switch (overhang) {
+    case RubyOverhang::Auto: ts << "auto"; break;
+    case RubyOverhang::None: ts << "none"; break;
     }
     return ts;
 }
 
-#if ENABLE(CSS_SCROLL_SNAP)
 TextStream& operator<<(TextStream& ts, ScrollSnapAxis axis)
 {
     switch (axis) {
@@ -1048,12 +1062,9 @@ TextStream& operator<<(TextStream& ts, ScrollSnapStop stop)
     }
     return ts;
 }
-#endif
-
 TextStream& operator<<(TextStream& ts, SpeakAs speakAs)
 {
     switch (speakAs) {
-    case SpeakAs::Normal: ts << "normal"; break;
     case SpeakAs::SpellOut: ts << "spell-out"; break;
     case SpeakAs::Digits: ts << "digits"; break;
     case SpeakAs::LiteralPunctuation: ts << "literal-punctuation"; break;
@@ -1068,11 +1079,11 @@ TextStream& operator<<(TextStream& ts, StyleDifference diff)
     case StyleDifference::Equal: ts << "equal"; break;
     case StyleDifference::RecompositeLayer: ts << "recomposite layer"; break;
     case StyleDifference::Repaint: ts << "repaint"; break;
-    case StyleDifference::RepaintIfTextOrBorderOrOutline: ts << "repaint if text or border or outline"; break;
+    case StyleDifference::RepaintIfText: ts << "repaint if text"; break;
     case StyleDifference::RepaintLayer: ts << "repaint layer"; break;
     case StyleDifference::LayoutPositionedMovementOnly: ts << "layout positioned movement only"; break;
-    case StyleDifference::SimplifiedLayout: ts << "simplified layout"; break;
-    case StyleDifference::SimplifiedLayoutAndPositionedMovement: ts << "simplified layout and positioned movement"; break;
+    case StyleDifference::Overflow: ts << "overflow"; break;
+    case StyleDifference::OverflowAndPositionedMovement: ts << "overflow and positioned movement"; break;
     case StyleDifference::Layout: ts << "layout"; break;
     case StyleDifference::NewStyle: ts << "new style"; break;
     }
@@ -1104,37 +1115,47 @@ TextStream& operator<<(TextStream& ts, TextAlignMode alignMode)
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, TextAlignLast textAlignLast)
+{
+    switch (textAlignLast) {
+    case TextAlignLast::Auto: ts << "auto"; break;
+    case TextAlignLast::Start: ts << "start"; break;
+    case TextAlignLast::End: ts << "end"; break;
+    case TextAlignLast::Left: ts << "left"; break;
+    case TextAlignLast::Right: ts << "right"; break;
+    case TextAlignLast::Center: ts << "center"; break;
+    case TextAlignLast::Justify: ts << "justify"; break;
+    }
+
+    return ts;
+}
+
 TextStream& operator<<(TextStream& ts, TextCombine textCombine)
 {
     switch (textCombine) {
     case TextCombine::None: ts << "none"; break;
-    case TextCombine::Horizontal: ts << "horizontal"; break;
+    case TextCombine::All: ts << "all"; break;
     }
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, TextDecoration textDecoration)
+TextStream& operator<<(TextStream& ts, TextDecorationLine line)
 {
-    switch (textDecoration) {
-    case TextDecoration::None: ts << "none"; break;
-    case TextDecoration::Underline: ts << "underline"; break;
-    case TextDecoration::Overline: ts << "overline"; break;
-    case TextDecoration::LineThrough: ts << "line-through"; break;
-    case TextDecoration::Blink: ts << "blink"; break;
-#if ENABLE(LETTERPRESS)
-    case TextDecoration::Letterpress: ts << "letterpress"; break;
-#endif
+    switch (line) {
+    case TextDecorationLine::Underline: ts << "underline"; break;
+    case TextDecorationLine::Overline: ts << "overline"; break;
+    case TextDecorationLine::LineThrough: ts << "line-through"; break;
+    case TextDecorationLine::Blink: ts << "blink"; break;
     }
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, TextDecorationSkip skip)
+TextStream& operator<<(TextStream& ts, TextDecorationSkipInk skip)
 {
     switch (skip) {
-    case TextDecorationSkip::None: ts << "none"; break;
-    case TextDecorationSkip::Ink: ts << "ink"; break;
-    case TextDecorationSkip::Objects: ts << "objects"; break;
-    case TextDecorationSkip::Auto: ts << "auto"; break;
+    case TextDecorationSkipInk::None: ts << "none"; break;
+    case TextDecorationSkipInk::Auto: ts << "auto"; break;
+    case TextDecorationSkipInk::All: ts << "all"; break;
     }
     return ts;
 }
@@ -1186,15 +1207,32 @@ TextStream& operator<<(TextStream& ts, TextEmphasisPosition position)
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, TextOrientation orientation)
+TextStream& operator<<(TextStream& ts, TextGroupAlign textGroupAlign)
 {
-    switch (orientation) {
-    case TextOrientation::Mixed: ts << "mixed"; break;
-    case TextOrientation::Upright: ts << "upright"; break;
-    case TextOrientation::Sideways: ts << "sideways"; break;
+    switch (textGroupAlign) {
+    case TextGroupAlign::None: ts << "none"; break;
+    case TextGroupAlign::Start: ts << "start"; break;
+    case TextGroupAlign::End: ts << "end"; break;
+    case TextGroupAlign::Left: ts << "left"; break;
+    case TextGroupAlign::Right: ts << "right"; break;
+    case TextGroupAlign::Center: ts << "center"; break;
     }
+
     return ts;
 }
+
+TextStream& operator<<(TextStream& ts, TextJustify justify)
+{
+    switch (justify) {
+    case TextJustify::Auto: ts << "auto"; break;
+    case TextJustify::InterCharacter: ts << "inter-character"; break;
+    case TextJustify::InterWord: ts << "inter-word"; break;
+    case TextJustify::None: ts << "none"; break;
+    }
+
+    return ts;
+}
+
 TextStream& operator<<(TextStream& ts, TextOverflow overflow)
 {
     switch (overflow) {
@@ -1221,17 +1259,65 @@ TextStream& operator<<(TextStream& ts, TextTransform textTransform)
     case TextTransform::Capitalize: ts << "capitalize"; break;
     case TextTransform::Uppercase: ts << "uppercase"; break;
     case TextTransform::Lowercase: ts << "lowercase"; break;
-    case TextTransform::None: ts << "none"; break;
+    case TextTransform::FullSizeKana: ts << "full-size-kana"; break;
+    case TextTransform::FullWidth: ts << "full-width"; break;
     }
     return ts;
 }
 
-TextStream& operator<<(TextStream& ts, TextUnderlinePosition underlinePosition)
+TextStream& operator<<(TextStream& ts, TextUnderlinePosition position)
 {
-    switch (underlinePosition) {
-    case TextUnderlinePosition::Auto: ts << "Auto"; break;
-    case TextUnderlinePosition::Under: ts << "Under"; break;
-    case TextUnderlinePosition::FromFont: ts << "FromFont"; break;
+    switch (position) {
+    case TextUnderlinePosition::FromFont: ts << "from-font"; break;
+    case TextUnderlinePosition::Under: ts << "under"; break;
+    case TextUnderlinePosition::Left: ts << "left"; break;
+    case TextUnderlinePosition::Right: ts << "right"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, TextWrapMode wrap)
+{
+    switch (wrap) {
+    case TextWrapMode::Wrap: ts << "wrap"; break;
+    case TextWrapMode::NoWrap: ts << "nowrap"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, TextWrapStyle style)
+{
+    switch (style) {
+    case TextWrapStyle::Auto: ts << "auto"; break;
+    case TextWrapStyle::Balance: ts << "balance"; break;
+    case TextWrapStyle::Pretty: ts << "pretty"; break;
+    case TextWrapStyle::Stable: ts << "stable"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, TextBoxTrim textBoxTrim)
+{
+    switch (textBoxTrim) {
+    case TextBoxTrim::None: ts << "None"; break;
+    case TextBoxTrim::TrimStart: ts << "trim-start"; break;
+    case TextBoxTrim::TrimEnd: ts << "trim-end"; break;
+    case TextBoxTrim::TrimBoth: ts << "trim-both"; break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, TextEdgeType textEdgeType)
+{
+    switch (textEdgeType) {
+    case TextEdgeType::Auto: ts << "auto"; break;
+    case TextEdgeType::Leading: ts << "half-leading"; break;
+    case TextEdgeType::Text: ts << "text-over/under baseline"; break;
+    case TextEdgeType::CapHeight: ts << "cap-height baseline"; break;
+    case TextEdgeType::ExHeight: ts << "x-height baseline"; break;
+    case TextEdgeType::Alphabetic: ts << "alphabetic baseline"; break;
+    case TextEdgeType::CJKIdeographic: ts << "ideographic-over baseline"; break;
+    case TextEdgeType::CJKIdeographicInk: ts << "ideographic-ink-over/ink-under baseline"; break;
     }
     return ts;
 }
@@ -1262,8 +1348,8 @@ TextStream& operator<<(TextStream& ts, TransformStyle3D transformStyle)
     switch (transformStyle) {
     case TransformStyle3D::Flat: ts << "flat"; break;
     case TransformStyle3D::Preserve3D: ts << "preserve-3d"; break;
-#if ENABLE(CSS_TRANSFORM_STYLE_OPTIMIZED_3D)
-    case TransformStyle3D::Optimized3D: ts << "optimized-3d"; break;
+#if HAVE(CORE_ANIMATION_SEPARATED_LAYERS)
+    case TransformStyle3D::Separated: ts << "separated"; break;
 #endif
     }
     return ts;
@@ -1335,12 +1421,22 @@ TextStream& operator<<(TextStream& ts, WhiteSpace whiteSpace)
     case WhiteSpace::PreWrap: ts << "pre-wrap"; break;
     case WhiteSpace::PreLine: ts << "pre-line"; break;
     case WhiteSpace::NoWrap: ts << "nowrap"; break;
-    case WhiteSpace::KHTMLNoWrap: ts << "khtml-nowrap"; break;
     case WhiteSpace::BreakSpaces: ts << "break-spaces"; break;
     }
     return ts;
 }
 
+TextStream& operator<<(TextStream& ts, WhiteSpaceCollapse whiteSpaceCollapse)
+{
+    switch (whiteSpaceCollapse) {
+    case WhiteSpaceCollapse::Collapse: ts << "collapse"; break;
+    case WhiteSpaceCollapse::Preserve: ts << "preserve"; break;
+    case WhiteSpaceCollapse::PreserveBreaks: ts << "preserve-breaks"; break;
+    case WhiteSpaceCollapse::BreakSpaces: ts << "break-spaces"; break;
+    }
+    return ts;
+}
+
 TextStream& operator<<(TextStream& ts, WordBreak wordBreak)
 {
     switch (wordBreak) {
@@ -1348,6 +1444,7 @@ TextStream& operator<<(TextStream& ts, WordBreak wordBreak)
     case WordBreak::BreakAll: ts << "break-all"; break;
     case WordBreak::KeepAll: ts << "keep-all"; break;
     case WordBreak::BreakWord: ts << "break-word"; break;
+    case WordBreak::AutoPhrase: ts << "auto-phrase"; break;
     }
     return ts;
 }
@@ -1361,11 +1458,41 @@ TextStream& operator<<(TextStream& ts, MathStyle mathStyle)
     return ts;
 }
 
-bool alwaysPageBreak(BreakBetween between)
+TextStream& operator<<(TextStream& ts, ContainIntrinsicSizeType containIntrinsicSizeType)
 {
-    return between >= BreakBetween::Page;
+    switch (containIntrinsicSizeType) {
+    case ContainIntrinsicSizeType::None: ts << "none"; break;
+    case ContainIntrinsicSizeType::Length: ts << "length"; break;
+    case ContainIntrinsicSizeType::AutoAndLength: ts << "autoandlength"; break;
+    case ContainIntrinsicSizeType::AutoAndNone: ts << "autoandnone"; break;
+    }
+    return ts;
 }
 
-const float defaultMiterLimit = 4;
+TextStream& operator<<(TextStream& ts, OverflowContinue overflowContinue)
+{
+    switch (overflowContinue) {
+    case OverflowContinue::Auto:
+        ts << "auto";
+        break;
+    case OverflowContinue::Discard:
+        ts << "discard";
+        break;
+    }
+    return ts;
+}
+
+TextStream& operator<<(TextStream& ts, StyleDifferenceContextSensitiveProperty property)
+{
+    switch (property) {
+    case StyleDifferenceContextSensitiveProperty::Transform: ts << "transform"; break;
+    case StyleDifferenceContextSensitiveProperty::Opacity: ts << "opacity"; break;
+    case StyleDifferenceContextSensitiveProperty::Filter: ts << "filter"; break;
+    case StyleDifferenceContextSensitiveProperty::ClipRect: ts << "clipRect"; break;
+    case StyleDifferenceContextSensitiveProperty::ClipPath: ts << "clipPath"; break;
+    case StyleDifferenceContextSensitiveProperty::WillChange: ts << "willChange"; break;
+    }
+    return ts;
+}
 
 } // namespace WebCore
