@@ -26,23 +26,32 @@
 #include "config.h"
 #include "SVGToOTFFontConversion.h"
 
-#if ENABLE(SVG_FONTS)
-
 #include "CSSStyleDeclaration.h"
-#include "ElementChildIterator.h"
+#include "ElementChildIteratorInlines.h"
 #include "Glyph.h"
+#include "HTMLParserIdioms.h"
+#include "SVGElementTypeHelpers.h"
 #include "SVGFontElement.h"
 #include "SVGFontFaceElement.h"
 #include "SVGGlyphElement.h"
 #include "SVGHKernElement.h"
 #include "SVGMissingGlyphElement.h"
 #include "SVGPathParser.h"
-#include "SVGPathStringSource.h"
+#include "SVGPathStringViewSource.h"
 #include "SVGVKernElement.h"
-#include <wtf/Optional.h>
 #include <wtf/Vector.h>
+#include <wtf/text/StringToIntegerConversion.h>
 #include <wtf/text/StringView.h>
 
+namespace WebCore {
+class SVGToOTFFontConverter;
+}
+
+namespace WTF {
+template<typename T> struct IsDeprecatedWeakRefSmartPointerException;
+template<> struct IsDeprecatedWeakRefSmartPointerException<WebCore::SVGToOTFFontConverter> : std::true_type { };
+}
+
 namespace WebCore {
 
 template <typename V>
@@ -54,12 +63,12 @@ static inline void append32(V& result, uint32_t value)
     result.append(value);
 }
 
-class SVGToOTFFontConverter {
+class SVGToOTFFontConverter : public CanMakeWeakPtr<SVGToOTFFontConverter> {
 public:
     SVGToOTFFontConverter(const SVGFontElement&);
     bool convertSVGToOTFFont();
 
-    Vector<char> releaseResult()
+    Vector<uint8_t> releaseResult()
     {
         return WTFMove(m_result);
     }
@@ -83,7 +92,7 @@ private:
         FloatRect boundingBox;
         Vector<char> charString;
         String codepoints;
-        const SVGGlyphElement* glyphElement;
+        WeakPtr<const SVGGlyphElement, WeakPtrImplWithEventTargetData> glyphElement;
         float horizontalAdvance;
         float verticalAdvance;
     };
@@ -93,9 +102,9 @@ private:
         Placeholder(SVGToOTFFontConverter& converter, size_t baseOfOffset)
             : m_converter(converter)
             , m_baseOfOffset(baseOfOffset)
-            , m_location(m_converter.m_result.size())
+            , m_location(m_converter->m_result.size())
         {
-            m_converter.append16(0);
+            m_converter->append16(0);
         }
 
         Placeholder(Placeholder&& other)
@@ -103,20 +112,17 @@ private:
             , m_baseOfOffset(other.m_baseOfOffset)
             , m_location(other.m_location)
 #if ASSERT_ENABLED
-            , m_active(other.m_active)
+            , m_active(std::exchange(other.m_active, false))
 #endif
         {
-#if ASSERT_ENABLED
-            other.m_active = false;
-#endif
         }
 
         void populate()
         {
             ASSERT(m_active);
-            size_t delta = m_converter.m_result.size() - m_baseOfOffset;
+            size_t delta = m_converter->m_result.size() - m_baseOfOffset;
             ASSERT(delta < std::numeric_limits<uint16_t>::max());
-            m_converter.overwrite16(m_location, delta);
+            m_converter->overwrite16(m_location, delta);
 #if ASSERT_ENABLED
             m_active = false;
 #endif
@@ -128,7 +134,7 @@ private:
         }
 
     private:
-        SVGToOTFFontConverter& m_converter;
+        WeakRef<SVGToOTFFontConverter> m_converter;
         const size_t m_baseOfOffset;
         const size_t m_location;
 #if ASSERT_ENABLED
@@ -158,12 +164,12 @@ private:
         WebCore::append32(m_result, value);
     }
 
-    void append32BitCode(const char code[4])
+    void append32BitCode(ASCIILiteral code)
     {
-        m_result.append(code[0]);
-        m_result.append(code[1]);
-        m_result.append(code[2]);
-        m_result.append(code[3]);
+        m_result.append(code[0uz]);
+        m_result.append(code[1uz]);
+        m_result.append(code[2uz]);
+        m_result.append(code[3uz]);
     }
 
     void append16(uint16_t value)
@@ -198,12 +204,12 @@ private:
 
     uint32_t calculateChecksum(size_t startingOffset, size_t endingOffset) const;
 
-    void processGlyphElement(const SVGElement& glyphOrMissingGlyphElement, const SVGGlyphElement*, float defaultHorizontalAdvance, float defaultVerticalAdvance, const String& codepoints, Optional<FloatRect>& boundingBox);
+    void processGlyphElement(const SVGElement& glyphOrMissingGlyphElement, const SVGGlyphElement*, float defaultHorizontalAdvance, float defaultVerticalAdvance, const String& codepoints, std::optional<FloatRect>& boundingBox);
 
     typedef void (SVGToOTFFontConverter::*FontAppendingFunction)();
-    void appendTable(const char identifier[4], FontAppendingFunction);
-    void appendFormat12CMAPTable(const Vector<std::pair<UChar32, Glyph>>& codepointToGlyphMappings);
-    void appendFormat4CMAPTable(const Vector<std::pair<UChar32, Glyph>>& codepointToGlyphMappings);
+    void appendTable(ASCIILiteral identifier, FontAppendingFunction);
+    void appendFormat12CMAPTable(const Vector<std::pair<char32_t, Glyph>>& codepointToGlyphMappings);
+    void appendFormat4CMAPTable(const Vector<std::pair<char32_t, Glyph>>& codepointToGlyphMappings);
     void appendCMAPTable();
     void appendGSUBTable();
     void appendHEADTable();
@@ -224,35 +230,37 @@ private:
 
     void appendValidCFFString(const String&);
 
-    Vector<char> transcodeGlyphPaths(float width, const SVGElement& glyphOrMissingGlyphElement, Optional<FloatRect>& boundingBox) const;
+    Vector<char> transcodeGlyphPaths(float width, const SVGElement& glyphOrMissingGlyphElement, std::optional<FloatRect>& boundingBox) const;
 
-    void addCodepointRanges(const UnicodeRanges&, HashSet<Glyph>& glyphSet) const;
-    void addCodepoints(const HashSet<String>& codepoints, HashSet<Glyph>& glyphSet) const;
-    void addGlyphNames(const HashSet<String>& glyphNames, HashSet<Glyph>& glyphSet) const;
-    void addKerningPair(Vector<KerningData>&, const SVGKerningPair&) const;
-    template<typename T> size_t appendKERNSubtable(bool (T::*buildKerningPair)(SVGKerningPair&) const, uint16_t coverage);
+    void addCodepointRanges(const UnicodeRanges&, UncheckedKeyHashSet<Glyph>& glyphSet) const;
+    void addCodepoints(const UncheckedKeyHashSet<String>& codepoints, UncheckedKeyHashSet<Glyph>& glyphSet) const;
+    void addGlyphNames(const UncheckedKeyHashSet<String>& glyphNames, UncheckedKeyHashSet<Glyph>& glyphSet) const;
+    void addKerningPair(Vector<KerningData>&, SVGKerningPair&&) const;
+    template<typename T> size_t appendKERNSubtable(std::optional<SVGKerningPair> (T::*buildKerningPair)() const, uint16_t coverage);
     size_t finishAppendingKERNSubtable(Vector<KerningData>, uint16_t coverage);
 
     void appendLigatureSubtable(size_t subtableRecordLocation);
-    void appendArabicReplacementSubtable(size_t subtableRecordLocation, const char arabicForm[]);
+    void appendArabicReplacementSubtable(size_t subtableRecordLocation, ASCIILiteral arabicForm);
     void appendScriptSubtable(unsigned featureCount);
-    Vector<Glyph, 1> glyphsForCodepoint(UChar32) const;
-    Glyph firstGlyph(const Vector<Glyph, 1>&, UChar32) const;
+    Vector<Glyph, 1> glyphsForCodepoint(char32_t) const;
+    Glyph firstGlyph(const Vector<Glyph, 1>&, char32_t) const;
 
     template<typename T> T scaleUnitsPerEm(T value) const
     {
         return value * s_outputUnitsPerEm / m_inputUnitsPerEm;
     }
 
+    Ref<const SVGFontElement> protectedFontElement() const { return m_fontElement.get(); }
+
     Vector<GlyphData> m_glyphs;
-    HashMap<String, Glyph> m_glyphNameToIndexMap; // SVG 1.1: "It is recommended that glyph names be unique within a font."
-    HashMap<String, Vector<Glyph, 1>> m_codepointsToIndicesMap;
-    Vector<char> m_result;
+    UncheckedKeyHashMap<String, Glyph> m_glyphNameToIndexMap; // SVG 1.1: "It is recommended that glyph names be unique within a font."
+    UncheckedKeyHashMap<String, Vector<Glyph, 1>> m_codepointsToIndicesMap;
+    Vector<uint8_t> m_result;
     Vector<char, 17> m_emptyGlyphCharString;
     FloatRect m_boundingBox;
-    const SVGFontElement& m_fontElement;
-    const SVGFontFaceElement* m_fontFaceElement;
-    const SVGMissingGlyphElement* m_missingGlyphElement;
+    WeakRef<const SVGFontElement, WeakPtrImplWithEventTargetData> m_fontElement;
+    WeakPtr<const SVGFontFaceElement, WeakPtrImplWithEventTargetData> m_fontFaceElement;
+    WeakPtr<const SVGMissingGlyphElement, WeakPtrImplWithEventTargetData> m_missingGlyphElement;
     String m_fontFamily;
     float m_advanceWidthMax;
     float m_advanceHeightMax;
@@ -288,7 +296,7 @@ static uint16_t integralLog2(uint16_t x)
     return result;
 }
 
-void SVGToOTFFontConverter::appendFormat12CMAPTable(const Vector<std::pair<UChar32, Glyph>>& mappings)
+void SVGToOTFFontConverter::appendFormat12CMAPTable(const Vector<std::pair<char32_t, Glyph>>& mappings)
 {
     // Braindead scheme: One segment for each character
     ASSERT(m_glyphs.size() < 0xFFFF);
@@ -306,7 +314,7 @@ void SVGToOTFFontConverter::appendFormat12CMAPTable(const Vector<std::pair<UChar
     overwrite32(subtableLocation + 12, mappings.size());
 }
 
-void SVGToOTFFontConverter::appendFormat4CMAPTable(const Vector<std::pair<UChar32, Glyph>>& bmpMappings)
+void SVGToOTFFontConverter::appendFormat4CMAPTable(const Vector<std::pair<char32_t, Glyph>>& bmpMappings)
 {
     auto subtableLocation = m_result.size();
     append16(4); // Format 4
@@ -367,11 +375,11 @@ void SVGToOTFFontConverter::appendCMAPTable()
     append16(10); // Unicode
     append32(28); // Byte offset of subtable
 
-    Vector<std::pair<UChar32, Glyph>> mappings;
-    UChar32 previousCodepoint = std::numeric_limits<UChar32>::max();
+    Vector<std::pair<char32_t, Glyph>> mappings;
+    char32_t previousCodepoint = std::numeric_limits<char32_t>::max();
     for (size_t i = 0; i < m_glyphs.size(); ++i) {
         auto& glyph = m_glyphs[i];
-        UChar32 codepoint;
+        char32_t codepoint;
         auto codePoints = StringView(glyph.codepoints).codePoints();
         auto iterator = codePoints.begin();
         if (iterator == codePoints.end())
@@ -390,7 +398,7 @@ void SVGToOTFFontConverter::appendCMAPTable()
 
     appendFormat12CMAPTable(mappings);
 
-    Vector<std::pair<UChar32, Glyph>> bmpMappings;
+    Vector<std::pair<char32_t, Glyph>> bmpMappings;
     for (auto& mapping : mappings) {
         if (mapping.first < 0x10000)
             bmpMappings.append(mapping);
@@ -491,13 +499,11 @@ void SVGToOTFFontConverter::appendNAMETable()
 void SVGToOTFFontConverter::appendOS2Table()
 {
     int16_t averageAdvance = s_outputUnitsPerEm;
-    bool ok;
-    int value = m_fontElement.attributeWithoutSynchronization(SVGNames::horiz_adv_xAttr).toInt(&ok);
-    if (!ok && m_missingGlyphElement)
-        value = m_missingGlyphElement->attributeWithoutSynchronization(SVGNames::horiz_adv_xAttr).toInt(&ok);
-    value = scaleUnitsPerEm(value);
-    if (ok)
-        averageAdvance = clampTo<int16_t>(value);
+    auto horizAdvX = parseHTMLInteger(m_fontElement->attributeWithoutSynchronization(SVGNames::horiz_adv_xAttr));
+    if (!horizAdvX && m_missingGlyphElement)
+        horizAdvX = parseHTMLInteger(m_missingGlyphElement->attributeWithoutSynchronization(SVGNames::horiz_adv_xAttr));
+    if (horizAdvX)
+        averageAdvance = clampTo<int16_t>(scaleUnitsPerEm(*horizAdvX));
 
     append16(2); // Version
     append16(clampTo<int16_t>(averageAdvance));
@@ -518,22 +524,22 @@ void SVGToOTFFontConverter::appendOS2Table()
     append16(0); // No classification
 
     unsigned numPanoseBytes = 0;
-    const unsigned panoseSize = 10;
-    char panoseBytes[panoseSize];
+    constexpr unsigned panoseSize = 10;
+    std::array<char, panoseSize> panoseBytes;
     if (m_fontFaceElement) {
-        Vector<String> segments = m_fontFaceElement->attributeWithoutSynchronization(SVGNames::panose_1Attr).string().split(' ');
-        if (segments.size() == panoseSize) {
-            for (auto& segment : segments) {
-                bool ok;
-                int value = segment.toInt(&ok);
-                if (ok && value >= std::numeric_limits<uint8_t>::min() && value <= std::numeric_limits<uint8_t>::max())
-                    panoseBytes[numPanoseBytes++] = value;
+        auto segments = StringView(m_fontFaceElement->attributeWithoutSynchronization(SVGNames::panose_1Attr)).split(' ');
+        for (auto segment : segments) {
+            if (numPanoseBytes == panoseSize) {
+                ++numPanoseBytes;
+                break;
             }
+            if (auto value = parseIntegerAllowingTrailingJunk<uint8_t>(segment))
+                panoseBytes[numPanoseBytes++] = *value;
         }
     }
     if (numPanoseBytes != panoseSize)
-        memset(panoseBytes, 0, panoseSize);
-    m_result.append(panoseBytes, panoseSize);
+        panoseBytes.fill(0);
+    m_result.append(std::span<char> { panoseBytes });
 
     for (int i = 0; i < 4; ++i)
         append32(0); // "Bit assignments are pending. Set to 0"
@@ -715,7 +721,7 @@ void SVGToOTFFontConverter::appendCFFTable()
         m_result.appendVector(glyph.charString);
 }
 
-Glyph SVGToOTFFontConverter::firstGlyph(const Vector<Glyph, 1>& v, UChar32 codepoint) const
+Glyph SVGToOTFFontConverter::firstGlyph(const Vector<Glyph, 1>& v, char32_t codepoint) const
 {
 #if !ASSERT_ENABLED
     UNUSED_PARAM(codepoint);
@@ -809,7 +815,7 @@ void SVGToOTFFontConverter::appendLigatureSubtable(size_t subtableRecordLocation
     }
 }
 
-void SVGToOTFFontConverter::appendArabicReplacementSubtable(size_t subtableRecordLocation, const char arabicForm[])
+void SVGToOTFFontConverter::appendArabicReplacementSubtable(size_t subtableRecordLocation, ASCIILiteral arabicForm)
 {
     Vector<std::pair<Glyph, Glyph>> arabicFinalReplacements;
     for (auto& pair : m_codepointsToIndicesMap) {
@@ -866,9 +872,9 @@ void SVGToOTFFontConverter::appendGSUBTable()
     // ScriptList
     auto scriptListLocation = m_result.size();
     append16(2); // Number of ScriptRecords
-    append32BitCode("DFLT");
+    append32BitCode("DFLT"_s);
     append16(0); // Placeholder for offset of Script table, relative to beginning of ScriptList
-    append32BitCode("arab");
+    append32BitCode("arab"_s);
     append16(0); // Placeholder for offset of Script table, relative to beginning of ScriptList
 
     overwrite16(scriptListLocation + 6, m_result.size() - scriptListLocation);
@@ -884,15 +890,15 @@ void SVGToOTFFontConverter::appendGSUBTable()
     size_t featureListSize = 2 + 6 * featureCount;
     size_t featureTableSize = 6;
     append16(featureCount); // FeatureCount
-    append32BitCode("liga");
+    append32BitCode("liga"_s);
     append16(featureListSize + featureTableSize * 0); // Offset of feature table, relative to beginning of FeatureList table
-    append32BitCode("fina");
+    append32BitCode("fina"_s);
     append16(featureListSize + featureTableSize * 1); // Offset of feature table, relative to beginning of FeatureList table
-    append32BitCode("medi");
+    append32BitCode("medi"_s);
     append16(featureListSize + featureTableSize * 2); // Offset of feature table, relative to beginning of FeatureList table
-    append32BitCode("init");
+    append32BitCode("init"_s);
     append16(featureListSize + featureTableSize * 3); // Offset of feature table, relative to beginning of FeatureList table
-    append32BitCode("rlig");
+    append32BitCode("rlig"_s);
     append16(featureListSize + featureTableSize * 4); // Offset of feature table, relative to beginning of FeatureList table
     ASSERT_UNUSED(featureListLocation, featureListLocation + featureListSize == m_result.size());
 
@@ -910,7 +916,7 @@ void SVGToOTFFontConverter::appendGSUBTable()
     append16(featureCount); // LookupCount
     for (unsigned i = 0; i < featureCount; ++i)
         append16(0); // Placeholder for offset to feature table, relative to beginning of LookupList
-    size_t subtableRecordLocations[featureCount];
+    std::array<size_t, featureCount> subtableRecordLocations;
     for (unsigned i = 0; i < featureCount; ++i) {
         subtableRecordLocations[i] = m_result.size();
         overwrite16(lookupListLocation + 2 + 2 * i, m_result.size() - lookupListLocation);
@@ -931,9 +937,9 @@ void SVGToOTFFontConverter::appendGSUBTable()
     }
 
     appendLigatureSubtable(subtableRecordLocations[0]);
-    appendArabicReplacementSubtable(subtableRecordLocations[1], "terminal");
-    appendArabicReplacementSubtable(subtableRecordLocations[2], "medial");
-    appendArabicReplacementSubtable(subtableRecordLocations[3], "initial");
+    appendArabicReplacementSubtable(subtableRecordLocations[1], "terminal"_s);
+    appendArabicReplacementSubtable(subtableRecordLocations[2], "medial"_s);
+    appendArabicReplacementSubtable(subtableRecordLocations[3], "initial"_s);
 
     // Manually append empty "rlig" subtable
     overwrite16(subtableRecordLocations[4] + 6, m_result.size() - subtableRecordLocations[4]);
@@ -949,20 +955,18 @@ void SVGToOTFFontConverter::appendVORGTable()
     append16(1); // Major version
     append16(0); // Minor version
 
-    bool ok;
-    int defaultVerticalOriginY = m_fontElement.attributeWithoutSynchronization(SVGNames::vert_origin_yAttr).toInt(&ok);
-    if (!ok && m_missingGlyphElement)
-        defaultVerticalOriginY = m_missingGlyphElement->attributeWithoutSynchronization(SVGNames::vert_origin_yAttr).toInt();
-    defaultVerticalOriginY = scaleUnitsPerEm(defaultVerticalOriginY);
-    append16(clampTo<int16_t>(defaultVerticalOriginY));
+    auto vertOriginY = parseHTMLInteger(m_fontElement->attributeWithoutSynchronization(SVGNames::vert_origin_yAttr));
+    if (!vertOriginY && m_missingGlyphElement)
+        vertOriginY = parseHTMLInteger(m_missingGlyphElement->attributeWithoutSynchronization(SVGNames::vert_origin_yAttr));
+    append16(clampTo<int16_t>(scaleUnitsPerEm(vertOriginY.value_or(0))));
 
     auto tableSizeOffset = m_result.size();
     append16(0); // Place to write table size.
     for (Glyph i = 0; i < m_glyphs.size(); ++i) {
-        if (auto* glyph = m_glyphs[i].glyphElement) {
-            if (int verticalOriginY = glyph->attributeWithoutSynchronization(SVGNames::vert_origin_yAttr).toInt()) {
+        if (auto& glyph = m_glyphs[i].glyphElement) {
+            if (auto verticalOriginY = parseHTMLInteger(glyph->attributeWithoutSynchronization(SVGNames::vert_origin_yAttr))) {
                 append16(i);
-                append16(clampTo<int16_t>(scaleUnitsPerEm(verticalOriginY)));
+                append16(clampTo<int16_t>(scaleUnitsPerEm(*verticalOriginY)));
             }
         }
     }
@@ -1000,21 +1004,21 @@ void SVGToOTFFontConverter::appendVMTXTable()
     }
 }
 
-static String codepointToString(UChar32 codepoint)
+static String codepointToString(char32_t codepoint)
 {
-    UChar buffer[2];
+    std::array<UChar, 2> buffer;
     uint8_t length = 0;
     UBool error = false;
     U16_APPEND(buffer, length, 2, codepoint, error);
-    return error ? String() : String(buffer, length);
+    return error ? String() : String(std::span<UChar> { buffer }.first(length));
 }
 
-Vector<Glyph, 1> SVGToOTFFontConverter::glyphsForCodepoint(UChar32 codepoint) const
+Vector<Glyph, 1> SVGToOTFFontConverter::glyphsForCodepoint(char32_t codepoint) const
 {
     return m_codepointsToIndicesMap.get(codepointToString(codepoint));
 }
 
-void SVGToOTFFontConverter::addCodepointRanges(const UnicodeRanges& unicodeRanges, HashSet<Glyph>& glyphSet) const
+void SVGToOTFFontConverter::addCodepointRanges(const UnicodeRanges& unicodeRanges, UncheckedKeyHashSet<Glyph>& glyphSet) const
 {
     for (auto& unicodeRange : unicodeRanges) {
         for (auto codepoint = unicodeRange.first; codepoint <= unicodeRange.second; ++codepoint) {
@@ -1024,7 +1028,7 @@ void SVGToOTFFontConverter::addCodepointRanges(const UnicodeRanges& unicodeRange
     }
 }
 
-void SVGToOTFFontConverter::addCodepoints(const HashSet<String>& codepoints, HashSet<Glyph>& glyphSet) const
+void SVGToOTFFontConverter::addCodepoints(const UncheckedKeyHashSet<String>& codepoints, UncheckedKeyHashSet<Glyph>& glyphSet) const
 {
     for (auto& codepointString : codepoints) {
         for (auto index : m_codepointsToIndicesMap.get(codepointString))
@@ -1032,7 +1036,7 @@ void SVGToOTFFontConverter::addCodepoints(const HashSet<String>& codepoints, Has
     }
 }
 
-void SVGToOTFFontConverter::addGlyphNames(const HashSet<String>& glyphNames, HashSet<Glyph>& glyphSet) const
+void SVGToOTFFontConverter::addGlyphNames(const UncheckedKeyHashSet<String>& glyphNames, UncheckedKeyHashSet<Glyph>& glyphSet) const
 {
     for (auto& glyphName : glyphNames) {
         if (Glyph glyph = m_glyphNameToIndexMap.get(glyphName))
@@ -1040,10 +1044,10 @@ void SVGToOTFFontConverter::addGlyphNames(const HashSet<String>& glyphNames, Has
     }
 }
 
-void SVGToOTFFontConverter::addKerningPair(Vector<KerningData>& data, const SVGKerningPair& kerningPair) const
+void SVGToOTFFontConverter::addKerningPair(Vector<KerningData>& data, SVGKerningPair&& kerningPair) const
 {
-    HashSet<Glyph> glyphSet1;
-    HashSet<Glyph> glyphSet2;
+    UncheckedKeyHashSet<Glyph> glyphSet1;
+    UncheckedKeyHashSet<Glyph> glyphSet2;
 
     addCodepointRanges(kerningPair.unicodeRange1, glyphSet1);
     addCodepointRanges(kerningPair.unicodeRange2, glyphSet2);
@@ -1059,13 +1063,12 @@ void SVGToOTFFontConverter::addKerningPair(Vector<KerningData>& data, const SVGK
     }
 }
 
-template<typename T> inline size_t SVGToOTFFontConverter::appendKERNSubtable(bool (T::*buildKerningPair)(SVGKerningPair&) const, uint16_t coverage)
+template<typename T> inline size_t SVGToOTFFontConverter::appendKERNSubtable(std::optional<SVGKerningPair> (T::*buildKerningPair)() const, uint16_t coverage)
 {
     Vector<KerningData> kerningData;
-    for (auto& element : childrenOfType<T>(m_fontElement)) {
-        SVGKerningPair kerningPair;
-        if ((element.*buildKerningPair)(kerningPair))
-            addKerningPair(kerningData, kerningPair);
+    for (auto& element : childrenOfType<T>(protectedFontElement())) {
+        if (auto kerningPair = (element.*buildKerningPair)())
+            addKerningPair(kerningData, WTFMove(*kerningPair));
     }
     return finishAppendingKERNSubtable(WTFMove(kerningData), coverage);
 }
@@ -1143,7 +1146,7 @@ public:
         m_cffData.append(rMoveTo);
     }
 
-    Optional<FloatRect> boundingBox() const
+    std::optional<FloatRect> boundingBox() const
     {
         return m_boundingBox;
     }
@@ -1234,11 +1237,11 @@ private:
     Vector<char>& m_cffData;
     FloatPoint m_startingPoint;
     FloatPoint m_current;
-    Optional<FloatRect> m_boundingBox;
+    std::optional<FloatRect> m_boundingBox;
     float m_unitsPerEmScalar;
 };
 
-Vector<char> SVGToOTFFontConverter::transcodeGlyphPaths(float width, const SVGElement& glyphOrMissingGlyphElement, Optional<FloatRect>& boundingBox) const
+Vector<char> SVGToOTFFontConverter::transcodeGlyphPaths(float width, const SVGElement& glyphOrMissingGlyphElement, std::optional<FloatRect>& boundingBox) const
 {
     Vector<char> result;
 
@@ -1262,7 +1265,7 @@ Vector<char> SVGToOTFFontConverter::transcodeGlyphPaths(float width, const SVGEl
         horizontalOriginY = scaleUnitsPerEm(m_fontFaceElement->horizontalOriginY());
 
     CFFBuilder builder(result, width, FloatPoint(horizontalOriginX, horizontalOriginY), static_cast<float>(s_outputUnitsPerEm) / m_inputUnitsPerEm);
-    SVGPathStringSource source(dAttribute);
+    SVGPathStringViewSource source(dAttribute);
 
     ok = SVGPathParser::parse(source, builder);
     if (!ok)
@@ -1274,7 +1277,7 @@ Vector<char> SVGToOTFFontConverter::transcodeGlyphPaths(float width, const SVGEl
     return result;
 }
 
-void SVGToOTFFontConverter::processGlyphElement(const SVGElement& glyphOrMissingGlyphElement, const SVGGlyphElement* glyphElement, float defaultHorizontalAdvance, float defaultVerticalAdvance, const String& codepoints, Optional<FloatRect>& boundingBox)
+void SVGToOTFFontConverter::processGlyphElement(const SVGElement& glyphOrMissingGlyphElement, const SVGGlyphElement* glyphElement, float defaultHorizontalAdvance, float defaultVerticalAdvance, const String& codepoints, std::optional<FloatRect>& boundingBox)
 {
     bool ok;
     float horizontalAdvance = scaleUnitsPerEm(glyphOrMissingGlyphElement.attributeWithoutSynchronization(SVGNames::horiz_adv_xAttr).toFloat(&ok));
@@ -1286,7 +1289,7 @@ void SVGToOTFFontConverter::processGlyphElement(const SVGElement& glyphOrMissing
         verticalAdvance = defaultVerticalAdvance;
     m_advanceHeightMax = std::max(m_advanceHeightMax, verticalAdvance);
 
-    Optional<FloatRect> glyphBoundingBox;
+    std::optional<FloatRect> glyphBoundingBox;
     auto path = transcodeGlyphPaths(horizontalAdvance, glyphOrMissingGlyphElement, glyphBoundingBox);
     if (!path.size()) {
         // It's better to use a fallback font rather than use a font without all its glyphs.
@@ -1299,19 +1302,19 @@ void SVGToOTFFontConverter::processGlyphElement(const SVGElement& glyphOrMissing
     if (glyphBoundingBox)
         m_minRightSideBearing = std::min(m_minRightSideBearing, horizontalAdvance - glyphBoundingBox.value().maxX());
 
-    m_glyphs.append(GlyphData(WTFMove(path), glyphElement, horizontalAdvance, verticalAdvance, glyphBoundingBox.valueOr(FloatRect()), codepoints));
+    m_glyphs.append(GlyphData(WTFMove(path), glyphElement, horizontalAdvance, verticalAdvance, valueOrDefault(glyphBoundingBox), codepoints));
 }
 
 void SVGToOTFFontConverter::appendLigatureGlyphs()
 {
-    HashSet<UChar32> ligatureCodepoints;
-    HashSet<UChar32> nonLigatureCodepoints;
+    UncheckedKeyHashSet<uint32_t> ligatureCodepoints;
+    UncheckedKeyHashSet<uint32_t> nonLigatureCodepoints;
     for (auto& glyph : m_glyphs) {
         auto codePoints = StringView(glyph.codepoints).codePoints();
         auto codePointsIterator = codePoints.begin();
         if (codePointsIterator == codePoints.end())
             continue;
-        UChar32 codepoint = *codePointsIterator;
+        char32_t codepoint = *codePointsIterator;
         ++codePointsIterator;
         if (codePointsIterator == codePoints.end())
             nonLigatureCodepoints.add(codepoint);
@@ -1321,7 +1324,6 @@ void SVGToOTFFontConverter::appendLigatureGlyphs()
                 ligatureCodepoints.add(*codePointsIterator);
         }
     }
-
     for (auto codepoint : nonLigatureCodepoints)
         ligatureCodepoints.remove(codepoint);
     for (auto codepoint : ligatureCodepoints) {
@@ -1338,7 +1340,7 @@ bool SVGToOTFFontConverter::compareCodepointsLexicographically(const GlyphData&
     auto iterator1 = codePoints1.begin();
     auto iterator2 = codePoints2.begin();
     while (iterator1 != codePoints1.end() && iterator2 != codePoints2.end()) {
-        UChar32 codepoint1, codepoint2;
+        char32_t codepoint1, codepoint2;
         codepoint1 = *iterator1;
         codepoint2 = *iterator2;
 
@@ -1352,8 +1354,8 @@ bool SVGToOTFFontConverter::compareCodepointsLexicographically(const GlyphData&
     }
 
     if (iterator1 == codePoints1.end() && iterator2 == codePoints2.end()) {
-        bool firstIsIsolated = data1.glyphElement && equalLettersIgnoringASCIICase(data1.glyphElement->attributeWithoutSynchronization(SVGNames::arabic_formAttr), "isolated");
-        bool secondIsIsolated = data2.glyphElement && equalLettersIgnoringASCIICase(data2.glyphElement->attributeWithoutSynchronization(SVGNames::arabic_formAttr), "isolated");
+        bool firstIsIsolated = data1.glyphElement && equalLettersIgnoringASCIICase(data1.glyphElement->attributeWithoutSynchronization(SVGNames::arabic_formAttr), "isolated"_s);
+        bool secondIsIsolated = data2.glyphElement && equalLettersIgnoringASCIICase(data2.glyphElement->attributeWithoutSynchronization(SVGNames::arabic_formAttr), "isolated"_s);
         return firstIsIsolated && !secondIsIsolated;
     }
     return iterator1 == codePoints1.end();
@@ -1370,8 +1372,8 @@ static void populateEmptyGlyphCharString(Vector<char, 17>& o, unsigned unitsPerE
 
 SVGToOTFFontConverter::SVGToOTFFontConverter(const SVGFontElement& fontElement)
     : m_fontElement(fontElement)
-    , m_fontFaceElement(childrenOfType<SVGFontFaceElement>(m_fontElement).first())
-    , m_missingGlyphElement(childrenOfType<SVGMissingGlyphElement>(m_fontElement).first())
+    , m_fontFaceElement(childrenOfType<SVGFontFaceElement>(fontElement).first())
+    , m_missingGlyphElement(childrenOfType<SVGMissingGlyphElement>(fontElement).first())
     , m_advanceWidthMax(0)
     , m_advanceHeightMax(0)
     , m_minRightSideBearing(std::numeric_limits<float>::max())
@@ -1409,21 +1411,21 @@ SVGToOTFFontConverter::SVGToOTFFontConverter(const SVGFontElement& fontElement)
 
     populateEmptyGlyphCharString(m_emptyGlyphCharString, s_outputUnitsPerEm);
 
-    Optional<FloatRect> boundingBox;
-    if (m_missingGlyphElement)
-        processGlyphElement(*m_missingGlyphElement, nullptr, defaultHorizontalAdvance, defaultVerticalAdvance, String(), boundingBox);
+    std::optional<FloatRect> boundingBox;
+    if (RefPtr missingGlyphElement = m_missingGlyphElement.get())
+        processGlyphElement(*missingGlyphElement, nullptr, defaultHorizontalAdvance, defaultVerticalAdvance, String(), boundingBox);
     else {
         m_glyphs.append(GlyphData(Vector<char>(m_emptyGlyphCharString), nullptr, s_outputUnitsPerEm, s_outputUnitsPerEm, FloatRect(), String()));
         boundingBox = FloatRect(0, 0, s_outputUnitsPerEm, s_outputUnitsPerEm);
     }
 
-    for (auto& glyphElement : childrenOfType<SVGGlyphElement>(m_fontElement)) {
+    for (auto& glyphElement : childrenOfType<SVGGlyphElement>(protectedFontElement())) {
         auto& unicodeAttribute = glyphElement.attributeWithoutSynchronization(SVGNames::unicodeAttr);
         if (!unicodeAttribute.isEmpty()) // If we can never actually trigger this glyph, ignore it completely
             processGlyphElement(glyphElement, &glyphElement, defaultHorizontalAdvance, defaultVerticalAdvance, unicodeAttribute, boundingBox);
     }
 
-    m_boundingBox = boundingBox.valueOr(FloatRect());
+    m_boundingBox = valueOrDefault(boundingBox);
 
     appendLigatureGlyphs();
 
@@ -1442,9 +1444,9 @@ SVGToOTFFontConverter::SVGToOTFFontConverter(const SVGFontElement& fontElement)
                 m_glyphNameToIndexMap.add(glyphName, i);
         }
         if (m_codepointsToIndicesMap.isValidKey(glyph.codepoints)) {
-            auto& glyphVector = m_codepointsToIndicesMap.add(glyph.codepoints, Vector<Glyph>()).iterator->value;
+            auto& glyphVector = m_codepointsToIndicesMap.add(glyph.codepoints, Vector<Glyph, 1>()).iterator->value;
             // Prefer isolated arabic forms
-            if (glyph.glyphElement && equalLettersIgnoringASCIICase(glyph.glyphElement->attributeWithoutSynchronization(SVGNames::arabic_formAttr), "isolated"))
+            if (glyph.glyphElement && equalLettersIgnoringASCIICase(glyph.glyphElement->attributeWithoutSynchronization(SVGNames::arabic_formAttr), "isolated"_s))
                 glyphVector.insert(0, i);
             else
                 glyphVector.append(i);
@@ -1453,22 +1455,18 @@ SVGToOTFFontConverter::SVGToOTFFontConverter(const SVGFontElement& fontElement)
 
     // FIXME: Handle commas.
     if (m_fontFaceElement) {
-        auto& fontWeightAttribute = m_fontFaceElement->attributeWithoutSynchronization(SVGNames::font_weightAttr);
-        for (auto segment : StringView(fontWeightAttribute).split(' ')) {
-            if (equalLettersIgnoringASCIICase(segment, "bold")) {
+        for (auto segment : StringView(m_fontFaceElement->attributeWithoutSynchronization(SVGNames::font_weightAttr)).split(' ')) {
+            if (equalLettersIgnoringASCIICase(segment, "bold"_s)) {
                 m_weight = 7;
                 break;
             }
-            bool ok;
-            int value = segment.toInt(ok);
-            if (ok && value >= 0 && value < 1000) {
-                m_weight = std::max(std::min((value + 50) / 100, static_cast<int>(std::numeric_limits<uint8_t>::max())), static_cast<int>(std::numeric_limits<uint8_t>::min()));
+            if (auto value = parseIntegerAllowingTrailingJunk<uint16_t>(segment); value && *value < 1000) {
+                m_weight = (*value + 50) / 100;
                 break;
             }
         }
-        auto& fontStyleAttribute = m_fontFaceElement->attributeWithoutSynchronization(SVGNames::font_styleAttr);
-        for (auto segment : StringView(fontStyleAttribute).split(' ')) {
-            if (equalLettersIgnoringASCIICase(segment, "italic") || equalLettersIgnoringASCIICase(segment, "oblique")) {
+        for (auto segment : StringView(m_fontFaceElement->attributeWithoutSynchronization(SVGNames::font_styleAttr)).split(' ')) {
+            if (equalLettersIgnoringASCIICase(segment, "italic"_s) || equalLettersIgnoringASCIICase(segment, "oblique"_s)) {
                 m_italic = true;
                 break;
             }
@@ -1497,7 +1495,7 @@ uint32_t SVGToOTFFontConverter::calculateChecksum(size_t startingOffset, size_t
     return sum;
 }
 
-void SVGToOTFFontConverter::appendTable(const char identifier[4], FontAppendingFunction appendingFunction)
+void SVGToOTFFontConverter::appendTable(ASCIILiteral identifier, FontAppendingFunction appendingFunction)
 {
     size_t offset = m_result.size();
     ASSERT(isFourByteAligned(offset));
@@ -1507,10 +1505,10 @@ void SVGToOTFFontConverter::appendTable(const char identifier[4], FontAppendingF
         m_result.append(0);
     ASSERT(isFourByteAligned(m_result.size()));
     size_t directoryEntryOffset = headerSize + m_tablesAppendedCount * directoryEntrySize;
-    m_result[directoryEntryOffset] = identifier[0];
-    m_result[directoryEntryOffset + 1] = identifier[1];
-    m_result[directoryEntryOffset + 2] = identifier[2];
-    m_result[directoryEntryOffset + 3] = identifier[3];
+    m_result[directoryEntryOffset] = identifier[0uz];
+    m_result[directoryEntryOffset + 1] = identifier[1uz];
+    m_result[directoryEntryOffset + 2] = identifier[2uz];
+    m_result[directoryEntryOffset + 3] = identifier[3uz];
     overwrite32(directoryEntryOffset + 4, calculateChecksum(offset, m_result.size()));
     overwrite32(directoryEntryOffset + 8, offset);
     overwrite32(directoryEntryOffset + 12, unpaddedSize);
@@ -1541,21 +1539,21 @@ bool SVGToOTFFontConverter::convertSVGToOTFFont()
     for (size_t i = 0; i < directoryEntrySize * numTables; ++i)
         m_result.append(0);
 
-    appendTable("CFF ", &SVGToOTFFontConverter::appendCFFTable);
-    appendTable("GSUB", &SVGToOTFFontConverter::appendGSUBTable);
-    appendTable("OS/2", &SVGToOTFFontConverter::appendOS2Table);
-    appendTable("VORG", &SVGToOTFFontConverter::appendVORGTable);
-    appendTable("cmap", &SVGToOTFFontConverter::appendCMAPTable);
+    appendTable("CFF "_s, &SVGToOTFFontConverter::appendCFFTable);
+    appendTable("GSUB"_s, &SVGToOTFFontConverter::appendGSUBTable);
+    appendTable("OS/2"_s, &SVGToOTFFontConverter::appendOS2Table);
+    appendTable("VORG"_s, &SVGToOTFFontConverter::appendVORGTable);
+    appendTable("cmap"_s, &SVGToOTFFontConverter::appendCMAPTable);
     auto headTableOffset = m_result.size();
-    appendTable("head", &SVGToOTFFontConverter::appendHEADTable);
-    appendTable("hhea", &SVGToOTFFontConverter::appendHHEATable);
-    appendTable("hmtx", &SVGToOTFFontConverter::appendHMTXTable);
-    appendTable("kern", &SVGToOTFFontConverter::appendKERNTable);
-    appendTable("maxp", &SVGToOTFFontConverter::appendMAXPTable);
-    appendTable("name", &SVGToOTFFontConverter::appendNAMETable);
-    appendTable("post", &SVGToOTFFontConverter::appendPOSTTable);
-    appendTable("vhea", &SVGToOTFFontConverter::appendVHEATable);
-    appendTable("vmtx", &SVGToOTFFontConverter::appendVMTXTable);
+    appendTable("head"_s, &SVGToOTFFontConverter::appendHEADTable);
+    appendTable("hhea"_s, &SVGToOTFFontConverter::appendHHEATable);
+    appendTable("hmtx"_s, &SVGToOTFFontConverter::appendHMTXTable);
+    appendTable("kern"_s, &SVGToOTFFontConverter::appendKERNTable);
+    appendTable("maxp"_s, &SVGToOTFFontConverter::appendMAXPTable);
+    appendTable("name"_s, &SVGToOTFFontConverter::appendNAMETable);
+    appendTable("post"_s, &SVGToOTFFontConverter::appendPOSTTable);
+    appendTable("vhea"_s, &SVGToOTFFontConverter::appendVHEATable);
+    appendTable("vmtx"_s, &SVGToOTFFontConverter::appendVMTXTable);
 
     ASSERT(numTables == m_tablesAppendedCount);
 
@@ -1565,16 +1563,14 @@ bool SVGToOTFFontConverter::convertSVGToOTFFont()
     return true;
 }
 
-Optional<Vector<char>> convertSVGToOTFFont(const SVGFontElement& element)
+std::optional<Vector<uint8_t>> convertSVGToOTFFont(const SVGFontElement& element)
 {
     SVGToOTFFontConverter converter(element);
     if (converter.error())
-        return WTF::nullopt;
+        return std::nullopt;
     if (!converter.convertSVGToOTFFont())
-        return WTF::nullopt;
+        return std::nullopt;
     return converter.releaseResult();
 }
 
 }
-
-#endif // ENABLE(SVG_FONTS)
