@@ -26,18 +26,25 @@
 #include "config.h"
 #include "ServiceWorkerInternals.h"
 
-#if ENABLE(SERVICE_WORKER)
-
 #include "FetchEvent.h"
+#include "FetchRequest.h"
+#include "JSDOMPromiseDeferred.h"
 #include "JSFetchResponse.h"
+#include "NotificationPayload.h"
+#include "PushSubscription.h"
+#include "PushSubscriptionData.h"
 #include "SWContextManager.h"
+#include "ServiceWorkerClient.h"
+#include "ServiceWorkerGlobalScope.h"
+#include "ServiceWorkerRegistration.h"
 #include <wtf/ProcessID.h>
 
 namespace WebCore {
 
-ServiceWorkerInternals::ServiceWorkerInternals(ServiceWorkerIdentifier identifier)
+ServiceWorkerInternals::ServiceWorkerInternals(ServiceWorkerGlobalScope& globalScope, ServiceWorkerIdentifier identifier)
     : m_identifier(identifier)
 {
+    globalScope.setIsProcessingUserGestureForTesting(true);
 }
 
 ServiceWorkerInternals::~ServiceWorkerInternals() = default;
@@ -45,7 +52,7 @@ ServiceWorkerInternals::~ServiceWorkerInternals() = default;
 void ServiceWorkerInternals::setOnline(bool isOnline)
 {
     callOnMainThread([identifier = m_identifier, isOnline] () {
-        if (auto* proxy = SWContextManager::singleton().workerByID(identifier))
+        if (auto* proxy = SWContextManager::singleton().serviceWorkerThreadProxy(identifier))
             proxy->notifyNetworkStateChange(isOnline);
     });
 }
@@ -57,14 +64,51 @@ void ServiceWorkerInternals::terminate()
     });
 }
 
+void ServiceWorkerInternals::schedulePushEvent(const String& message, RefPtr<DeferredPromise>&& promise)
+{
+    auto counter = ++m_pushEventCounter;
+    m_pushEventPromises.add(counter, WTFMove(promise));
+
+    std::optional<Vector<uint8_t>> data;
+    if (!message.isNull())
+        data = Vector(byteCast<uint8_t>(message.utf8().span()));
+    callOnMainThread([identifier = m_identifier, data = WTFMove(data), weakThis = WeakPtr { *this }, counter]() mutable {
+        SWContextManager::singleton().firePushEvent(identifier, WTFMove(data), std::nullopt, [identifier, weakThis = WTFMove(weakThis), counter](bool result, std::optional<NotificationPayload>&&) mutable {
+            if (auto* proxy = SWContextManager::singleton().serviceWorkerThreadProxy(identifier)) {
+                proxy->thread().runLoop().postTaskForMode([weakThis = WTFMove(weakThis), counter, result](auto&) {
+                    if (!weakThis)
+                        return;
+                    if (auto promise = weakThis->m_pushEventPromises.take(counter))
+                        promise->resolve<IDLBoolean>(result);
+                }, WorkerRunLoop::defaultMode());
+            }
+        });
+    });
+}
+
+void ServiceWorkerInternals::schedulePushSubscriptionChangeEvent(PushSubscription* newSubscription, PushSubscription* oldSubscription)
+{
+    std::optional<PushSubscriptionData> newSubscriptionData;
+    std::optional<PushSubscriptionData> oldSubscriptionData;
+
+    if (newSubscription)
+        newSubscriptionData = newSubscription->data().isolatedCopy();
+    if (oldSubscription)
+        oldSubscriptionData = oldSubscription->data().isolatedCopy();
+
+    callOnMainThread([identifier = m_identifier, newSubscriptionData = WTFMove(newSubscriptionData), oldSubscriptionData = WTFMove(oldSubscriptionData)]() mutable {
+        SWContextManager::singleton().firePushSubscriptionChangeEvent(identifier, WTFMove(newSubscriptionData), WTFMove(oldSubscriptionData));
+    });
+}
+
 void ServiceWorkerInternals::waitForFetchEventToFinish(FetchEvent& event, DOMPromiseDeferred<IDLInterface<FetchResponse>>&& promise)
 {
-    event.onResponse([promise = WTFMove(promise), event = makeRef(event)] (auto&& result) mutable {
+    event.onResponse([promise = WTFMove(promise), event = Ref { event }] (auto&& result) mutable {
         if (!result.has_value()) {
             String description;
             if (auto& error = result.error())
                 description = error->localizedDescription();
-            promise.reject(TypeError, description);
+            promise.reject(ExceptionCode::TypeError, description);
             return;
         }
         promise.resolve(WTFMove(result.value()));
@@ -87,18 +131,16 @@ Ref<FetchResponse> ServiceWorkerInternals::createOpaqueWithBlobBodyResponse(Scri
     ResourceResponse response;
     response.setType(ResourceResponse::Type::Cors);
     response.setTainting(ResourceResponse::Tainting::Opaque);
-    auto fetchResponse = FetchResponse::create(context, FetchBody::fromFormData(context, formData), FetchHeaders::Guard::Response, WTFMove(response));
+    auto fetchResponse = FetchResponse::create(&context, FetchBody::fromFormData(context, WTFMove(formData)), FetchHeaders::Guard::Response, WTFMove(response));
     fetchResponse->initializeOpaqueLoadIdentifierForTesting();
     return fetchResponse;
 }
 
 Vector<String> ServiceWorkerInternals::fetchResponseHeaderList(FetchResponse& response)
 {
-    Vector<String> headerNames;
-    headerNames.reserveInitialCapacity(response.internalResponseHeaders().size());
-    for (auto keyValue : response.internalResponseHeaders())
-        headerNames.uncheckedAppend(keyValue.key);
-    return headerNames;
+    return WTF::map(response.internalResponseHeaders(), [](auto& keyValue) {
+        return keyValue.key;
+    });
 }
 
 #if !PLATFORM(MAC)
@@ -123,8 +165,8 @@ void ServiceWorkerInternals::lastNavigationWasAppInitiated(Ref<DeferredPromise>&
 {
     ASSERT(!m_lastNavigationWasAppInitiatedPromise);
     m_lastNavigationWasAppInitiatedPromise = WTFMove(promise);
-    callOnMainThread([identifier = m_identifier, weakThis = makeWeakPtr(this)]() mutable {
-        if (auto* proxy = SWContextManager::singleton().workerByID(identifier)) {
+    callOnMainThread([identifier = m_identifier, weakThis = WeakPtr { *this }]() mutable {
+        if (auto* proxy = SWContextManager::singleton().serviceWorkerThreadProxy(identifier)) {
             proxy->thread().runLoop().postTaskForMode([weakThis = WTFMove(weakThis), appInitiated = proxy->lastNavigationWasAppInitiated()](auto&) {
                 if (!weakThis || !weakThis->m_lastNavigationWasAppInitiatedPromise)
                     return;
@@ -136,6 +178,34 @@ void ServiceWorkerInternals::lastNavigationWasAppInitiated(Ref<DeferredPromise>&
     });
 }
 
-} // namespace WebCore
+RefPtr<PushSubscription> ServiceWorkerInternals::createPushSubscription(const String& endpoint, std::optional<EpochTimeStamp> expirationTime, const ArrayBuffer& serverVAPIDPublicKey, const ArrayBuffer& clientECDHPublicKey, const ArrayBuffer& auth)
+{
+    return PushSubscription::create(PushSubscriptionData { std::nullopt, { endpoint }, expirationTime, serverVAPIDPublicKey.toVector(), clientECDHPublicKey.toVector(), auth.toVector() });
+}
 
-#endif
+bool ServiceWorkerInternals::fetchEventIsSameSite(FetchEvent& event)
+{
+    return event.request().internalRequest().isSameSite();
+}
+
+String ServiceWorkerInternals::serviceWorkerClientInternalIdentifier(const ServiceWorkerClient& client)
+{
+    return client.identifier().toString();
+}
+
+void ServiceWorkerInternals::setAsInspected(bool isInspected)
+{
+    SWContextManager::singleton().setAsInspected(m_identifier, isInspected);
+}
+
+void ServiceWorkerInternals::enableConsoleMessageReporting(ScriptExecutionContext& context)
+{
+    downcast<ServiceWorkerGlobalScope>(context).enableConsoleMessageReporting();
+}
+
+void ServiceWorkerInternals:: logReportedConsoleMessage(ScriptExecutionContext& context, const String& value)
+{
+    downcast<ServiceWorkerGlobalScope>(context).addConsoleMessage(MessageSource::Storage, MessageLevel::Info, value, 0);
+}
+
+} // namespace WebCore
