@@ -33,59 +33,68 @@
 #include "DownloadMonitor.h"
 #include "DownloadProxyMessages.h"
 #include "Logging.h"
+#include "MessageSenderInlines.h"
 #include "NetworkDataTask.h"
 #include "NetworkProcess.h"
 #include "NetworkSession.h"
 #include "SandboxExtension.h"
-#include "WebCoreArgumentCoders.h"
 #include <WebCore/NotImplemented.h>
+#include <wtf/TZoneMallocInlines.h>
 
 #if PLATFORM(COCOA)
 #include "NetworkDataTaskCocoa.h"
 #endif
 
 #define DOWNLOAD_RELEASE_LOG(fmt, ...) RELEASE_LOG(Network, "%p - Download::" fmt, this, ##__VA_ARGS__)
+#define DOWNLOAD_RELEASE_LOG_WITH_THIS(thisPtr, fmt, ...) RELEASE_LOG(Network, "%p - Download::" fmt, thisPtr, ##__VA_ARGS__)
 
 namespace WebKit {
 using namespace WebCore;
 
+WTF_MAKE_TZONE_ALLOCATED_IMPL(Download);
+
+Ref<Download> Download::create(DownloadManager& downloadManager, DownloadID downloadID, NetworkDataTask& download, NetworkSession& session, const String& suggestedName)
+{
+    return adoptRef(*new Download(downloadManager, downloadID, download, session, suggestedName));
+}
+
 Download::Download(DownloadManager& downloadManager, DownloadID downloadID, NetworkDataTask& download, NetworkSession& session, const String& suggestedName)
     : m_downloadManager(downloadManager)
     , m_downloadID(downloadID)
     , m_client(downloadManager.client())
     , m_download(&download)
     , m_sessionID(session.sessionID())
-    , m_suggestedName(suggestedName)
     , m_testSpeedMultiplier(session.testSpeedMultiplier())
 {
-    ASSERT(m_downloadID);
-
-    m_downloadManager.didCreateDownload();
+    downloadManager.didCreateDownload();
 }
 
 #if PLATFORM(COCOA)
+Ref<Download> Download::create(DownloadManager& downloadManager, DownloadID downloadID, NSURLSessionDownloadTask* download, NetworkSession& session, const String& suggestedName)
+{
+    return adoptRef(*new Download(downloadManager, downloadID, download, session, suggestedName));
+}
+
 Download::Download(DownloadManager& downloadManager, DownloadID downloadID, NSURLSessionDownloadTask* download, NetworkSession& session, const String& suggestedName)
     : m_downloadManager(downloadManager)
     , m_downloadID(downloadID)
     , m_client(downloadManager.client())
     , m_downloadTask(download)
     , m_sessionID(session.sessionID())
-    , m_suggestedName(suggestedName)
     , m_testSpeedMultiplier(session.testSpeedMultiplier())
 {
-    ASSERT(m_downloadID);
-
-    m_downloadManager.didCreateDownload();
+    downloadManager.didCreateDownload();
 }
 #endif
 
 Download::~Download()
 {
     platformDestroyDownload();
-    m_downloadManager.didDestroyDownload();
+    if (CheckedPtr downloadManager = m_downloadManager)
+        downloadManager->didDestroyDownload();
 }
 
-void Download::cancel(CompletionHandler<void(const IPC::DataReference&)>&& completionHandler, IgnoreDidFailCallback ignoreDidFailCallback)
+void Download::cancel(CompletionHandler<void(std::span<const uint8_t>)>&& completionHandler, IgnoreDidFailCallback ignoreDidFailCallback)
 {
     RELEASE_ASSERT(isMainRunLoop());
 
@@ -94,14 +103,16 @@ void Download::cancel(CompletionHandler<void(const IPC::DataReference&)>&& compl
     // completionHandler will inform the API that the cancellation succeeded.
     m_ignoreDidFailCallback = ignoreDidFailCallback;
 
-    auto completionHandlerWrapper = [this, weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)] (const IPC::DataReference& resumeData) mutable {
+    auto completionHandlerWrapper = [weakThis = WeakPtr { *this }, completionHandler = WTFMove(completionHandler)] (std::span<const uint8_t> resumeData) mutable {
         completionHandler(resumeData);
-        if (!weakThis || m_ignoreDidFailCallback == IgnoreDidFailCallback::No)
+        RefPtr protectedThis = weakThis.get();
+        if (!protectedThis || protectedThis->m_ignoreDidFailCallback == IgnoreDidFailCallback::No)
             return;
-        DOWNLOAD_RELEASE_LOG("didCancel: (id = %" PRIu64 ")", downloadID().toUInt64());
-        if (auto extension = std::exchange(m_sandboxExtension, nullptr))
+        DOWNLOAD_RELEASE_LOG_WITH_THIS(protectedThis.get(), "didCancel: (id = %" PRIu64 ")", protectedThis->downloadID().toUInt64());
+        if (auto extension = std::exchange(protectedThis->m_sandboxExtension, nullptr))
             extension->revoke();
-        m_downloadManager.downloadFinished(*this);
+        if (CheckedPtr downloadManager = protectedThis->m_downloadManager)
+            downloadManager->downloadFinished(*protectedThis);
     };
 
     if (m_download) {
@@ -130,12 +141,16 @@ void Download::didCreateDestination(const String& path)
 void Download::didReceiveData(uint64_t bytesWritten, uint64_t totalBytesWritten, uint64_t totalBytesExpectedToWrite)
 {
     if (!m_hasReceivedData) {
-        DOWNLOAD_RELEASE_LOG("didReceiveData: Started receiving data (id = %" PRIu64 ")", downloadID().toUInt64());
+        DOWNLOAD_RELEASE_LOG("didReceiveData: Started receiving data (id = %" PRIu64 ", expected length = %" PRIu64 ")", downloadID().toUInt64(), totalBytesExpectedToWrite);
         m_hasReceivedData = true;
     }
     
     m_monitor.downloadReceivedBytes(bytesWritten);
 
+#if HAVE(MODERN_DOWNLOADPROGRESS)
+    updateProgress(totalBytesWritten, totalBytesExpectedToWrite);
+#endif
+
     send(Messages::DownloadProxy::DidReceiveData(bytesWritten, totalBytesWritten, totalBytesExpectedToWrite));
 }
 
@@ -143,17 +158,24 @@ void Download::didFinish()
 {
     DOWNLOAD_RELEASE_LOG("didFinish: (id = %" PRIu64 ")", downloadID().toUInt64());
 
-    send(Messages::DownloadProxy::DidFinish());
+    platformDidFinish([weakThis = WeakPtr { *this }] {
+        RELEASE_ASSERT(isMainRunLoop());
+        RefPtr protectedThis = weakThis.get();
+        if (!protectedThis)
+            return;
+        protectedThis->send(Messages::DownloadProxy::DidFinish());
 
-    if (m_sandboxExtension) {
-        m_sandboxExtension->revoke();
-        m_sandboxExtension = nullptr;
-    }
+        if (protectedThis->m_sandboxExtension) {
+            protectedThis->m_sandboxExtension->revoke();
+            protectedThis->m_sandboxExtension = nullptr;
+        }
 
-    m_downloadManager.downloadFinished(*this);
+        if (CheckedPtr downloadManager = protectedThis->m_downloadManager)
+            downloadManager->downloadFinished(*protectedThis);
+    });
 }
 
-void Download::didFail(const ResourceError& error, const IPC::DataReference& resumeData)
+void Download::didFail(const ResourceError& error, std::span<const uint8_t> resumeData)
 {
     if (m_ignoreDidFailCallback == IgnoreDidFailCallback::Yes)
         return;
@@ -161,18 +183,26 @@ void Download::didFail(const ResourceError& error, const IPC::DataReference& res
     DOWNLOAD_RELEASE_LOG("didFail: (id = %" PRIu64 ", isTimeout = %d, isCancellation = %d, errCode = %d)",
         downloadID().toUInt64(), error.isTimeout(), error.isCancellation(), error.errorCode());
 
+#if HAVE(MODERN_DOWNLOADPROGRESS)
+    auto resumeDataWithPlaceholder = updateResumeDataWithPlaceholderURL(m_placeholderURL.get(), resumeData);
+    resumeData = resumeDataWithPlaceholder.span();
+#endif
+
     send(Messages::DownloadProxy::DidFail(error, resumeData));
 
     if (m_sandboxExtension) {
         m_sandboxExtension->revoke();
         m_sandboxExtension = nullptr;
     }
-    m_downloadManager.downloadFinished(*this);
+    if (CheckedPtr downloadManager = m_downloadManager)
+        downloadManager->downloadFinished(*this);
 }
 
 IPC::Connection* Download::messageSenderConnection() const
 {
-    return m_downloadManager.downloadProxyConnection();
+    if (CheckedPtr downloadManager = m_downloadManager)
+        return downloadManager->downloadProxyConnection();
+    return nullptr;
 }
 
 uint64_t Download::messageSenderDestinationID() const
@@ -181,7 +211,7 @@ uint64_t Download::messageSenderDestinationID() const
 }
 
 #if !PLATFORM(COCOA)
-void Download::platformCancelNetworkLoad(CompletionHandler<void(const IPC::DataReference&)>&& completionHandler)
+void Download::platformCancelNetworkLoad(CompletionHandler<void(std::span<const uint8_t>)>&& completionHandler)
 {
     completionHandler({ });
 }
@@ -189,6 +219,11 @@ void Download::platformCancelNetworkLoad(CompletionHandler<void(const IPC::DataR
 void Download::platformDestroyDownload()
 {
 }
+
+void Download::platformDidFinish(CompletionHandler<void()>&& completionHandler)
+{
+    completionHandler();
+}
 #endif
 
 } // namespace WebKit
