@@ -25,10 +25,9 @@
 
 #pragma once
 
+#include <span>
 #include <wtf/FileSystem.h>
-#include <wtf/FunctionDispatcher.h>
 #include <wtf/SHA1.h>
-#include <wtf/Span.h>
 #include <wtf/ThreadSafeRefCounted.h>
 #include <wtf/text/WTFString.h>
 
@@ -45,16 +44,18 @@
 #include <wtf/Box.h>
 #endif
 
-namespace WebKit {
-
+namespace WebCore {
 class SharedMemory;
+}
+
+namespace WebKit {
 
 namespace NetworkCache {
 
 class Data {
 public:
     Data() { }
-    Data(const uint8_t*, size_t);
+    Data(std::span<const uint8_t>);
 
     ~Data() { }
 
@@ -69,19 +70,19 @@ public:
     Data(GRefPtr<GBytes>&&, FileSystem::PlatformFileHandle fd = FileSystem::invalidPlatformFileHandle);
 #elif USE(CURL)
     Data(std::variant<Vector<uint8_t>, FileSystem::MappedFileData>&&);
+    Data(Vector<uint8_t>&& data) : Data(std::variant<Vector<uint8_t>, FileSystem::MappedFileData> { WTFMove(data) }) { }
 #endif
     bool isNull() const;
-    bool isEmpty() const { return !m_size; }
+    bool isEmpty() const { return !size(); }
 
-    const uint8_t* data() const;
-    size_t size() const { return m_size; }
-    Span<const uint8_t> span() const { return { data(), size() }; }
+    std::span<const uint8_t> span() const;
+    size_t size() const;
     bool isMap() const { return m_isMap; }
-    RefPtr<SharedMemory> tryCreateSharedMemory() const;
+    RefPtr<WebCore::SharedMemory> tryCreateSharedMemory() const;
 
     Data subrange(size_t offset, size_t) const;
 
-    bool apply(const Function<bool(Span<const uint8_t>)>&) const;
+    bool apply(NOESCAPE const Function<bool(std::span<const uint8_t>)>&) const;
 
     Data mapToFile(const String& path) const;
 
@@ -95,6 +96,7 @@ public:
 private:
 #if PLATFORM(COCOA)
     mutable OSObjectPtr<dispatch_data_t> m_dispatchData;
+    mutable std::span<const uint8_t> m_data;
 #endif
 #if USE(GLIB)
     mutable GRefPtr<GBytes> m_buffer;
@@ -103,15 +105,12 @@ private:
 #if USE(CURL)
     Box<std::variant<Vector<uint8_t>, FileSystem::MappedFileData>> m_buffer;
 #endif
-    mutable const uint8_t* m_data { nullptr };
-    size_t m_size { 0 };
     bool m_isMap { false };
 };
 
 Data concatenate(const Data&, const Data&);
 bool bytesEqual(const Data&, const Data&);
 Data adoptAndMapFile(FileSystem::PlatformFileHandle, size_t offset, size_t);
-Data mapFile(const char* path);
 Data mapFile(const String& path);
 
 using Salt = FileSystem::Salt;
