@@ -24,12 +24,13 @@
 #include <WebCore/CSSParser.h>
 
 /**
- * SECTION: WebKitColor
- * @Short_description: A boxed type representing a RGBA color
- * @Title: WebKitColor
- * @See_also: #WebKitWebView.
+ * WebKitColor:
+ * @red: Red channel, between 0.0 and 1.0 inclusive
+ * @green: Green channel, between 0.0 and 1.0 inclusive
+ * @blue: Blue channel, between 0.0 and 1.0 inclusive
+ * @alpha: Alpha channel, between 0.0 and 1.0 inclusive
  *
- * A WebKitColor is a boxed type representing a RGBA color.
+ * Boxed type representing a RGBA color.
  *
  * Since: 2.24
  */
@@ -108,7 +109,7 @@ gboolean webkit_color_parse(WebKitColor* color, const gchar* colorString)
     g_return_val_if_fail(color, FALSE);
     g_return_val_if_fail(colorString, FALSE);
 
-    auto webCoreColor = WebCore::CSSParser::parseColorWithoutContext({ colorString });
+    auto webCoreColor = WebCore::CSSParser::parseColorWithoutContext(String::fromLatin1(colorString));
     if (!webCoreColor.isValid())
         return FALSE;
 
