@@ -28,13 +28,11 @@
 
 #if ENABLE(MOMENTUM_EVENT_DISPATCHER)
 
-#include "EventDispatcher.h"
 #include "Logging.h"
-#include "WebProcess.h"
-#include "WebProcessProxyMessages.h"
-#include <WebCore/DisplayRefreshMonitor.h>
+#include <WebCore/AnimationFrameRate.h>
 #include <WebCore/Scrollbar.h>
 #include <wtf/SystemTracing.h>
+#include <wtf/TZoneMallocInlines.h>
 
 namespace WebKit {
 
@@ -43,9 +41,10 @@ static constexpr Seconds deltaHistoryMaximumInterval = 150_ms;
 static constexpr WebCore::FramesPerSecond idealCurveFrameRate = 60;
 static constexpr Seconds idealCurveFrameInterval = 1_s / idealCurveFrameRate;
 
-MomentumEventDispatcher::MomentumEventDispatcher(EventDispatcher& dispatcher)
-    : m_observerID(DisplayLinkObserverID::generate())
-    , m_dispatcher(dispatcher)
+WTF_MAKE_TZONE_ALLOCATED_IMPL(MomentumEventDispatcher);
+
+MomentumEventDispatcher::MomentumEventDispatcher(Client& client)
+    : m_client(client)
 {
 }
 
@@ -59,8 +58,8 @@ bool MomentumEventDispatcher::eventShouldStartSyntheticMomentumPhase(WebCore::Pa
     if (event.momentumPhase() != WebWheelEvent::PhaseBegan)
         return false;
 
-    auto curveIterator = m_accelerationCurves.find(pageIdentifier);
-    if (curveIterator == m_accelerationCurves.end() || !curveIterator->value) {
+    auto curveIterator = scrollingAccelerationCurveForPage(pageIdentifier);
+    if (!curveIterator) {
         RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher not using synthetic momentum phase: no acceleration curve");
         return false;
     }
@@ -73,7 +72,7 @@ bool MomentumEventDispatcher::eventShouldStartSyntheticMomentumPhase(WebCore::Pa
     return true;
 }
 
-bool MomentumEventDispatcher::handleWheelEvent(WebCore::PageIdentifier pageIdentifier, const WebWheelEvent& event, WebCore::RectEdges<bool> rubberBandableEdges)
+bool MomentumEventDispatcher::handleWheelEvent(WebCore::PageIdentifier pageIdentifier, const WebWheelEvent& event, WebCore::RectEdges<WebCore::RubberBandingBehavior> rubberBandableEdges)
 {
     m_lastRubberBandableEdges = rubberBandableEdges;
     m_lastIncomingEvent = event;
@@ -178,7 +177,7 @@ void MomentumEventDispatcher::dispatchSyntheticMomentumEvent(WebWheelEvent::Phas
     // FIXME: Ideally we would stick legitimate rawPlatformDeltas on the event,
     // but currently nothing will consume them, and we'd have to keep track of them separately.
     WebWheelEvent syntheticEvent(
-        WebEvent::Wheel,
+        { WebEventType::Wheel, m_lastIncomingEvent->modifiers(), time },
         m_currentGesture.initiatingEvent->position(),
         m_currentGesture.initiatingEvent->globalPosition(),
         appKitAcceleratedDelta,
@@ -190,12 +189,11 @@ void MomentumEventDispatcher::dispatchSyntheticMomentumEvent(WebWheelEvent::Phas
         true,
         m_currentGesture.initiatingEvent->scrollCount(),
         delta,
-        m_lastIncomingEvent->modifiers(),
-        time,
         time,
         { },
         WebWheelEvent::MomentumEndType::Unknown);
-    m_dispatcher.internalWheelEvent(m_currentGesture.pageIdentifier, syntheticEvent, m_lastRubberBandableEdges, EventDispatcher::WheelEventOrigin::MomentumEventDispatcher);
+
+    m_client->handleSyntheticWheelEvent(*m_currentGesture.pageIdentifier, syntheticEvent, m_lastRubberBandableEdges);
 
 #if ENABLE(MOMENTUM_EVENT_DISPATCHER_TEMPORARY_LOGGING)
     m_currentLogState.totalGeneratedOffset += appKitAcceleratedDelta.height();
@@ -217,12 +215,7 @@ void MomentumEventDispatcher::didStartMomentumPhase(WebCore::PageIdentifier page
     m_currentGesture.currentOffset = { };
     m_currentGesture.startTime = MonotonicTime::now();
     m_currentGesture.displayNominalFrameRate = displayProperties->nominalFrameRate;
-    m_currentGesture.accelerationCurve = [&] () -> std::optional<ScrollingAccelerationCurve> {
-        auto curveIterator = m_accelerationCurves.find(m_currentGesture.pageIdentifier);
-        if (curveIterator == m_accelerationCurves.end())
-            return { };
-        return curveIterator->value;
-    }();
+    m_currentGesture.accelerationCurve = scrollingAccelerationCurveForPage(pageIdentifier);
 
     startDisplayLink();
 
@@ -248,9 +241,7 @@ void MomentumEventDispatcher::didEndMomentumPhase()
 
 #if ENABLE(MOMENTUM_EVENT_DISPATCHER_TEMPORARY_LOGGING)
     RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher ending synthetic momentum phase with total offset %.1f %.1f, duration %f (event offset would have been %.1f %.1f) (tail index %d of %zu)", m_currentGesture.currentOffset.width(), m_currentGesture.currentOffset.height(), (MonotonicTime::now() - m_currentGesture.startTime).seconds(), m_currentGesture.accumulatedEventOffset.width(), m_currentGesture.accumulatedEventOffset.height(), m_currentGesture.currentTailDeltaIndex, m_currentGesture.tailDeltaTable.size());
-    m_dispatcher.queue().dispatchAfter(1_s, [this] {
-        flushLog();
-    });
+    m_client->flushMomentumEventLoggingSoon();
 #endif
 
     stopDisplayLink();
@@ -261,18 +252,28 @@ void MomentumEventDispatcher::didEndMomentumPhase()
 
 void MomentumEventDispatcher::setScrollingAccelerationCurve(WebCore::PageIdentifier pageIdentifier, std::optional<ScrollingAccelerationCurve> curve)
 {
+    Locker locker { m_accelerationCurvesLock };
     m_accelerationCurves.set(pageIdentifier, curve);
 
 #if ENABLE(MOMENTUM_EVENT_DISPATCHER_TEMPORARY_LOGGING)
     WTF::TextStream stream(WTF::TextStream::LineMode::SingleLine);
     stream << curve;
-    RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher set curve %{public}s", stream.release().utf8().data());
+    RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher set curve %" PUBLIC_LOG_STRING, stream.release().utf8().data());
 #endif
 }
 
+std::optional<ScrollingAccelerationCurve> MomentumEventDispatcher::scrollingAccelerationCurveForPage(WebCore::PageIdentifier pageIdentifier) const
+{
+    Locker locker { m_accelerationCurvesLock };
+
+    auto curveIterator = m_accelerationCurves.find(pageIdentifier);
+    if (curveIterator == m_accelerationCurves.end())
+        return { };
+    return curveIterator->value;
+}
+
 std::optional<MomentumEventDispatcher::DisplayProperties> MomentumEventDispatcher::displayProperties(WebCore::PageIdentifier pageIdentifier) const
 {
-    ASSERT(pageIdentifier);
     auto displayPropertiesIterator = m_displayProperties.find(pageIdentifier);
     if (displayPropertiesIterator == m_displayProperties.end())
         return std::nullopt;
@@ -281,14 +282,14 @@ std::optional<MomentumEventDispatcher::DisplayProperties> MomentumEventDispatche
 
 void MomentumEventDispatcher::startDisplayLink()
 {
-    auto displayProperties = this->displayProperties(m_currentGesture.pageIdentifier);
+    auto displayProperties = this->displayProperties(*m_currentGesture.pageIdentifier);
     if (!displayProperties) {
         RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher failed to start display link");
         return;
     }
 
     // FIXME: Switch down to lower-than-full-speed frame rates for the tail end of the curve.
-    WebProcess::singleton().parentProcessConnection()->send(Messages::WebProcessProxy::StartDisplayLink(m_observerID, displayProperties->displayID, WebCore::FullSpeedFramesPerSecond), 0);
+    m_client->startDisplayDidRefreshCallbacks(displayProperties->displayID);
 #if ENABLE(MOMENTUM_EVENT_DISPATCHER_TEMPORARY_LOGGING)
     RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher starting display link for display %d", displayProperties->displayID);
 #endif
@@ -296,13 +297,16 @@ void MomentumEventDispatcher::startDisplayLink()
 
 void MomentumEventDispatcher::stopDisplayLink()
 {
-    auto displayProperties = this->displayProperties(m_currentGesture.pageIdentifier);
+    if (!m_currentGesture.active)
+        return;
+
+    auto displayProperties = this->displayProperties(*m_currentGesture.pageIdentifier);
     if (!displayProperties) {
         RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher failed to stop display link");
         return;
     }
 
-    WebProcess::singleton().parentProcessConnection()->send(Messages::WebProcessProxy::StopDisplayLink(m_observerID, displayProperties->displayID), 0);
+    m_client->stopDisplayDidRefreshCallbacks(displayProperties->displayID);
 #if ENABLE(MOMENTUM_EVENT_DISPATCHER_TEMPORARY_LOGGING)
     RELEASE_LOG(ScrollAnimations, "MomentumEventDispatcher stopping display link for display %d", displayProperties->displayID);
 #endif
@@ -346,12 +350,12 @@ std::optional<WebCore::FloatSize> MomentumEventDispatcher::consumeDeltaForCurren
     return delta;
 }
 
-void MomentumEventDispatcher::displayWasRefreshed(WebCore::PlatformDisplayID displayID, const WebCore::DisplayUpdate&)
+void MomentumEventDispatcher::displayDidRefresh(WebCore::PlatformDisplayID displayID)
 {
     if (!m_currentGesture.active)
         return;
 
-    auto displayProperties = this->displayProperties(m_currentGesture.pageIdentifier);
+    auto displayProperties = this->displayProperties(*m_currentGesture.pageIdentifier);
     if (!displayProperties || displayID != displayProperties->displayID)
         return;
 
@@ -462,16 +466,16 @@ void MomentumEventDispatcher::equalizeTailGaps()
         return;
 
     enum Axis { Horizontal, Vertical };
-    Vector<float> deltas[2];
-    unsigned firstZeroIndex[2] = { 0, 0 };
+    std::array<Vector<float>, 2> deltas;
+    std::array<unsigned, 2> firstZeroIndex { 0, 0 };
     deltas[Horizontal].reserveInitialCapacity(initialTableSize);
     deltas[Vertical].reserveInitialCapacity(initialTableSize);
     for (unsigned i = 0; i < initialTableSize; i++) {
-        deltas[Horizontal].uncheckedAppend(table[i].width());
+        deltas[Horizontal].append(table[i].width());
         if (!firstZeroIndex[Horizontal] && !table[i].width())
             firstZeroIndex[Horizontal] = i;
 
-        deltas[Vertical].uncheckedAppend(table[i].height());
+        deltas[Vertical].append(table[i].height());
         if (!firstZeroIndex[Vertical] && !table[i].height())
             firstZeroIndex[Vertical] = i;
     }
@@ -490,11 +494,11 @@ void MomentumEventDispatcher::equalizeTailGaps()
     sortDeltas(Vertical);
 
     // GapSize is a count of contiguous frames with zero deltas.
-    typedef unsigned GapSize[2];
-    GapSize minimumGap = { 0, 0 };
-    GapSize currentGap = { 0, 0 };
-    GapSize remainingGapToGenerate = { 0, 0 };
-    unsigned originalTableIndex[2] = { 0, 0 };
+    typedef std::array<unsigned, 2> GapSize;
+    GapSize minimumGap { 0, 0 };
+    GapSize currentGap { 0, 0 };
+    GapSize remainingGapToGenerate { 0, 0 };
+    std::array<unsigned, 2> originalTableIndex { 0, 0 };
 
     auto takeNextDelta = [&] (uint8_t axis) -> float {
         if (originalTableIndex[axis] >= initialTableSize)
