@@ -36,7 +36,18 @@
 #endif
 
 #if USE(GSTREAMER)
-#include <gst/gst.h>
+#include <WebCore/GStreamerCommon.h>
+#endif
+
+#if USE(SKIA)
+#include <skia/core/SkGraphics.h>
+#endif
+
+#if USE(SYSPROF_CAPTURE)
+#include <wtf/SystemTracing.h>
+#if USE(SKIA_OPENTYPE_SVG)
+#include <skia/modules/svg/SkSVGOpenTypeSVGDecoder.h>
+#endif
 #endif
 
 namespace WebKit {
@@ -46,10 +57,21 @@ class WebProcessMainWPE final : public AuxiliaryProcessMainBase<WebProcess> {
 public:
     bool platformInitialize() override
     {
+#if USE(SYSPROF_CAPTURE)
+        SysprofAnnotator::createIfNeeded("WebKit (Web)"_s);
+#endif
+
 #if USE(GCRYPT)
         PAL::GCrypt::initialize();
 #endif
 
+#if USE(SKIA)
+        SkGraphics::Init();
+#if USE(SKIA_OPENTYPE_SVG)
+        SkGraphics::SetOpenTypeSVGDecoderFactory(SkSVGOpenTypeSVGDecoder::Make);
+#endif
+#endif
+
 #if ENABLE(DEVELOPER_MODE)
         if (g_getenv("WEBKIT2_PAUSE_WEB_PROCESS_ON_LAUNCH"))
             WTF::sleep(30_s);
@@ -65,7 +87,7 @@ public:
     void platformFinalize() override
     {
 #if USE(GSTREAMER)
-        gst_deinit();
+        deinitializeGStreamer();
 #endif
     }
 };
