@@ -54,6 +54,8 @@ bool pas_page_malloc_decommit_zero_fill = false;
 
 #if PAS_OS(DARWIN)
 #define PAS_VM_TAG VM_MAKE_TAG(VM_MEMORY_TCMALLOC)
+#elif PAS_PLATFORM(PLAYSTATION) && defined(VM_MAKE_TAG)
+#define PAS_VM_TAG VM_MAKE_TAG(VM_TYPE_USER1)
 #else
 #define PAS_VM_TAG -1
 #endif
@@ -83,11 +85,30 @@ PAS_NEVER_INLINE size_t pas_page_malloc_alignment_shift_slow(void)
     return result;
 }
 
+static void*
+pas_page_malloc_try_map_pages(size_t size, bool may_contain_small_or_medium)
+{
+    void* mmap_result = NULL;
+
+    PAS_PROFILE(PAGE_ALLOCATION, size, may_contain_small_or_medium, PAS_VM_TAG);
+
+    mmap_result = mmap(NULL, size, PROT_READ | PROT_WRITE, MAP_PRIVATE | MAP_ANON | PAS_NORESERVE, PAS_VM_TAG, 0);
+    if (mmap_result == MAP_FAILED) {
+        errno = 0; /* Clear the error so that we don't leak errno in those
+                      cases where we handle the allocation failure
+                      internally. If we want to set errno for clients then we
+                      do that explicitly. */
+        return NULL;
+    }
+
+    return mmap_result;
+}
+
 pas_aligned_allocation_result
 pas_page_malloc_try_allocate_without_deallocating_padding(
-    size_t size, pas_alignment alignment)
+    size_t size, pas_alignment alignment, bool may_contain_small_or_medium)
 {
-    static const bool verbose = false;
+    static const bool verbose = PAS_SHOULD_LOG(PAS_LOG_OTHER);
     
     size_t aligned_size;
     size_t mapped_size;
@@ -119,21 +140,13 @@ pas_page_malloc_try_allocate_without_deallocating_padding(
         if (__builtin_add_overflow(page_allocation_alignment, aligned_size, &mapped_size))
             return result;
     }
-    
-#if PAS_PLATFORM(PLAYSTATION)
-    mmap_result = mmap_np(NULL, mapped_size, PROT_READ | PROT_WRITE,
-                          MAP_PRIVATE | MAP_ANON | PAS_NORESERVE, PAS_VM_TAG, 0, "SceNKLibpas");
-#else
-    mmap_result = mmap(NULL, mapped_size, PROT_READ | PROT_WRITE,
-                       MAP_PRIVATE | MAP_ANON | PAS_NORESERVE, PAS_VM_TAG, 0);
-#endif
-    if (mmap_result == MAP_FAILED) {
-        errno = 0; /* Clear the error so that we don't leak errno in those
-                      cases where we handle the allocation failure
-                      internally. If we want to set errno for clients then we
-                      do that explicitly. */
+
+    mmap_result = pas_page_malloc_try_map_pages(mapped_size, may_contain_small_or_medium);
+    if (!mmap_result)
         return result;
-    }
+
+    uintptr_t pages_begin = (uintptr_t)mmap_result;
+    mmap_result = (void*)pages_begin;
     
     mapped = (char*)mmap_result;
     mapped_end = mapped + mapped_size;
@@ -184,12 +197,10 @@ void pas_page_malloc_zero_fill(void* base, size_t size)
     PAS_ASSERT(pas_is_aligned((uintptr_t)base, page_size));
     PAS_ASSERT(pas_is_aligned(size, page_size));
 
-    result_ptr = mmap(base,
-                      size,
-                      PROT_READ | PROT_WRITE,
-                      MAP_PRIVATE | MAP_ANON | MAP_FIXED | PAS_NORESERVE,
-                      PAS_VM_TAG,
-                      0);
+    int flags = MAP_PRIVATE | MAP_ANON | MAP_FIXED | PAS_NORESERVE;
+    int tag = PAS_VM_TAG;
+    PAS_PROFILE(ZERO_FILL_PAGE, base, size, flags, tag);
+    result_ptr = mmap(base, size, PROT_READ | PROT_WRITE, flags, tag, 0);
     PAS_ASSERT(result_ptr == base);
 }
 
@@ -238,7 +249,7 @@ static void decommit_impl(void* ptr, size_t size,
                           bool do_mprotect,
                           pas_mmap_capability mmap_capability)
 {
-    static const bool verbose = false;
+    static const bool verbose = PAS_SHOULD_LOG(PAS_LOG_OTHER);
     
     uintptr_t base_as_int;
     uintptr_t end_as_int;
@@ -290,8 +301,10 @@ void pas_page_malloc_deallocate(void* ptr, size_t size)
     uintptr_t ptr_as_int;
     
     ptr_as_int = (uintptr_t)ptr;
+    PAS_PROFILE(PAGE_DEALLOCATION, ptr_as_int);
     PAS_ASSERT(pas_is_aligned(ptr_as_int, pas_page_malloc_alignment()));
     PAS_ASSERT(pas_is_aligned(size, pas_page_malloc_alignment()));
+    ptr = (void*)ptr_as_int;
     
     if (!size)
         return;
