@@ -43,7 +43,7 @@ bool eq(const Vector<uint8_t>& cbor, const CString& expect)
 {
     if (cbor.size() != expect.length())
         return false;
-    return !memcmp(cbor.data(), reinterpret_cast<const uint8_t*>(expect.data()), cbor.size());
+    return !memcmp(cbor.data(), expect.data(), cbor.size());
 }
 
 bool eq(const Vector<uint8_t>& cbor, const uint8_t* expect, const size_t expectLength)
@@ -63,20 +63,20 @@ TEST(CBORWriterTest, TestWriteUint)
     static const UintTestCase kUintTestCases[] = {
         // Reminder: must specify length when creating string pieces
         // with null bytes, else the string will truncate prematurely.
-        {0, CString("\x00", 1)},
-        {1, CString("\x01")},
-        {10, CString("\x0a")},
-        {23, CString("\x17")},
-        {24, CString("\x18\x18")},
-        {25, CString("\x18\x19")},
-        {100, CString("\x18\x64")},
-        {1000, CString("\x19\x03\xe8")},
-        {1000000, CString("\x1a\x00\x0f\x42\x40", 5)},
-        {0xFFFFFFFF, CString("\x1a\xff\xff\xff\xff")},
-        {0x100000000,
-            CString("\x1b\x00\x00\x00\x01\x00\x00\x00\x00", 9)},
-        {std::numeric_limits<int64_t>::max(),
-            CString("\x1b\x7f\xff\xff\xff\xff\xff\xff\xff")}
+        { 0, CString({ "\x00", 1 }) },
+        { 1, CString("\x01") },
+        { 10, CString("\x0a") },
+        { 23, CString("\x17") },
+        { 24, CString("\x18\x18") },
+        { 25, CString("\x18\x19") },
+        { 100, CString("\x18\x64") },
+        { 1000, CString("\x19\x03\xe8") },
+        { 1000000, CString({ "\x1a\x00\x0f\x42\x40", 5 }) },
+        { 0xFFFFFFFF, CString("\x1a\xff\xff\xff\xff") },
+        { 0x100000000,
+            CString({ "\x1b\x00\x00\x00\x01\x00\x00\x00\x00", 9 }) },
+        { std::numeric_limits<int64_t>::max(),
+            CString("\x1b\x7f\xff\xff\xff\xff\xff\xff\xff") }
     };
 
     for (const UintTestCase& testCase : kUintTestCases) {
@@ -101,7 +101,7 @@ TEST(CBORWriterTest, TestWriteNegativeInteger)
         { -1000LL, CString("\x39\x03\xe7") },
         { -4294967296LL, CString("\x3a\xff\xff\xff\xff") },
         { -4294967297LL,
-            CString("\x3b\x00\x00\x00\x01\x00\x00\x00\x00", 9) },
+            CString({ "\x3b\x00\x00\x00\x01\x00\x00\x00\x00", 9 }) },
         { std::numeric_limits<int64_t>::min(),
             CString("\x3b\x7f\xff\xff\xff\xff\xff\xff\xff") },
     };
@@ -140,10 +140,10 @@ TEST(CBORWriterTest, TestWriteString)
     } StringTestCase;
 
     static const StringTestCase kStringTestCases[] = {
-        { "", CString("\x60") },
-        { "a", CString("\x61\x61") },
-        { "IETF", CString("\x64\x49\x45\x54\x46") },
-        { "\"\\", CString("\x62\x22\x5c") },
+        { emptyString(), CString("\x60") },
+        { "a"_s, CString("\x61\x61") },
+        { "IETF"_s, CString("\x64\x49\x45\x54\x46") },
+        { "\"\\"_s, CString("\x62\x22\x5c") },
         { String::fromUTF8("\xc3\xbc"), CString("\x62\xc3\xbc") },
         { String::fromUTF8("\xe6\xb0\xb4"), CString("\x63\xe6\xb0\xb4") },
         { String::fromUTF8("\xf0\x90\x85\x91"), CString("\x64\xf0\x90\x85\x91") }
