@@ -36,34 +36,40 @@ namespace WebCore {
 std::ostream& operator<<(std::ostream& os, MarkedText::Type type)
 {
     switch (type) {
-    case MarkedText::Correction:
+    case MarkedText::Type::Correction:
         return os << "Correction";
-    case MarkedText::DictationAlternatives:
+    case MarkedText::Type::DictationAlternatives:
         return os << "DictationAlternatives";
 #if PLATFORM(IOS_FAMILY)
     // FIXME: See <rdar://problem/8933352>. Also, remove the PLATFORM(IOS_FAMILY)-guard.
-    case MarkedText::DictationPhraseWithAlternatives:
+    case MarkedText::Type::DictationPhraseWithAlternatives:
         return os << "DictationPhraseWithAlternatives";
 #endif
-    case MarkedText::DraggedContent:
+    case MarkedText::Type::DraggedContent:
         return os << "DraggedContent";
-    case MarkedText::GrammarError:
+    case MarkedText::Type::GrammarError:
         return os << "GrammarError";
-    case MarkedText::Selection:
+    case MarkedText::Type::Selection:
         return os << "Selection";
-    case MarkedText::SpellingError:
+    case MarkedText::Type::SpellingError:
         return os << "SpellingError";
-    case MarkedText::TextMatch:
+    case MarkedText::Type::TextMatch:
         return os << "TextMatch";
-    case MarkedText::Highlight:
+    case MarkedText::Type::Highlight:
         return os << "Highlight";
-    case MarkedText::FragmentHighlight:
+    case MarkedText::Type::FragmentHighlight:
         return os << "FragmentHighlight";
 #if ENABLE(APP_HIGHLIGHTS)
-    case MarkedText::AppHighlight:
+    case MarkedText::Type::AppHighlight:
         return os << "AppHighlight";
 #endif
-    case MarkedText::Unmarked:
+#if ENABLE(WRITING_TOOLS)
+    case MarkedText::Type::WritingToolsTextSuggestion:
+        return os << "WritingToolsTextSuggestion";
+#endif
+    case MarkedText::Type::TransparentContent:
+        return os << "TransparentContent";
+    case MarkedText::Type::Unmarked:
         return os << "Unmarked";
     }
 }
@@ -88,7 +94,7 @@ TEST(MarkedText, SubdivideEmpty)
 
 TEST(MarkedText, SubdivideSimple)
 {
-    MarkedText markedText { 0, 9, MarkedText::SpellingError };
+    MarkedText markedText { 0, 9, MarkedText::Type::SpellingError };
     auto results = MarkedText::subdivide({ markedText });
     ASSERT_EQ(1U, results.size());
     EXPECT_EQ(markedText, results[0]);
@@ -96,10 +102,10 @@ TEST(MarkedText, SubdivideSimple)
 
 TEST(MarkedText, SubdivideSpellingAndGrammarSimple)
 {
-    RenderedDocumentMarker grammarErrorMarker { DocumentMarker { DocumentMarker::Grammar, { 7, 8 } } };
+    RenderedDocumentMarker grammarErrorMarker { DocumentMarker { DocumentMarkerType::Grammar, { 7, 8 } } };
     Vector<MarkedText> expectedMarkedTexts {
-        MarkedText { grammarErrorMarker.startOffset(), grammarErrorMarker.endOffset(), MarkedText::GrammarError, &grammarErrorMarker },
-        MarkedText { 22, 32, MarkedText::SpellingError },
+        MarkedText { grammarErrorMarker.startOffset(), grammarErrorMarker.endOffset(), MarkedText::Type::GrammarError, &grammarErrorMarker },
+        MarkedText { 22, 32, MarkedText::Type::SpellingError },
     };
     auto results = MarkedText::subdivide(expectedMarkedTexts);
     ASSERT_EQ(expectedMarkedTexts.size(), results.size());
@@ -110,19 +116,19 @@ TEST(MarkedText, SubdivideSpellingAndGrammarSimple)
 TEST(MarkedText, SubdivideSpellingAndGrammarOverlap)
 {
     Vector<MarkedText> markedTexts {
-        MarkedText { 0, 40, MarkedText::GrammarError },
-        MarkedText { 2, 17, MarkedText::SpellingError },
-        MarkedText { 20, 40, MarkedText::SpellingError },
-        MarkedText { 41, 45, MarkedText::SpellingError },
+        MarkedText { 0, 40, MarkedText::Type::GrammarError },
+        MarkedText { 2, 17, MarkedText::Type::SpellingError },
+        MarkedText { 20, 40, MarkedText::Type::SpellingError },
+        MarkedText { 41, 45, MarkedText::Type::SpellingError },
     };
     Vector<MarkedText> expectedMarkedTexts {
-        MarkedText { 0, 2, MarkedText::GrammarError },
-        MarkedText { 2, 17, MarkedText::GrammarError },
-        MarkedText { 2, 17, MarkedText::SpellingError },
-        MarkedText { 17, 20, MarkedText::GrammarError },
-        MarkedText { 20, 40, MarkedText::GrammarError },
-        MarkedText { 20, 40, MarkedText::SpellingError },
-        MarkedText { 41, 45, MarkedText::SpellingError },
+        MarkedText { 0, 2, MarkedText::Type::GrammarError },
+        MarkedText { 2, 17, MarkedText::Type::GrammarError },
+        MarkedText { 2, 17, MarkedText::Type::SpellingError },
+        MarkedText { 17, 20, MarkedText::Type::GrammarError },
+        MarkedText { 20, 40, MarkedText::Type::GrammarError },
+        MarkedText { 20, 40, MarkedText::Type::SpellingError },
+        MarkedText { 41, 45, MarkedText::Type::SpellingError },
     };
     auto results = MarkedText::subdivide(markedTexts);
     ASSERT_EQ(expectedMarkedTexts.size(), results.size());
@@ -133,17 +139,17 @@ TEST(MarkedText, SubdivideSpellingAndGrammarOverlap)
 TEST(MarkedText, SubdivideSpellingAndGrammarOverlapFrontmost)
 {
     Vector<MarkedText> markedTexts {
-        MarkedText { 0, 40, MarkedText::GrammarError },
-        MarkedText { 2, 17, MarkedText::SpellingError },
-        MarkedText { 20, 40, MarkedText::SpellingError },
-        MarkedText { 41, 45, MarkedText::SpellingError },
+        MarkedText { 0, 40, MarkedText::Type::GrammarError },
+        MarkedText { 2, 17, MarkedText::Type::SpellingError },
+        MarkedText { 20, 40, MarkedText::Type::SpellingError },
+        MarkedText { 41, 45, MarkedText::Type::SpellingError },
     };
     Vector<MarkedText> expectedMarkedTexts {
-        MarkedText { 0, 2, MarkedText::GrammarError },
-        MarkedText { 2, 17, MarkedText::SpellingError },
-        MarkedText { 17, 20, MarkedText::GrammarError },
-        MarkedText { 20, 40, MarkedText::SpellingError },
-        MarkedText { 41, 45, MarkedText::SpellingError },
+        MarkedText { 0, 2, MarkedText::Type::GrammarError },
+        MarkedText { 2, 17, MarkedText::Type::SpellingError },
+        MarkedText { 17, 20, MarkedText::Type::GrammarError },
+        MarkedText { 20, 40, MarkedText::Type::SpellingError },
+        MarkedText { 41, 45, MarkedText::Type::SpellingError },
     };
     auto results = MarkedText::subdivide(markedTexts, MarkedText::OverlapStrategy::Frontmost);
     ASSERT_EQ(expectedMarkedTexts.size(), results.size());
@@ -154,27 +160,27 @@ TEST(MarkedText, SubdivideSpellingAndGrammarOverlapFrontmost)
 TEST(MarkedText, SubdivideSpellingAndGrammarComplicatedFrontmost)
 {
     Vector<MarkedText> markedTexts {
-        MarkedText { 0, 6, MarkedText::SpellingError },
-        MarkedText { 0, 46, MarkedText::GrammarError },
-        MarkedText { 7, 16, MarkedText::SpellingError },
-        MarkedText { 22, 27, MarkedText::SpellingError },
-        MarkedText { 34, 44, MarkedText::SpellingError },
-        MarkedText { 46, 50, MarkedText::SpellingError },
-        MarkedText { 51, 58, MarkedText::SpellingError },
-        MarkedText { 59, 63, MarkedText::GrammarError },
+        MarkedText { 0, 6, MarkedText::Type::SpellingError },
+        MarkedText { 0, 46, MarkedText::Type::GrammarError },
+        MarkedText { 7, 16, MarkedText::Type::SpellingError },
+        MarkedText { 22, 27, MarkedText::Type::SpellingError },
+        MarkedText { 34, 44, MarkedText::Type::SpellingError },
+        MarkedText { 46, 50, MarkedText::Type::SpellingError },
+        MarkedText { 51, 58, MarkedText::Type::SpellingError },
+        MarkedText { 59, 63, MarkedText::Type::GrammarError },
     };
     Vector<MarkedText> expectedMarkedTexts {
-        MarkedText { 0, 6, MarkedText::SpellingError },
-        MarkedText { 6, 7, MarkedText::GrammarError },
-        MarkedText { 7, 16, MarkedText::SpellingError },
-        MarkedText { 16, 22, MarkedText::GrammarError },
-        MarkedText { 22, 27, MarkedText::SpellingError },
-        MarkedText { 27, 34, MarkedText::GrammarError },
-        MarkedText { 34, 44, MarkedText::SpellingError },
-        MarkedText { 44, 46, MarkedText::GrammarError },
-        MarkedText { 46, 50, MarkedText::SpellingError },
-        MarkedText { 51, 58, MarkedText::SpellingError },
-        MarkedText { 59, 63, MarkedText::GrammarError },
+        MarkedText { 0, 6, MarkedText::Type::SpellingError },
+        MarkedText { 6, 7, MarkedText::Type::GrammarError },
+        MarkedText { 7, 16, MarkedText::Type::SpellingError },
+        MarkedText { 16, 22, MarkedText::Type::GrammarError },
+        MarkedText { 22, 27, MarkedText::Type::SpellingError },
+        MarkedText { 27, 34, MarkedText::Type::GrammarError },
+        MarkedText { 34, 44, MarkedText::Type::SpellingError },
+        MarkedText { 44, 46, MarkedText::Type::GrammarError },
+        MarkedText { 46, 50, MarkedText::Type::SpellingError },
+        MarkedText { 51, 58, MarkedText::Type::SpellingError },
+        MarkedText { 59, 63, MarkedText::Type::GrammarError },
     };
     auto results = MarkedText::subdivide(markedTexts, MarkedText::OverlapStrategy::Frontmost);
     ASSERT_EQ(expectedMarkedTexts.size(), results.size());
@@ -185,17 +191,17 @@ TEST(MarkedText, SubdivideSpellingAndGrammarComplicatedFrontmost)
 TEST(MarkedText, SubdivideGrammarAndSelectionOverlap)
 {
     Vector<MarkedText> markedTexts {
-        MarkedText { 0, 40, MarkedText::GrammarError },
-        MarkedText { 2, 60, MarkedText::Selection },
-        MarkedText { 50, 60, MarkedText::GrammarError },
+        MarkedText { 0, 40, MarkedText::Type::GrammarError },
+        MarkedText { 2, 60, MarkedText::Type::Selection },
+        MarkedText { 50, 60, MarkedText::Type::GrammarError },
     };
     Vector<MarkedText> expectedMarkedTexts {
-        MarkedText { 0, 2, MarkedText::GrammarError },
-        MarkedText { 2, 40, MarkedText::GrammarError },
-        MarkedText { 2, 40, MarkedText::Selection },
-        MarkedText { 40, 50, MarkedText::Selection },
-        MarkedText { 50, 60, MarkedText::GrammarError },
-        MarkedText { 50, 60, MarkedText::Selection },
+        MarkedText { 0, 2, MarkedText::Type::GrammarError },
+        MarkedText { 2, 40, MarkedText::Type::GrammarError },
+        MarkedText { 2, 40, MarkedText::Type::Selection },
+        MarkedText { 40, 50, MarkedText::Type::Selection },
+        MarkedText { 50, 60, MarkedText::Type::GrammarError },
+        MarkedText { 50, 60, MarkedText::Type::Selection },
     };
     auto results = MarkedText::subdivide(markedTexts);
     ASSERT_EQ(expectedMarkedTexts.size(), results.size());
@@ -206,15 +212,15 @@ TEST(MarkedText, SubdivideGrammarAndSelectionOverlap)
 TEST(MarkedText, SubdivideGrammarAndSelectionOverlapFrontmost)
 {
     Vector<MarkedText> markedTexts {
-        MarkedText { 0, 40, MarkedText::GrammarError },
-        MarkedText { 2, 60, MarkedText::Selection },
-        MarkedText { 50, 60, MarkedText::GrammarError },
+        MarkedText { 0, 40, MarkedText::Type::GrammarError },
+        MarkedText { 2, 60, MarkedText::Type::Selection },
+        MarkedText { 50, 60, MarkedText::Type::GrammarError },
     };
     Vector<MarkedText> expectedMarkedTexts {
-        MarkedText { 0, 2, MarkedText::GrammarError },
-        MarkedText { 2, 40, MarkedText::Selection },
-        MarkedText { 40, 50, MarkedText::Selection },
-        MarkedText { 50, 60, MarkedText::Selection },
+        MarkedText { 0, 2, MarkedText::Type::GrammarError },
+        MarkedText { 2, 40, MarkedText::Type::Selection },
+        MarkedText { 40, 50, MarkedText::Type::Selection },
+        MarkedText { 50, 60, MarkedText::Type::Selection },
     };
     auto results = MarkedText::subdivide(markedTexts, MarkedText::OverlapStrategy::Frontmost);
     ASSERT_EQ(expectedMarkedTexts.size(), results.size());
